<!--

   The Contents of this file are made available subject to the terms of
   either of the following licenses

          - GNU Lesser General Public License Version 2.1
          - Sun Industry Standards Source License Version 1.1

   Sun Microsystems Inc., October, 2000

   GNU Lesser General Public License Version 2.1
   =============================================
   Copyright 2000 by Sun Microsystems, Inc.
   901 San Antonio Road, Palo Alto, CA 94303, USA

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License version 2.1, as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA  02111-1307  USA


   Sun Industry Standards Source License Version 1.1
   =================================================
   The contents of this file are subject to the Sun Industry Standards
   Source License Version 1.1 (the "License"); You may not use this file
   except in compliance with the License. You may obtain a copy of the
   License at http://www.openoffice.org/license.html.

   Software provided under this License is provided on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
   WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
   See the License for the specific provisions governing your rights and
   obligations concerning the Software.

   The Initial Developer of the Original Code is: Sun Microsystems, Inc.

   Copyright: 2000 by Sun Microsystems, Inc.

   All Rights Reserved.

   Contributor(s): _______________________________________

-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xt="http://www.jclark.com/xt"
                xmlns:office="http://openoffice.org/2000/office"
                xmlns:style="http://openoffice.org/2000/style"
                xmlns:text="http://openoffice.org/2000/text"
                xmlns:table="http://openoffice.org/2000/table"
                xmlns:draw="http://openoffice.org/2000/drawing"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:number="http://openoffice.org/2000/datastyle"
                xmlns:svg="http://www.w3.org/2000/svg"
                xmlns:chart="http://openoffice.org/2000/chart"
                xmlns:dr3d="http://openoffice.org/2000/dr3d"
                xmlns:math="http://www.w3.org/1998/Math/MathML"
                xmlns:form="http://openoffice.org/2000/form"
                xmlns:script="http://openoffice.org/2000/script"
                office:class="text"
                office:version="1.0"
                xmlns:dc="http://purl.org/dc/elements/1.1/"
                xmlns:meta="http://openoffice.org/2000/meta"
                xmlns:config="http://openoffice.org/2001/config"
                xmlns:help="http://openoffice.org/2000/help">



    <!-- ******************* -->
    <!-- *** main table  *** -->
    <!-- ******************* -->

    <!-- creation of the main table -->
    <xsl:template match="table:table">
        <xsl:call-template name="create-table">
            <xsl:with-param name="isMaintable" select="true()"/>
        </xsl:call-template>
    </xsl:template>

    <!-- creation of an inner table -->
    <xsl:template match="table:sub-table">
        <xsl:call-template name="create-table"/>
    </xsl:template>



    <xsl:template name="create-table">
        <xsl:param name="isMaintable"/>
        <!-- a table will only be created if scenario is active -->
        <xsl:if test="string-length(table:scenario/@table:is-active) = 0">
            <xsl:choose>
                <!-- no table for WAP/WML devices  -->
                <xsl:when test="not($outputType = 'WML')">
                    <xsl:element name="table">
                        <xsl:if test="$isMaintable">
                            <xsl:call-template name="check-for-table-border"/>
                        </xsl:if>
                        <xsl:call-template name="write-table">
                            <xsl:with-param name="isMaintable" select="$isMaintable"/>
                        </xsl:call-template>
                    </xsl:element>
                </xsl:when>
                <xsl:otherwise>
                <!-- only ASCII table written for WML devices as tables are not implemented widley
                     from 'repeat-write-row' the templates are handled by the main_wap.xsl stylesheet -->
                    <xsl:call-template name="write-table">
                        <xsl:with-param name="isMaintable" select="$isMaintable"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>


    <!-- only table border for HTML4 or CSS devices which contain one or more 'fo:border-top' attributes (if one exist the other exist, too) -->
    <xsl:template name="check-for-table-border">

            <!-- check this table styles for a certain table-cell style (e.g. fo:border-top)
                 if a single border element in the table exist, the whole table will get border (better browser display) -->
            <xsl:if test="descendant-or-self::*[@table:style-name=/office:document-content/*/style:style[style:properties/@fo:border-top]/@style:name]">
                <xsl:attribute name="border">1</xsl:attribute>
                <xsl:attribute name="bordercolor">#000000</xsl:attribute>
                <xsl:attribute name="cellpadding">2</xsl:attribute>
                <xsl:attribute name="cellspacing">0</xsl:attribute>
                <xsl:attribute name="page-break-inside">page-break-inside:avoid</xsl:attribute>
            </xsl:if>
    </xsl:template>


    <xsl:template name="write-table">
        <xsl:param name="isMaintable"/>
        <!--
            creates a variable and whether reads out the column styles or signes a column as hidden:
                - if a column is hidden it contains only the hidden attribute otherwise the style-properties will be stored
                - if a style of a column is being repeated, the styles in the variable will be iterated.
                  Later the position() of the variable column can be compared with the implicit cell number (cell of a certain column).
                  (result tree fragment)
        -->
        <xsl:variable name="all-column-styles-RTF">
            <xsl:choose>
                <xsl:when test="$isMaintable">
                    <!-- for all capsulated table:table-column, but not the columns of a subtable -->
                    <xsl:for-each select="descendant::table:table-column[not(ancestor::table:sub-table)]">
                        <xsl:choose>
                            <xsl:when test="string-length(@table:number-columns-repeated) = 0">
                                <!-- writes an entry of a column in the columns-variable -->
                                <xsl:call-template name="write-columns-variable-entry">
                                    <xsl:with-param name="column-style-attributes" select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <!-- repeated colums will be written explicit several times in the variable-->
                                <xsl:call-template name="repeat-write-columns-variable-entry">
                                    <xsl:with-param name="column-style-attributes" select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                                    <xsl:with-param name="number-columns-repeated" select="@table:number-columns-repeated"/>
                                </xsl:call-template>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <!-- for all by another table surrounded table:table-column-->
                    <xsl:for-each select="descendant::table:table-column">
                        <xsl:choose>
                            <xsl:when test="string-length(@table:number-columns-repeated) = 0">
                                <!-- writes an entry of a column in the columns-variable -->
                                <xsl:call-template name="write-columns-variable-entry">
                                    <xsl:with-param name="column-style-attributes" select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <!-- repeated colums will be written explicit several times in the variable-->
                                <xsl:call-template name="repeat-write-columns-variable-entry">
                                    <xsl:with-param name="column-style-attributes" select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                                    <xsl:with-param name="number-columns-repeated" select="@table:number-columns-repeated"/>
                                </xsl:call-template>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="all-column-styles"  select="xt:node-set($all-column-styles-RTF)"/>



        <xsl:if test="$DEBUG">
<xsl:element name="debug_all-column-styles"><xsl:text>
</xsl:text>
    <xsl:for-each select="$all-column-styles/column-style-attributes">
    <xsl:choose>
    <xsl:when test="@column-hidden">
<xsl:text>  </xsl:text><xsl:value-of select="@style-name"/><xsl:text>Column is hidden</xsl:text><xsl:text>
</xsl:text>
    </xsl:when>
    <xsl:otherwise>
<xsl:text>  </xsl:text><xsl:value-of select="@style-name"/><xsl:text> = </xsl:text><xsl:value-of select="."/><xsl:text>
</xsl:text>
    </xsl:otherwise>
    </xsl:choose>
               </xsl:for-each>
            </xsl:element>
        </xsl:if>



        <xsl:choose>
            <xsl:when test="not($outputType = 'WML')">
                <xsl:choose>
                    <xsl:when test="$isMaintable">
                        <!-- all rows - we can not use xsl:apply-template with a node-set parameter as by a bug in XT (James Clark)
                             (here: columns-variable) will be interpreted as a result tree fragment, where no search expression (XPath) can be used -->
                        <xsl:for-each select="descendant::table:table-row[not(@table:visibility = ('collapse' or 'filter')) and not(ancestor::table:sub-table)]">
                            <xsl:call-template name="repeat-write-row">
                                <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                                <xsl:with-param name="number-rows-repeated" select="@table:number-rows-repeated"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- all rows - we can not use xsl:apply-template with a node-set parameter as by a bug in XT (James Clark)
                             (in this case: columns-variable) will be interpreted as a result tree fragment, where no search expression (XPath) can be used -->
                        <xsl:for-each select="descendant::table:table-row[not(@table:visibility = ('collapse' or 'filter'))]">
                            <xsl:call-template name="repeat-write-row">
                                <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                                <xsl:with-param name="number-rows-repeated" select="@table:number-rows-repeated"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <!-- WML devices will show no table elements as tables are not widely implemented in devices, yet -->
                <xsl:choose>
                    <xsl:when test="$isMaintable">
                        <!-- all rows - we can not use xsl:apply-template with a node-set parameter as by a bug in XT (James Clark)
                             (here: columns-variable) will be interpreted as a result tree fragment, where no search expression (XPath) can be used -->
                        <xsl:for-each select="descendant::table:table-row[not(@table:visibility = ('collapse' or 'filter')) and not(ancestor::table:sub-table)]">
                            <xsl:call-template name="wml-repeat-write-row">
                                <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                                <xsl:with-param name="number-rows-repeated" select="@table:number-rows-repeated"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- all rows - we can not use xsl:apply-template with a node-set parameter as by a bug in XT (James Clark)
                             (here: columns-variable) will be interpreted as a result tree fragment, where no search expression (XPath) can be used -->
                        <xsl:for-each select="descendant::table:table-row[not(@table:visibility = ('collapse' or 'filter'))]">
                            <xsl:call-template name="wml-repeat-write-row">
                                <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                                <xsl:with-param name="number-rows-repeated" select="@table:number-rows-repeated"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <!-- **************************************************** -->
    <!-- *** Step 1: Create table columns style variable  *** -->
    <!-- **************************************************** -->

    <!-- writes the repeated column style explicit as an element in the columns-variable -->
    <xsl:template name="repeat-write-columns-variable-entry">
        <xsl:param name="column-style-attributes"/>
        <xsl:param name="number-columns-repeated"/>

        <xsl:choose>
            <xsl:when test="$number-columns-repeated > 1">
                <!-- writes an entry of a column in the columns-variable -->
                <xsl:call-template name="write-columns-variable-entry">
                    <xsl:with-param name="column-style-attributes" select="$column-style-attributes"/>
                </xsl:call-template>
                <!-- repeat calling this method until all elements written out -->
                <xsl:call-template name="repeat-write-columns-variable-entry">
                    <xsl:with-param name="column-style-attributes" select="$column-style-attributes"/>
                    <xsl:with-param name="number-columns-repeated" select="$number-columns-repeated - 1"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <!-- writes an entry of a column in the columns-variable -->
                <xsl:call-template name="write-columns-variable-entry">
                    <xsl:with-param name="column-style-attributes" select="$column-style-attributes"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <!-- the column-style write-pattern for each column written in a variable -->
    <xsl:template name="write-columns-variable-entry">
        <xsl:param name="column-style-attributes"/>

        <xsl:element name="column-style-attributes">
            <xsl:choose>
                <xsl:when test="@table:visibility = ('collapse' or 'filter')">
                    <xsl:attribute name="column-hidden">true</xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="style-name"><xsl:value-of select="translate(@table:style-name, '. ', '')"/></xsl:attribute>
                    <xsl:value-of select="$column-style-attributes"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
    </xsl:template>



    <!-- ***************************************************** -->
    <!-- *** Step 2: write (explicit) repeating table rows *** -->
    <!-- ***************************************************** -->

    <xsl:template name="repeat-write-row">
        <xsl:param name="all-column-styles"/>
        <xsl:param name="number-rows-repeated" select="1"/>

        <xsl:choose>
            <!-- write an entry of a row and repeat calling this method until all elements are written out -->
            <xsl:when test="$number-rows-repeated > 1">
                <xsl:call-template name="write-row">
                    <xsl:with-param name="all-column-styles" select="$all-column-styles"/>
                </xsl:call-template>

                <xsl:call-template name="repeat-write-row">
                    <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                    <xsl:with-param name="number-rows-repeated" select="$number-rows-repeated - 1"/>
                </xsl:call-template>
            </xsl:when>
            <!-- write an entry of a row -->
            <xsl:otherwise>
                <xsl:call-template name="write-row">
                    <xsl:with-param name="all-column-styles" select="$all-column-styles"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>



    <xsl:template name="write-row">
        <xsl:param name="all-column-styles"/>

        <xsl:element name="tr">
            <xsl:call-template name='add-style-properties'/>
            <xsl:for-each select="child::table:table-cell">
                <!-- calls afterwards WRITE-CELL -->
                <xsl:call-template name="get-cell-column-number">
                    <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                    <xsl:with-param name="number-columns-repeated"  select="@table:number-columns-repeated"/>
                    <xsl:with-param name="end-position"             select="position()"/>
                    <xsl:with-param name="temp-position"            select="1"/>
                    <xsl:with-param name="absolute-position"        select="1"/>
                </xsl:call-template>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>



    <xsl:template name="get-cell-column-number">
        <xsl:param name="all-column-styles"/>
        <!-- repetition of the working cell -->
        <xsl:param name="number-columns-repeated"/>
        <!-- position of the working cell -->
        <xsl:param name="end-position"/>
        <!-- member variable to see which cell is currently checkt for number-columns-repeated
             runs up to the calling cell s end-position -->
        <xsl:param name="temp-position"/>
        <!-- needed position of the working in the output table -->
        <xsl:param name="absolute-position"/>


        <xsl:choose>
            <xsl:when test="$end-position > $temp-position">

                <!-- get any @table:number-columns-repeated number and add it to the absolute-position -->
                <xsl:variable name="columns-repeated" select="parent::*/table:table-cell[position() = $temp-position]/@table:number-columns-repeated"/>
                <xsl:choose>
                    <!-- when the columns are repeated the next absolute-positions raises up to the amount of repeated columns, otherwise up to 1 -->
                    <xsl:when test="string-length($columns-repeated) = 0">
                        <xsl:call-template name="get-cell-column-number">
                            <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                            <xsl:with-param name="number-columns-repeated"  select="$number-columns-repeated"/>
                            <xsl:with-param name="end-position"             select="$end-position"/>
                            <xsl:with-param name="temp-position"            select="$temp-position + 1"/>
                            <xsl:with-param name="absolute-position"        select="$absolute-position + 1"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="get-cell-column-number">
                            <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                            <xsl:with-param name="number-columns-repeated"  select="$number-columns-repeated"/>
                            <xsl:with-param name="end-position"             select="$end-position"/>
                            <xsl:with-param name="temp-position"            select="$temp-position + 1"/>
                            <xsl:with-param name="absolute-position"        select="$absolute-position + $columns-repeated"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="repeat-write-cell">
                    <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                    <xsl:with-param name="number-columns-repeated"  select="$number-columns-repeated"/>
                    <xsl:with-param name="absolute-position"        select="$absolute-position"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>





    <!-- *********************************** -->
    <!-- *** write repeating table cells *** -->
    <!-- *********************************** -->

    <xsl:template name="repeat-write-cell">
        <xsl:param name="all-column-styles"/>
        <xsl:param name="number-columns-repeated"/>
        <xsl:param name="absolute-position"/>


        <xsl:choose>
            <xsl:when test="$number-columns-repeated > 1">
                <!-- writes an entry of a cell -->
                <xsl:call-template name="write-cell">
                    <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                    <xsl:with-param name="absolute-position"        select="$absolute-position"/>
                </xsl:call-template>
                <!-- repeat calling this method until all elements written out -->
                <xsl:call-template name="repeat-write-cell">
                    <xsl:with-param name="all-column-styles"        select="$all-column-styles"/>
                    <xsl:with-param name="number-columns-repeated"  select="$number-columns-repeated - 1"/>
                    <xsl:with-param name="absolute-position"        select="$absolute-position + 1"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <!-- writes an entry of a cell -->
                <xsl:call-template name="write-cell">
                    <xsl:with-param name="all-column-styles"    select="$all-column-styles"/>
                    <xsl:with-param name="absolute-position"    select="$absolute-position"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>



    <xsl:template name="write-cell">
        <xsl:param name="all-column-styles"/>
        <xsl:param name="absolute-position"/>

        <!-- Setting the table cell type (table data or table header)
                in my point of view, all children from  able:table-header-rows are table header cells
                ??? possible that a table-header-rows has a subtable with contain table data cells ??? -->
        <xsl:variable name="tableType">
            <xsl:choose>
                <xsl:when test="ancestor::table:table-header-rows">
                    <xsl:text>th</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>td</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:choose>
            <!-- a hidden column will give out nothing -->
            <xsl:when test="$all-column-styles/column-style-attributes[position() = $absolute-position]/@column-hidden">
            </xsl:when>
            <!-- NOT a hidden column -->
            <xsl:otherwise>
                <xsl:choose>
                    <!--+++++ CSS (CASCADING STLYE SHEET) HEADER STYLE WAY +++++-->
                    <xsl:when test="$outputType = 'CSS_HEADER'">
                        <xsl:element name="{$tableType}">
                            <xsl:if test="@table:number-columns-spanned">
                                <xsl:attribute name="colspan">
                                    <xsl:value-of select="@table:number-columns-spanned"/>
                                </xsl:attribute>
                            </xsl:if>
                            <xsl:if test="@table:number-rows-spanned">
                                <xsl:attribute name="rowspan">
                                    <xsl:value-of select="@table:number-rows-spanned"/>
                                </xsl:attribute>
                            </xsl:if>

                            <xsl:if test="@table:style-name">
                                <!-- *** the cell-style *** -->
                                <xsl:attribute name="class">
                                    <xsl:value-of select="translate(@table:style-name, '. ', '')"/>
                                </xsl:attribute>
                            </xsl:if>

                            <!-- TABLE:VALUE-TYPE - the value of a table-cell will be aligned left by default only exlicit non-string is aligned:right-->
                            <xsl:choose>
                                <xsl:when test="@table:value-type and not(@table:value-type = 'string')">
                                    <xsl:attribute name="style">text-align:right;</xsl:attribute>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="style">text-align:left;</xsl:attribute>
                                </xsl:otherwise>
                            </xsl:choose>

                            <xsl:choose>
                                <!-- IF NO CELL CONTENT: insert &#160: it is a non-breakable space, necessary to make to the browser show the table-cell grid -->
                                <xsl:when test="not(child::text()) and not(child::*)">&#160;</xsl:when>
                                <!-- IF CELL CONTENT -->
                                <xsl:otherwise>
                                    <!-- *** the column-style ***
                                         the column style has no conclusion with the cell style, so it can be positioned AFTER the cell style -->
                                    <xsl:element name="span">
                                        <xsl:attribute name="class">
                                            <xsl:value-of select="$all-column-styles/column-style-attributes[position() = $absolute-position]/@style-name"/>
                                        </xsl:attribute>
                                        <xsl:apply-templates/>
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:element>
                    </xsl:when>

                    <!--+++++ HTML 4.0 INLINED WAY  +++++-->
                    <xsl:when test="$outputType = 'CSS_INLINED'">
                        <xsl:element name="{$tableType}">

                            <xsl:if test="@table:number-columns-spanned">
                                <xsl:attribute name="colspan">
                                    <xsl:value-of select="@table:number-columns-spanned"/>
                                </xsl:attribute>
                            </xsl:if>
                            <xsl:if test="@table:number-rows-spanned">
                                <xsl:attribute name="rowspan">
                                    <xsl:value-of select="@table:number-rows-spanned"/>
                                </xsl:attribute>
                            </xsl:if>

                            <xsl:attribute name="style">
                                <!-- cell-style -->
                                <xsl:value-of select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                                <!-- column-style -->
                                <xsl:value-of select="$all-column-styles/column-style-attributes[position()=$absolute-position]"/>
                                <!-- TABLE:VALUE-TYPE - the value of a table-cell will be aligned left by default only exlicit non-string is aligned:right-->
                                <xsl:choose>
                                    <xsl:when test="@table:value-type and not(@table:value-type = 'string')">
                                        <xsl:text>text-align:right;</xsl:text>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:text>text-align:left;</xsl:text>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:attribute>

                            <!-- &#160 is a non-breakable space, necessary to make to the browser show the table-cell grid -->
                            <xsl:if test="not(child::text()) and not(child::*)">&#160;</xsl:if>
                            <xsl:apply-templates/>
                        </xsl:element>
                    </xsl:when>
                    <!--+++++ PALM INLINED WAY  +++++-->
                    <xsl:when test="$outputType = 'PALM'">
                        <xsl:element name="{$tableType}">
                            <xsl:if test="@table:number-columns-spanned">
                                <xsl:attribute name="colspan">
                                    <xsl:value-of select="@table:number-columns-spanned"/>
                                </xsl:attribute>
                            </xsl:if>

                            <xsl:if test="@table:number-rows-spanned">
                                <xsl:attribute name="rowspan">
                                    <xsl:value-of select="@table:number-rows-spanned"/>
                                </xsl:attribute>
                            </xsl:if>
                            <xsl:apply-templates/>
                        </xsl:element>
                    </xsl:when>
                    <!--+++++ WML WAY  +++++-->
                    <xsl:when test="$outputType = 'WML'">
                        <xsl:choose>
                            <xsl:when test="not($all-column-styles/column-style-attributes[last() = $absolute-position])">
                                <xsl:apply-templates/>
                                <xsl:text>, </xsl:text>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:apply-templates/>
                                <xsl:text>; </xsl:text>
                                <xsl:element name="br"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>



    <!-- **************************** -->
    <!-- *** HELPER: table styles *** -->
    <!-- **************************** -->

    <xsl:template name="add-style-properties">
        <xsl:param name="all-column-styles"/>
        <xsl:param name="absolute-position"/>

        <xsl:choose>
            <!--+++++ CSS (CASCADING STLYE SHEET) HEADER STYLE WAY +++++-->
            <xsl:when test="$outputType = 'CSS_HEADER'">
                <xsl:attribute name="class">
                    <xsl:value-of select="translate(@table:style-name, '. ', '')"/>
                </xsl:attribute>
            </xsl:when>

            <!--+++++ HTML 4.0 INLINED WAY  +++++-->
            <xsl:when test="$outputType = 'CSS_INLINED'">
                <xsl:attribute name="style">
                    <xsl:value-of select="$style-attribute-tree/allstyles/*[name()=current()/@table:style-name]"/>
                </xsl:attribute>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>
