

#ifndef _SB_SBSTAR_HXX
#define _SB_SBSTAR_HXX

#ifndef _SBX_SBXOBJECT_HXX 
#include <svtools/sbxobj.hxx>
#endif

#include <basic/sbdef.hxx>
#include <basic/sberrors.hxx>

class SbModule;                     
class SbiInstance;                  
class SbiRuntime;                   
class SbiImage;                     
class BasicLibInfo;					
class SbiBreakpoints;
class SbTextPortions;
class SbMethod;
class BasicManager;

class StarBASIC : public SbxObject
{
	friend class SbiScanner;
	friend class SbiExpression;			
	friend class SbiInstance;
	friend class SbiRuntime;

	SbxArrayRef		pModules;			
	SbxObjectRef	pRtl;				
	SbxObjectRef	pJavaRtl;			

	Link			aErrorHdl;			
	Link			aBreakHdl;			
	BOOL			bNoRtl;				
	BOOL			bBreak;				
	BasicLibInfo*	pLibInfo;			
	SbLanguageMode	eLanguageMode;		
protected:
	BOOL 			CError( SbError, const String&, xub_StrLen, xub_StrLen, xub_StrLen );
private:
#if 0 // _SOLAR__PRIVATE
	BOOL 			RTError( SbError, xub_StrLen, xub_StrLen, xub_StrLen );
	BOOL 			RTError( SbError, const String& rMsg, xub_StrLen, xub_StrLen, xub_StrLen );
	USHORT 			BreakPoint( xub_StrLen nLine, xub_StrLen nCol1, xub_StrLen nCol2 );
	USHORT 			StepPoint( xub_StrLen nLine, xub_StrLen nCol1, xub_StrLen nCol2 );
#endif
	virtual BOOL LoadData( SvStream&, USHORT );
	virtual BOOL StoreData( SvStream& ) const;

protected:

	virtual	BOOL  	ErrorHdl();
	virtual	USHORT	BreakHdl();
	virtual ~StarBASIC();

public:

	SBX_DECL_PERSIST_NODATA(SBXCR_SBX,SBXID_BASIC,1);
	TYPEINFO();

	StarBASIC( StarBASIC* pParent = NULL );

	virtual void SetModified( BOOL );

	void* operator 	new( size_t );
	void operator 	delete( void* );

	virtual void    Insert( SbxVariable* );
	virtual void    Remove( SbxVariable* );
	virtual void	Clear();

	BasicLibInfo*	GetLibInfo()					{ return pLibInfo;	}
	void			SetLibInfo( BasicLibInfo* p )   { pLibInfo = p;		}

	SbModule*   	MakeModule( const String& rName, const String& rSrc );
	BOOL			Compile( SbModule* );
	BOOL 			Disassemble( SbModule*, String& rText );
	static void 	Stop();
	static void 	Error( SbError );
	static void 	Error( SbError, const String& rMsg );
	static void 	FatalError( SbError );
	static BOOL 	IsRunning();
	static SbError 	GetErr();

	static String	GetErrorMsg();
	static xub_StrLen GetErl();

	void 			Highlight( const String& rSrc, SbTextPortions& rList );

	virtual SbxVariable* Find( const String&, SbxClassType );
	virtual BOOL Call( const String&, SbxArray* = NULL );

	SbxArray*		GetModules() { return pModules; }
	SbxObject*		GetRtl()	 { return pRtl;		}
	SbModule*		FindModule( const String& );

	void			InitAllModules( void );
	void			ClearAllModuleVars( void );
	void			ActivateObject( const String*, BOOL );
	BOOL 			LoadOldModules( SvStream& );

	void			ClearGlobalVars( void );

	static USHORT	GetLine();
	static USHORT	GetCol1();
	static USHORT	GetCol2();
	static void		SetErrorData( SbError nCode, USHORT nLine,
					  USHORT nCol1, USHORT nCol2 );

	static void		MakeErrorText( SbError, const String& aMsg );
	static const	String&	GetErrorText();
	static SbError	GetErrorCode();
	static BOOL		IsCompilerError();
	static USHORT	GetVBErrorCode( SbError nError );
	static SbError	GetSfxFromVBError( USHORT nError );
	static void		SetGlobalLanguageMode( SbLanguageMode eLangMode );
	static SbLanguageMode GetGlobalLanguageMode();

	void SetLanguageMode( SbLanguageMode eLangMode )
		{ eLanguageMode = eLangMode; }
	SbLanguageMode GetLanguageMode();

	BOOL   			IsBreak() const				{ return bBreak; }

	static Link 	GetGlobalErrorHdl();
	static void 	SetGlobalErrorHdl( const Link& rNewHdl );
	Link 			GetErrorHdl() const { return aErrorHdl; }
	void 			SetErrorHdl( const Link& r ) { aErrorHdl = r; }

	static Link 	GetGlobalBreakHdl();
	static void 	SetGlobalBreakHdl( const Link& rNewHdl );
	Link 			GetBreakHdl() const { return aBreakHdl; }
	void 			SetBreakHdl( const Link& r ) { aBreakHdl = r; }

	static SbxBase*	FindSBXInCurrentScope( const String& rName );
	static SbxVariable*	FindVarInCurrentScopy
		( const String& rName, USHORT& rStatus );
	static SbMethod* GetActiveMethod( USHORT nLevel = 0 );
	static SbModule* GetActiveModule();

	static void StaticSuppressSfxResource( BOOL bSuppress );
};

#ifndef __SB_SBSTARBASICREF_HXX
#define __SB_SBSTARBASICREF_HXX

SV_DECL_IMPL_REF(StarBASIC)

#endif

#endif

