

#ifndef _SFXSMPLHINT_HXX
#include <svtools/smplhint.hxx>
#endif

#define TT_SIGNATURE_FOR_UNICODE_TEXTFILES "'encoding UTF-8	 Do not remove or change this line!"

#define ADD_ERROR_QUIET(nNr, aStr)										\
{																		\
	ErrorEntry *pErr;													\
	if ( BasicRuntimeAccess::HasRuntime() )								\
	{																	\
		BasicRuntime aRun = BasicRuntimeAccess::GetRuntime();			\
		xub_StrLen aErrLn = StarBASIC::GetErl();						\
		if ( 0 == aErrLn )												\
			aErrLn = aRun.GetLine();									\
		pErr = new ErrorEntry(nNr, aStr,								\
			aErrLn, aRun.GetCol1(), aRun.GetCol2());					\
	}																	\
	else																\
	{																	\
		pErr = new ErrorEntry(nNr, aStr);								\
	}																	\
	P_FEHLERLISTE->C40_INSERT(ErrorEntry, pErr, P_FEHLERLISTE->Count());\
}

#define ADD_ERROR(nNr, aStr) { \
		if ( !SbxBase::IsError() )				\
			SbxBase::SetError( nNr );			\
		ADD_ERROR_QUIET(nNr, aStr);	\
}

#define POP_ERROR() P_FEHLERLISTE->DeleteAndDestroy(0)
#define GET_ERROR() P_FEHLERLISTE->GetObject(0)
#define IS_ERROR() ( P_FEHLERLISTE->Count() > 0 )

enum TTLogType { LOG_RUN, LOG_TEST_CASE, LOG_ERROR, LOG_CALL_STACK, LOG_MESSAGE, LOG_WARNING, LOG_ASSERTION };

struct TTDebugData
{
public:
	TTLogType aLogType;
	String aMsg;
	String aFilename;
	xub_StrLen nLine;
	xub_StrLen nCol1;
	xub_StrLen nCol2;
};

struct TTLogMsg
{
public:
	String aLogFileName;
	TTDebugData aDebugData;
};

struct WinInfoRec
{
public:
	ULONG nUId;
	String aKurzname;
	String aSlotname;
	String aLangname;
	ULONG nRType;
	String aRName;
	BOOL bIsReset;
};

#define TT_KEYWORD		((SbTextType)100)	
#define TT_REMOTECMD	((SbTextType)101)	
#define TT_LOCALCMD		((SbTextType)102)	
#define TT_CONTROL		((SbTextType)103)	
#define TT_SLOT			((SbTextType)104)	
#define TT_METHOD		((SbTextType)105)	
#define TT_NOMETHOD		((SbTextType)106)	

#define FILELIST1		((SbTextType)111)	
#define FILELIST2		((SbTextType)112)	
#define FILELIST3		((SbTextType)113)	
#define FILELIST4		((SbTextType)114)	

#define SBX_HINT_LANGUAGE_EXTENSION_LOADED		SFX_HINT_USER06
#define SBX_HINT_EXECUTION_STATUS_INFORMATION	SFX_HINT_USER07

#define TT_EXECUTION_ENTERWAIT	0x01
#define TT_EXECUTION_LEAVEWAIT	0x02
#define TT_EXECUTION_SHOW_ACTION	0x03

class TTExecutionStatusHint : public SfxSimpleHint
{
private:
	String maExecutionStatus;
	String maAdditionalExecutionStatus;
	USHORT mnType;

public:
			TYPEINFO();
			TTExecutionStatusHint( USHORT nType, sal_Char *pExecutionStatus, sal_Char *pAdditionalExecutionStatus = "" )
	: SfxSimpleHint(SBX_HINT_EXECUTION_STATUS_INFORMATION)
	, mnType( nType )
	, maExecutionStatus( pExecutionStatus, RTL_TEXTENCODING_ASCII_US )
	, maAdditionalExecutionStatus( pAdditionalExecutionStatus, RTL_TEXTENCODING_ASCII_US )
	{;}
	TTExecutionStatusHint( USHORT nType, const String &aExecutionStatus = String(), const String &aAdditionalExecutionStatus = String() )
	: SfxSimpleHint(SBX_HINT_EXECUTION_STATUS_INFORMATION)
	, mnType( nType )
	, maExecutionStatus( aExecutionStatus )
	, maAdditionalExecutionStatus( aAdditionalExecutionStatus )
	{;}

	const String& GetExecutionStatus() const { return maExecutionStatus; }
	const String& GetAdditionalExecutionStatus() const { return maAdditionalExecutionStatus; }
	USHORT GetType(){ return mnType; }
};

