

#ifndef _ENVSET_HXX
#define _ENVSET_HXX

#include <tools/string.hxx>
#include <tools/list.hxx>

class GenericInformationList;

class Switch : public ByteString
{
private:
	ByteString sValue;
	BOOL bValid;

public:
	Switch( ByteString &rSwitch, ByteString &rVal )
		: ByteString( rSwitch ),
		sValue( rVal ), bValid( FALSE ) {};
	ByteString &GetValue() { return sValue; }
	void Valid() { bValid = TRUE; }
	BOOL IsValid() { return bValid; }
};

DECLARE_LIST( SwitchList_Impl, Switch * );

class SwitchList : public SwitchList_Impl
{
public:
	SwitchList() {};

	void InsertUnique( Switch *pObject );
};

class EnvironmentVariable : public ByteString
{
private:
	ByteString sValue;	
public:
	EnvironmentVariable( const ByteString &rVar, const ByteString &rVal );
	EnvironmentVariable( const ByteString &rVar );

	ByteString &GetValue( BOOL bOutOfEnv = FALSE );
	void SetValue( const ByteString &rVal, BOOL bIntoEnv = FALSE );

	void PutEnv();
	void PutEnv( const ByteString &rVal );
	ByteString &GetEnv();

	ByteString GetSetter( BOOL bUnset = FALSE ); 
};

DECLARE_LIST( EnvironmentList, EnvironmentVariable * );
DECLARE_LIST( ScriptLines, ByteString* );

#define ENVSET_NO_ERROR			0x0000
#define ENVSET_NO_FILE			0x0001
#define ENVSET_FILE_ERROR			   	0x0002
#define ENVSET_SWITCHES_ERROR		   	0x0004
#define ENVSET_NO_ENVIRONMENT		   	0x0008
#define ENVSET_NO_SWITCH			   	0x000F
#define ENVSET_MISSING_VALUE	  		0x0010
#define ENVSET_NO_ORDER					0x0020
#define ENVSET_UNKNOWN_BLOCK			0x0040
#define ENVSET_VERLISTENTRY_NOT_FOUND   0x0080

class EnvironmentSetter : public EnvironmentList
{
private:
	USHORT nError;
	SwitchList *pPossibleSwitches;

	ScriptLines aPreScript;
	ScriptLines aPostScript;

	GenericInformationList *pInfoList;
	GenericInformationList *pVersionList;

protected:
	virtual void PrintStatus( const char* pStatus ) {};

public:
	EnvironmentSetter( const ByteString &rConfigFile,
		GenericInformationList *pVerList,
		EnvironmentSetter* pSourceSetter = NULL );
	~EnvironmentSetter();

	USHORT GetError() { return nError; }

	SwitchList *GetPossibleSwitches();
	USHORT Execute( SwitchList &rSwitches, ByteString sEnvironment );

	BOOL CheckIfStatement( const ByteString &rStatement );
	void Expand( ByteString &rVariableToExpand );

	void Insert( const ByteString &rVar, const ByteString &rVal );

	void Set();
	BOOL WriteToFile( const ByteString &rFileName ); 
};

#endif
