
#ifndef _CONNECTIVITY_SQLNODE_HXX
#define _CONNECTIVITY_SQLNODE_HXX

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _COM_SUN_STAR_UTIL_XNUMBERFORMATTYPES_HPP_
#include <com/sun/star/util/XNumberFormatTypes.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_XDATABASEMETADATA_HPP_
#include <com/sun/star/sdbc/XDatabaseMetaData.hpp>
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace sdbc
			{
	class XDatabaseMetaData;
			}
			namespace beans
			{
	class XPropertySet;
			}
			namespace util
			{
	class XNumberFormatter;
			}
		}
	}
}

namespace connectivity
{

	class OSQLParseNode;
	class OParseContext;

	typedef ::std::vector< OSQLParseNode* > OSQLParseNodes;

	enum SQLNodeType	{SQL_NODE_RULE, SQL_NODE_LISTRULE, SQL_NODE_COMMALISTRULE,
			 SQL_NODE_KEYWORD, SQL_NODE_COMPARISON, SQL_NODE_NAME,
			 SQL_NODE_STRING,	SQL_NODE_INTNUM, SQL_NODE_APPROXNUM,
			 SQL_NODE_EQUAL,SQL_NODE_LESS,SQL_NODE_GREAT,SQL_NODE_LESSEQ,SQL_NODE_GREATEQ,SQL_NODE_NOTEQUAL,
			 SQL_NODE_PUNCTUATION, SQL_NODE_AMMSC, SQL_NODE_ACCESS_DATE,SQL_NODE_DATE};

	class OSQLParseNode
	{
		friend class OSQLParser;

		OSQLParseNodes					m_aChilds;
		OSQLParseNode*	 				m_pParent;		
		::rtl::OUString 				m_aNodeValue;	

		SQLNodeType 					m_eNodeType;	
		sal_uInt32						m_nNodeID; 		

	protected:
		struct SQLParseNodeParameter
		{
			const ::com::sun::star::lang::Locale&	rLocale;
			const ::rtl::OUString					aIdentifierQuote;
			const ::rtl::OUString					aCatalogSeparator;	
			::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > xFormatter;
			::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >  xField;
			const OParseContext& rContext;
			sal_Char			cDecSep ;
			sal_Bool			bQuote : 1;
			sal_Bool			bInternational : 1;
			sal_Bool			bPredicate : 1;
	SQLParseNodeParameter(const ::rtl::OUString& _rIdentifierQuote,
					  const ::rtl::OUString& _rCatalogSep,
					  const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & _xFormatter,
					  const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField,
					  const ::com::sun::star::lang::Locale& _rIntl,
					  const OParseContext* _pContext,
					  sal_Bool _bIntl = sal_False, 
					  sal_Bool _bQuote= sal_True, 
					  sal_Char _cDecSep = '.',							  
					  sal_Bool _bPredicate = sal_False);
		};							

	public:
		enum Rule
		{
			select_statement = 0,
			table_exp,
			table_ref_commalist,
			table_ref,
			catalog_name,
			schema_name,
			table_name,
			opt_column_commalist,
			column_commalist,
			column_ref_commalist,
			column_ref,
			opt_order_by_clause,
			ordering_spec_commalist,
			ordering_spec,
			opt_asc_desc,
			where_clause,
			opt_where_clause,
			search_condition,
			comparison_predicate,
			between_predicate,
			like_predicate,
			opt_escape,
			test_for_null,
			scalar_exp_commalist,
			scalar_exp,
			parameter_ref,
			parameter,
			general_set_fct,
			range_variable,
			column,
			delete_statement_positioned,
			delete_statement_searched,
			update_statement_positioned,
			update_statement_searched,
			assignment_commalist,
			assignment,
			values_or_query_spec,
			insert_statement,
			insert_atom_commalist,
			insert_atom,
			predicate_check,
			from_clause,
			qualified_join,
			cross_union,
			select_sublist,
			derived_column,
			column_val,
			set_fct_spec,
			boolean_term,
			boolean_primary,
			num_value_exp,
			join_type,
			position_exp,
			extract_exp,
			length_exp,
			char_value_fct,
			odbc_call_spec,
			in_predicate,
			existence_test,
			unique_test,
			all_or_any_predicate,
			named_columns_join,
			join_condition,
			joined_table,
			boolean_factor,
			not,
			boolean_test,
			manipulative_statement,
			subquery,
			value_exp_commalist,
			odbc_fct_spec,
			union_statement,
			outer_join_type,
			char_value_exp,
			term,
			value_exp_primary,
			value_exp,
			rule_count						
		};

		OSQLParseNode(const sal_Char* _pValueStr,
		  SQLNodeType _eNodeType,
		  sal_uInt32 _nNodeID = 0);

		OSQLParseNode(const ::rtl::OString& _rValue,
		  SQLNodeType eNewNodeType,
		  sal_uInt32 nNewNodeID=0);	

		OSQLParseNode(const sal_Unicode* _pValue,
		  SQLNodeType _eNodeType,
		  sal_uInt32 _nNodeID = 0);

		OSQLParseNode(const ::rtl::OUString& _rValue,
		  SQLNodeType _eNodeType,
		  sal_uInt32 _nNodeID = 0);

		OSQLParseNode(const OSQLParseNode& rParseNode);
		OSQLParseNode& operator=(const OSQLParseNode& rParseNode);

		sal_Bool operator==(OSQLParseNode& rParseNode) const;

		virtual ~OSQLParseNode();

		OSQLParseNode* getParent() const {return m_pParent;};

		void setParent(OSQLParseNode* pParseNode) {m_pParent = pParseNode;};

		sal_uInt32 count() const {return m_aChilds.size();};
		inline OSQLParseNode* getChild(sal_uInt32 nPos) const;	

		void append(OSQLParseNode* pNewSubTree);			
		void insert(sal_uInt32 nPos, OSQLParseNode* pNewSubTree);	

		OSQLParseNode* replaceAt(sal_uInt32 nPos, OSQLParseNode* pNewSubTree);
		OSQLParseNode* replace(OSQLParseNode* pOldSubTree, OSQLParseNode* pNewSubTree);

		OSQLParseNode* removeAt(sal_uInt32 nPos);
		OSQLParseNode* remove(OSQLParseNode* pSubTree);	

		void replaceNodeValue(const ::rtl::OUString& rTableAlias,const ::rtl::OUString& rColumnName);

		void parseNodeToStr(::rtl::OUString& rString, 
				const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,
				const OParseContext* pContext = NULL,
				sal_Bool _bIntl = sal_False, 
				sal_Bool _bQuote= sal_True) const;

		void parseNodeToPredicateStr(::rtl::OUString& rString, 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,														 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
						 const ::com::sun::star::lang::Locale& rIntl,
						 sal_Char _cDec,
						 const OParseContext* pContext = NULL ) const;	

		void parseNodeToPredicateStr(::rtl::OUString& rString, 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,														 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
						 const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField,
						 const ::com::sun::star::lang::Locale& rIntl,
						 sal_Char _cDec,
						 const OParseContext* pContext = NULL ) const;	

		OSQLParseNode* getByRule(OSQLParseNode::Rule eRule) const;

		void showParseTree(::rtl::OUString& rString, sal_uInt32 nLevel=0);

		SQLNodeType getNodeType() const {return m_eNodeType;};

		sal_uInt32 getRuleID() const {return m_nNodeID;}

		sal_uInt32 getTokenID() const {return m_nNodeID;}

		sal_Bool isRule() const 
			{ return (m_eNodeType == SQL_NODE_RULE) || (m_eNodeType == SQL_NODE_LISTRULE)
	|| (m_eNodeType == SQL_NODE_COMMALISTRULE);}

		sal_Bool isToken() const {return !isRule();} 

		const ::rtl::OUString& getTokenValue() const {return m_aNodeValue;}

		void setTokenValue(const ::rtl::OUString& rString) {	if (isToken()) m_aNodeValue = rString;}

		sal_Bool isLeaf() const {return m_aChilds.empty();}

		static void negateSearchCondition(OSQLParseNode*& pSearchCondition,sal_Bool bNegate=sal_False);

		static void disjunctiveNormalForm(OSQLParseNode*& pSearchCondition);

		static void absorptions(OSQLParseNode*& pSearchCondition);

		static void eraseBraces(OSQLParseNode*& pSearchCondition);

		static void compress(OSQLParseNode*& pSearchCondition);

		static sal_Bool getTableComponents(const OSQLParseNode* _pTableNode,
								::com::sun::star::uno::Any &_rCatalog,
								::rtl::OUString &_rSchema,
								::rtl::OUString &_rTable);

	protected:

		void parseNodeToStr(::rtl::OUString& rString, 
				const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,						
				const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
				const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField,
				const ::com::sun::star::lang::Locale& rIntl,
				const OParseContext* pContext,
				sal_Bool _bIntl, 
				sal_Bool _bQuote, 
				sal_Char _cDecSep,						
				sal_Bool bPredicate) const;

		virtual void parseNodeToStr(::rtl::OUString& rString, 
						const SQLParseNodeParameter& rParam) const;				
	private:
		void likeNodeToStr(::rtl::OUString& rString, 
			   const SQLParseNodeParameter& rParam) const;				
		void tableRangeNodeToStr(::rtl::OUString& rString, 
					 const SQLParseNodeParameter& rParam) const;				
		sal_Bool addDateValue(::rtl::OUString& rString, const SQLParseNodeParameter& rParam) const;
		::rtl::OUString convertDateTimeString(const SQLParseNodeParameter& rParam, const ::rtl::OUString& rString) const;
		::rtl::OUString convertDateString(const SQLParseNodeParameter& rParam, const ::rtl::OUString& rString) const;
		::rtl::OUString convertTimeString(const SQLParseNodeParameter& rParam, const ::rtl::OUString& rString) const;	
		void parseLeaf(::rtl::OUString & rString, const SQLParseNodeParameter& rParam) const;
	};

	inline OSQLParseNode* OSQLParseNode::getChild(sal_uInt32 nPos) const 
	{
		OSL_ENSURE(nPos < m_aChilds.size(), "Invalid Position");
		return m_aChilds[nPos];
	}	

	#define SQL_ISRULE(pParseNode, eRule) 	((pParseNode)->isRule() && (pParseNode)->getRuleID() == OSQLParser::RuleID(OSQLParseNode::##eRule))
	#define SQL_ISTOKEN(pParseNode, token) ((pParseNode)->isToken() && (pParseNode)->getTokenID() == SQL_TOKEN_##token)
	#define SQL_ISPUNCTUATION(pParseNode, aString) ((pParseNode)->getNodeType() == SQL_NODE_PUNCTUATION && !(pParseNode)->getTokenValue().compareToAscii(aString))
}

#endif	
