

#ifndef _B2D_MBASE_HXX
#define _B2D_MBASE_HXX

#ifndef _B2D_MATRIX3D_HXX
#include <goodies/matrix3d.hxx>
#endif

#ifndef _SV_GEN_HXX
#include <tools/gen.hxx>
#endif

#ifndef _SV_COLOR_HXX
#include <vcl/color.hxx>
#endif

#ifndef _SV_BITMAPEX_HXX
#include <vcl/bitmapex.hxx>
#endif

#ifndef _SV_REGION_HXX
#include <vcl/region.hxx>
#endif

#ifndef _SV_POLY_HXX
#include <vcl/poly.hxx>
#endif

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _VDEVCACHE_HXX
#include <goodies/vdevcache.hxx>
#endif

class B2dIAOManager;
class OutputDevice;
class Window;
class B2dIAOElementProvider;

#define	B2D_NUM_IAOPIXEL_TO_ALLOCATE			(2048)
#define	B2D_NUM_IAOBITMAP_TO_ALLOCATE			(256)
#define	B2D_NUM_IAOBMPREF_TO_ALLOCATE			(1024)
#define	B2D_NUM_IAOBMPVDEV_TO_ALLOCATE			(256)

#define	B2D_NUM_IAOPIXEL_PER_IOOPERATION		(4096)

#define	B2D_IAO_ELEMENT_PIXEL			(0x0000)
#define	B2D_IAO_ELEMENT_BITMAP			(0x0001)
#define	B2D_IAO_ELEMENT_BITMAPREFERENCE	(0x0002)
#define	B2D_IAO_ELEMENT_BMPVDEV			(0x0003)

class B2dIAOElement
{
	friend class				B2dIAOPixelProvider;
	friend class				B2dIAOBitmapProvider;
	friend class				B2dIAOBmpRefProvider;
	friend class				B2dIAOBmpVDevProvider;

	B2dIAOElement*				pNext;	
	signed						nXpos : 15;
	signed						nYpos : 15;
	unsigned					nType : 2;

public:

	BOOL IsPixel() const { return (nType == B2D_IAO_ELEMENT_PIXEL); }
	BOOL IsBitmap() const { return (nType == B2D_IAO_ELEMENT_BITMAP); }
	BOOL IsBmpRef() const { return (nType == B2D_IAO_ELEMENT_BITMAPREFERENCE); }
	BOOL IsBmpVDev() const { return (nType == B2D_IAO_ELEMENT_BMPVDEV); }

	void Move(const Point& rDelta) { nXpos += rDelta.X(); nYpos += rDelta.Y(); }

	INT16 GetPositionX() const { return nXpos; }
	INT16 GetPositionY() const { return nYpos; }
	void SetPosition(const Point& rNew) { nXpos = rNew.X(); nYpos = rNew.Y(); }

	BOOL IsInside(const Region& rRegion);
	BOOL IsOutside(const Region& rRegion);

	B2dIAOElement* GetNext() const { return pNext; }

	B2dIAOElement* AddToList(B2dIAOElement* pList) { pNext = pList; return this; }
	B2dIAOElement* RemFromList(B2dIAOElement* pList, B2dIAOElement* pPrev)
		{ if(pPrev)	pPrev->pNext = pNext; else pList = pNext; pNext = NULL;	return pList; }
	B2dIAOElement* RemFirstFromList(B2dIAOElement* pList)
		{ pList = pNext; pNext = NULL;	return pList; }
};

class B2dIAOPixel : public B2dIAOElement
{
	friend class				B2dIAOPixelProvider;

	Color						aColor;

public:
	Color GetColor() const { return aColor; }
	void SetColor(Color aNew) { aColor = aNew; }
};

class B2dIAOBitmap : public B2dIAOElement
{
	friend class				B2dIAOBitmapProvider;

	BitmapEx					aBitmap;

public:
	const BitmapEx& GetBitmap() const { return aBitmap; }
	void SetBitmap(const BitmapEx& rNew) { aBitmap = rNew; }
};

class B2dIAOBmpRef : public B2dIAOElement
{
	friend class				B2dIAOBmpRefProvider;

	BitmapEx*					pBitmap;

public:
	B2dIAOBmpRef() : pBitmap(NULL) {}
	const BitmapEx* GetBitmap() const { return pBitmap; }
	void SetBitmap(BitmapEx* pNew) { pBitmap = pNew; }
};

class B2dIAOBmpVDev : public B2dIAOElement
{
	friend class				B2dIAOBmpVDevProvider;

	VDevCacheEntry*				pEntry;

public:
	B2dIAOBmpVDev() : pEntry(NULL) {}
	VDevCacheEntry* GetEntry() { return pEntry; }
	void SetEntry(VDevCacheEntry* pNew) { pEntry = pNew; }
};

class B2dIAOPixelProvider
{
	friend class				B2dIAOManager;

	List						aMemList;

	B2dIAOPixel*				pFirst;

	void CreateNewEntries();

	void TryToReleaseSomeMemory();

public:
	B2dIAOPixelProvider() 
	:	aMemList(64, 4, 4), 
		pFirst(NULL)
	{}
	~B2dIAOPixelProvider() 
		{ for(void* pObj = aMemList.First(); pObj; pObj = aMemList.Next()) delete[] (B2dIAOPixel*)pObj; }

	B2dIAOPixel* GetB2dIAOPixel()
		{ if(!pFirst) CreateNewEntries(); B2dIAOPixel* pRet = pFirst; pFirst = (B2dIAOPixel*)pFirst->pNext; return pRet; }
	void PutB2dIAOPixel(B2dIAOPixel* pOld)
		{ pOld->pNext = pFirst; pFirst = pOld; }
};

class B2dIAOBitmapProvider
{
	friend class				B2dIAOManager;

	List						aMemList;

	B2dIAOBitmap*				pFirst;

	void CreateNewEntries();

	void TryToReleaseSomeMemory();

public:
	B2dIAOBitmapProvider() 
	:	aMemList(64, 4, 4), 
		pFirst(NULL)
	{}
	~B2dIAOBitmapProvider() 
		{ for(void* pObj = aMemList.First(); pObj; pObj = aMemList.Next()) delete[] (B2dIAOBitmap*)pObj; }

	B2dIAOBitmap* GetB2dIAOBitmap() 
		{ if(!pFirst) CreateNewEntries(); B2dIAOBitmap* pRet = pFirst; pFirst = (B2dIAOBitmap*)pFirst->pNext; return pRet; }
	void PutB2dIAOBitmap(B2dIAOBitmap* pOld) 
		{ pOld->aBitmap = Bitmap(); pOld->pNext = pFirst; pFirst = pOld; }
};

class B2dIAOBmpRefProvider
{
	friend class				B2dIAOManager;

	List						aMemList;

	B2dIAOBmpRef*				pFirst;

	void CreateNewEntries();

	void TryToReleaseSomeMemory();

public:
	B2dIAOBmpRefProvider() 
	:	aMemList(64, 4, 4), 
		pFirst(NULL)
	{}
	~B2dIAOBmpRefProvider() 
		{ for(void* pObj = aMemList.First(); pObj; pObj = aMemList.Next()) delete[] (B2dIAOBmpRef*)pObj; }

	B2dIAOBmpRef* GetB2dIAOBmpRef() 
		{ if(!pFirst) CreateNewEntries(); B2dIAOBmpRef* pRet = pFirst; pFirst = (B2dIAOBmpRef*)pFirst->pNext; return pRet; }
	void PutB2dIAOBmpRef(B2dIAOBmpRef* pOld) 
		{ pOld->pBitmap = NULL; pOld->pNext = pFirst; pFirst = pOld; }
};

class B2dIAOBmpVDevProvider
{
	friend class				B2dIAOManager;

	List						aMemList;

	B2dIAOBmpVDev*				pFirst;

	void CreateNewEntries();

	void TryToReleaseSomeMemory();

public:
	B2dIAOBmpVDevProvider() 
	:	aMemList(64, 4, 4), 
		pFirst(NULL)
	{}
	~B2dIAOBmpVDevProvider() 
		{ for(void* pObj = aMemList.First(); pObj; pObj = aMemList.Next()) delete[] (B2dIAOBmpVDev*)pObj; }

	B2dIAOBmpVDev* GetB2dIAOBmpVDev() 
		{ if(!pFirst) CreateNewEntries(); B2dIAOBmpVDev* pRet = pFirst; pFirst = (B2dIAOBmpVDev*)pFirst->pNext; return pRet; }
	void PutB2dIAOBmpVDev(B2dIAOBmpVDev* pOld) 
		{ pOld->pEntry = NULL; pOld->pNext = pFirst; pFirst = pOld; }
};

class B2dPositionEntity
{
	B2dPositionEntity*			pNext;
	Point						aPosition;
	Point						aPositionPixel;

public:
	B2dPositionEntity(B2dPositionEntity*& rpList, Point aPos = Point(0, 0))
		{ pNext = rpList; rpList = this; aPosition = aPos; }

	const Point& GetPosition() const { return aPosition; }
	void SetPosition(const Point& rNew) { aPosition = rNew; }

	const Point& GetPositionPixel() const { return aPositionPixel; }
	void SetPositionPixel(const Point& rNew) { aPositionPixel = rNew; }

	B2dPositionEntity* GetNext() const { return pNext; }
};

class B2dIAObject
{

	friend class				B2dIAOManager;

	B2dIAOManager*				pManager;

	B2dIAObject*				pNext;
	B2dIAObject*				pPrev;

	B2dIAOElement*				pGeometry;

	B2dPositionEntity*			pGeomPositions;

	B2dPositionEntity			aBasePosition;

	Rectangle					aBaseRect;

	Color						aBaseColor;

	unsigned					bVisible : 1;

	unsigned					bGeometryValid : 1;

	unsigned					bBaseRectValid : 1;

	unsigned					bAnimationOn : 1;
	unsigned					bAnimRegisteredAtManager : 1;

	unsigned					bIsHittable : 1;

	void FreeGeometry();

	void CheckAnimationState();

protected:

	virtual void CreateGeometry() = 0;

	virtual void CreateBaseRect() = 0;

	virtual void AnimationStep(UINT32 nAnimCounter);

	void ApplyDevice(OutputDevice *pOut);

	const Rectangle& GetBaseRect() const;
	void SetBaseRect(const Rectangle& rNew) { aBaseRect = rNew; }

	B2dIAOElement* GetGeometry();

	void GeometryChange() { if(bGeometryValid) FreeGeometry(); }
	void BaseRectChange() { bBaseRectValid = FALSE; }

	BOOL AddPixel(const Point& rPos, const Color& rCol);

	BOOL AddBitmap(const Point& rPos, const BitmapEx& rBmp);
	BOOL AddBmpRef(const Point& rPos, BitmapEx* pBmp);

	virtual BOOL AddLinePixel(const Point& rPos, UINT32 nPixelNum);
	BOOL AddLine(const Point& rPos1, const Point& rPos2);

	virtual BOOL AddTrianglePixel(const Point& rPos);
	BOOL AddTriangle(const Point& rPos1, const Point& rPos2, const Point& rPos3);

	const Point& GetBasePositionPixel() const { return aBasePosition.GetPositionPixel(); }

	B2dPositionEntity*& GetGeomPositions() { return pGeomPositions; }

public:
	TYPEINFO();
	B2dIAObject(B2dIAOManager* pMan, Point aBasPos = Point(0,0), Color aBasCol = Color(COL_LIGHTGREEN));
	virtual ~B2dIAObject();

	void Transform(const Matrix3D& rMat);

	B2dIAOManager* GetManager() const { return pManager; }

	B2dIAObject* GetNext() const { return pNext; }

	const Point& GetBasePosition() const { return aBasePosition.GetPosition(); }
	void SetBasePosition(Point aNew);

	BOOL IsVisible() const { return bVisible; }
	void SetVisible(BOOL bNew);

	BOOL IsHittable() const { return bIsHittable; }
	void SetHittable(BOOL bNew) { bIsHittable = bNew; }

	const Color& GetBaseColor() const { return aBaseColor; }
	void SetBaseColor(Color aNew);

	BOOL IsAnimation() const { return bAnimationOn; }
	void SetAnimation(BOOL bNew);

	virtual BOOL IsHit(const Point& rPixelPos, UINT16 nTol = 0) const;

	inline void InvalidateRectangle();
};

class B2dIAOGroup
{
	B2dIAObject*				pIAO;
	Container*					pIAOList;

public:
	B2dIAOGroup();
	~B2dIAOGroup();

	void Clear();

	void Delete();

	BOOL InsertIAO(B2dIAObject* pNew);
	BOOL RemoveIAO(B2dIAObject* pOld);

	UINT32 GetIAOCount() const;
	B2dIAObject* GetIAObject(UINT32 nNum) const;

	BOOL IsHit(const Point& rPixelPos, UINT16 nTol = 0) const;
};

class B2dIAOManager
{

	friend class				B2dIAObject;

	Window*						pWindow;

	static B2dIAOPixelProvider	aPixelProvider;
	static B2dIAOBitmapProvider	aBitmapProvider;
	static B2dIAOBmpRefProvider	aBmpRefProvider;
	static B2dIAOBmpVDevProvider aBmpVDevProvider;

	VDevCache					aVDevCache;

	B2dIAObject*				pIAObjectList;
	B2dIAObject*				pIAObjectListEnd;
	UINT32						nIAONumber;

	B2dIAOElement*				pSaveList;

	MapMode						maLastMapMode;

	Polygon						aTmpPoly;
	Color*						pTmpColor;
	UINT16						nNextFree;

	Region						aClipRegion;

	Rectangle					aInvalidateRectangle;
	 AutoTimer					aAnimator;
	UINT32						nAnimatorCount;
	List						aAnimatedObjectList;

	unsigned					bChanged : 1;

	unsigned					bVisible : 1;

	unsigned					bAnimationOn : 1;
	unsigned					bTimerIsOn : 1;

	B2dIAOPixel* GetB2dIAOPixel() { return aPixelProvider.GetB2dIAOPixel(); }
	B2dIAOBitmap* GetB2dIAOBitmap() { return aBitmapProvider.GetB2dIAOBitmap(); }
	B2dIAOBmpRef* GetB2dIAOBmpRef() { return aBmpRefProvider.GetB2dIAOBmpRef(); }
	B2dIAOBmpVDev* GetB2dIAOBmpVDev() { return aBmpVDevProvider.GetB2dIAOBmpVDev(); }
	void PutB2dIAOElement(B2dIAOElement* pOld) 
		{ if(pOld->IsPixel()) aPixelProvider.PutB2dIAOPixel((B2dIAOPixel*)pOld);
	else if(pOld->IsBitmap()) aBitmapProvider.PutB2dIAOBitmap((B2dIAOBitmap*)pOld);
	else if(pOld->IsBmpRef()) aBmpRefProvider.PutB2dIAOBmpRef((B2dIAOBmpRef*)pOld); 
	else aBmpVDevProvider.PutB2dIAOBmpVDev((B2dIAOBmpVDev*)pOld); }

	void RestoreBackground(const Region& rClipRegion, const Region& rWindowRegion, BOOL bInPaint);
	BOOL SaveBackground(const Region& rClipRegion);
	void ForgetBackground();
	void Paint(const Region& rClipRegion);

	void PixelArrayAdd(B2dIAOPixel* pNew);
	void PixelArrayFlushWrite();
	 DECL_LINK(AnimatorHdl, AutoTimer*);

	VDevCache& GetVDevCache() { return aVDevCache; }

protected:
	void ApplyClipRegion(const Region& rClipRegion);
	void ApplyDevice(OutputDevice *pOut, BOOL bInPaint);
	BOOL InsertIAO(B2dIAObject* pNew);
	BOOL RemoveIAO(B2dIAObject* pOld);
	B2dIAOElement* GetSave() const { return pSaveList; }
	void SetSave(B2dIAOElement* pNew) { pSaveList = pNew; }
	void MoveSavedElementsPixel(const Point& rDelta);
	void CheckTimerState();
	void RegisterAnimatedObject(B2dIAObject* pNew);
	void UnregisterAnimatedObject(B2dIAObject* pOld);

public:
	B2dIAOManager(Window* pTarget);
	virtual ~B2dIAOManager();

	Window* GetWindow() const { return pWindow; }

	void Transform(const Matrix3D& rMat);

	const Region& GetClipRegion() const { return aClipRegion; }

	B2dIAObject* GetIAObjectList() const { return pIAObjectList; }
	UINT32 GetIAOCount() const { return nIAONumber; }
	B2dIAObject* GetIAObject(UINT32 nNum) const;

	BOOL IsVisible() const;
	void SetVisible(BOOL bNew); 

	void InvalidateRectangle(const Rectangle& rRect);

	BOOL UpdateDisplay();

	void ForceHide();

	BOOL IsAnimation() const { return bAnimationOn; }
	void SetAnimation(BOOL bNew);
};

#endif          
