

#ifndef _B3D_B3DENTITY_HXX
#define _B3D_B3DENTITY_HXX

#ifndef _B3D_B3DCOLOR_HXX
#include <goodies/b3dcolor.hxx>
#endif

#ifndef _B3D_POINT4D_HXX
#include <goodies/point4d.hxx>
#endif

#ifndef _B3D_BUCKET_HXX
#include <goodies/bucket.hxx>
#endif

class Base3DCommon;
class Base3D;
class B3dTransformationSet;
class Matrix4D;

class B3dEntity
{
private:

	Point4D					aPoint;
	Vector3D				aNormal;
	Vector3D				aPlaneNormal;
	Vector3D				aTexCoor;
	B3dColor				aColor;

	UINT8					bEdgeFlag;

	unsigned				bValid			: 1;

	unsigned				bNormalUsed		: 1;

	unsigned				bTexCoorUsed	: 1;

	unsigned				bDeviceCoor		: 1;

public:
	B3dEntity() { Reset(); }

	BOOL IsValid() { return bValid; }
	void SetValid(BOOL bNew=TRUE) { bValid = bNew; }

	BOOL IsNormalUsed() { return bNormalUsed; }
	void SetNormalUsed(BOOL bNew=TRUE) { bNormalUsed = bNew; }

	BOOL IsTexCoorUsed() { return bTexCoorUsed; }
	void SetTexCoorUsed(BOOL bNew=TRUE) { bTexCoorUsed = bNew; }

	BOOL IsDeviceCoor() { return bDeviceCoor; }
	void SetDeviceCoor(BOOL bNew=TRUE) { bDeviceCoor = bNew; }

	BOOL IsEdgeVisible() { return bEdgeFlag; }
	void SetEdgeVisible(BOOL bNew=TRUE) { bEdgeFlag = bNew; }

	Point4D& Point() { return aPoint; }
	Vector3D& Normal() { return aNormal; }
	Vector3D& PlaneNormal() { return aPlaneNormal; }
	Vector3D& TexCoor() { return aTexCoor; }
	B3dColor& Color() { return aColor; }
	UINT8& EdgeFlag() { return bEdgeFlag; }

	double GetX() { return aPoint.X(); }
	void SetX(double fNew) { aPoint.X() = fNew; }

	double GetY() { return aPoint.Y(); }
	void SetY(double fNew) { aPoint.Y() = fNew; }

	double GetZ() { return aPoint.Z(); }
	void SetZ(double fNew) { aPoint.Z() = fNew; }

	void Reset();

	void Copy(B3dEntity& rEnt);
	void ToDeviceCoor(B3dTransformationSet* pSet)
		{ if(!IsDeviceCoor()) ImplToDeviceCoor(pSet); }
	void To3DCoor(B3dTransformationSet* pSet)
		{ if(IsDeviceCoor()) ImplTo3DCoor(pSet); }

	void ForceEqualBase(B3dTransformationSet* pSet, B3dEntity& rOld);
	void ForceEqualBase(B3dTransformationSet* pSet, B3dEntity& rOld1,
		B3dEntity& rOld2);
	void CalcInBetween(B3dEntity& rOld1,
		B3dEntity& rOld2, double t);
	void CalcMiddle(B3dEntity& rOld1,
		B3dEntity& rOld2);
	void CalcMiddle(B3dEntity& rOld1,
		B3dEntity& rOld2, B3dEntity& rOld3);

	void Transform(const Matrix4D&);

protected:
	void ImplToDeviceCoor(B3dTransformationSet* pSet);
	void ImplTo3DCoor(B3dTransformationSet* pSet);
};

BASE3D_DECL_BUCKET(B3dEntity, Bucket)

#endif          
