

#ifndef _B3D_B3DTEX_HXX
#define _B3D_B3DTEX_HXX

#ifndef _SV_OPENGL_HXX
#include <vcl/opengl.hxx>
#endif

#ifndef _SV_BITMAPEX_HXX
#include <vcl/bitmapex.hxx>
#endif

#ifndef _SV_COLOR_HXX
#include <vcl/color.hxx>
#endif

#ifndef _SV_SALBTYPE_HXX
#include <vcl/salbtype.hxx>
#endif

#ifndef _TOOLS_TIME_HXX
#include <tools/time.hxx>
#endif

class BitmapReadAccess;
class BitmapColor;

#define	B3D_TEXTURE_LIFETIME				0, 1, 0

#ifdef WIN
typedef UINT8 huge*	GL_UINT8;
#else
typedef UINT8*		GL_UINT8;
#endif

enum Base3DTextureKind
{
	Base3DTextureLuminance = 1,
	Base3DTextureIntensity,
	Base3DTextureColor
};

enum Base3DTextureMode
{
	Base3DTextureReplace = 1,
	Base3DTextureModulate,
	Base3DTextureBlend
};

enum Base3DTextureFilter
{
	Base3DTextureNearest = 1,
	Base3DTextureLinear
};

enum Base3DTextureWrap
{
	Base3DTextureClamp = 1,
	Base3DTextureRepeat,
	Base3DTextureSingle
};

#define	B3D_TXT_KIND_LUM		0x00
#define	B3D_TXT_KIND_INT		0x01
#define	B3D_TXT_KIND_COL		0x02

#define	B3D_TXT_MODE_REP		0x04
#define	B3D_TXT_MODE_MOD		0x08
#define	B3D_TXT_MODE_BND		0x0C

#define	B3D_TXT_FLTR_NEA		0x10

#define	TEXTURE_ATTRIBUTE_TYPE_COLOR			0x0000
#define	TEXTURE_ATTRIBUTE_TYPE_BITMAP			0x0001
#define	TEXTURE_ATTRIBUTE_TYPE_GRADIENT			0x0002
#define	TEXTURE_ATTRIBUTE_TYPE_HATCH			0x0003

class TextureAttributes
{
private:
	void*			mpFloatTrans;
	BOOL			mbGhosted;

public:
	TextureAttributes(BOOL bGhosted, void* pFT);

	virtual BOOL operator==(const TextureAttributes&) const;
	virtual UINT16 GetTextureAttributeType() const =0;

	BOOL GetGhostedAttribute() { return mbGhosted; }
	void* GetFloatTransAttribute() { return mpFloatTrans; }
};

class TextureAttributesColor : public TextureAttributes
{
private:
	Color			maColorAttribute;

public:
	TextureAttributesColor(BOOL bGhosted, void* pFT, Color aColor);

	virtual BOOL operator==(const TextureAttributes&) const;
	virtual UINT16 GetTextureAttributeType() const;

	Color GetColorAttribute() { return maColorAttribute; }
};

class TextureAttributesBitmap : public TextureAttributes
{
private:
	Bitmap			maBitmapAttribute;

public:
	TextureAttributesBitmap(BOOL bGhosted, void* pFT, Bitmap aBmp);

	virtual BOOL operator==(const TextureAttributes&) const;
	virtual UINT16 GetTextureAttributeType() const;

	Bitmap GetBitmapAttribute() { return maBitmapAttribute; }
};

class TextureAttributesGradient : public TextureAttributes
{
private:
	void*			mpFill;
	void*			mpStepCount;

public:
	TextureAttributesGradient(BOOL bGhosted, void* pFT, void* pF, void *pSC);

	virtual BOOL operator==(const TextureAttributes&) const;
	virtual UINT16 GetTextureAttributeType() const;

	void* GetFillAttribute() { return mpFill; }
	void* GetStepCountAttribute() { return mpStepCount; }
};

class TextureAttributesHatch : public TextureAttributes
{
private:
	void*			mpFill;

public:
	TextureAttributesHatch(BOOL bGhosted, void* pFT, void* pF);

	virtual BOOL operator==(const TextureAttributes&) const;
	virtual UINT16 GetTextureAttributeType() const;

	void* GetHatchFillAttribute() { return mpFill; }
};

class B3dTexture
{
protected:

	Bitmap					aBitmap;
	AlphaMask				aAlphaMask;
	BitmapReadAccess*		pReadAccess;
	BitmapReadAccess*		pAlphaReadAccess;

	TextureAttributes*		pAttributes;

	Time					maTimeStamp;

	BitmapColor				aColBlend;

	BitmapColor				aColTexture;

	Base3DTextureKind		eKind;

	Base3DTextureMode		eMode;

	Base3DTextureFilter		eFilter;

	Base3DTextureWrap		eWrapS;
	Base3DTextureWrap		eWrapT;

	UINT8					nSwitchVal;

	unsigned				bTextureKindChanged	: 1;

	B3dTexture(TextureAttributes& rAtt,
		BitmapEx& rBmpEx,
		Base3DTextureKind=Base3DTextureColor,
		Base3DTextureMode=Base3DTextureReplace,
		Base3DTextureFilter=Base3DTextureNearest,
		Base3DTextureWrap eS=Base3DTextureSingle,
		Base3DTextureWrap eT=Base3DTextureSingle);
	virtual ~B3dTexture();

	inline const BitmapColor GetBitmapColor(long nX, long nY);
	inline const sal_uInt8 GetBitmapTransparency(long nX, long nY);

	void SetSwitchVal();

	void Touch() { maTimeStamp = Time() + Time(B3D_TEXTURE_LIFETIME); }
	const Time& GetTimeStamp() const { return maTimeStamp; }

public:

	TextureAttributes& GetAttributes();

	Bitmap& GetBitmap() { return aBitmap; }
	AlphaMask& GetAlphaMask() { return aAlphaMask; }
	BitmapEx GetBitmapEx() { return BitmapEx(aBitmap, aAlphaMask); }
	const Size GetBitmapSize() { return aBitmap.GetSizePixel(); }

	void ModifyColor(Color& rCol, double fS, double fT);

	void SetTextureKind(Base3DTextureKind eNew);
	Base3DTextureKind GetTextureKind() { return eKind; }

	void SetTextureMode(Base3DTextureMode eNew);
	Base3DTextureMode GetTextureMode() { return eMode; }

	void SetTextureFilter(Base3DTextureFilter eNew);
	Base3DTextureFilter GetTextureFilter() { return eFilter; }

	void SetTextureWrapS(Base3DTextureWrap eNew);
	Base3DTextureWrap GetTextureWrapS() { return eWrapS; }
	void SetTextureWrapT(Base3DTextureWrap eNew);
	Base3DTextureWrap GetTextureWrapT() { return eWrapT; }

	void SetBlendColor(Color rNew);
	Color GetBlendColor();

	void SetTextureColor(Color rNew);
	Color GetTextureColor();

protected:

	friend class Base3D;
	friend class Base3DOpenGL;
	friend class B3dGlobalData;
	friend class B3dTextureStore;
};

class B3dTextureOpenGL : public B3dTexture
{
private:

	GLuint					nTextureName;

	B3dTextureOpenGL(TextureAttributes& rAtt,
		BitmapEx& rBmpEx,
		OpenGL& rOGL,
		Base3DTextureKind=Base3DTextureColor,
		Base3DTextureMode=Base3DTextureReplace,
		Base3DTextureFilter=Base3DTextureNearest,
		Base3DTextureWrap eS=Base3DTextureClamp,
		Base3DTextureWrap eT=Base3DTextureClamp);
	virtual ~B3dTextureOpenGL();

	void DestroyOpenGLTexture(OpenGL&);

public:

	void MakeCurrentTexture(OpenGL&);

	void CreateOpenGLTexture(OpenGL&);

protected:

	friend class Base3D;
	friend class Base3DOpenGL;
	friend class B3dTextureStore;
};

#endif          
