

#ifndef _SVX_VECTOR3D_HXX
#define _SVX_VECTOR3D_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _INC_MATH
#include <math.h>
#endif

class Point;
class SvStream;

class Vector3D
{
 protected:
	double V[3];

 public:
	Vector3D(double fX = 0, double fY = 0, double fZ = 0)
		{ V[0] = fX; V[1] = fY; V[2] = fZ; }
	Vector3D(const Point& rPnt, double fZ = 0);
	const double& X() const { return V[0]; }
	const double& Y() const { return V[1]; }
	const double& Z() const { return V[2]; }
	double& X() { return V[0]; }
	double& Y() { return V[1]; }
	double& Z() { return V[2]; }
	const double& operator[] (int nPos) const { return V[nPos]; }
	double& operator[] (int nPos) { return V[nPos]; }

	double	GetLength(void) const;
	double	GetXYLength(void) const;
	double	GetXZLength(void) const;
	double	GetYZLength(void) const;

	void	Normalize();
	void	Min(const Vector3D& rVec);
	void	Max(const Vector3D& rVec);
	void	Abs();

	void CalcInBetween(const Vector3D& rOld1, const Vector3D& rOld2, double t);
	void CalcMiddle(const Vector3D& rOld1, const Vector3D& rOld2);
	void CalcMiddle(const Vector3D& rOld1, const Vector3D& rOld2, Vector3D& rOld3);

	Vector3D GetPerpendicular2D() const;
	Vector3D GetPerpendicular(const Vector3D& rNormalizedVec) const;
	Vector3D GetProjectionOnPlane(const Vector3D& rNormalizedPlane) const;

	Vector3D&	operator+=	(const Vector3D&);
	Vector3D&	operator-=	(const Vector3D&);
	Vector3D	operator+ 	(const Vector3D&) const;
	Vector3D	operator-	(const Vector3D&) const;
	Vector3D	operator-	(void) const;

	Vector3D&	operator|=	(const Vector3D&);
	Vector3D	operator|	(const Vector3D&) const;

	double Scalar(const Vector3D&) const;

	Vector3D&	operator/=	(const Vector3D&);
	Vector3D	operator/	(const Vector3D&) const;
	Vector3D&	operator*=	(const Vector3D&);
	Vector3D	operator*	(const Vector3D&) const;

	Vector3D&	operator*=	(double);
	Vector3D	operator*	(double) const;
	Vector3D&	operator/=	(double);
	Vector3D	operator/	(double) const;

	BOOL		operator==	(const Vector3D&) const;
	BOOL		operator!=	(const Vector3D&) const;

	friend SvStream& operator>>(SvStream& rIStream, Vector3D&);
	friend SvStream& operator<<(SvStream& rOStream, const Vector3D&);
};

#endif 
