

#ifndef _INETFTP_HXX
#define _INETFTP_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _VOS_OBJECT_HXX_
#include <vos/object.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif

#ifndef _DATETIME_HXX
#include <tools/datetime.hxx>
#endif

#ifndef _INET_CLIENT_HXX
#include <inet/client.hxx>
#endif

class List;
class SvLockBytes;
class SvOpenLockBytes;

#define INETCOREFTP_CMD_PORT 21

#define INETCOREFTP_FILEMODE_UNKNOWN 0x00
#define INETCOREFTP_FILEMODE_READ    0x01
#define INETCOREFTP_FILEMODE_WRITE   0x02
#define INETCOREFTP_FILEMODE_ISDIR   0x04
#define INETCOREFTP_FILEMODE_ISLINK  0x08

#ifdef _USE_NAMESPACE
namespace inet {
#endif

struct INetFTPDirentry
{
	rtl::OUString m_aName;
	DateTime      m_aDate;
	sal_uInt32    m_nMode;
	sal_uInt32    m_nSize;

	INetFTPDirentry (void)
		: m_aDate (Date(0), Time(0)),
	m_nMode (INETCOREFTP_FILEMODE_UNKNOWN),
	m_nSize ((sal_uInt32)(-1))
	{}
};

class INetFTPConnection;

typedef sal_Bool INetFTPCallback (
	INetFTPConnection *pConnection,
	sal_Int32          nReplyCode,
	const sal_Char    *pReplyText,
	void              *pData);

class INetFTPConnection :
	public NAMESPACE_INET(INetClientConnection_Impl)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetFTPConnection, inet));

public:

	enum Reply
	{

		REPLY_TRANSFER_WAIT      = -18,

		REPLY_RESPONSE_DONE      = -16,
		REPLY_RESPONSE_ERROR     = -15,
		REPLY_RESPONSE_WAIT      = -14,

		REPLY_REQUEST_DONE       = -12,
		REPLY_REQUEST_ERROR      = -11,
		REPLY_REQUEST_WAIT       = -10,

		REPLY_CONNECT_DONE       =  -8,
		REPLY_CONNECT_ERROR      =  -7,
		REPLY_CONNECT_WAIT       =  -6,

		REPLY_RESOLVER_DONE      =  -4,
		REPLY_RESOLVER_ERROR     =  -3,
		REPLY_RESOLVER_WAIT      =  -2,

		REPLY_NETWORK_ERROR      =  -1,

		REPLY_SERVICE_DELAYED    = 120,
		REPLY_TRANSFER_STARTING  = 125,
		REPLY_OPENING_DCON       = 150,

		REPLY_CMD_OK             = 200,
		REPLY_CMD_NOT_NECESSARY  = 202,
		REPLY_SERVICE_READY      = 220,
		REPLY_PASSIVE_MODE       = 227,
		REPLY_USER_LOGGED_IN     = 230,
		REPLY_FILE_ACTION_OK     = 250,
		REPLY_PATHNAME_CREATED   = 257,

		REPLY_NEED_PASSWORD      = 331,
		REPLY_NEED_ACCOUNT       = 332,
		REPLY_NEED_XFERCMD       = 350,

		REPLY_SERVICE_UNAVAIL    = 421,
		REPLY_OPEN_DCON_ERROR    = 425,
		REPLY_TRANSFER_ABORTED   = 426,

		REPLY_SYNTAX_ERROR       = 500,
		REPLY_PARAM_SYNTAX_ERROR = 501,
		REPLY_BAD_CMD_SEQUENCE   = 503,
		REPLY_NOT_LOGGED_IN      = 530
	};

	static sal_Bool createInstance (
		NAMESPACE_VOS(ORef)<INetFTPConnection> &rxConnection);

	virtual sal_Bool open (
		const rtl::OUString &rHost, sal_uInt16 nPort,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool isOpen (void) const = 0;

	virtual sal_Bool loginUsername (
		const rtl::OUString &rUsername,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool loginPassword (
		const rtl::OUString &rPassword,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool loginAccount (
		const rtl::OUString &rAccount,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool isLoggedIn (void) const = 0;

	virtual sal_Bool close (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual void abort (void) = 0;

	virtual sal_Bool noop (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool setCurDir (
		const rtl::OUString &rPathname,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool getCurDir (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual const rtl::OUString& getCurDir (void) const = 0;

	virtual sal_Bool makeDir (
		const rtl::OUString &rPathname,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool removeDir (
		const rtl::OUString &rPathname,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool remove (
		const rtl::OUString &rPathname,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool renameFrom (
		const rtl::OUString &rFromPath,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;
	virtual sal_Bool renameTo (
		const rtl::OUString &rToPath,
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	enum DataType
	{
		DATA_TYPE_UNKNOWN,
		DATA_TYPE_ASCII,
		DATA_TYPE_IMAGE
	};

	virtual sal_Bool setTypeAscii (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool setTypeImage (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual DataType getDataType (void) const = 0;

	enum ListType
	{
		LIST_TYPE_UNDETERMINED = -1, 
		LIST_TYPE_UNKNOWN      =  0, 
		LIST_TYPE_DOS          =  1, 
		LIST_TYPE_UNIX         =  2, 
		LIST_TYPE_VMS          =  3  
	};

	virtual sal_Bool getNameList (
		const rtl::OUString &rPathname,
		List                &rNameList,
		INetFTPCallback     *pfnCB,
		void                *pDataCB) = 0;

	virtual ListType getListType (void) const = 0;

	virtual sal_Bool retrieve (
		const rtl::OUString &rFromPath,
		SvOpenLockBytes     *pToPath,
		sal_uInt32           nRestartOffset,
		INetFTPCallback     *pfnCB,
		void                *pDataCB) = 0;

	virtual sal_uInt32 getRetrieveCount (void) const = 0;

	virtual sal_Bool store (
		SvLockBytes         *pFromPath,
		const rtl::OUString &rToPath,
		sal_uInt32           nRestartOffset,
		INetFTPCallback     *pfnCB,
		void                *pDataCB) = 0;

	virtual sal_uInt32 getStoreCount (void) const = 0;

	virtual sal_Bool abortTransfer (sal_Bool bSilent = sal_True) = 0;

	virtual sal_Bool setTransferCallback (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual sal_Bool setTerminateCallback (
		INetFTPCallback *pfnCB, void *pDataCB) = 0;

	virtual const rtl::OUString& getSocksGatewayName (void) const = 0;
	virtual sal_uInt16           getSocksGatewayPort (void) const = 0;

	virtual sal_Bool hasSocksGateway (void) const = 0;
	virtual void     setSocksGateway (
		const rtl::OUString &rName, sal_uInt16 nPort) = 0;

protected:

	INetFTPConnection (void);
	virtual ~INetFTPConnection (void);

private:

	INetFTPConnection (const INetFTPConnection&);
	INetFTPConnection& operator= (const INetFTPConnection&);
};

#ifdef _USE_NAMESPACE
}
#endif

#endif 

