

#ifndef _INETHTTP_HXX
#define _INETHTTP_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _VOS_OBJECT_HXX_
#include <vos/object.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif

#ifndef _TOOLS_INETMSG_HXX
#include <tools/inetmsg.hxx>
#endif

#ifndef _INET_MACROS_HXX
#include <inet/macros.hxx>
#endif
#ifndef _INET_CLIENT_HXX
#include <inet/client.hxx>
#endif

#ifdef _USE_NAMESPACE
namespace inet {
#endif

class  INetHTTPRequest;
struct INetHTTPRequestContext;

typedef sal_Bool INetHTTPRequestCallback (
	INetHTTPRequest *pRequest,
	sal_Int32        nReplyCode,
	const sal_Char  *pReplyText,
	void            *pData);

class INetHTTPConnection :
	public NAMESPACE_INET(INetClientConnection_Impl)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPConnection, inet));

public:
	static sal_Bool createInstance (
		NAMESPACE_VOS(ORef)<INetHTTPConnection> &rxConnection);

	virtual sal_Bool startRequest (
		INetHTTPRequestContext *pCtx) = 0;

	virtual sal_Bool abortRequest (
		INetHTTPRequestContext *pCtx) = 0;

protected:
	INetHTTPConnection (void);
	virtual ~INetHTTPConnection (void);

private:

	INetHTTPConnection (const INetHTTPConnection&);
	INetHTTPConnection& operator= (const INetHTTPConnection&);
};

class INetHTTPRequest :
	public NAMESPACE_VOS(OReference),
	public NAMESPACE_VOS(OObject)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPRequest, inet));

public:

	enum Method
	{
		METHOD_GET,
		METHOD_HEAD,
		METHOD_POST,
		METHOD_PUT,
		METHOD_DELETE
	};

	enum Scheme
	{
		SCHEME_FTP,
		SCHEME_HTTP,
		SCHEME_HTTPS,
		SCHEME_OTHER
	};

	enum Reply
	{

		REPLY_CONNECTION_TERMINATED = -18,

		REPLY_RESPONSE_DONE         = -16,
		REPLY_RESPONSE_ERROR        = -15,
		REPLY_RESPONSE_WAIT         = -14,

		REPLY_REQUEST_DONE          = -12,
		REPLY_REQUEST_ERROR         = -11,
		REPLY_REQUEST_WAIT          = -10,

		REPLY_CONNECT_DONE          =  -8,
		REPLY_CONNECT_ERROR         =  -7,
		REPLY_CONNECT_WAIT          =  -6,

		REPLY_RESOLVER_DONE         =  -4,
		REPLY_RESOLVER_ERROR        =  -3,
		REPLY_RESOLVER_WAIT         =  -2,

		REPLY_NETWORK_ERROR         =  -1,

		REPLY_09                    =   0,

		REPLY_OK                    = 200,
		REPLY_CREATED               = 201,
		REPLY_ACCEPTED              = 202,
		REPLY_NO_CONTENT            = 204,

		REPLY_MULTIPLE_CHOICE       = 300,
		REPLY_MOVED_PERMANENT       = 301,
		REPLY_MOVED_TEMPORARY       = 302,
		REPLY_NOT_MODIFIED          = 304,

		REPLY_BAD_REQUEST           = 400,
		REPLY_UNAUTHORIZED          = 401,
		REPLY_FORBIDDEN             = 403,
		REPLY_NOT_FOUND             = 404,
		REPLY_PROXY_UNAUTHORIZED    = 407,

		REPLY_INTERNAL_ERROR        = 500,
		REPLY_NOT_IMPLEMENTED       = 501,
		REPLY_BAD_GATEWAY           = 502,
		REPLY_SERVICE_UNAVAIL       = 503
	};

	virtual sal_Bool start (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

	virtual void abort (void);

	virtual sal_Bool getEndpoint (
		rtl::OUString &rHost, sal_uInt16 &rPort) const;

	virtual sal_Int32  getStatusCode (void) const;

	virtual sal_Bool   isRequestHeaderGenerated  (void) const;
	virtual sal_Bool   isResponseHeaderParsed    (void) const;

	virtual sal_uInt32 getRequestDocumentLength  (void) const;
	virtual sal_uInt32 getResponseDocumentLength (void) const;

protected:
	INetHTTPRequest (Method eMethod);
	virtual ~INetHTTPRequest (void);

private:

	NAMESPACE_VOS(ORef)<INetHTTPConnection>  m_xConnection;
	INetHTTPRequestContext                  *m_pContext;

	INetHTTPRequest (const INetHTTPRequest&);
	INetHTTPRequest& operator= (const INetHTTPRequest&);
};

class INetHTTPGetRequest : public NAMESPACE_INET(INetHTTPRequest)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPGetRequest, inet));

public:
	inline INetHTTPGetRequest (void);

	inline sal_Bool loadDocument (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

	inline sal_Bool reloadDocument (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

protected:
	virtual ~INetHTTPGetRequest (void) {}

private:

	INetHTTPGetRequest (const INetHTTPGetRequest&);
	INetHTTPGetRequest& operator= (const INetHTTPGetRequest&);
};

inline INetHTTPGetRequest::INetHTTPGetRequest (void)
	: INetHTTPRequest (METHOD_GET)
{
}

inline sal_Bool INetHTTPGetRequest::loadDocument (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

inline sal_Bool INetHTTPGetRequest::reloadDocument (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	rReqMsg.SetPragma (rtl::OUString::createFromAscii ("no-cache"));
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

class INetHTTPHeadRequest : public NAMESPACE_INET(INetHTTPRequest)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPHeadRequest, inet));

public:
	inline INetHTTPHeadRequest (void);

	inline sal_Bool loadHeader (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

	inline sal_Bool reloadHeader (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

protected:
	virtual ~INetHTTPHeadRequest (void) {}

private:

	INetHTTPHeadRequest (const INetHTTPHeadRequest&);
	INetHTTPHeadRequest& operator= (const INetHTTPHeadRequest&);
};

inline INetHTTPHeadRequest::INetHTTPHeadRequest (void)
	: INetHTTPRequest (METHOD_HEAD)
{
}

inline sal_Bool INetHTTPHeadRequest::loadHeader (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

inline sal_Bool INetHTTPHeadRequest::reloadHeader (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	rReqMsg.SetPragma (rtl::OUString::createFromAscii ("no-cache"));
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

class INetHTTPPostRequest : public NAMESPACE_INET(INetHTTPRequest)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPPostRequest, inet));

public:
	inline INetHTTPPostRequest (void);

	inline sal_Bool postDocument (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

protected:
	virtual ~INetHTTPPostRequest (void) {}

private:

	INetHTTPPostRequest (const INetHTTPPostRequest&);
	INetHTTPPostRequest& operator= (const INetHTTPPostRequest&);
};

inline INetHTTPPostRequest::INetHTTPPostRequest (void)
	: INetHTTPRequest (METHOD_POST)
{
}

inline sal_Bool INetHTTPPostRequest::postDocument (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

class INetHTTPPutRequest : public NAMESPACE_INET(INetHTTPRequest)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPPutRequest, inet));

public:
	inline INetHTTPPutRequest (void);

	inline sal_Bool storeDocument (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

protected:
	virtual ~INetHTTPPutRequest (void) {}

private:

	INetHTTPPutRequest (const INetHTTPPutRequest&);
	INetHTTPPutRequest& operator= (const INetHTTPPutRequest&);
};

inline INetHTTPPutRequest::INetHTTPPutRequest (void)
	: INetHTTPRequest (METHOD_PUT)
{
}

inline sal_Bool INetHTTPPutRequest::storeDocument (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

class INetHTTPDeleteRequest : public NAMESPACE_INET(INetHTTPRequest)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetHTTPDeleteRequest, inet));

public:
	inline INetHTTPDeleteRequest (void);

	inline sal_Bool deleteDocument (
		const rtl::OUString     &rReqURI,
		INetHTTPMessage         &rReqMsg,
		INetHTTPMessage         &rResMsg,
		INetHTTPRequestCallback *pfnCB,
		void                    *pDataCB);

protected:
	virtual ~INetHTTPDeleteRequest (void) {}

private:

	INetHTTPDeleteRequest (const INetHTTPDeleteRequest&);
	INetHTTPDeleteRequest& operator= (const INetHTTPDeleteRequest&);
};

inline INetHTTPDeleteRequest::INetHTTPDeleteRequest (void)
	: INetHTTPRequest (METHOD_DELETE)
{
}

inline sal_Bool INetHTTPDeleteRequest::deleteDocument (
	const rtl::OUString     &rReqURI,
	INetHTTPMessage         &rReqMsg,
	INetHTTPMessage         &rResMsg,
	INetHTTPRequestCallback *pfnCB,
	void                    *pDataCB)
{
	return start (rReqURI, rReqMsg, rResMsg, pfnCB, pDataCB);
}

#ifdef _USE_NAMESPACE
}
#endif

#endif 

