

#ifndef _INETCORELDAP_HXX
#define _INETCORELDAP_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _INET_CLIENT_HXX
#include <inet/client.hxx>
#endif

#ifndef _LDAPMSG_HXX
#include <inet/ldapmsg.hxx>
#endif

#define INETCORELDAP_DEF_PORT 389

class  INetCoreLDAPConnection;
struct INetCoreLDAPConnectionContext;

typedef int INetCoreLDAPCallback (
	INetCoreLDAPConnection *pConnection,
	int                    nReplyCode,
	const void             *pReplyData,
	sal_uInt16              nMessageID,
	void                   *pData);

class INetCoreLDAPConnection :
	public NAMESPACE_INET(INetClientConnection_Impl)
{
public:
	INetCoreLDAPConnection (void);

	virtual sal_Bool Open (
		const rtl::OUString  &rHost,
		sal_uInt16            nPort,
		INetCoreLDAPCallback *pfnCallback,
		void                 *pData);
	virtual sal_Bool IsOpen (void);

	virtual void Abort (void);
	virtual sal_uInt16 Bind (
		const rtl::OUString &rUserDN,
		const rtl::OUString &rPassword);
	virtual sal_uInt16 Unbind (void);

	virtual sal_uInt16 Add (
		const INetCoreLDAPEntry& rNewEntry);
	virtual sal_uInt16 Compare (
		const rtl::OUString                  &rObjectDN,
		const INetCoreLDAPAttributeValuePair &rAssertion);
	virtual sal_uInt16 Delete (
		const rtl::OUString &rObjectDN);
	virtual sal_uInt16 Modify (
		const INetCoreLDAPModifyRequestMessage& rMessage);
	virtual sal_uInt16 ModifyRDN (
		const rtl::OUString &rObjectDN,
		const rtl::OUString &rNewRDN);

	virtual sal_uInt16 List (
		const rtl::OUString &rBaseObject,
		sal_Bool             bAttrsOnly = sal_False);
	virtual sal_uInt16 Read (
		const rtl::OUString &rBaseObject,
		sal_Bool             bAttrsOnly = sal_False);
	virtual sal_uInt16 Search (
		const INetCoreLDAPSearchRequestMessage& rMessage);

	virtual sal_uInt16 Abandon (sal_uInt16 nMessageID);

protected:
	virtual ~INetCoreLDAPConnection (void);

private:
	typedef NAMESPACE_INET(INetSocket) socket_type;
	INetCoreLDAPConnectionContext *m_pConCtx;

	virtual sal_Bool handleResolverEvent (
		sal_Int32 nStatus, INetCoreDNSHostEntry *pHostEntry);

	virtual sal_Bool handleSocketEvent (
		const NAMESPACE_VOS(ORef)<socket_type> &rxSocket, sal_Int32 nEvent);

	sal_uInt16 startRequest (INetCoreLDAPMessage &rMessage);
};

#endif 

