

#ifndef _INETPOP3_HXX
#define _INETPOP3_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _INET_CLIENT_HXX
#include <inet/client.hxx>
#endif

class List;

#define INETCOREPOP3_DEF_PORT 110

enum INetCorePOP3ConnectionReply
{

	INETCOREPOP3_REPLY_TRANSFER_WAIT  = -6,
	INETCOREPOP3_REPLY_CONNECT_ERROR  = -5,
	INETCOREPOP3_REPLY_CONNECT_WAIT   = -4,
	INETCOREPOP3_REPLY_RESOLVER_ERROR = -3,
	INETCOREPOP3_REPLY_RESOLVER_WAIT  = -2,
	INETCOREPOP3_REPLY_NETWORK_ERROR  = -1,

	INETCOREPOP3_REPLY_ERROR          =  0,
	INETCOREPOP3_REPLY_OK             =  1
};

struct INetCorePOP3MailListEntry
{
	sal_uInt32 m_nIndex;
	sal_uInt32 m_nSize;
};

typedef rtl::OUString INetCorePOP3UniqueIdListEntry;

class INetCoreMessageOStream;

#ifdef _USE_NAMESPACE
namespace inet {
#endif

class  INetCorePOP3Connection;
struct INetCorePOP3CmdContext;
struct INetCorePOP3ConnectionContext;

typedef sal_Bool INetCorePOP3Callback (
	INetCorePOP3Connection *pConnection,
	sal_Int32               nReplyCode,
	const sal_Char         *pReplyText,
	void                   *pData);

class INetCorePOP3Connection :
	public NAMESPACE_INET(INetClientConnection_Impl)
{
public:
	INetCorePOP3Connection (void);

	sal_Bool Open (
		const rtl::OUString &rHost, sal_uInt16 nPort,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool IsOpen (void);

	sal_Bool Close (
		INetCorePOP3Callback *pfnCallback, void *pData);
	void Destroy (void);

	sal_Bool LoginUser (
		const rtl::OUString &rUsername,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool LoginPassword (
		const rtl::OUString &rPassword,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool LoginSecure (
		const rtl::OUString &rUsername,
		const rtl::OUString &rPassword,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool IsLoggedIn (void);

	sal_Bool GetMailListEntry (
		INetCorePOP3MailListEntry &rMailEntry,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool GetMailList (
		List& rMailList,
		INetCorePOP3Callback *pfnCallback, void *pData);

	sal_Bool GetUniqueIdList (
		List& rUniqueIdList,
		INetCorePOP3Callback *pfnCallback, void *pData);

	sal_Bool GetMailHeader (
		sal_uInt32 nIndex, INetCoreMessageOStream& rMessageStream,
		INetCorePOP3Callback *pfnCallback, void *pData);

	sal_Bool DeleteMail (
		sal_uInt32 nIndex,
		INetCorePOP3Callback *pfnCallback, void *pData);

	sal_Bool RetrieveMail (
		sal_uInt32 nIndex, INetCoreMessageOStream& rMessageStream,
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_uInt32 GetRetrieveCount (void);

	sal_Bool SetRetrieveCallback (
		INetCorePOP3Callback *pfnCallback, void *pData);
	sal_Bool SetTerminateCallback (
		INetCorePOP3Callback *pfnCallback, void *pData);

protected:
	virtual ~INetCorePOP3Connection (void);

private:
	INetCorePOP3ConnectionContext *m_pConCtx;

	virtual sal_Bool handleResolverEvent (
		sal_Int32 nStatus, INetCoreDNSHostEntry *pHostEntry);

	virtual sal_Bool handleSocketEvent (
		const NAMESPACE_VOS(ORef)<INetSocket> &rxSocket, sal_Int32 nEvent);

	sal_Bool StartCommand (INetCorePOP3CmdContext *pCtx);
};

#ifdef _USE_NAMESPACE
}
#endif

#endif 

