

#ifndef _INETSMTP_HXX
#define _INETSMTP_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _INET_CLIENT_HXX
#include <inet/client.hxx>
#endif

class INetCoreMessageIStream;

#define INETCORESMTP_DEF_PORT 25

enum INetCoreSMTPConnectionReply
{

	INETCORESMTP_REPLY_TRANSFER_WAIT      =  -6,
	INETCORESMTP_REPLY_CONNECT_ERROR      =  -5,
	INETCORESMTP_REPLY_CONNECT_WAIT       =  -4,
	INETCORESMTP_REPLY_RESOLVER_ERROR     =  -3,
	INETCORESMTP_REPLY_RESOLVER_WAIT      =  -2,
	INETCORESMTP_REPLY_NETWORK_ERROR      =  -1,

	INETCORESMTP_REPLY_SYSTEM_STATUS      = 211,
	INETCORESMTP_REPLY_HELP_MESSAGE       = 214,
	INETCORESMTP_REPLY_SERVICE_READY      = 220,
	INETCORESMTP_REPLY_SERVICE_CLOSING    = 221,
	INETCORESMTP_REPLY_ACTION_OK          = 250,
	INETCORESMTP_REPLY_FORWARDING         = 251,

	INETCORESMTP_REPLY_START_MAILING      = 354,

	INETCORESMTP_REPLY_SERVICE_UNAVAIL    = 421,
	INETCORESMTP_REPLY_MAILBOX_UNAVAIL    = 450,
	INETCORESMTP_REPLY_ACTION_ABORTED     = 451,
	INETCORESMTP_REPLY_STORAGE_UNAVAIL    = 452,

	INETCORESMTP_REPLY_COMMAND_SYNTAX     = 500,
	INETCORESMTP_REPLY_PARAM_SYNTAX       = 501,
	INETCORESMTP_REPLY_COMMAND_UNAVAIL    = 502,
	INETCORESMTP_REPLY_COMMAND_SEQUENCE   = 503,
	INETCORESMTP_REPLY_PARAM_UNAVAIL      = 504,
	INETCORESMTP_REPLY_MAILBOX_INVALID    = 550,
	INETCORESMTP_REPLY_REMOTE_USER        = 551,
	INETCORESMTP_REPLY_STORAGE_EXCEEDED   = 552,
	INETCORESMTP_REPLY_MAILBOX_NAME       = 553,
	INETCORESMTP_REPLY_TRANSACTION_FAILED = 554
};

#ifdef _USE_NAMESPACE
namespace inet {
#endif

class  INetCoreSMTPConnection;
struct INetCoreSMTPConnectionContext;
struct INetCoreSMTPCommandContext;

typedef sal_Bool INetCoreSMTPCallback (
	INetCoreSMTPConnection *pConnection,
	sal_Int32               nReplyCode,
	const sal_Char         *pReplyText,
	void                   *pData);

class INetCoreSMTPConnection :
	public NAMESPACE_INET(INetClientConnection_Impl)
{
public:
	INetCoreSMTPConnection (void);

	sal_Bool Open (
		const rtl::OUString  &rHost, sal_uInt16 nPort,
		INetCoreSMTPCallback *pfnCallback, void *pData);
	sal_Bool IsOpen (void);

	sal_Bool Close (
		INetCoreSMTPCallback *pfnCallback, void *pData);
	void Destroy (void);

	sal_Bool MailFrom (
		const rtl::OUString  &rSender,
		INetCoreSMTPCallback *pfnCallback, void *pData);
	sal_Bool MailTo (
		const rtl::OUString  &rRecipient,
		INetCoreSMTPCallback *pfnCallback, void *pData);

	sal_Bool TransferData (
		INetCoreMessageIStream &rMessageStream,
		INetCoreSMTPCallback *pfnCallback, void *pData);
	sal_uInt32 GetTransferCount (void);

	sal_Bool SetTransferCallback (
		INetCoreSMTPCallback *pfnCallback, void *pData);
	sal_Bool SetTerminateCallback (
		INetCoreSMTPCallback *pfnCallback, void *pData);

protected:
	virtual ~INetCoreSMTPConnection (void);

private:
	INetCoreSMTPConnectionContext *m_pConCtx;

	virtual sal_Bool handleResolverEvent (
		sal_Int32 nStatus, INetCoreDNSHostEntry *pHostEntry);

	virtual sal_Bool handleSocketEvent (
		const NAMESPACE_VOS(ORef)<INetSocket> &rxSocket, sal_Int32 nEvent);

	sal_Bool StartCommand (INetCoreSMTPCommandContext *pCtx);
};

#ifdef _USE_NAMESPACE
}
#endif

#endif 

