

#ifndef _INET_WRAPPER_HXX
#define _INET_WRAPPER_HXX "$Revision:   1.0"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _VOS_DYNLOAD_HXX_
#include <vos/dynload.hxx>
#endif
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
#ifndef _VOS_OBJECT_HXX_
#include <vos/object.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif

#ifndef _INET_MACROS_HXX
#include <inet/macros.hxx>
#endif

class INetIMAPClientRef;
class INetCoreSSLConfig;

class INetSession;

#ifdef _USE_NAMESPACE
namespace inet {
#endif

class INetConfig;
class INetCoreMailer;
class INetFTPConnection;
class INetHTTPWrapper;
class INetLDAPWrapper;

class INetModuleApi
{
public:
	virtual void acquire (void) = 0;
	virtual void release (void) = 0;

	virtual sal_Bool getINetConfig (
		NAMESPACE_VOS(ORef)<INetConfig> &rxConfig) = 0;
	virtual sal_Bool getINetCoreSSLConfig (
		INetCoreSSLConfig *&rpConfig) = 0;

	virtual sal_Bool newINetIMAPClient (
		INetIMAPClientRef &rxClient) = 0;
	virtual sal_Bool newINetFTPConnection (
		NAMESPACE_VOS(ORef)<INetFTPConnection> &rxConnection) = 0;
	virtual sal_Bool newINetHTTPWrapper (
		INetHTTPWrapper *&rpWrapper) = 0;
	virtual sal_Bool newINetLDAPWrapper (
		INetLDAPWrapper *&rpWrapper) = 0;
	virtual sal_Bool newINetCoreMailer (
		INetCoreMailer *&rpMailer) = 0;

	virtual sal_Bool newINetSession (
		INetSession *&rpSession) = 0;
};

class INetWrapper :
	public NAMESPACE_INET(INetModuleApi),
	public NAMESPACE_VOS(OObject)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (INetWrapper, inet));

public:
	INetWrapper (void);
	virtual ~INetWrapper (void);

	virtual sal_Bool getINetConfig (
		NAMESPACE_VOS(ORef)<INetConfig> &rxConfig);

	virtual sal_Bool getINetCoreSSLConfig (
		INetCoreSSLConfig *&rpConfig);

	virtual sal_Bool newINetIMAPClient (
		INetIMAPClientRef &rxClient);

	virtual sal_Bool newINetFTPConnection (
		NAMESPACE_VOS(ORef)<INetFTPConnection> &rxConnection);

	virtual sal_Bool newINetHTTPWrapper (
		INetHTTPWrapper *&rpWrapper);

	virtual sal_Bool newINetLDAPWrapper (
		INetLDAPWrapper *&rpWrapper);

	virtual sal_Bool newINetCoreMailer (
		INetCoreMailer *&rpMailer);

	virtual sal_Bool newINetSession (
		INetSession *&rpSession);

private:

	NAMESPACE_VOS(ODynamicLoader)<INetModuleApi>  m_aLoader;
	NAMESPACE_VOS(OMutex)                         m_aMutex;
	INetModuleApi                                *m_pApi;

	virtual void acquire (void);
	virtual void release (void);

	INetWrapper (const INetWrapper&);
	INetWrapper& operator= (const INetWrapper&);
};

#ifdef _USE_NAMESPACE
}
#endif

#endif 

