

#ifndef _SDPAGE_HXX
#define _SDPAGE_HXX

#include <com/sun/star/uno/XInterface.hpp>
#ifndef _COM_SUN_STAR_PRESENTATION_FADEEFFECT_HPP_
#include <com/sun/star/presentation/FadeEffect.hpp>
#endif

#ifndef _SVDOBJ_HXX 
#include <svx/svdobj.hxx>
#endif
#ifndef _FM_FMPAGE_HXX 
#include <svx/fmpage.hxx>
#endif
#ifndef _SD_FADEDEF_H
#include <sd/fadedef.h>
#endif
#ifndef _SD_DIADEF_H
#include <sd/diadef.h>
#endif
#ifndef _PRESENTATION
#include <sd/pres.hxx>
#endif

class SfxStyleSheet;
class SdDrawDocument;
class SdrTextObj;
class SdPageLink;
class StarBASIC;
struct StyleRequestData;

enum PresObjKind
{
	PRESOBJ_NONE,
	PRESOBJ_TITLE,
	PRESOBJ_OUTLINE,
	PRESOBJ_TEXT,
	PRESOBJ_GRAPHIC,
	PRESOBJ_OBJECT,
	PRESOBJ_CHART,
	PRESOBJ_ORGCHART,
	PRESOBJ_TABLE,
	PRESOBJ_IMAGE,
	PRESOBJ_BACKGROUND,
	PRESOBJ_PAGE,
	PRESOBJ_HANDOUT,
	PRESOBJ_NOTES
};

class SdPage : public FmFormPage, public SdrObjUserCall
{
friend class SdGenericDrawPage;

protected:
	PageKind	ePageKind;				  
	AutoLayout	eAutoLayout;			  
	List		aPresObjList;			  
	BOOL		bOwnArrangement;		  

	BOOL		bSelected;				  
	FadeSpeed	eFadeSpeed; 			  
	::com::sun::star::presentation::FadeEffect	eFadeEffect;			  
	PresChange	ePresChange;			  
	UINT32		nTime;					  
	BOOL		bSoundOn;				  
	BOOL		bExcluded;				  

	String		aLayoutName;			  
	String      aSoundFile;               
	String      aCreatedPageName;         
	String      aFileName;                
	String		aBookmarkName;			  
	BOOL		bScaleObjects;			  
	BOOL		bBackgroundFullSize;	  
	rtl_TextEncoding eCharSet;            
	USHORT      nPaperBin;                
	Orientation eOrientation;             
	SdPageLink* pPageLink;				  

	BOOL		InsertPresObj(SdrObject* pObj, PresObjKind eObjKind, BOOL bVertical,
				  Rectangle rRect, BOOL bInit, List& rObjList);

	void		AdjustBackgroundSize();
	Rectangle   GetTitleRect() const;
	Rectangle   GetLayoutRect() const;

	virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > createUnoPage();

public:
	TYPEINFO();

	SdPage(SdDrawDocument& rNewDoc, StarBASIC* pBasic, BOOL bMasterPage=FALSE);
	SdPage(const SdPage& rSrcPage);
	~SdPage();
	virtual SdrPage* Clone() const;

	virtual void	SetSize(const Size& aSize);
	virtual void	SetBorder(INT32 nLft, INT32 nUpp, INT32 nRgt, INT32 Lwr);
	virtual void	SetLftBorder(INT32 nBorder);
	virtual void	SetRgtBorder(INT32 nBorder);
	virtual void	SetUppBorder(INT32 nBorder);
	virtual void	SetLwrBorder(INT32 nBorder);
	virtual void	SetModel(SdrModel* pNewModel);
	virtual FASTBOOL IsReadOnly() const;

	List*           GetPresObjList() { return &aPresObjList; }
	SdrObject*      CreatePresObj(PresObjKind eObjKind, BOOL bVertical, const Rectangle& rRect, BOOL bInsert=FALSE);
	SdrObject*		GetPresObj(PresObjKind eObjKind, USHORT nIndex = 1);
	PresObjKind     GetPresObjKind(SdrObject* pObj);
	String          GetPresObjText(PresObjKind eObjKind);
	SfxStyleSheet*	GetStyleSheetForPresObj(PresObjKind eObjKind);
	void			SetAutoLayout(AutoLayout eLayout, BOOL bInit=FALSE, BOOL bAPICall=FALSE);
	AutoLayout		GetAutoLayout() const { return eAutoLayout; }
	void            CreateTitleAndLayout(BOOL bInit=FALSE, BOOL bAPICall=FALSE);

	virtual void       NbcInsertObject(SdrObject* pObj, ULONG nPos=CONTAINER_APPEND,
						   const SdrInsertReason* pReason=NULL);
	virtual SdrObject* NbcRemoveObject(ULONG nObjNum);
	virtual SdrObject* RemoveObject(ULONG nObjNum);

	virtual void SetLinkData(const String& rLinkName, const String& rLinkData);

	void		SetObjText(SdrTextObj* pObj, SdrOutliner* pOutliner, PresObjKind eObjKind, const String& rStr );

	void		SetPageKind(PageKind ePgType)		 { ePageKind = ePgType; }
	PageKind	GetPageKind() const 				 { return ePageKind; }

	void		SetSelected(BOOL bSel)				 { bSelected = bSel; }
	BOOL		IsSelected() const					 { return bSelected; }

	void		SetFadeSpeed(FadeSpeed eNewSpeed)	 { eFadeSpeed = eNewSpeed; }
	FadeSpeed	GetFadeSpeed() const				 { return eFadeSpeed; }

	void		SetFadeEffect(::com::sun::star::presentation::FadeEffect eNewEffect) { eFadeEffect = eNewEffect; }
	::com::sun::star::presentation::FadeEffect	GetFadeEffect() const				 { return eFadeEffect; }

	void		SetPresChange(PresChange eChange)	 { ePresChange = eChange; }
	PresChange	GetPresChange() const				 { return ePresChange; }

	void		SetTime(UINT32 nNewTime)			 { nTime = nNewTime; }
	UINT32		GetTime() const 					 { return nTime; }

	void		SetSound(BOOL bNewSoundOn)			 { bSoundOn = bNewSoundOn; }
	BOOL		IsSoundOn() const					 { return bSoundOn; }

	void		SetExcluded(BOOL bNewExcluded)		{ bExcluded = bNewExcluded; }
	BOOL		IsExcluded() const					{ return bExcluded; }

	void		SetScaleObjects(BOOL bScale)		{ bScaleObjects = bScale; }
	BOOL		IsScaleObjects() const				{ return bScaleObjects; }

	void		SetSoundFile(const String& rStr)	{ aSoundFile = rStr; }
	String		GetSoundFile() const				{ return aSoundFile; }

#ifndef SVX_LIGHT

	virtual void Changed(const SdrObject& rObj, SdrUserCallType eType,
			 const Rectangle& rOldBoundRect);

	virtual void RequestBasic();

	virtual void WriteData(SvStream& rOut) const;
#endif
	virtual void ReadData(const SdrIOHeader& rHead, SvStream& rIn);

	void			SetLayoutName(String aName);
	virtual String	GetLayoutName() const		{ return aLayoutName; }

	void			SetFileName(const String& aName) { aFileName = aName; }
	virtual String	GetFileName() const 	  { return aFileName; }
	void			SetBookmarkName(const String& aName) { aBookmarkName = aName; }
	virtual String	GetBookmarkName() const 	  { return aBookmarkName; }
	SdPageLink* 	GetLink() { return pPageLink; }

#ifndef SVX_LIGHT
	void			ConnectLink();
	void			DisconnectLink();

	void    ScaleObjects(const Size& rNewPageSize, const Rectangle& rNewBorderRect,
			 BOOL bScaleAllObj);
#endif

	const String&   GetName();
	String			GetRealName() const { return aPageName; };

	void    SetPresentationLayout(const String& rLayoutName,
					  BOOL bReplaceStyleSheets = TRUE,
					  BOOL bSetMasterPage = TRUE,
					  BOOL bReverseOrder = FALSE);
	void	EndListenOutlineText();

	void	SetBackgroundFullSize( BOOL bIn );
	BOOL	GetBackgroundFullSize() const { return bBackgroundFullSize; }

	rtl_TextEncoding GetCharSet() { return(eCharSet); }

	void    SetPaperBin(USHORT nBin) { nPaperBin = nBin; }
	USHORT  GetPaperBin() const { return nPaperBin; }
	void        SetOrientation(const Orientation eOrient) { eOrientation = eOrient; }
	Orientation GetOrientation() const { return eOrientation; }

	virtual SfxStyleSheet* GetTextStyleSheetForObject( SdrObject* pObj ) const;
};

#endif	   

