
#ifndef __COMPILER_HXX
#define __COMPILER_HXX

#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif

#include <setup2/parser.hxx>
#include <setup2/hashtbl.hxx>
#include <setup2/os.hxx>

DECLARE_HASHTABLE(ConstantSet,ByteString,char const*)

class SiDeclarator;
class SiCompiledScript;

class SiCompiler : public SiParser, public SvRefBase
{
private:
	SiCompiledScript*		m_pCompiledScript;
	SiDeclarator*			m_pCurrentDeclarator;
	ByteString			   	m_CurrentPropertyname;
	int				   		m_nCurrentPropertyLanguage;
	com::sun::star::setup::OSType m_eOsType;
	ConstantSet        		m_Constants;

	BOOL               		m_bSymanticError;
	BOOL					m_bOpenRefs;
	BOOL					m_bIgnoreError;

	BOOL					m_bWebbasedMode;
	BOOL					m_bScpMode;

	ByteString 				m_aDefDefaultLanguage;
	ByteString 				m_aDefLanguages;

	ByteString 				m_aSuiteName;
	ByteString 				m_aProductName;
	ByteString 				m_aProductVersion;

	friend class SiDeclarator;
	friend class SiCompiledScript;
	 void	SymanticError(ByteString const&);
	void    Warning(ByteString const&);

	SiDeclarator* CheckLanguage();
	com::sun::star::setup::OSType GetOsType() const { return m_eOsType; }

	void	AddSystemObjectsTo(SiCompiledScript *);

protected:
	virtual void OnDeclaration    (SiLexemType,SiIdentifier const&);
	virtual void OnDeclarationEnd ();
	virtual void OnProperty       (ByteString const&, int nLanguage);
	virtual void OnSingleValue    (sal_Int32);
	virtual void OnSingleValue    (sal_uInt32);
	virtual void OnSingleValue    (ByteString const&);
	virtual void OnSingleValue    (SiIdentifier const&);
	virtual void OnListValue      (sal_Int32);
	virtual void OnListValue      (ByteString const&);
	virtual void OnListValue      (SiIdentifier const&);

	virtual void DisplayError(int nLine, ByteString const& ErrorType,
						ByteString const& Message) const;

public:
	 SiCompiler( SvStream& aStream, com::sun::star::setup::OSType eOsType = OS::GetType(), int nLanguage = 0 );
		~SiCompiler();
	 BOOL				CompileTo( SiCompiledScript*, int nMaxErrors = 1 );

	void				SetIgnoreError(BOOL bNew) 							{ m_bIgnoreError = bNew; }
	void				SetScpZipMode(BOOL bNew) 							{ m_bScpMode = bNew; }
	BOOL 				SecondLevelCompile( SiCompiledScript* );
	void 				SetDefDefaultLanguage(const ByteString& rNew)		{ m_aDefDefaultLanguage = rNew; }
	void 				SetDefLanguages(const ByteString& rNew)				{ m_aDefLanguages = rNew; }

	SiCompiledScript*	GetCScript() const									{ return m_pCompiledScript; }
};

SV_DECL_IMPL_REF( SiCompiler );

#endif 
