

#ifndef SETUP_LEXEM_HXX
#define SETUP_LEXEM_HXX

#include <setup2/ident.hxx>

enum SiLexemType
{
	LT_NULL,
	LT_ERR,
	LT_EOF,

	LT_ICONST,      
	LT_SCONST,      
	LT_ID,          

	LT_COMMA,
	LT_SEMICOLON,
	LT_ASSIGN,

	LT_POPEN,       
	LT_PCLOSE,      

	LT_MODULE,      
	LT_DIRECTORY,
	LT_FILE,
	LT_DATACARRIER,
	LT_FOLDER,
	LT_FOLDERITEM,
	LT_PROFILE,
	LT_PROFILEITEM,
	LT_PROCEDURE,
	LT_CUSTOM,
	LT_SLIDE,
	LT_REGISTRYITEM,
	LT_REGISTRYAREA,
	LT_END,
	LT_OS2_CREATOR,
	LT_OS2_TEMPLATE,
	LT_OS2_CLASS,
	LT_SHORTCUT,
	LT_INSTALLATION,
	LT_HELPTEXT,
	LT_SCPACTION,
	LT_STARREGISTRY,
	LT_STARREGISTRY_ITEM,
	LT_CONFIGURATION_ITEM
};

enum SiLexemError
{
	LE_NULL,          
	LE_NOTPRINTABLE,  
	LE_NOTBEGIN       
};

class SiLexem
{
	SiLexemType		m_LexemType;
	SiLexemError	m_LexemError;
	ByteString		m_Value;

	void Construct( SiLexemType 		= LT_NULL,
		SiLexemError		= LE_NULL,
		ByteString const&	= ByteString("") );

public:
		SiLexem();
		SiLexem( SiLexemType eType);
		SiLexem( SiLexemType eType, ByteString const& s );
		SiLexem( SiLexemError eError, ByteString const& s );

	SiLexemType			GetType()			   const { return m_LexemType; }
	SiLexemError		GetError()			   const;
	sal_Int32			GetValueAsInt32()	   const;
	sal_uInt32			GetValueAsUInt32()	   const;
	ByteString const&	GetValueAsString()     const { return m_Value; }
	SiIdentifier		GetValueAsIdentifier() const { return m_Value; }
};

inline SiLexem::SiLexem()
{ Construct(); }

inline SiLexem::SiLexem(SiLexemType eType)
{ Construct(eType); }

inline SiLexem::SiLexem(SiLexemType eType, ByteString const& s)
{ Construct(eType, LE_NULL, s); }

inline SiLexem::SiLexem(SiLexemError eError, ByteString const& s)
{ Construct(LT_ERR, eError, s); }

#endif 
