

#ifndef SETUP_SCRIPT_HXX
#define SETUP_SCRIPT_HXX

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif

#include <setup2/hashtbl.hxx>
#include <setup2/decltor.hxx>

class SiCompiler;
class SiEnvironment;
class SiProcedure;

DECLARE_HASHTABLE(DeclaratorSet,ByteString,SiDeclarator*)

DECLARE_LIST(SiSlideList,SiSlide*)
DECLARE_LIST(SiHelpTextList,SiHelpText*)
DECLARE_LIST(SiScpActionList,SiScpAction*)

class SiCompiledScript
{

	SiInstallation  *m_pInstallation;
	SiProfileItem   *m_pInstallInfo;	

	SiModule        *m_pRootModule;
	SiFile			*m_pSetup;			
	SiFile			*m_pCompiledHelp;	

	SiProfile		*m_pORegINI;		
	SiProfile		*m_pOfficeINI;		
	SiProfile		*m_pServerINI;		

	SiDirList		 m_Directories;

	SiSlideList		 m_Slides;
	SiHelpTextList	 m_HelpTextLst;
	SiScpActionList  m_ScpActionLst;
	 DeclaratorSet   *m_pDeclarators;
	BOOL             m_bUncomplete; 

	BOOL 			 m_bWriteScpAction;
	BOOL			 m_bSecondLevel;
	BOOL			 m_bBindUnlinked;
	BOOL			 m_bBindUnlinkedWarning;

	void Join (SiModule*, DeclaratorSet const&);
	void CreateNameSpace(ByteString const& rPrefix);
	BOOL PrepareForLocalInstallation(SiModule*);

public:
	SiCompiledScript();
	~SiCompiledScript();

	BOOL 				DoWriteScpAction() const { return m_bWriteScpAction; }
	void				SetWriteScpAction(BOOL bNew) { m_bWriteScpAction = bNew; }

	BOOL 				IsSecondLevel() const { return m_bSecondLevel; }
	void				SetSecondLevel(BOOL bNew) { m_bSecondLevel = bNew; }

	void				RemoveDeclarator(const ByteString& rID);
	BOOL				Add(SiDeclarator*, SiCompiler*);

	void				DoBindUnlinkedObjects(BOOL bNew, BOOL bWarn = TRUE) { m_bBindUnlinked = bNew; m_bBindUnlinkedWarning = bWarn; }
	void				AddUnlinkedObjectsToRoot();

	void				SetCompiledHelp(const SiFile* pFile, const ByteString& rFileName);
	SiFile*				GetCompiledHelp() const { return m_pCompiledHelp; }
	ByteString 				GetSetupExePath(const SiEnvironment& rEnv);

	SiModule*					GetRootModule() const;
	SiModule*					GetModule(ByteString const& ID) const;
	SiInstallation*				GetInstallation() const;
	SiProfileItem*				GetInstallInfo() const;

	SiDirList   const&			GetDirectoryList() const;
	SiSlideList const&			GetSlideList() const;
	SiHelpTextList const&		GetHelpTextList() const;
	SiScpActionList const&		GetScpActionList() const;

	SiFile*						GetReadMe() const;
	SiProfile*					GetOfficeINI() const { return m_pOfficeINI; }
	SiProfile*					GetORegINI() const { return m_pORegINI; }
	SiProfile*					GetServerINI() const { return m_pServerINI; }

	SiHelpText*			GetHelpTextForId( USHORT nId, USHORT nLanguage );
	 SiDeclarator*		Find(ByteString const& ID) const;
	SiDeclarator*		FindProcedureByName(ByteString const& aName) const;

	SiProcedure*		FindPreSelectProc() const;
	SiProcedure*		FindLanguageSelectProc() const;

	void SetUncomplete(BOOL b = TRUE) { m_bUncomplete = TRUE; }
	BOOL IsUncomplete() const         { return m_bUncomplete; }

	void Join(SiCompiledScript*);
	void PrepareForLocalInstallation();
};

inline SiModule* SiCompiledScript::GetRootModule() const
{
	return m_pRootModule;
}

inline SiModule* SiCompiledScript::GetModule(ByteString const& ID) const
{
	return PTR_CAST(SiModule,m_pDeclarators->Find(ID));
}

inline SiInstallation* SiCompiledScript::GetInstallation() const
{
	return m_pInstallation;
}

inline SiProfileItem* SiCompiledScript::GetInstallInfo() const
{
	return m_pInstallInfo;
}

inline SiDirectoryList const& SiCompiledScript::GetDirectoryList() const
{
	return m_Directories;
}

inline SiSlideList const& SiCompiledScript::GetSlideList() const
{
	return m_Slides;
}

inline SiHelpTextList const& SiCompiledScript::GetHelpTextList() const
{
	return m_HelpTextLst;
}

inline SiScpActionList const& SiCompiledScript::GetScpActionList() const
{
	return m_ScpActionLst;
}

inline SiDeclarator* SiCompiledScript::Find(ByteString const& ID) const
{
	return m_pDeclarators->Find(ID);
}

#endif 

