
#ifndef _SICUSTOM_HXX
#define _SICUSTOM_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif

#define SICUSTOM_VERSION	1

#ifndef _SV_CALL
#if defined(WIN) || defined(WNT) || ( defined(OS2) && !defined( ICC ))
#define _SV_CALL	_cdecl
#elif (defined(OS2) && defined(ICC))
#define _SV_CALL 	__cdecl
#else
#define _SV_CALL
#endif
#endif

class SiEnvironment;
struct EnvironmentImpl;

class SiCustomEnvironment
{
		EnvironmentImpl*	pImpl;
protected:
	friend class SiCustomAction;

	void				SetPreFlag(BOOL bPre);
	void 				SetInstallContext(BOOL bInst);
	void				SetFirstInstallation(BOOL bNew = TRUE);
	void				SetResponse(BOOL bNew = TRUE);
	void 				SetResponseFileName(const ByteString& rStr);
	void				SetStartPath(const ByteString& rStr);
	void				SetSourcePath(const ByteString& rStr);
	void				SetDestPath(const ByteString& rStr);
	void				SetInstalledPath(const ByteString& rStr);
	void				SetEnvPtr(SiEnvironment* pPtr);

public:
			SiCustomEnvironment();
			~SiCustomEnvironment();

	SiEnvironment*			GetBigEnv()				const;

	BOOL					IsResponseFileMode()  	const;
	ByteString				GetResponseFileName()	const;

	BOOL					IsFirstInstallation()	const;

	BOOL 					IsPre()					const;
	BOOL 					IsPost()				const;

	BOOL 					IsInstallation()		const;
	BOOL 					IsDeinstallation()		const;

	const ByteString&			GetStartPath() 			const;
	const ByteString&			GetSourcePath() 		const;
	const ByteString&			GetDestPath() 			const;
	const ByteString&			GetInstalledPath()		const;
};

struct ModuleInfoImpl;
class SiCustomAction;

class SiCustomModuleInfo
{
private:
	ModuleInfoImpl*		pImpl;
protected:
	friend class SiCustomAction;
	void				SetModuleID(const ByteString& rStr);
	void				SetModuleName(const ByteString& rStr);
public:
		SiCustomModuleInfo();
		~SiCustomModuleInfo();

	const ByteString&	 	GetModuleID()			const;
	const ByteString&		GetModuleName()			const;
	BOOL				IsSelected()			const;

	void				SelectModule();
	void				DeselectModule();
};

DECLARE_LIST(SiCustomModuleList, SiCustomModuleInfo*);

typedef BOOL (_SV_CALL *FncPtrCustomMain)(SiCustomEnvironment*, SiCustomModuleList*);
extern "C" BOOL _SV_CALL CustomMain(SiCustomEnvironment*, SiCustomModuleList*);

typedef void (_SV_CALL *FncPtrKill)();
extern "C" void _SV_CALL Kill();

typedef BOOL (_SV_CALL *FncPtrSupportVersion)(const ByteString& rVersionKey);
extern "C" BOOL _SV_CALL SupportVersion(const ByteString& rVersionKey);

typedef void (_SV_CALL *FncPtrExecute)();
extern "C" void _SV_CALL Execute();

typedef ULONG (_SV_CALL *FncPtrExecuteCalc)();
extern "C" ULONG _SV_CALL ExecuteCalc();

typedef void		(_SV_CALL *FncPtrHideSetup)();
typedef void		(_SV_CALL *FncPtrShowSetup)();
typedef ByteString	(_SV_CALL *FncPtrGetCodeFilename)();

struct SiCustomFnc
{
	USHORT						nVersion;
	FncPtrCustomMain			fncMain;
	FncPtrKill	 				fncKill;
	FncPtrSupportVersion		fncSupportVersion;
	FncPtrExecute				fncPtrExecute;
	FncPtrExecuteCalc			fncPtrExecuteCalc;
};

struct SiSetupFnc
{
	USHORT						nVersion;
	FncPtrHideSetup				fncHideSetup;
	FncPtrShowSetup				fncShowSetup;
	FncPtrGetCodeFilename		fncGetCodeFilename;
};

extern "C" {
typedef BOOL (_SV_CALL *FncPtrCustomInit)(SiSetupFnc*, SiCustomFnc*);
BOOL _SV_CALL CustomInit(SiSetupFnc* pSetupFnc, SiCustomFnc* pCustomFnc);
};

#endif	

