
#ifndef _BASEDLGS_HXX
#define _BASEDLGS_HXX

#ifndef _FLOATWIN_HXX 
#include <vcl/floatwin.hxx>
#endif
#ifndef _TIMER_HXX 
#include <vcl/timer.hxx>
#endif
#ifndef _DIALOG_HXX 
#include <vcl/dialog.hxx>
#endif

class SfxTabPage;
class SfxViewFrame;
class SfxBindings;
class SfxChildWindow;
struct SfxChildWinInfo;
class SfxItemSet;
class SfxItemPool;
class OKButton;
class CancelButton;
class HelpButton;
class Button;

class SfxModalDefParentHelper
{
private:
	Window *pOld;

public:
	SfxModalDefParentHelper(Window* pWindow);
	~SfxModalDefParentHelper();
};

class SfxModalDialog: public ModalDialog
{
	SfxViewFrame*		pFrame;
	USHORT				nUniqId;
	String              aExtraData;
	Timer				aTimer;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK(			TimerHdl_Impl, Timer* );
#endif

protected:
	SfxModalDialog(SfxViewFrame *pViewFrame, Window *pParent, const ResId &);
	SfxModalDialog(SfxViewFrame *pViewFrame, Window* pParent,
	   USHORT nUniqueId, WinBits nWinStyle = WB_STDMODAL);
	SfxModalDialog(Window *pParent, const ResId &);
	SfxModalDialog(Window* pParent, USHORT nUniqueId,
	   WinBits nWinStyle = WB_STDMODAL);
	~SfxModalDialog();

	String& 	GetExtraData()		{ return aExtraData; }
	USHORT		GetUniqId() const	{ return nUniqId; }
};

class SfxModelessDialog: public ModelessDialog
{
	SfxBindings*			pBindings;
	Size					aSize;
	SfxChildWindow*     	pMgr;

protected:
				SfxModelessDialog( SfxBindings*, SfxChildWindow*,
					Window*, const ResId& );
				SfxModelessDialog( SfxBindings*, SfxChildWindow*,
					Window*, WinBits nWinStyle = WB_STDMODELESS );
				~SfxModelessDialog();
	virtual BOOL			Close();
	virtual void			Resize();

public:
	virtual void        	FillInfo(SfxChildWinInfo&) const;
	void					Initialize (SfxChildWinInfo* pInfo);
	virtual long			Notify( NotifyEvent& rNEvt );
};

class SfxFloatingWindow: public FloatingWindow
{
	SfxBindings*			pBindings;
	Size					aSize;
	SfxChildWindow*     	pMgr;

protected:
				SfxFloatingWindow( SfxBindings *pBindings,
								  SfxChildWindow *pCW,
								  Window* pParent,
								  WinBits nWinBits=WB_STDMODELESS);
				SfxFloatingWindow( SfxBindings *pBindings,
								  SfxChildWindow *pCW,
								  Window* pParent,
								  const ResId& rResId);
				~SfxFloatingWindow();
	virtual BOOL			Close();
	virtual void			Resize();
	virtual long			Notify( NotifyEvent& rNEvt );
	SfxBindings&            GetBindings()
	                        { return *pBindings; }

public:
	virtual void        	FillInfo(SfxChildWinInfo&) const;
	void					Initialize (SfxChildWinInfo* pInfo);
};

typedef USHORT*	(*GetTabPageRanges)(); 

class SfxSingleTabDialog : public SfxModalDialog
{
public:
#if SUPD < 637
	SfxSingleTabDialog( SfxViewFrame* pViewFrame, Window* pParent,
			const SfxItemSet& rOptionsSet, USHORT nUniqueId,
			BOOL bGroupBox = FALSE );
	SfxSingleTabDialog( Window* pParent, const SfxItemSet& rOptionsSet,
			USHORT nUniqueId, BOOL bGroupBox = FALSE );
	SfxSingleTabDialog( SfxViewFrame* pViewFrame, Window* pParent,
			USHORT nUniqueId,	const SfxItemSet* pInSet = 0,
			BOOL bGroupBox = FALSE );
	SfxSingleTabDialog( Window* pParent, USHORT nUniqueId,
			const SfxItemSet* pInSet = 0,
			BOOL bGroupBox = FALSE );
#else
	SfxSingleTabDialog( SfxViewFrame* pViewFrame, Window* pParent,
			const SfxItemSet& rOptionsSet, USHORT nUniqueId );
	SfxSingleTabDialog( Window* pParent, const SfxItemSet& rOptionsSet, USHORT nUniqueId );
	SfxSingleTabDialog( SfxViewFrame* pViewFrame, Window* pParent,
			USHORT nUniqueId,	const SfxItemSet* pInSet = 0 );
	SfxSingleTabDialog( Window* pParent, USHORT nUniqueId, const SfxItemSet* pInSet = 0 );
#endif
	virtual             ~SfxSingleTabDialog();

	void				SetTabPage( SfxTabPage* pTabPage,
						GetTabPageRanges pRangesFunc = 0 );
	SfxTabPage*			GetTabPage() const { return pPage; }

	const USHORT*		GetInputRanges( const SfxItemPool& rPool );
	void				SetInputSet( const SfxItemSet* pInSet )
				{ pOptions = pInSet; }
	const SfxItemSet*	GetOutputItemSet() const { return pOutSet; }
	OKButton*			GetOKButton() const { return pOKBtn; }
	CancelButton*		GetCancelButton() const { return pCancelBtn; }

private:
	SfxViewFrame*       pFrame;
	GetTabPageRanges	fnGetRanges;	
	USHORT*				pRanges;

	OKButton*			pOKBtn;
	CancelButton*		pCancelBtn;
	HelpButton*			pHelpBtn;
#if SUPD < 637
	void*				pDummy;
#endif

	SfxTabPage*			pPage;
	const SfxItemSet*	pOptions;
	SfxItemSet*			pOutSet;
#if SUPD < 637
	BOOL				bGrpBox;
#endif

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( OKHdl_Impl, Button * );
#endif
};

#endif

