
#ifndef _SFX_BINDINGS_HXX
#define _SFX_BINDINGS_HXX

#ifndef _COM_SUN_STAR_FRAME_XFRAME_HPP_
#include <com/sun/star/frame/XFrame.hpp>
#endif

#ifndef _COM_SUN_STAR_FRAME_XDISPATCHPROVIDER_HPP_
#include <com/sun/star/frame/XDispatchProvider.hpp>
#endif

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#include <sfx2/minarray.hxx>
#include <sfx2/app.hxx>

class SfxConfigManager;
class SystemWindow;
class SfxArg;
class SfxSlot;
class SfxSlotServer;
class SfxControllerItem;
class SfxStateCache;
class SfxItemSet;
class SfxDispatcher;
class SfxBindings;
class Timer;
struct SfxFoundCache_Impl;
class SfxFoundCacheArr_Impl;
struct SfxSlotInterceptor_Impl;
class SfxWorkWindow;
class SfxUnoControllerItem;
typedef SfxUnoControllerItem* SfxUnoControllerItemPtr;
SV_DECL_PTRARR( SfxUnoControllerArr_Impl, SfxUnoControllerItemPtr, 20, 20 );

#define SFX_CALLMODE_SLOT           0x00    
#define SFX_CALLMODE_SYNCHRON		0x01    
#define SFX_CALLMODE_ASYNCHRON		0x02    
#define SFX_CALLMODE_RECORD			0x04    
#define SFX_CALLMODE_API        	0x08    
#define SFX_CALLMODE_MODAL	     	0x10    

#define SFX_CALLMODE_STANDARD       SFX_CALLMODE_RECORD
typedef sal_uInt16 SfxCallMode;

enum SfxPopupAction
{
	SFX_POPUP_DELETE,
	SFX_POPUP_HIDE,
	SFX_POPUP_SHOW
};

class SfxSlotInterceptor
{
	SfxSlotInterceptor_Impl*_pImp;

public:
				SfxSlotInterceptor( SfxBindings *pBindings,
									sal_Bool bActivate = sal_True );
				~SfxSlotInterceptor();

	void                    SetBindings( SfxBindings* pBindings );
	SfxBindings*            GetBindings() const;
	void                    Activate( sal_Bool bActivate = sal_True );
	sal_Bool					IsActive() const;

	virtual sal_Bool			Execute( sal_uInt16 nSID, SfxPoolItem **ppArgs=0 ) = 0;
	virtual SfxItemState	QueryState( sal_uInt16 nSID, SfxPoolItem* &rpState ) = 0;
};

class SfxSlotInterceptorWrapper : public SvRefBase
{
	SfxSlotInterceptor* _pInterceptor;
public:
	SfxSlotInterceptorWrapper( SfxSlotInterceptor* pInterceptor )
		: _pInterceptor( pInterceptor ) {}
	~SfxSlotInterceptorWrapper() { delete _pInterceptor; }
	SfxSlotInterceptor* GetInterceptor() { return _pInterceptor; }
};

SV_DECL_IMPL_REF( SfxSlotInterceptorWrapper )

class SfxImageManager;
class SfxBindings: public SfxBroadcaster

{
friend class SfxApplication;
friend class SfxShell;
friend class SfxSlotInterceptor;
friend class SfxBindings_Impl;

	SfxBindings_Impl*pImp;			
	SfxDispatcher*	 pDispatcher;	
	sal_uInt16       nRegLevel;      

#if 0 // _SOLAR__PRIVATE
private:
	const SfxPoolItem*  Execute_Impl( sal_uInt16 nSlot, const SfxPoolItem **pArgs, sal_uInt16 nModi,
	                                SfxCallMode nCall, const SfxPoolItem **pInternalArgs);
	void			 SetSubBindings_Impl( SfxBindings* );
	void			 UpdateSlotServer_Impl(); 
	SfxItemSet* 	 CreateSet_Impl( SfxStateCache* &pCache,
						const SfxSlot* &pRealSlot,
						const SfxSlotServer**,
						SfxFoundCacheArr_Impl& );
	sal_uInt16       GetSlotPos( sal_uInt16 nId, sal_uInt16 nStartSearchAt = 0 );
	void			 Update_Impl( SfxStateCache* pCache );
	void			 UpdateControllers_Impl(
				const SfxInterface* pIF,
				const SfxFoundCache_Impl* pFound,
				const SfxPoolItem *pItem,
				SfxItemState eItemState );
	DECL_LINK( NextJob_Impl, Timer * );
#endif

public:
		 SfxBindings();
		 ~SfxBindings();

	void			 HidePopups( FASTBOOL bHide = sal_True );
	void			 HidePopupCtrls_Impl( FASTBOOL bHide = sal_True );

	void			 SetDispatcher(SfxDispatcher *pDisp);

	void			 Update( sal_uInt16 nId ); 
	void			 Update();
	void			 StartUpdate_Impl(sal_Bool bComplete=sal_False);
	void			 Invalidate( sal_uInt16 nId );
	void			 Invalidate( const sal_uInt16* pIds );
	void			 InvalidateShell( const SfxShell &rSh, sal_Bool bDeep = sal_False );
	void			 InvalidateAll( sal_Bool bWithMsg );
	void			 SetState( const SfxItemSet &rSet );
	void			 SetState( const SfxPoolItem &rItem );
	void 			 Invalidate( sal_uInt16 nId, sal_Bool bWithItem, sal_Bool bWithMsg=sal_False);
	void 			 Invalidate( sal_uInt16 nId, sal_Bool bWithMsg);
	sal_Bool			 IsInUpdate() const;

	sal_Bool			 IsBound( sal_uInt16 nMsgId, sal_uInt16 nStartSearchAt = 0 );

	const SfxSlot* 	 GetSlot( sal_uInt16 nMsgId );
	SfxStateCache*	 GetStateCache( sal_uInt16 nId, sal_uInt16 *pPos = 0 );
	SfxStateCache*	 GetAnyStateCache_Impl( sal_uInt16 nId );
	SfxItemState	 QueryState( sal_uInt16 nSID, SfxPoolItem* &rpState );
	 const SfxPoolItem*  ExecuteSynchron( sal_uInt16 nSlot,
					 const SfxPoolItem **pArgs = 0,
					 sal_uInt16 nModi = 0,
					 const SfxPoolItem **pInternalArgs = 0);
	sal_Bool         Execute( sal_uInt16 nSlot,
					 const SfxPoolItem **pArgs = 0,
					 sal_uInt16 nModi = 0,
					 SfxCallMode nCall = SFX_CALLMODE_SLOT,
					 const SfxPoolItem **pInternalArgs = 0);

	void			 SetDispatchProvider_Impl( const ::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider > & rFrame );
	const ::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider > & GetDispatchProvider_Impl() const;
	void			 SetActiveFrame( const ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > & rFrame );

	int 			 IsInRegistrations() const;
	sal_uInt16			 EnterRegistrations(char *pFile = 0, int nLine = 0);
	void			 LeaveRegistrations( sal_uInt16 nLevel = USHRT_MAX, char *pFile = 0, int nLine = 0 );
	void			 Register( SfxControllerItem& rBinding );
	void			 Release( SfxControllerItem& rBinding );
	SystemWindow*    GetSystemWindow() const;
	SfxDispatcher*   GetDispatcher() const
	                 { return pDispatcher; }
	SfxImageManager* GetImageManager() const;
	SfxConfigManager* GetConfigManager( USHORT nType ) const;
	SfxToolBoxConfig* GetToolBoxConfig() const;

#if 0 // _SOLAR__PRIVATE
	void             DeleteControllers_Impl();
	SfxPopupAction   GetPopupAction_Impl() const;
	SfxDispatcher*	 GetDispatcher_Impl()  { return pDispatcher; }
	void			 ClearCache_Impl( sal_uInt16 nSlotId );
	sal_Bool         IsInUpdate_Impl() const
	                 { return IsInUpdate(); }
	SfxWorkWindow*   GetWorkWindow_Impl() const;
	void             SetWorkWindow_Impl( SfxWorkWindow* );
	SfxBindings*     GetSubBindings_Impl( sal_Bool bTop = sal_False ) const;
	void             InvalidateUnoControllers_Impl();
	void             RegisterUnoController_Impl( SfxUnoControllerItem* );
	void             ReleaseUnoController_Impl( SfxUnoControllerItem* );
	BOOL             ExecuteCommand_Impl( const String& rCommand );
#endif
};

#ifdef DBG_UTIL
#define ENTERREGISTRATIONS() EnterRegistrations(__FILE__, __LINE__)
#define LEAVEREGISTRATIONS() LeaveRegistrations(USHRT_MAX, __FILE__, __LINE__)
#define DENTERREGISTRATIONS(  ) \
		EnterRegistrations( (ByteString(__FILE__).Append('(').Append(ByteString::CreateFromInt32((sal_Int32)this).Append(')'))).GetBufferAccess(), __LINE__ )
#define DLEAVEREGISTRATIONS(  ) \
		LeaveRegistrations( USHRT_MAX, (ByteString(__FILE__).Append('(').Append(ByteString::CreateFromInt32((sal_Int32)this).Append(')'))).GetBufferAccess(), __LINE__ )
#else
#define ENTERREGISTRATIONS() EnterRegistrations()
#define LEAVEREGISTRATIONS() LeaveRegistrations()
#define DENTERREGISTRATIONS(  ) EnterRegistrations()
#define DLEAVEREGISTRATIONS(  ) LeaveRegistrations()
#endif

inline int SfxBindings::IsInRegistrations() const

{
	return 0 != nRegLevel;
}

#if SUPD<606
#define SFX_BINDINGS() SfxGetpApp()->GetBindings()
#endif

#endif

