
#ifndef _SFX_DOCFILT_HACK_HXX
#define _SFX_DOCFILT_HACK_HXX

#ifndef _COM_SUN_STAR_PLUGIN_PLUGINDESCRIPTION_HPP_
#include <com/sun/star/plugin/PluginDescription.hpp>
#endif

#ifndef _WLDCRD_HXX 
#include <tools/wldcrd.hxx>
#endif

#define SFX_FILTER_IMPORT            0x00000001L
#define SFX_FILTER_EXPORT            0x00000002L
#define SFX_FILTER_TEMPLATE          0x00000004L
#define SFX_FILTER_INTERNAL          0x00000008L
#define SFX_FILTER_TEMPLATEPATH      0x00000010L
#define SFX_FILTER_OWN 		         0x00000020L
#define SFX_FILTER_ALIEN	         0x00000040L
#define SFX_FILTER_USESOPTIONS       0x00000080L
#define SFX_FILTER_NOTINFILEDLG      0x00001000L
#define SFX_FILTER_NOTINCHOOSER      0x00002000L

#define SFX_FILTER_DEFAULT           0x00000100L

#define SFX_FILTER_EXECUTABLE        0x00000200L
#define SFX_FILTER_REDIRECT          0x00000400L
#define SFX_FILTER_MAPTOAPPPLUG      0x00000800L
#define SFX_FILTER_ASYNC             0x00004000L

#define SFX_FILTER_CREATOR           0x00008000L
#define SFX_FILTER_OPENREADONLY      0x00010000L
#define SFX_FILTER_MUSTINSTALL		 0x00020000L
#define SFX_FILTER_CONSULTSERVICE	 0x00040000L

#define SFX_FILTER_STARONEFILTER	 0x00080000L
#define SFX_FILTER_PACKED	 		 0x00100000L
#define SFX_FILTER_SILENTEXPORT      0x00200000L

#define SFX_FILTER_BROWSERPREFERED   0x00400000L
#define SFX_FILTER_PREFERED          0x10000000L

#define SFX_FILTER_VERSION_NONE      0
#define SFX_FILTER_NOTINSTALLED		 SFX_FILTER_MUSTINSTALL | SFX_FILTER_CONSULTSERVICE

#include <sfx2/sfxdefs.hxx>

#if SUPD<582
class SfxJSMimeTypeObject_Impl;
class SfxJSPluginObject_Impl;
#endif
class SfxFilterContainer;

class SfxFilter
{
friend class SfxFilterContainer;

	String			aName;
	WildCard		aWildCard;
	ULONG			lFormat;
	String			aMacType;
	String			aTypeName;
	String			aUserData;
	SfxFilterFlags	nFormatType;
	USHORT          nDocIcon;
	const SfxFilterContainer* pContainer;
	String          aMimeType;
	String          aFilterName;
	String          aPattern;
	ULONG           nVersion;
	BOOL            bPlugDataSearched;
	::com::sun::star::plugin::PluginDescription* pPlugData;
#if SUPD<582
	SfxJSMimeTypeObject_Impl* pJSMimeTypeObj;
	SfxJSPluginObject_Impl* pJSPluginObj;
#endif
	String          aUIName;
	String          aDefaultTemplate;

public:
		SfxFilter( const String &rName,
				   const String &rWildCard,
				   SfxFilterFlags nFormatType,
				   ULONG lFormat,		
				   const String &rMacType,	
				   const String &rTypeName, 
				   USHORT nDocIcon,         
				   const String &rMimeType,
				   const SfxFilterContainer* pFactoryP,
				   const String &rUserData );
		SfxFilter( const char* pName, const String &rWildCard,
				   SfxFilterFlags nFormatType,
				   const SfxFilterContainer* pFactoryP );
		SfxFilter( const char* pName, const String &rWildCard,
				   SfxFilterFlags nFormatType, const String &rTypeName,
				   const SfxFilterContainer* pFactoryP );
			~SfxFilter();

	const SfxFilterContainer* GetFilterContainer( ) const { return pContainer; }

	FASTBOOL		IsAllowedAsTemplate() const
		{ return nFormatType & SFX_FILTER_TEMPLATE; }
	FASTBOOL		IsOwnFormat() const
		{ return nFormatType & SFX_FILTER_OWN; }
	FASTBOOL		IsOwnTemplateFormat() const
		{ return nFormatType & SFX_FILTER_TEMPLATEPATH; }
	FASTBOOL		IsAlienFormat() const
		{ return nFormatType & SFX_FILTER_ALIEN; }
	FASTBOOL		CanImport() const
		{ return nFormatType & SFX_FILTER_IMPORT; }
	FASTBOOL		CanExport() const
		{ return nFormatType & SFX_FILTER_EXPORT; }
	FASTBOOL		IsInternal() const
		{ return nFormatType & SFX_FILTER_INTERNAL; }
	SfxFilterFlags  GetFilterFlags() const	{ return nFormatType; }
	const String&   GetFilterName() const { return aFilterName; }
	const String&   GetMimeType() const { return aMimeType; }
	const String&	GetName() const { return  aName; }
	const WildCard& GetWildcard() const { return aWildCard; }
	const String&	GetMacType() const { return aMacType; }
	const String&	GetRealTypeName() const { return aTypeName; }
	ULONG			GetFormat() const { return lFormat; }
	const String&	GetTypeName() const;
	const String&   GetUIName() const { return aUIName; }
	USHORT          GetDocIconId() const { return nDocIcon; }
	const String&	GetUserData() const { return aUserData; }
	const String&   GetDefaultTemplate() const { return aDefaultTemplate; }
	void            SetDefaultTemplate( const String& rStr )
			{ aDefaultTemplate = rStr; }
	BOOL			UsesStorage() const { return GetFormat() != 0; }
	void            SetURLPattern( const String& rStr )
		{
			aPattern = rStr;
			aPattern.ToLowerAscii();
		}
	String          GetURLPattern() const { return aPattern; }
	void            SetUIName( const String& rName ) { aUIName = rName; }
	void            SetVersion( ULONG nVersionP ) { nVersion = nVersionP; }
	ULONG           GetVersion() const { return nVersion; }
	String          GetSuffixes() const;
	String          GetDefaultExtension() const;
	const ::com::sun::star::plugin::PluginDescription* GetPlugData() const {
		return ((SfxFilter*)this)->GetPlugData();	}

	const ::com::sun::star::plugin::PluginDescription* GetPlugData();
	BOOL            IsFirstPlugin() const;
#if 0 // _SOLAR__PRIVATE
	void            InitMembers_Impl();
#endif
#if SUPD<582
	SfxJSMimeTypeObject_Impl*    GetJSMimeTypeObject() const;
	SfxJSPluginObject_Impl* GetJSPluginObject() const;
#endif
};

#if 0 // _SOLAR__PRIVATE
class SfxOfficeFilter: public SfxFilter
{
public:
	SfxOfficeFilter( const SfxFilterContainer* pContainer);
};
#endif

#endif

