
#ifndef _SFX_MACROCONF_HXX
#define _SFX_MACROCONF_HXX

#ifndef _ERRCODE_HXX 
#include <tools/errcode.hxx>
#endif
#define _SVSTDARR_USHORTS
#include <svtools/svstdarr.hxx>		
#include <sfx2/evntconf.hxx>

class SfxMacroInfo;
class SfxSlot;
class SfxMacroInfoItem;
class SfxObjectShell;
class BasicManager;
struct SfxMacroConfig_Impl;
class SbMethod;
class SbxValue;
class SbxObject;
class SbxArray;
class SvStream;
class SvxMacro;

typedef SfxMacroInfo* SfxMacroInfoPtr;
#if 0 // _SOLAR__PRIVATE
SV_DECL_PTRARR(SfxMacroInfoArr_Impl, SfxMacroInfoPtr, 5, 5);
#else
class SfxMacroInfoArr_Impl;
#endif

class SfxMacroInfo
{
friend class SfxMacroConfig;
friend class SfxEventConfiguration;
friend SvStream& operator >> (SvStream& rStream, SfxMacroInfo& rInfo);
friend SvStream& operator << (SvStream& rStream, const SfxMacroInfo& rInfo);

	SfxObjectShell* 		pDocShell;			
	String*   				pHelpText;
	sal_uInt16                  nRefCnt;
	sal_Bool					bAppBasic;
	String					aLibName;
	String					aModuleName;
	String					aMethodName;
	sal_uInt16					nSlotId;
	SfxSlot*				pSlot;

public:
	SfxMacroInfo( const String& rURL );
	SfxMacroInfo(SfxObjectShell *pDoc=NULL);
	SfxMacroInfo( SfxObjectShell* pDoc, const String& rQualifiedName );
	SfxMacroInfo(SfxMacroInfo& rOther);
	SfxMacroInfo(SfxObjectShell* pMgr, const String& rLibName,
		const String& rModuleName, const String& rMethodName);
	~SfxMacroInfo();
	sal_Bool operator==(const SfxMacroInfo& rOther) const;
	int Load (SvStream&);
	int Store (SvStream&);
	String              GetMacroName() const;
	String				GetQualifiedName() const;
	String				GetFullQualifiedName() const;
	BasicManager*		GetBasicManager() const;
	String				GetBasicName() const;
	String				GetHelpText() const;
	sal_Bool				IsAppMacro() const
			{ return bAppBasic; }
	const String&		GetModuleName() const
			{ return aModuleName; }
	const String&		GetLibName() const
			{ return aLibName; }
	const String&		GetMethodName() const
			{ return aMethodName; }
	sal_uInt16				GetSlotId() const
			{ return nSlotId; }
	SfxSlot*			GetSlot() const
			{ return pSlot; }

	sal_Bool				Compare( const SvxMacro& ) const;
	void 				SetHelpText( const String& rText );
	String				GetURL();
};

class SfxMacroConfig
{
friend class SfxEventConfiguration;

	static SfxMacroConfig*	pMacroConfig;

	SfxMacroConfig_Impl* 	pImp;
	SvUShorts               aIdArray;

public:
				SfxMacroConfig();
				~SfxMacroConfig();

	static SfxMacroConfig*	GetOrCreate();

	static String			RequestHelp( sal_uInt16 nId );
	static sal_Bool 			IsMacroSlot( sal_uInt16 nId );
	static sal_Bool				IsBasic( SbxObject*, const String&, BasicManager* );
	static ErrCode			Call( SbxObject*, const String&, BasicManager*,
					SbxArray *pArgs=NULL, SbxValue *pRet=NULL );

	static SbMethod*		GetMethod_Impl( const String&, BasicManager* );

	sal_uInt16					GetSlotId(SfxMacroInfoPtr);
	void					ReleaseSlotId(sal_uInt16 nId);
	void					RegisterSlotId(sal_uInt16 nId);
	const SfxMacroInfoPtr 	GetMacroInfo(sal_uInt16 nId) const;
	sal_Bool					ExecuteMacro(sal_uInt16 nId, const String& rArgs ) const;
	sal_Bool					ExecuteMacro( SfxObjectShell*, const SvxMacro*, const String& ) const;
	sal_Bool					CheckMacro(sal_uInt16 nId) const;
	sal_Bool					CheckMacro( SfxObjectShell*, const SvxMacro* ) const;

#if 0 // _SOLAR__PRIVATE
	static void				Release_Impl();
	const SfxMacroInfoPtr   GetMacroInfo_Impl( const SvxMacro *pMacro ) const;
	DECL_LINK(				CallbackHdl_Impl, SfxMacroConfig*);
	DECL_LINK(				EventHdl_Impl, SfxMacroInfo*);
#endif
};

#endif
