
#ifndef _SFXMNUITEM_HXX
#define _SFXMNUITEM_HXX

class SfxVirtualMenu;
class SfxBindings;
class SfxModule;
class Menu;
class PopupMenu;
class SfxUnoMenuControl;
class SfxUnoControllerItem;

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#include <sfx2/ctrlitem.hxx>

class SfxMenuControl: public SfxControllerItem
{

	String					aTitle;
	String					aHelpText;
	SfxVirtualMenu* 		pOwnMenu;
	SfxVirtualMenu* 		pSubMenu;
	BOOL					b_ShowStrings;
	BOOL					b_UnusedDummy;

public:
				SfxMenuControl();
				SfxMenuControl( BOOL bShowStrings );
				SfxMenuControl( USHORT, SfxBindings&);

	static SfxMenuControl* 	CreateImpl( USHORT nId, Menu &rMenu, SfxBindings &rBindings );
	static void 			RegisterControl( USHORT nSlotId = 0, SfxModule *pMod=NULL );
					~SfxMenuControl();

	void					Bind( SfxVirtualMenu* pOwnMenu, USHORT nId,
					  const String& rTitle, const String &rHelpText,
					  SfxBindings & );
	void					Bind( SfxVirtualMenu* pOwnMenu, USHORT nId,
					  SfxVirtualMenu& rSubMenu,
					  const String& rTitle, const String &rHelpText,
					  SfxBindings & );

	String					GetTitle() const;
	SfxVirtualMenu* 		GetPopupMenu() const;
	virtual PopupMenu*		GetPopup() const;
	void					SetOwnMenu( SfxVirtualMenu* pMenu );
	void 					RemovePopup();

	const String&			GetHelpText() const { return aHelpText; }
	void					SetHelpText(const String &rStr) { aHelpText  = rStr; }

	virtual void			StateChanged( USHORT nSID, SfxItemState eState,
							  const SfxPoolItem* pState );

	static SfxMenuControl*	CreateControl( USHORT nId, Menu &, SfxBindings & );
	static SfxUnoMenuControl*
				CreateControl( const String&, USHORT, Menu&, SfxBindings&, SfxVirtualMenu* );
	static BOOL             IsSpecialControl( USHORT nId, SfxModule* );
};

class SfxUnoMenuControl : public SfxMenuControl
{
	SfxUnoControllerItem*	pUnoCtrl;
public:
				SfxUnoMenuControl( const String&, USHORT nId, Menu&,
									SfxBindings&, SfxVirtualMenu* );
				~SfxUnoMenuControl();
	void					Select();
};

typedef SfxMenuControl* (*SfxMenuControlCtor)( USHORT nId, Menu &, SfxBindings & );

struct SfxMenuCtrlFactory
{
	SfxMenuControlCtor	pCtor;
	TypeId				nTypeId;
	USHORT				nSlotId;

	SfxMenuCtrlFactory( SfxMenuControlCtor pTheCtor,
			TypeId nTheTypeId, USHORT nTheSlotId ):
		pCtor(pTheCtor),
		nTypeId(nTheTypeId),
		nSlotId(nTheSlotId)
	{}
};

inline String SfxMenuControl::GetTitle() const
{
	return aTitle;
}

inline SfxVirtualMenu* SfxMenuControl::GetPopupMenu() const
{
	return pSubMenu;
}

#define SFX_DECL_MENU_CONTROL() \
		static SfxMenuControl* CreateImpl( USHORT nId, Menu &rMenu, SfxBindings &rBindings ); \
		static void RegisterControl(USHORT nSlotId = 0, SfxModule *pMod=NULL)

#define SFX_IMPL_MENU_CONTROL(Class, nItemClass) \
		SfxMenuControl* __EXPORT Class::CreateImpl( USHORT nId, Menu &rMenu, SfxBindings &rBindings ) \
	  { return new Class(nId, rMenu, rBindings); } \
		void Class::RegisterControl(USHORT nSlotId, SfxModule *pMod) \
	  { SFX_APP()->RegisterMenuControl( pMod, new SfxMenuCtrlFactory( \
		Class::CreateImpl, TYPE(nItemClass), nSlotId ) ); }

#if 0 // _SOLAR__PRIVATE

class SfxExplorerMenuView;
class SfxAppMenuControl_Impl : public SfxMenuControl
{
	SfxExplorerMenuView* pMenu;
public:
	SFX_DECL_MENU_CONTROL();
	SfxAppMenuControl_Impl(
		USHORT nPos, Menu& rMenu, SfxBindings& rBindings );
	~SfxAppMenuControl_Impl();
	DECL_LINK ( Select, Menu* );
};

#endif

#endif
