
#ifndef _SFXMNUMGR_HXX
#define _SFXMNUMGR_HXX

#include <stdarg.h>

#ifndef _MENU_HXX 
#include <vcl/menu.hxx>
#endif
#ifndef VCL
#ifndef _WINBIT_HXX 
#include <vcl/winbit.hxx>
#endif
#else
#ifndef _SV_WINTYPES_HXX
#include <vcl/wintypes.hxx>
#endif
#endif
#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif

#include <tools/stream.hxx>
#include <sfx2/cfgitem.hxx>
#include <sfx2/sfx.hrc>

class SfxMenuCfgItemArr;
class SfxMenuCfgItemArrStack;
class SfxVirtualMenu;
class SfxPoolItem;
class SfxBindings;
class SfxMenuIter_Impl;
class ResMgr;
class Menu;
class MenuBar;
class PopupMenu;
class SfxBindings;
class ResId;
class SvVerbList;
class Point;
class SfxObjectShell;
class SfxModule;

class SfxMenuManager : public SfxConfigItem
{
friend class SfxPopupMenuManager;
friend class SfxMenuBarManager;

	SfxVirtualMenu* 		pMenu;			
	SfxVirtualMenu* 		pOldMenu;		
	BOOL					bMenuBar;		
	SfxMenuIter_Impl*		pIterator;		
	SfxMenuCfgItemArr*		pCfg;
	SfxMenuCfgItemArrStack* pCfgStack;
	SfxBindings*			pBindings;
	ResMgr* 				pResMgr;
	BOOL                    bOLE : 1;
	BOOL					bAddClipboardFuncs : 1;

	void					Construct( SfxVirtualMenu& rMenu );
	void					ConstructSvMenu ( Menu*, SfxMenuCfgItemArr& );
	static void             ConstructSvMenu ( Menu*, SvStream&, BOOL, BOOL );
	 int                     Load( SvStream& rStream );
	BOOL                    Store( SvStream& rStream );

protected:
	                        SfxMenuManager( const ResId&, SfxBindings&, SfxConfigManager*, BOOL );
				~SfxMenuManager();
	USHORT                  GetItemPos( USHORT nId );
public:
	 static Menu*            LoadMenu( SvStream& rStream );
	static BOOL             StoreMenu( SvStream& rStream, Menu* pMenu, SfxModule* pMod );
	virtual int             Load( SotStorage& );
	virtual BOOL            Store( SotStorage& );
	virtual void			UseDefault();
	virtual String          GetStreamName() const;

	DECL_LINK( Select, Menu* );

	SfxVirtualMenu* 		GetMenu() const
				{ return pMenu; }

	SfxBindings&			GetBindings() { return *pBindings; }
	const SfxBindings&		GetBindings() const { return *pBindings; }
	void					SetResMgr(ResMgr* pMgr)  {pResMgr = pMgr; }
	ResMgr*                 GetResMgr() const { return pResMgr; }
	void					SetPopupMenu( USHORT nId, PopupMenu *pMenu );

	BOOL    FirstItem();         
	BOOL    NextItem();          
	BOOL    IsBinding() const;   
	BOOL    IsSeparator() const; 
	BOOL    IsPopup() const;     
	USHORT  GetLevel() const;    
	USHORT	GetItemId() const;	 
	String  GetTitle() const;    
	String  GetHelpText() const; 

	void SetForceCtrlCreateMode( BOOL bCreate = TRUE );
	void Clear();                           
	void Reconfigure();                     
	void AppendItem(const String &rText, const String &rHelpText, USHORT nId );	
	void AppendSeparator();                 
	void EnterPopup(const String& rTitle,  
		const String &rHelpText,
		USHORT nId = 0 );      
	void LeavePopup();                      
	void InvalidateKeyCodes();				

	static BOOL 	IsPopupFunction( USHORT nId );
	void            InsertVerbs(const SvVerbList *pVerblist);

#if 0 // _SOLAR__PRIVATE
	void			Construct_Impl( Menu* pMenu, BOOL bWithHelp );
#endif
};

class SfxPopupMenuManager : public SfxMenuManager
{
private:
	DECL_LINK( SelectHdl, void * );
	Menu*				pSVMenu;

public:
			SfxPopupMenuManager( const ResId& , SfxBindings& );

	USHORT				Execute( const Point& rPos, Window *pWindow );
	USHORT				Execute( const Point& rPos );
	USHORT				Execute( Window*, va_list pArgs, const SfxPoolItem *pArg1 );

	USHORT				Execute( Window*, const SfxPoolItem *pArg1 ... );
	void				StartInsert();
	void				EndInsert();
	void				CheckItem( USHORT, BOOL );
	void				RemoveItem( USHORT );
	void 				InsertItem( USHORT, const String&, MenuItemBits,
					USHORT nPos = MENU_APPEND );
	void 				InsertSeparator( USHORT nPos = MENU_APPEND );
	void				RemoveDisabledEntries();
	void				AddClipboardFunctions();
};

class SfxMenuBarManager : public SfxMenuManager
{
private:
	struct SfxObjectMenu_Impl
	{
		USHORT					nId;
		ResMgr* 				pResMgr;
		SfxPopupMenuManager*	pPMMgr;
	};

	SfxObjectMenu_Impl			aObjMenus[4];
	SystemWindow*               pWindow;
	BOOL                        bDowning;

public:
	 SfxMenuBarManager( const ResId&, SfxBindings& rBindings, SfxConfigManager*, BOOL bOLE );
	SfxMenuBarManager( const SfxMenuBarManager& rOther, SfxConfigManager* );
	~SfxMenuBarManager();

	static MenuBar*	LoadMenuBar( SvStream& rStream );
	static BOOL		StoreMenuBar( SvStream& rStream, MenuBar* pMenuBar );
	 virtual int             Load( SotStorage& );
	virtual BOOL            Store( SotStorage& );

	void			ResetObjectMenus();
	void			SetObjectMenu( USHORT nPos, const ResId& rResId );
	PopupMenu*		GetObjectMenu( USHORT nPos, USHORT& rConfigId );
	void            UpdateObjectMenus();
	MenuBar*		GetMenuBar() const;
	void			ReconfigureObjectMenus();
	SystemWindow*   GetWindow() const { return pWindow; }
};

#endif 

