
#ifndef _SFXTBXCTRL_HXX
#define _SFXTBXCTRL_HXX

#ifndef _TIMER_HXX 
#include <vcl/timer.hxx>
#endif
#ifndef _MENU_HXX 
#include <vcl/menu.hxx>
#endif
#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif
#ifndef _FLOATWIN_HXX 
#include <vcl/floatwin.hxx>
#endif
#include <sfx2/ctrlitem.hxx>

class SfxUnoToolBoxControl;
class SfxToolBoxControl;
class SfxBindings;
class SfxModule;
class SfxUnoControllerItem;

typedef SfxToolBoxControl* (*SfxToolBoxControlCtor)( USHORT nId, ToolBox &rTbx, SfxBindings & );

struct SfxTbxCtrlFactory
{
	SfxToolBoxControlCtor	pCtor;
	TypeId					nTypeId;
	USHORT					nSlotId;

	SfxTbxCtrlFactory( SfxToolBoxControlCtor pTheCtor,
			TypeId nTheTypeId, USHORT nTheSlotId ):
		pCtor(pTheCtor),
		nTypeId(nTheTypeId),
		nSlotId(nTheSlotId)
	{}
};

enum SfxPopupWindowType
{
	SFX_POPUPWINDOW_NONE,
	SFX_POPUPWINDOW_ONCLICK,
	SFX_POPUPWINDOW_ONTIMEOUT,
	SFX_POPUPWINDOW_ONCLICKANDMOVE,
	SFX_POPUPWINDOW_ONTIMEOUTANDMOVE,
	SFX_POPUPWINDOW_CONTEXTMENU
};

class SfxPopupWindow: public FloatingWindow, public SfxControllerItem
{
friend class SfxToolBox_Impl;
	BOOL					bFloating;
	ULONG					nEventId;
	BOOL					bCascading;
	Link					aDeleteLink;

	private:
#if 0 // _SOLAR__PRIVATE
	DECL_LINK( Delete, void * );
#endif

protected:
	virtual void			PopupModeEnd();
	virtual BOOL			Close();
	virtual void			DeleteFloatingWindow();
	virtual void			StateChanged( USHORT nSID, SfxItemState eState,
							  const SfxPoolItem* pState );

public:
				SfxPopupWindow( USHORT nId, WinBits nBits, SfxBindings & );
				SfxPopupWindow( USHORT nId, const ResId &rId, SfxBindings & );
	virtual SfxPopupWindow* Clone() const;
	virtual void			MouseMove( const MouseEvent& rMEvt );

#if 0 // _SOLAR__PRIVATE
	void					StartCascading_Impl();
	void					EndCascading_Impl();
	void					SetDeleteLink_Impl( const Link& rLink )
				{
					aDeleteLink = rLink;
				}
#endif
};

#define SFX_DECL_TOOLBOX_CONTROL() \
		static SfxToolBoxControl* CreateImpl( USHORT nId, ToolBox &rTbx, SfxBindings &rBindings ); \
		static void RegisterControl(USHORT nSlotId = 0, SfxModule *pMod=NULL)

#define SFX_IMPL_TOOLBOX_CONTROL(Class, nItemClass) \
		SfxToolBoxControl* __EXPORT Class::CreateImpl( USHORT nId, ToolBox &rTbx, SfxBindings &rBindings ) \
	  { return new Class(nId, rTbx, rBindings); } \
		void Class::RegisterControl(USHORT nSlotId, SfxModule *pMod) \
	  { SFX_APP()->RegisterToolBoxControl( pMod, new SfxTbxCtrlFactory( \
		Class::CreateImpl, TYPE(nItemClass), nSlotId ) ); }

#define SFX_IMPL_TOOLBOX_CONTROL_ARG(Class, nItemClass, Arg) \
		SfxToolBoxControl* __EXPORT Class::CreateImpl( USHORT nId, ToolBox &rTbx, SfxBindings &rBindings ) \
	  { return new Class(nId, rTbx, rBindings, Arg); } \
		void Class::RegisterControl(USHORT nSlotId, SfxModule *pMod) \
	  { SFX_APP()->RegisterToolBoxControl( pMod, new SfxTbxCtrlFactory( \
		Class::CreateImpl, TYPE(nItemClass), nSlotId ) ); }

class SfxToolBoxControl: public SfxControllerItem
{
friend class SfxToolBoxManager;
friend class SfxToolbox;
friend class SfxToolBox_Impl;
friend class SfxToolboxCustomizer;
friend class SfxPopupWindow;
friend struct SfxTbxCtrlFactory;

	ToolBox*				pBox;
	BOOL					bShowString;
	USHORT                  nSelectModifier;
	SfxTbxCtrlFactory*		pFact;

protected:
	virtual void			StateChanged( USHORT nSID, SfxItemState eState,
							  const SfxPoolItem* pState );
	virtual void			Select( BOOL bMod1 = FALSE );
	virtual void			Select( USHORT nModifier );

	virtual void			DoubleClick();
	virtual void			Click();
	virtual SfxPopupWindowType
				GetPopupWindowType() const;
	virtual SfxPopupWindow* CreatePopupWindow();
	virtual	SfxPopupWindow* CreatePopupWindowCascading();
	virtual Window*         CreateItemWindow( Window *pParent );

public:
				SFX_DECL_TOOLBOX_CONTROL();
					SfxToolBoxControl( USHORT nId, ToolBox& rBox,
								   SfxBindings &,
								   BOOL bShowStrings = FALSE );
	virtual 				~SfxToolBoxControl();

	ToolBox&				GetToolBox() const
				{ return *pBox; }

	static SfxToolBoxControl* CreateControl( USHORT nId, ToolBox *pBox,
								 SfxBindings&, SfxModule *pMod );

	static SfxUnoToolBoxControl* CreateControl( const String&, USHORT nId,
								 ToolBox *pBox,
								 SfxBindings& );
#if 0 // _SOLAR__PRIVATE
	SfxToolBoxControl*		ReInit_Impl();
#endif
};

class SfxUnoToolBoxControl : public SfxToolBoxControl
{
	SfxUnoControllerItem*	pUnoCtrl;
public:
				SfxUnoToolBoxControl( const String&, USHORT nId,
								 ToolBox *pBox,
								 SfxBindings& );
				~SfxUnoToolBoxControl();
	virtual void			Select( BOOL bMod1 = FALSE );
};

#if 0 // _SOLAR__PRIVATE

class SfxDragButton_Impl : public FixedImage
{
	SfxBindings*    pBindings;
public:
	                 SfxDragButton_Impl( Window *pParent, SfxBindings& rBind );
	virtual void 	Command ( const CommandEvent& rCEvt );
	virtual void	MouseMove( const MouseEvent& rMEvt );
	virtual void	MouseButtonDown( const MouseEvent& rMEvt );
};

class SfxDragToolBoxControl_Impl : public SfxToolBoxControl

{
public:
				SFX_DECL_TOOLBOX_CONTROL();
				SfxDragToolBoxControl_Impl(
					USHORT nId, ToolBox& rBox, SfxBindings& );
	virtual Window*         CreateItemWindow( Window *pParent );
	virtual void			Select( BOOL bMod1 = FALSE );
};

class SfxAppToolBoxControl_Impl : public SfxToolBoxControl

{
	String                  aLastURL;
	Timer					aTimer;
	BOOL					bBigImages;

private:
				DECL_LINK( Timeout, Timer * );

protected:
	virtual void			Click( );
	virtual void			Select( BOOL );
	virtual void			StateChanged( USHORT nSID, SfxItemState eState,
							  const SfxPoolItem* pState );
public:
				SFX_DECL_TOOLBOX_CONTROL();
				SfxAppToolBoxControl_Impl(
					USHORT nId, ToolBox& rBox,  SfxBindings& );
	void					SetImage( const String& rFacName );
};

class SfxHistoryToolBoxControl_Impl : public SfxToolBoxControl
{
	Timer					aTimer;

private:
				DECL_LINK( Timeout, Timer * );

protected:
	virtual void			Click( );
	virtual void			Select( BOOL );

public:
				SFX_DECL_TOOLBOX_CONTROL();
				SfxHistoryToolBoxControl_Impl(
					USHORT nId, ToolBox& rBox,  SfxBindings& );
};

class SfxReloadToolBoxControl_Impl : public SfxToolBoxControl
{
protected:
	virtual void			Select( USHORT nSelectModifier );

public:
				SFX_DECL_TOOLBOX_CONTROL();
				SfxReloadToolBoxControl_Impl(
					USHORT nId, ToolBox& rBox,  SfxBindings& );
};

#endif

#endif

