
#ifndef _SFXTBXMGR_HXX
#define _SFXTBXMGR_HXX

#ifndef _TOOLBOX_HXX 
#include <vcl/toolbox.hxx>
#endif

#include <vcl/bitmap.hxx>
#include <framework/toolboxconfiguration.hxx>

#include <sfx2/sfx.hrc>
#include <sfx2/app.hxx>
#include <sfx2/cfgitem.hxx>
#include <sfx2/imgmgr.hxx>

class SfxToolBoxControl;
class SfxToolBoxControlArr_Impl;
class SfxViewFrame;
class SfxBindings;
class SfxInterface;

class SfxToolBoxManager: public SfxConfigItem
{
friend class SfxToolbox;

	SfxViewFrame*			pFrame;
	Window*                 pParent;
	ToolBox*				pBox;
	SfxToolBoxControlArr_Impl* pControls;
	SfxBindings*			pBindings;
	USHORT					nIteratorPos;
	SfxSymbolSet			eSymbolSet;
	ResMgr* 				pResMgr;
	SfxInterface*			pIFace;
	BOOL                    bInternal : 1;

private:
	void            Construct();
	void			Align();
	void            CreateFromSVToolBox();
	SfxToolBoxControl* FindControl_Impl( USHORT nId ) const;
			SfxToolBoxManager( SfxConfigManager*, USHORT );

public:
			SfxToolBoxManager( Window*, SfxBindings&, const ResId& );
			SfxToolBoxManager( Window*, SfxBindings&, const ResId& rId,
			SfxInterface*, USHORT nPos, SfxShell* pTask );
		SfxToolBoxManager( const ResId& rId, SfxInterface*, const String&,
			SfxToolBoxManager*, SfxShell* pTask );
		~SfxToolBoxManager();

	void			SetFrame(SfxViewFrame* pFr) { pFrame = pFr;}
	SfxViewFrame*   GetFrame() const {return pFrame; }

	SfxBindings&	GetBindings() { return *pBindings; }
	const SfxBindings&	GetBindings() const { return *pBindings; }

	static void		CreateConfiguration( SfxConfigManager*, USHORT );
	static void		RemoveConfiguration( SfxConfigManager*, USHORT );
	static BOOL     Import( SvStream& rInStream, SvStream& rOutStream );
	static BOOL     Export( SvStream& rInStream, SvStream& rOutStream );
	static int      LoadToolBox( SvStream&, ::framework::ToolBoxDescriptor& );
	static BOOL     StoreToolBox( SvStream&, ::framework::ToolBoxDescriptor& );

	virtual int     Load(SotStorage&);
	virtual BOOL    Store(SotStorage&);
	virtual void    UseDefault();
	virtual String  GetStreamName() const;

	DECL_LINK( Activate, ToolBox * );
	DECL_LINK( Deactivate, ToolBox * );
	DECL_LINK( Select, ToolBox * );

	Size			CalcWindowSizePixel() const
			{ return pBox->CalcWindowSizePixel(); }
	void            SetPosSizePixel( const Point& rPos, const Size& rSize )
			{ pBox->SetPosSizePixel( rPos, rSize ); }
	void			SetBackground( const Wallpaper& rBackground )
			{ pBox->SetBackground( rBackground ); }

	void			Clear();
	BOOL			FirstItem();		 
	BOOL			NextItem(); 		 
	BOOL			IsBinding() const;	 
	BOOL			IsSeparator() const; 
	USHORT			GetId() const;		 

	ToolBox&		GetToolBox() const { return *pBox; }
	SfxSymbolSet    GetSymbolSet() const
		{return eSymbolSet; }
	void			SetSymbolSet(SfxSymbolSet eNewSet);
	ButtonType      GetButtonType() const
		{ return pBox->GetButtonType(); }
	void			SetButtonType( ButtonType eNewType);
	WindowAlign		GetAlign() const { return pBox->GetAlign(); }
	void			SetItemText( USHORT, const String& );

	void            Show();
	void            Hide();
	BOOL			IsVisible() const { return pBox && pBox->IsVisible(); }

	void            SetFloatingModeData (const Point&, USHORT, long);
	void            Customize( const ToolBoxCustomizeEvent& rCEvt );
	virtual BOOL    ReInitialize();
	void			Delete();
	void 			NextToolBox();
	BOOL            IsInternal();

#if 0 // _SOLAR__PRIVATE
	void 			RefreshImages_Impl();
	void            UpdateControls_Impl();
	static BOOL		IsUserDefToolBox_Impl(USHORT nId);
	static USHORT	GetUserDefToolBoxId_Impl();
	void			ReInitControllers_Impl();
	USHORT			GetNextToolBox_Impl( String *pStr = 0 );
	void			Reconfigure_Impl( SfxToolBoxConfig*  );
#endif

#undef C_VERSION
};

#endif 

