

#ifndef _CLIENT_HXX
#define _CLIENT_HXX

#include <so3/protocol.hxx>
#include <so3/iface.hxx>
#include <tools/gen.hxx>
#include <tools/fract.hxx>

class Window;
class OutputDevice;
class ImpClientSite;
class ImpAdviseSink;
class ImpOleObject;
class SvEmbeddedObject;
class SvEmbeddedClient;
class INetURLObject;
struct IOleClientSite;
struct IAdviseSink;

#ifndef SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
#define SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
SO2_DECL_REF(SvEmbeddedObject);
#endif

class SvClientData
{
friend class SvEmbeddedClient;
	SvEmbeddedClient * pClient;
	Rectangle		aObjRect;	

	Fraction		aScaleWidth;
	Fraction		aScaleHeight;
	Window *        pEditWin;   
	BOOL			bInvalidate;

protected:
	void			SetEditWin( Window * pWin ) { pEditWin = pWin; }
public:
		TYPEINFO();
		SvClientData( SvEmbeddedClient *, Window * pWin = NULL );
	virtual 		~SvClientData();
	SvEmbeddedClient * GetClient() const { return pClient; }

	void			SetInvalidate( BOOL b ) { bInvalidate = b; }
	BOOL			IsInvalidate() const { return bInvalidate; }

	virtual void    Invalidate();
	virtual Window* GetEditWin() const;

	virtual void	SetSizeScale( const Fraction & rScaleWidth,
					  const Fraction & rScaleHeight );
	const Fraction & GetScaleWidth() const { return aScaleWidth; }
	const Fraction & GetScaleHeight() const { return aScaleHeight; }

	virtual BOOL	SetObjArea( const Rectangle & );
	virtual Rectangle GetObjArea() const;
	Rectangle 		PixelObjAreaToLogic( const Rectangle & rRectPixel ) const;
	Rectangle 		LogicObjAreaToPixel( const Rectangle & rRect ) const;
	void            SetObjAreaPixel( const Rectangle & rRect )
		{
			SetObjArea( PixelObjAreaToLogic( rRect ) );
		}
	Rectangle       GetObjAreaPixel() const
		{
			return LogicObjAreaToPixel( GetObjArea() );
		}
};

class SvAreaConstrain
{
	Size	aMaxSize; 
	Size	aMinSize; 
	Size	aStepSize;
public:
			SvAreaConstrain() {};
			SvAreaConstrain( const Size & rMax,
				const Size & rMin, const Size & rStep )
	: aMaxSize( rMax )
	, aMinSize( rMin )
	, aStepSize( rStep ) {};

	void	SetMaxSize( const Size & r ) { aMaxSize = r; }
	void	SetMinSize( const Size & r ) { aMinSize = r; }
	void	SetStepSize( const Size & r ) { aStepSize = r; }
	const Size & GetMaxSize() const { return aMaxSize; }
	const Size & GetMinSize() const { return aMinSize; }
	const Size & GetStepSize() const { return aStepSize; }

	Size	ComputeConstrain( const Size & rSize,
				Fraction * pWidthScale = NULL ,
				Fraction * pHeightScale = NULL  ) const;
};

class SvEmbeddedClient : public SvObject
{
friend ImpClientSite;
friend ImpAdviseSink;
friend SvEmbeddedObject;
friend SvInPlaceClient;
friend SvEditObjectProtocol;
friend ImplSvEditObjectProtocol;

	SvClientData *  	pData;		
	SvEmbeddedClient *	pParent;
	USHORT          	nAspect;
	BOOL            	bAutoForce:1,
			bDeleteData:1,
			bCanPlugIn;

	void            SetStandardAdvises( SvEmbeddedObject * );
protected:
	SvEditObjectProtocol aProt;
		~SvEmbeddedClient();

	void			SetViewData( SvClientData * pD )
		{
			pData = pD;
		}
	BOOL			HasViewData() const { return pData != NULL; }
	virtual void	MakeViewData();
	virtual void	FreeViewData( SvClientData * );

	void			SetCanPlugIn( BOOL b )
		{ bCanPlugIn = b; }

	virtual void    Connected( BOOL bConnect );
	virtual void    Closed();
	virtual void    Opened( BOOL bOpen );
	virtual void    Embedded( BOOL bEmbedding );
	virtual void    PlugIn( BOOL bPlugIn );
	virtual void    ViewChanged( USHORT nAspect );
public:
	Rectangle      	aDoVerbRectPixel;
			SvEmbeddedClient();
		SvEmbeddedClient( Window * pObjWin );
		SO2_DECL_STANDARD_CLASS_DLL(SvEmbeddedClient,SOAPP)

	SvClientData *	GetClientData();

	void			SetParent( SvEmbeddedClient * pPar )
		{ pParent = pPar; }
	SvEmbeddedClient * GetParent() const { return pParent; }

	SvEmbeddedObject * GetEmbedObj() const
		{ return aProt.GetObj(); }
	void            DoDisconnect() { aProt.Reset(); }
	BOOL            IsEmbed() const { return aProt.IsEmbed(); }

	virtual BOOL    SaveObject();  
	virtual SvEmbeddedObjectRef  GetContainer();
	virtual void    MakeVisible();

	const SvEditObjectProtocol &  GetProtocol() const { return aProt; }
	SvEditObjectProtocol &  GetProtocol() { return aProt; }

	BOOL			CanPlugIn();
	BOOL            IsAutoForceMoniker() const { return bAutoForce; }
	void            SetAutoForceMoniker( BOOL bSet )
		{ bAutoForce = bSet; }
	virtual ErrCode	GetContURL( INetURLObject & );

	void            SetAspect( USHORT nAspectP, BOOL bInvalidate = TRUE  );
	USHORT          GetAspect() const { return nAspect; }
};
#ifndef SO2_DECL_SVEMBEDDEDCLIENT_DEFINED
#define SO2_DECL_SVEMBEDDEDCLIENT_DEFINED
SO2_DECL_REF(SvEmbeddedClient)
#endif
SO2_IMPL_REF(SvEmbeddedClient)

#endif 
