

#ifndef _PERSIST_HXX
#define _PERSIST_HXX

#ifndef _TIME_HXX 
#include <tools/time.hxx>
#endif
#include <so3/so2ref.hxx>
#include <so3/iface.hxx>

#ifndef SO2_DECL_SVSTORAGE_DEFINED
#define SO2_DECL_SVSTORAGE_DEFINED
SO2_DECL_REF(SvStorage)
#endif

class  ImpPersistStorage;
class  ImpPersistFile;
class  ImpItemContainer;
class  SvFactory;
class  SvStorage;

class SvObjectContainer : virtual public SvObject

{
friend ImpItemContainer;
friend SvFactory;
protected:
		~SvObjectContainer();
public:
		SvObjectContainer();
		SO2_DECL_STANDARD_CLASS_DLL(SvObjectContainer,SOAPP)

};
#ifndef SO2_DECL_SVOBJECTCONTAINER_DEFINED
#define SO2_DECL_SVOBJECTCONTAINER_DEFINED
SO2_DECL_REF(SvObjectContainer)
#endif
SO2_IMPL_REF(SvObjectContainer)

class SvInfoObjectMemberList;
class SvInfoObject;

#ifndef SO2_DECL_SVPERSIST_DEFINED
#define SO2_DECL_SVPERSIST_DEFINED
SO2_DECL_REF(SvPersist)
#endif

#define SVEXT_PERSIST_STREAM "\002OlePres000"

class SvPersist : public SvObjectContainer
{
friend SvInfoObject;
friend ImpPersistStorage;
friend ImpPersistFile;
friend ImpItemContainer;
friend SvFactory;
	BOOL            bIsModified:1,
		bIsInit:1,
		bOpSave:1,
		bOpSaveAs:1,
		bSaveExtern:1,  
		bOpHandsOff:1,  
		bCreateTempStor:1, 
		bSaveFailed:1, 	
		bEnableSetModified:1,
		bDummy0:1,
		bDummy1:1;
	USHORT          nModifyCount;
	SvStorageRef    aStorage;

	SvPersist *     pParent;    
private:
	SvInfoObjectMemberList * pChildList;
	Time            aModifiedTime;
	String			aFileName;	

	SvInfoObjectMemberList *GetInfoList();
	void            CountModified( BOOL bDirection );
	void            dtorClear();
	BOOL            SaveElement( SvStorage*, SvInfoObject* );
protected:
	void            InitMembers( SvStorage * );
		~SvPersist();
#ifdef DBG_UTIL
#define ASSERT_INIT()     AssertInit();
#else
#define ASSERT_INIT()
#endif
	void            AssertInit() const;
	void			SetSaveFailed( BOOL bFail = TRUE )
		{ bSaveFailed = bFail; }
	BOOL    		GetSaveFailed() const { return bSaveFailed; }

	virtual void    ModifyChanged();

	virtual BOOL    InitNew( SvStorage * );         
	virtual BOOL    Load( SvStorage * );            
	virtual BOOL    Save();                         
	virtual BOOL    SaveAs( SvStorage * pNewStg );  
	virtual void    HandsOff();                     
	virtual BOOL    SaveCompleted( SvStorage * );   

	BOOL            DoLoadContent( SvStorage *, BOOL bOwner );
	virtual void    LoadContent( SvStream & rStm, BOOL bOwner );
	BOOL            DoSaveContent( SvStorage *, BOOL bOwner );
	virtual void    SaveContent( SvStream & rStm, BOOL bOwner );

	BOOL            LoadChilds();                       
	BOOL            SaveChilds();                       
	BOOL            SaveAsChilds( SvStorage * pNewStg );
	BOOL            SaveCompletedChilds( SvStorage * ); 
	BOOL			HasStorage() const { return aStorage.Is(); }
public:
		SvPersist();
		SO2_DECL_STANDARD_CLASS_DLL(SvPersist,SOAPP)
	SvStorage *     GetStorage() const;
	SvPersist *     GetParent() const { return pParent; }
	void            SetupStorage( SvStorage * pStor ) const;

	virtual void    FillClass( SvGlobalName * pClassName,
				   ULONG * pFormat,
				   String * pAppName,
				   String * pFullTypeName,
				   String * pShortTypeName,
	                           long nFileFormat = SOFFICE_FILEFORMAT_CURRENT ) const;

	SvGlobalName    GetClassName() const;
	ULONG           GetFileFormat() const;
	String			GetFullTypeName() const;

	BOOL            Insert  ( SvInfoObject * pInfoObj );
	BOOL            Move    ( SvInfoObject * pInfoObj, const String & rStorName, BOOL bCopyStorage=FALSE );
	BOOL			Copy	( const String & rNewObjName, const String & rNewStorName,
				  SvInfoObject * pSrcInfoObj, SvPersist * pSrc );

	BOOL			Unload	( SvInfoObject * pInfoObj );
	BOOL			Unload	( SvPersist * pEle );

	void            Remove  ( const String & rEleName );
	void            Remove  ( SvPersist * pEle );
	void            Remove  ( SvInfoObject * pInfoObj );
	void            Clear   ();
	SvInfoObject *  Find    ( const String & rEleName ) const;
	SvInfoObject *  Find    ( const SvPersist * pEle ) const;
	const SvInfoObjectMemberList * GetObjectList() const
		{ return pChildList; }

	BOOL    		HasObject( const String & rObjName );
	SvPersistRef 	GetObject( const String & rObjName );
	SvStorageRef 	GetObjectStorage( const String & rObjName );
	SvStorageRef    GetObjectStorage( SvInfoObject* pEle );

	BOOL            IsRoot() const;

	void			EnableSetModified( BOOL bEnable = TRUE );
	BOOL			IsEnableSetModified() const
		{ return bEnableSetModified; }
	virtual void    SetModified( BOOL = TRUE );
	BOOL            IsModified();                   

	SvPersistRef    CopyObject( SvStorage * );

	virtual BOOL    DoInitNew( SvStorage * );
	const String &	GetFileName() const { return aFileName; }
	void			SetFileName( const String & rFileName )
		{ aFileName = rFileName; }
	virtual BOOL    DoLoad( const String & rFileName, StreamMode,
				short nStorMode );
	virtual BOOL    DoLoad( SvStorage * );
	BOOL			DoOwnerLoad( SvStorage * );
	virtual BOOL    DoSave();
	virtual BOOL    DoSaveAs( SvStorage * pNewStg );
	virtual void    DoHandsOff();
	virtual BOOL    DoSaveCompleted( SvStorage * = NULL );
	void            CleanUp(BOOL bRecurse=FALSE); 

	const Time &    GetModifyTime() const { return aModifiedTime; }
	void			SetModifyTime( const Time & rTime )
		{ aModifiedTime = rTime; }
	BOOL            IsHandsOff() const { return bOpHandsOff; }
	void            StartActivation( SvPersist* pObj );
};
SO2_IMPL_REF(SvPersist)

SV_DECL_REF(SvInfoObject)
class SvInfoObject_Impl;
class SvInfoObject : public SvPersistBase
{
friend SvPersist;
	SvPersistRef    aObj;
	String          aObjName;           
	String          aStorName;          
	SvGlobalName    aSvClassName;       
#if SUPD>631
	SvInfoObject_Impl* pImp;
#endif
	BOOL            bDeleted;

protected:
		~SvInfoObject();
	virtual void	Assign( const SvInfoObject * pObj );
	void            SetStorageName( const String & rName )
		{ aStorName = rName; }
public:
		SV_DECL_PERSIST1(SvInfoObject,SvPersistBase,1)
		SvInfoObject();
		SvInfoObject( SvPersist * );
		SvInfoObject( SvPersist *, const String & rObjName );
		SvInfoObject( const String& rObjName,
				   	  const SvGlobalName& rClassName );
	SvInfoObjectRef CreateCopy() const;

	virtual void    SetObj( SvPersist * );

	SvObject  *     GetObj() const { return &aObj; }
	SvPersist *     GetPersist() const { return aObj; }

	void            SetObjName( const String & rName )
		{ aObjName = rName; }
	String          GetObjName() const;

	String          GetStorageName() const;
	 String          GetRealStorageName() const;
	SvGlobalName    GetClassName() const;
	BOOL            IsDeleted() const
		{ return bDeleted; }
	void            SetDeleted( BOOL bDel=TRUE );
};

SV_IMPL_REF(SvInfoObject)
SV_DECL_IMPL_PERSIST_LIST(SvInfoObject,SvInfoObject*)

#endif 
