

#ifndef _STGDIR_HXX
#define _STGDIR_HXX

#ifndef _STGAVL_HXX
#include <sot/stgavl.hxx>
#endif
#ifndef _STGELEM_HXX
#include <sot/stgelem.hxx>
#endif
#ifndef _STGSTRMS_HXX
#include <sot/stgstrms.hxx>
#endif

class StgIo;
class StgEntry;
class StgDirEntry;
class StgDirStrm;

class BaseStorageStream;
class StgDirEntry : public StgAvlNode
{
	friend class StgIterator;
	friend class StgDirStrm;
	StgEntry	 aSave;						
	StgDirEntry*  pUp;						
	StgDirEntry*  pDown;					
	StgDirEntry** ppRoot;					
	StgStrm*     pStgStrm; 	                
	StgTmpStrm*	 pTmpStrm;					
	StgTmpStrm*  pCurStrm;					
	INT32		 nEntry;					
	INT32		 nPos;						
	BOOL		 bDirty;					
	BOOL		 bCreated;					
	BOOL		 bRemoved;					
	BOOL		 bRenamed;					
	void		 InitMembers();				
	virtual short Compare( const StgAvlNode* ) const;
	BOOL		 StoreStream( StgIo& );		
	BOOL		 StoreStreams( StgIo& );	
	void		 RevertAll();				
	BOOL		 Strm2Tmp();				
	BOOL		 Tmp2Strm();				
public:
	StgEntry 	 aEntry;					
	INT32  		 nRefCnt;					
	StreamMode   nMode;             		
	BOOL		 bTemp;						
	BOOL		 bDirect; 		  			
	BOOL         bZombie;                   
	BOOL		 bInvalid;					
	StgDirEntry( const void*, BOOL * pbOk );
	StgDirEntry( const StgEntry& );
	~StgDirEntry();

	void Invalidate( BOOL=FALSE ); 			
	void Enum( INT32& );					
	void DelTemp( BOOL );					
	BOOL Store( StgDirStrm& );				
	BOOL IsContained( StgDirEntry* );		

	void SetDirty()  { bDirty = TRUE;     }
	BOOL IsDirty();
	void ClearDirty();

	BOOL Commit();
	BOOL Revert();

	void  OpenStream( StgIo&, BOOL=FALSE );		
	void  Close();
	INT32 GetSize();
	BOOL  SetSize( INT32 );
	INT32 Seek( INT32 );
	INT32 Tell() { return nPos; }
	INT32 Read( void*, INT32 );
	INT32 Write( const void*, INT32 );
	void  Copy( StgDirEntry& );
	void  Copy( BaseStorageStream& );
};

class StgDirStrm : public StgDataStrm
{
	friend class StgIterator;
	StgDirEntry* pRoot;							
	short 		 nEntries;						
	void		 SetupEntry( INT32, StgDirEntry* );
public:
	StgDirStrm( StgIo& );
	~StgDirStrm();
	virtual BOOL SetSize( INT32 );				
	BOOL		 Store();
	void*	     GetEntry( INT32 n, BOOL=FALSE );
	StgDirEntry* GetRoot() { return pRoot; }
	StgDirEntry* Find( StgDirEntry&, const String& );
	StgDirEntry* Create( StgDirEntry&, const String&, StgEntryType );
	BOOL         Remove( StgDirEntry&, const String& );
	BOOL         Rename( StgDirEntry&, const String&, const String& );
	BOOL         Move( StgDirEntry&, StgDirEntry&, const String& );
};

class StgIterator : public StgAvlIterator
{
public:
	StgIterator( StgDirEntry& rStg ) : StgAvlIterator( rStg.pDown ) {}
	StgDirEntry* First() { return (StgDirEntry*) StgAvlIterator::First(); }
	StgDirEntry* Next()  { return (StgDirEntry*) StgAvlIterator::Next();  }
	StgDirEntry* Last()  { return (StgDirEntry*) StgAvlIterator::Last();  }
	StgDirEntry* Prev()  { return (StgDirEntry*) StgAvlIterator::Prev();  }
};

#endif
