

#ifndef _STGELEM_HXX
#define _STGELEM_HXX

#ifndef _TOOLS_SOLAR_H
#include <tools/solar.h>
#endif

#include <sot/stg.hxx>

class StgIo;
class SvStream;
class String;

SvStream& operator>>( SvStream&, ClsId& );
SvStream& operator<<( SvStream&, const ClsId& );

class StgHeader
{
	BYTE 	cSignature[ 8 ];			
	ClsId	aClsId;						
	INT32	nVersion;					
	UINT16	nByteOrder;					
	INT16	nPageSize;					
	INT16	nDataPageSize;				
	BYTE	bDirty;						
	BYTE	cReserved[ 9 ];				
	INT32	nFATSize; 					
	INT32	nTOCstrm;					
	INT32	nReserved;					
	INT32	nThreshold;					
	INT32	nDataFAT;					
	INT32	nDataFATSize;				
	INT32	nMasterChain;				
	INT32	nMaster;					
	INT32	nMasterFAT[ 109 ];			
public:
	StgHeader();
	void  Init();						
	BOOL  Load( StgIo& );
	BOOL  Load( SvStream& );
	BOOL  Store( StgIo& );
	BOOL  Check();						
	short GetByteOrder() const			{ return nByteOrder;	}
	INT32 GetTOCStart() const			{ return nTOCstrm;		}
	void  SetTOCStart( INT32 n );
	INT32 GetDataFATStart() const		{ return nDataFAT;		}
	void  SetDataFATStart( INT32 n );
	INT32 GetDataFATSize() const		{ return nDataFATSize; 	}
	void  SetDataFATSize( INT32 n );
	INT32 GetThreshold() const			{ return nThreshold;	}
	short GetPageSize() const			{ return nPageSize;		}
	short GetDataPageSize() const		{ return nDataPageSize;	}
	INT32 GetFATSize() const			{ return nFATSize;		}
	void  SetFATSize( INT32 n );
	INT32 GetFATChain() const			{ return nMasterChain;	}
	void  SetFATChain( INT32 n );
	INT32 GetMasters() const			{ return nMaster;		}
	void  SetMasters( INT32 n );
	short GetFAT1Size() const			{ return 109;			}
	const ClsId& GetClassId() const		{ return aClsId;		}
	void  SetClassId( const ClsId& );
	INT32 GetFATPage( short ) const;
	void  SetFATPage( short, INT32 );
};

enum StgEntryType {						
	STG_EMPTY	  = 0,
	STG_STORAGE   = 1,
	STG_STREAM    = 2,
	STG_LOCKBYTES = 3,
	STG_PROPERTY  = 4,
	STG_ROOT	  = 5
};

enum StgEntryRef {						
	STG_LEFT	  = 0,					
	STG_RIGHT	  = 1,					
	STG_CHILD	  = 2,					
	STG_DATA	  = 3					
};

enum StgEntryTime {						
	STG_MODIFIED  = 0,					
	STG_ACCESSED  = 1					
};

class StgStream;

#define STGENTRY_SIZE 128

class StgEntry {						
	UINT16	nName[ 32 ];				
	INT16	nNameLen;					
	BYTE	cType;						
	BYTE	cFlags;						
	INT32	nLeft;						
	INT32	nRight;						
	INT32	nChild;						
	ClsId	aClsId;						
	INT32	nFlags;						
	INT32	nMtime[ 2 ];				
	INT32	nAtime[ 2 ];				
	INT32	nPage1;						
	INT32	nSize;						
	INT32	nUnknown;					
	String	aName;		                
public:
	BOOL	Init();						
	BOOL	SetName( const String& );	
	void	GetName( String& rName ) const;

	short	Compare( const StgEntry& ) const;	
	BOOL	Load( const void* );
	void    Store( void* );
	StgEntryType GetType() const		{ return (StgEntryType) cType;	}
	INT32   GetStartPage() const        { return nPage1; }
	void	SetType( StgEntryType t )	{ cType = (BYTE) t;				}
	BYTE	GetFlags() const			{ return cFlags;				}
	void	SetFlags( BYTE c )			{ cFlags = c;					}
	INT32	GetSize() const				{ return nSize;					}
	void	SetSize( INT32 n )			{ nSize = n;					}
	const   ClsId& GetClassId() const  	{ return aClsId;				}
	void	SetClassId( const ClsId& );
	INT32	GetLeaf( StgEntryRef ) const;
	void	SetLeaf( StgEntryRef, INT32 );
	const	INT32* GetTime( StgEntryTime ) const;
	void	SetTime( StgEntryTime, INT32* );
};

#define STG_FREE	-1L					
#define	STG_EOF		-2L					
#define	STG_FAT		-3L					
#define	STG_MASTER	-4L					

#endif
