

#ifndef _CALENDAR_HXX
#define _CALENDAR_HXX

#ifndef _INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _CTRL_HXX
#include <vcl/ctrl.hxx>
#endif
#ifndef _TIMER_HXX
#include <vcl/timer.hxx>
#endif
#ifndef _FIELD_HXX
#include <vcl/field.hxx>
#endif

class Table;
class MouseEvent;
class TrackingEvent;
class KeyEvent;
class HelpEvent;
class DataChangedEvent;
class FloatingWindow;
class PushButton;
struct ImplDateInfo;
class ImplDateTable;
class ImplCFieldFloatWin;

#define WB_QUICKHELPSHOWSDATEINFO	((WinBits)0x00004000)
#define WB_BOLDTEXT 				((WinBits)0x00008000)
#define WB_FRAMEINFO				((WinBits)0x00010000)
#define WB_WEEKNUMBER				((WinBits)0x00020000)

#ifndef WB_RANGESELECT
#define WB_RANGESELECT				((WinBits)0x00200000)
#endif
#ifndef WB_MULTISELECT
#define WB_MULTISELECT				((WinBits)0x00400000)
#endif

#define DIB_BOLD					((USHORT)0x0001)

class Calendar : public Control
{
private:
	ImplDateTable*	mpDateTable;
	Table*			mpSelectTable;
	Table*			mpOldSelectTable;
	Table*			mpRestoreSelectTable;
	XubString*		mpDayText[31];
	XubString		maDayText;
	XubString		maWeekText;
	International	maIntn;
	Rectangle		maPrevRect;
	Rectangle		maNextRect;
	String			maDayOfWeekText;
	long			mnDayOfWeekAry[7];
	Date			maOldFormatFirstDate;
	Date			maOldFormatLastDate;
	Date			maFirstDate;
	Date			maOldFirstDate;
	Date			maCurDate;
	Date			maOldCurDate;
	Date			maAnchorDate;
	Date			maDropDate;
	Color			maSelColor;
	Color			maOtherColor;
	Color*			mpStandardColor;
	Color*			mpSaturdayColor;
	Color*			mpSundayColor;
	ULONG			mnDayCount;
	long			mnDaysOffX;
	long			mnWeekDayOffY;
	long			mnDaysOffY;
	long			mnMonthHeight;
	long			mnMonthWidth;
	long			mnMonthPerLine;
	long			mnLines;
	long			mnDayWidth;
	long			mnDayHeight;
	long			mnWeekWidth;
	long			mnDummy2;
	long			mnDummy3;
	long			mnDummy4;
	WinBits 		mnWinStyle;
	USHORT			mnFirstYear;
	USHORT			mnLastYear;
	USHORT			mnRequestYear;
	BOOL			mbCalc:1,
		mbFormat:1,
		mbDrag:1,
		mbSelection:1,
		mbMultiSelection:1,
		mbWeekSel:1,
		mbUnSel:1,
		mbMenuDown:1,
		mbSpinDown:1,
		mbPrevIn:1,
		mbNextIn:1,
		mbDirect:1,
		mbInSelChange:1,
		mbTravelSelect:1,
		mbScrollDateRange:1,
		mbSelLeft:1,
		mbAllSel:1,
		mbDropPos:1;
	Link			maSelectionChangingHdl;
	Link			maDateRangeChangedHdl;
	Link			maRequestDateInfoHdl;
	Link			maDoubleClickHdl;
	Link			maSelectHdl;
	Timer			maDragScrollTimer;
	USHORT			mnDragScrollHitTest;

#ifdef _SV_CALENDAR_CXX
	void			ImplInit( WinBits nWinStyle );
	void			ImplInitSettings();
	void			ImplGetWeekFont( Font& rFont ) const;
	void			ImplFormat();
	USHORT			ImplHitTest( const Point& rPos, Date& rDate ) const;
	void			ImplDrawSpin( BOOL bDrawPrev = TRUE, BOOL bDrawNext = TRUE );
	void			ImplDrawDate( long nX, long nY,
					  USHORT nDay, USHORT nMonth, USHORT nYear,
					  DayOfWeek eDayOfWeek,
					  BOOL bBack = TRUE, BOOL bOther = FALSE,
					  ULONG nToday = 0 );
	void			ImplDraw( BOOL bPaint = FALSE );
	void			ImplUpdateDate( const Date& rDate );
	void			ImplUpdateSelection( Table* pOld );
	void			ImplMouseSelect( const Date& rDate, USHORT nHitTest,
						 BOOL bMove, BOOL bExpand, BOOL bExtended );
	void			ImplUpdate( BOOL bCalcNew = FALSE );
	void			ImplScroll( BOOL bPrev );
	void			ImplInvertDropPos();
	void			ImplShowMenu( const Point& rPos, const Date& rDate );
	void			ImplTracking( const Point& rPos, BOOL bRepeat );
	void			ImplEndTracking( const Point& rPos, BOOL bCancel );
#endif

protected:
	BOOL			ShowDropPos( const Point& rPos, Date& rDate );
	void			HideDropPos();

	DECL_STATIC_LINK( Calendar, ScrollHdl, Timer *);

public:
		Calendar( Window* pParent, WinBits nWinStyle = 0 );
		Calendar( Window* pParent, const ResId& rResId );
		~Calendar();

	virtual void	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void	MouseButtonUp( const MouseEvent& rMEvt );
	virtual void	MouseMove( const MouseEvent& rMEvt );
	virtual void	Tracking( const TrackingEvent& rMEvt );
	virtual void	KeyInput( const KeyEvent& rKEvt );
	virtual void	Paint( const Rectangle& rRect );
	virtual void	Resize();
	virtual void	GetFocus();
	virtual void	LoseFocus();
	virtual void	RequestHelp( const HelpEvent& rHEvt );
	virtual void	Command( const CommandEvent& rCEvt );
	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

	virtual void	SelectionChanging();
	virtual void	DateRangeChanged();
	virtual void	RequestDateInfo();
	virtual void	DoubleClick();
	virtual void	Select();

	void					SetInternational( const International& rIntn );
	const International&	GetInternational() const { return maIntn; }

	void			SelectDate( const Date& rDate, BOOL bSelect = TRUE );
	void			SelectDateRange( const Date& rStartDate, const Date& rEndDate,
						 BOOL bSelect = TRUE );
	void			SetNoSelection();
	BOOL			IsDateSelected( const Date& rDate ) const;
	ULONG			GetSelectDateCount() const;
	Date			GetSelectDate( ULONG nIndex = 0 ) const;
	void			EnableCallEverySelect( BOOL bEvery = TRUE ) { mbAllSel = bEvery; }
	BOOL			IsCallEverySelectEnabled() const { return mbAllSel; }

	USHORT			GetRequestYear() const { return mnRequestYear; }
	void			SetCurDate( const Date& rNewDate );
	Date			GetCurDate() const { return maCurDate; }
	void			SetFirstDate( const Date& rNewFirstDate );
	Date			GetFirstDate() const { return maFirstDate; }
	Date			GetLastDate() const { return maFirstDate+mnDayCount; }
	ULONG			GetDayCount() const { return mnDayCount; }
	Date			GetFirstMonth() const;
	Date			GetLastMonth() const;
	USHORT			GetMonthCount() const;
	BOOL			GetDate( const Point& rPos, Date& rDate ) const;
	Rectangle		GetDateRect( const Date& rDate ) const;
	BOOL			GetDropDate( Date& rDate ) const;

	long			GetCurMonthPerLine() const { return mnMonthPerLine; }
	long			GetCurLines() const { return mnLines; }

	void			SetStandardColor( const Color& rColor );
	const Color&	GetStandardColor() const;
	void			SetSaturdayColor( const Color& rColor );
	const Color&	GetSaturdayColor() const;
	void			SetSundayColor( const Color& rColor );
	const Color&	GetSundayColor() const;

	void			AddDateInfo( const Date& rDate, const XubString& rText,
					 const Color* pTextColor = NULL,
					 const Color* pFrameColor = NULL,
					 USHORT nFlags = 0 );
	void			RemoveDateInfo( const Date& rDate );
	void			ClearDateInfo();
	XubString		GetDateInfoText( const Date& rDate );

	void			StartSelection();
	void			EndSelection();

	BOOL			IsTravelSelect() const { return mbTravelSelect; }
	BOOL			IsScrollDateRangeChanged() const { return mbScrollDateRange; }
	BOOL			IsSelectLeft() const { return mbSelLeft; }

	Size			CalcWindowSizePixel( long nCalcMonthPerLine = 1,
							 long nCalcLines = 1 ) const;

	void			SetSelectionChangingHdl( const Link& rLink ) { maSelectionChangingHdl = rLink; }
	const Link& 	GetSelectionChangingHdl() const { return maSelectionChangingHdl; }
	void			SetDateRangeChangedHdl( const Link& rLink ) { maDateRangeChangedHdl = rLink; }
	const Link& 	GetDateRangeChangedHdl() const { return maDateRangeChangedHdl; }
	void			SetRequestDateInfoHdl( const Link& rLink ) { maRequestDateInfoHdl = rLink; }
	const Link& 	GetRequestDateInfoHdl() const { return maRequestDateInfoHdl; }
	void			SetDoubleClickHdl( const Link& rLink ) { maDoubleClickHdl = rLink; }
	const Link& 	GetDoubleClickHdl() const { return maDoubleClickHdl; }
	void			SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link& 	GetSelectHdl() const { return maSelectHdl; }
};

inline const Color& Calendar::GetStandardColor() const
{
	if ( mpStandardColor )
		return *mpStandardColor;
	else
		return GetFont().GetColor();
}

inline const Color& Calendar::GetSaturdayColor() const
{
	if ( mpSaturdayColor )
		return *mpSaturdayColor;
	else
		return GetFont().GetColor();
}

inline const Color& Calendar::GetSundayColor() const
{
	if ( mpSundayColor )
		return *mpSundayColor;
	else
		return GetFont().GetColor();
}

class CalendarField : public DateField
{
private:
	ImplCFieldFloatWin* mpFloatWin;
	Calendar*			mpCalendar;
	WinBits 			mnCalendarStyle;
	PushButton* 		mpTodayBtn;
	PushButton* 		mpNoneBtn;
	Date				maDefaultDate;
	BOOL				mbToday;
	BOOL				mbNone;
	Link				maSelectHdl;

#ifdef _SV_CALENDAR_CXX
			DECL_LINK( ImplSelectHdl, Calendar* );
			DECL_LINK( ImplClickHdl, PushButton* );
			DECL_LINK( ImplPopupModeEndHdl, FloatingWindow* );
#endif

public:
			CalendarField( Window* pParent, WinBits nWinStyle );
			CalendarField( Window* pParent, const ResId& rResId );
			~CalendarField();

	virtual void		Select();

	virtual BOOL		ShowDropDown( BOOL bShow );
	virtual Calendar*	CreateCalendar( Window* pParent );
	Calendar*			GetCalendar();

	void				SetDefaultDate( const Date& rDate ) { maDefaultDate = rDate; }
	Date				GetDefaultDate() const { return maDefaultDate; }

	void				EnableToday( BOOL bToday = TRUE ) { mbToday = bToday; }
	BOOL				IsTodayEnabled() const { return mbToday; }
	void				EnableNone( BOOL bNone = TRUE ) { mbNone = bNone; }
	BOOL				IsNoneEnabled() const { return mbNone; }

	void				SetCalendarStyle( WinBits nStyle ) { mnCalendarStyle = nStyle; }
	WinBits 			GetCalendarStyle() const { return mnCalendarStyle; }

	void				SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link& 		GetSelectHdl() const { return maSelectHdl; }
};

#endif	
