
#ifndef _SFXCANCEL_HXX
#define _SFXCANCEL_HXX

#if SUPD>=364
#define SFX_INSERT_CANCELLABLE InsertCancellable
#define SFX_REMOVE_CANCELLABLE RemoveCancellable
#else
#define SFX_INSERT_CANCELLABLE AddJob
#define SFX_REMOVE_CANCELLABLE RemoveJob
#endif

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef _REF_HXX 
#include <tools/ref.hxx>
#endif

#ifndef _SFXBRDCST_HXX
#include <svtools/brdcst.hxx>
#endif

#ifndef _SFXSMPLHINT_HXX
#include <svtools/smplhint.hxx>
#endif

class SfxCancellable;

#ifdef _SFX_CANCEL_CXX

#ifndef _SVARRAY_HXX
#include <svtools/svarray.hxx>
#endif

SV_DECL_PTRARR( SfxCancellables_Impl, SfxCancellable*, 0, 4 );

#else

typedef SvPtrarr SfxCancellables_Impl;

#endif

class SfxCancelManager: public SfxBroadcaster
, public SvWeakBase

{
	SfxCancelManager*		_pParent;
	SfxCancellables_Impl 	_aJobs;

public:
				SfxCancelManager( SfxCancelManager *pParent = 0 );
				~SfxCancelManager();

	BOOL                    CanCancel() const;
	void					Cancel( BOOL bDeep );
	SfxCancelManager*		GetParent() const { return _pParent; }

	void					SFX_INSERT_CANCELLABLE( SfxCancellable *pJob );
	void					SFX_REMOVE_CANCELLABLE( SfxCancellable *pJob );
	USHORT					GetCancellableCount() const
				{ return _aJobs.Count(); }
	SfxCancellable* 		GetCancellable( USHORT nPos ) const
				{ return (SfxCancellable*) _aJobs[nPos]; }
};

SV_DECL_WEAK( SfxCancelManager )

class SfxCancellable

{
	SfxCancelManager*		_pMgr;
	BOOL					_bCancelled;
	String					_aTitle;

public:
				SfxCancellable( SfxCancelManager *pMgr,
								const String &rTitle )
				:	_pMgr( pMgr ),
					_bCancelled( FALSE ),
					_aTitle( rTitle )
				{ pMgr->SFX_INSERT_CANCELLABLE( this ); }

	virtual 				~SfxCancellable();

	void					SetManager( SfxCancelManager *pMgr );
	SfxCancelManager*       GetManager() const { return _pMgr; }

	virtual void			Cancel();
	BOOL					IsCancelled() const { return _bCancelled; }
	operator 				BOOL() const { return _bCancelled; }
	const String&			GetTitle() const { return _aTitle; }
};

#endif

