

#ifndef _CTRLBOX_HXX
#define _CTRLBOX_HXX

#ifndef _INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _LSTBOX_HXX
#include <vcl/lstbox.hxx>
#endif
#ifndef _COMBOBOX_HXX
#include <vcl/combobox.hxx>
#endif
#ifndef _IMAGE_HXX
#include <vcl/image.hxx>
#endif
#ifndef _VIRDEV_HXX
#include <vcl/virdev.hxx>
#endif
#ifndef _METRIC_HXX
#include <vcl/metric.hxx>
#endif
#ifndef _FIELD_HXX
#include <vcl/field.hxx>
#endif

class ImplFontList;
class ImpColorList;
class ImpLineList;
class FontList;

class ColorListBox : public ListBox
{
	ImpColorList*	pColorList; 
	Size			aImageSize;

#ifdef _CTRLBOX_CXX
	void			ImplInit();
	void			ImplDestroyColorEntries();
#endif

public:
		ColorListBox( Window* pParent,
					  WinBits nWinStyle = WB_BORDER );
		ColorListBox( Window* pParent, const ResId& rResId );
	virtual 		~ColorListBox();

	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

	USHORT			InsertEntry( const XubString& rStr,
					 USHORT nPos = LISTBOX_APPEND );
	USHORT			InsertEntry( const Color& rColor, const XubString& rStr,
					 USHORT nPos = LISTBOX_APPEND );
	void			InsertAutomaticEntry();
	void			RemoveEntry( USHORT nPos );
	void			Clear();
	void			CopyEntries( const ColorListBox& rBox );

	USHORT			GetEntryPos( const XubString& rStr ) const
			{ return ListBox::GetEntryPos( rStr ); }

	USHORT			GetEntryPos( const Color& rColor ) const;
	Color			GetEntryColor( USHORT nPos ) const;
	Size			GetImageSize() const { return aImageSize; }

	void			SelectEntry( const XubString& rStr, BOOL bSelect = TRUE )
			{ ListBox::SelectEntry( rStr, bSelect ); }
	void			SelectEntry( const Color& rColor, BOOL bSelect = TRUE );
	XubString		GetSelectEntry( USHORT nSelIndex = 0 ) const
			{ return ListBox::GetSelectEntry( nSelIndex ); }
	Color			GetSelectEntryColor( USHORT nSelIndex = 0 ) const;
	BOOL			IsEntrySelected( const XubString& rStr ) const
			{ return ListBox::IsEntrySelected( rStr ); }

	BOOL			IsEntrySelected( const Color& rColor ) const;

private:
			ColorListBox( const ColorListBox& );
	ColorListBox&	operator =( const ColorListBox& );
	USHORT			GetEntryPos( const void* pData ) const;
	void			SetEntryData( USHORT nPos, void* pNewData );
	void*			GetEntryData( USHORT nPos ) const;
};

inline void ColorListBox::SelectEntry( const Color& rColor, BOOL bSelect )
{
	USHORT nPos = GetEntryPos( rColor );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		ListBox::SelectEntryPos( nPos, bSelect );
}

inline BOOL ColorListBox::IsEntrySelected( const Color& rColor ) const
{
	USHORT nPos = GetEntryPos( rColor );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		return IsEntryPosSelected( nPos );
	else
		return FALSE;
}

inline Color ColorListBox::GetSelectEntryColor( USHORT nSelIndex ) const
{
	USHORT nPos = GetSelectEntryPos( nSelIndex );
	Color aColor;
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		aColor = GetEntryColor( nPos );
	return aColor;
}

class LineListBox : public ListBox
{
	ImpLineList*	pLineList;
	VirtualDevice	aVirDev;
	International	aIntn;
	Size			aTxtSize;
	Color			aColor;
	FieldUnit		eUnit;
	FieldUnit		eSourceUnit;

#ifdef _CTRLBOX_CXX
	void			ImpGetLine( long nLine1, long nLine2, long nDistance,
					Bitmap& rBmp, XubString& rStr );
	void			ImplInit();
#endif

public:
		LineListBox( Window* pParent,
					 WinBits nWinStyle = WB_BORDER );
		LineListBox( Window* pParent, const ResId& rResId );
	virtual 		~LineListBox();

	USHORT			InsertEntry( const XubString& rStr,
					 USHORT nPos = LISTBOX_APPEND );
	USHORT			InsertEntry( long nLine1, long nLine2 = 0,
					 long nDistance = 0,
					 USHORT nPos = LISTBOX_APPEND );
	void			RemoveEntry( USHORT nPos );
	void			Clear();

	USHORT			GetEntryPos( const XubString& rStr ) const
			{ return ListBox::GetEntryPos( rStr ); }
	USHORT			GetEntryPos( long nLine1, long nLine2 = 0,
					 long nDistance = 0 ) const;
	long			GetEntryLine1( USHORT nPos ) const;
	long			GetEntryLine2( USHORT nPos ) const;
	long			GetEntryDistance( USHORT nPos ) const;

	void			SelectEntry( const XubString& rStr, BOOL bSelect = TRUE )
			{ ListBox::SelectEntry( rStr, bSelect ); }
	void			SelectEntry( long nLine1, long nLine2 = 0,
					 long nDistance = 0, BOOL bSelect = TRUE );
	long			GetSelectEntryLine1( USHORT nSelIndex = 0 ) const;
	long			GetSelectEntryLine2( USHORT nSelIndex = 0 ) const;
	long			GetSelectEntryDistance( USHORT nSelIndex = 0 ) const;
	BOOL			IsEntrySelected( const XubString& rStr ) const
			{ return ListBox::IsEntrySelected( rStr ); }
	BOOL			IsEntrySelected( long nLine1, long nLine2 = 0,
						 long nDistance = 0 ) const;

	void			SetUnit( FieldUnit eNewUnit ) { eUnit = eNewUnit; }
	FieldUnit		GetUnit() const { return eUnit; }
	void			SetSourceUnit( FieldUnit eNewUnit ) { eSourceUnit = eNewUnit; }
	FieldUnit		GetSourceUnit() const { return eSourceUnit; }

	void			SetColor( const Color& rColor );
	Color			GetColor() const { return aColor; }

private:
			LineListBox( const LineListBox& );
	LineListBox&	operator =( const LineListBox& );
	USHORT			GetEntryPos( const void* pData ) const;
	void			SetEntryData( USHORT nPos, void* pNewData );
	void*			GetEntryData( USHORT nPos ) const;
};

inline void LineListBox::SelectEntry( long nLine1, long nLine2,
						  long nDistance, BOOL bSelect )
{
	USHORT nPos = GetEntryPos( nLine1, nLine2, nDistance );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		ListBox::SelectEntryPos( nPos, bSelect );
}

inline long LineListBox::GetSelectEntryLine1( USHORT nSelIndex ) const
{
	USHORT nPos = GetSelectEntryPos( nSelIndex );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		return GetEntryLine1( nPos );
	else
		return 0;
}

inline long LineListBox::GetSelectEntryLine2( USHORT nSelIndex ) const
{
	USHORT nPos = GetSelectEntryPos( nSelIndex );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		return GetEntryLine2( nPos );
	else
		return 0;
}

inline long LineListBox::GetSelectEntryDistance( USHORT nSelIndex ) const
{
	USHORT nPos = GetSelectEntryPos( nSelIndex );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		return GetEntryDistance( nPos );
	else
		return 0;
}

inline BOOL LineListBox::IsEntrySelected( long nLine1, long nLine2,
							  long nDistance ) const
{
	USHORT nPos = GetEntryPos( nLine1, nLine2, nDistance );
	if ( nPos != LISTBOX_ENTRY_NOTFOUND )
		return IsEntryPosSelected( nPos );
	else
		return FALSE;
}

class FontNameBox : public ComboBox
{
private:
	ImplFontList*	mpFontList;
	Image			maImagePrinterFont;
	Image			maImageBitmapFont;
	Image			maImageScalableFont;
	BOOL			mbWYSIWYG;
	BOOL			mbSymbols;

#ifdef _CTRLBOX_CXX
	void			ImplCalcUserItemSize();
	void			ImplDestroyFontList();
#endif

public:
		FontNameBox( Window* pParent,
					 WinBits nWinStyle = WB_SORT );
		FontNameBox( Window* pParent, const ResId& rResId );
	virtual 		~FontNameBox();

	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

	void			Fill( const FontList* pList );

	void			EnableWYSIWYG( BOOL bEnable = TRUE );
	BOOL			IsWYSIWYGEnabled() const { return mbWYSIWYG; }

	void			EnableSymbols( BOOL bEnable = TRUE );
	BOOL			IsSymbolsEnabled() const { return mbSymbols; }

private:
			FontNameBox( const FontNameBox& );
	FontNameBox&	operator =( const FontNameBox& );
};

class FontStyleBox : public ComboBox
{
	XubString		aLastStyle;

public:
		FontStyleBox( Window* pParent, WinBits nWinStyle = 0 );
		FontStyleBox( Window* pParent, const ResId& rResId );
	virtual 		~FontStyleBox();

	virtual void	Select();
	virtual void	LoseFocus();
	virtual void	Modify();

	void			SetText( const XubString& rText );
	void			Fill( const XubString& rName, const FontList* pList );

private:
			FontStyleBox( const FontStyleBox& );
	FontStyleBox&	operator =( const FontStyleBox& );
};

inline void FontStyleBox::SetText( const XubString& rText )
{
	aLastStyle = rText;
	ComboBox::SetText( rText );
}

class FontSizeBox : public MetricBox
{
	FontInfo		aFontInfo;
	const FontList* pFontList;
	USHORT			nRelMin;
	USHORT			nRelMax;
	USHORT			nRelStep;
	short			nPtRelMin;
	short			nPtRelMax;
	short			nPtRelStep;
	BOOL			bRelativeMode:1,
		bRelative:1,
		bPtRelative:1,
		bStdSize:1;

#ifdef _CTRLBOX_CXX
	void			ImplInit();
#endif

protected:
	virtual XubString CreateFieldText( long nValue ) const;

public:
		FontSizeBox( Window* pParent, WinBits nWinStyle = 0 );
		FontSizeBox( Window* pParent, const ResId& rResId );
	virtual 		~FontSizeBox();

	void			Reformat();
	void			Modify();

	void			Fill( const FontInfo& rInfo, const FontList* pList );

	void			EnableRelativeMode( USHORT nMin = 50, USHORT nMax = 150,
							USHORT nStep = 5 );
	void			EnablePtRelativeMode( short nMin = -200, short nMax = 200,
							  short nStep = 10 );
	BOOL			IsRelativeMode() const { return bRelativeMode; }
	void			SetRelative( BOOL bRelative = FALSE );
	BOOL			IsRelative() const { return bRelative; }
	void			SetPtRelative( BOOL bPtRel = TRUE )
			{ bPtRelative = bPtRel; SetRelative( TRUE ); }
	BOOL			IsPtRelative() const { return bPtRelative; }

	virtual void	SetValue( long nNewValue, FieldUnit eInUnit );
	virtual void	SetValue( long nNewValue  );
	virtual long	GetValue( FieldUnit eOutUnit ) const;
	virtual long	GetValue() const;
	long			GetValue( USHORT nPos, FieldUnit eOutUnit ) const;
	void			SetUserValue( long nNewValue, FieldUnit eInUnit );
	void			SetUserValue( long nNewValue ) { SetUserValue( nNewValue, FUNIT_NONE ); }

private:
			FontSizeBox( const FontSizeBox& );
	FontSizeBox&	operator =( const FontSizeBox& );
};

#endif	
