

#ifndef _SVTOOLS_EDITBROWSEBOX_HXX_
#define _SVTOOLS_EDITBROWSEBOX_HXX_

#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

#ifndef _WINDOW_HXX
#include <vcl/window.hxx>
#endif

#ifndef _SV_COMBOBOX_HXX
#include <vcl/combobox.hxx>
#endif

#ifndef _SV_LSTBOX_HXX
#include <vcl/lstbox.hxx>
#endif

#ifndef _IMAGEBTN_HXX
#include <vcl/imagebtn.hxx>
#endif

#ifndef _SVX_BRWBOX_HXX
#include <svtools/brwbox.hxx>
#endif

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif

#ifndef _BRWHEAD_HXX
#include <svtools/brwhead.hxx>
#endif

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#define EBBF_NONE					((sal_Int32)0x0000)
#define EBBF_NOROWPICTURE			((sal_Int32)0x0001)

#define EBBF_ACTIVATE_ON_BUTTONDOWN	((sal_Int32)0x0002)

class Edit;
class ListBoxFrame;
class ButtonCtrl;
class SpinField;

namespace svt
{

	class CellControllerRef;

	class CellController : public SvRefBase
	{
		friend class EditBrowseBox;

	protected:
		Window* pWindow;

	public:
		TYPEINFO();

		CellController(Window* pW);
		virtual ~CellController();

		Window& GetWindow() const {return *((CellController*) this)->pWindow;}

		virtual void SetModified();
		virtual void ClearModified() = 0;
		virtual sal_Bool IsModified()  const = 0;

	protected:
		virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
		virtual void SetModifyHdl(const Link& rLink) = 0;
		virtual sal_Bool WantMouseEvent() const;
	};

	SV_DECL_IMPL_REF(CellController);

	class EditCellController : public CellController
	{
	public:
		TYPEINFO();
		EditCellController(Edit* pEdit);
		Edit& GetEditWindow() const {return (Edit &)GetWindow();}

		virtual void SetModified();
		virtual sal_Bool IsModified() const;
		virtual void ClearModified();

	protected:
		virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
		virtual void SetModifyHdl(const Link& rLink);
	};

	class SpinCellController : public CellController
	{
	public:
		TYPEINFO();
		SpinCellController(SpinField* pSpinField);
		SpinField& GetSpinWindow() const {return (SpinField &)GetWindow();}

		virtual void SetModified();
		virtual sal_Bool IsModified() const;
		virtual void ClearModified();

	protected:
		virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
		virtual void SetModifyHdl(const Link& rLink);
	};

	class CheckBoxControl : public Control
	{
		CheckBox*	pBox;
		Rectangle	aFocusRect;
		Link		m_aClickLink,m_aModifyLink;

	public:
		CheckBoxControl(Window* pParent, WinBits nWinStyle = 0);
		~CheckBoxControl();

		virtual void GetFocus();
		virtual long PreNotify(NotifyEvent& rEvt);
		virtual void Paint(const Rectangle& rClientRect);
		virtual void Resize();

		void SetClickHdl(const Link& rHdl) {m_aClickLink = rHdl;}
		const Link& GetClickHdl() const {return m_aClickLink;}

		void SetModifyHdl(const Link& rHdl) {m_aModifyLink = rHdl;}
		const Link& GetModifyHdl() const {return m_aModifyLink;}

		CheckBox&	GetBox() {return *pBox;};

	private:
		DECL_LINK( OnClick, void* );
	};

	class CheckBoxCellController : public CellController
	{
	public:
		TYPEINFO();

		CheckBoxCellController(CheckBoxControl* pWin):CellController(pWin){}
		CheckBox& GetCheckBox() const;

		virtual sal_Bool IsModified() const;
		virtual void ClearModified();

	protected:
		virtual void SetModifyHdl(const Link& rLink);
		virtual sal_Bool WantMouseEvent() const;
	};

	class ComboBoxControl : public ComboBox
	{
		friend class ComboBoxCellController;

	public:
		ComboBoxControl(Window* pParent, WinBits nWinStyle = 0);

	protected:
		virtual long PreNotify( NotifyEvent& rNEvt );
	};

	class ComboBoxCellController : public CellController
	{
	public:
		TYPEINFO();

		ComboBoxCellController(ComboBoxControl* pParent);
		ComboBoxControl& GetComboBox() const {return (ComboBoxControl &)GetWindow();}

		virtual sal_Bool IsModified() const;
		virtual void ClearModified();

	protected:
		virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
		virtual void SetModifyHdl(const Link& rLink);
	};

	class ListBoxControl : public ListBox
	{
		friend class ListBoxCellController;

	public:
		ListBoxControl(Window* pParent, WinBits nWinStyle = 0);

	protected:
		virtual long PreNotify( NotifyEvent& rNEvt );
	};

	class ListBoxCellController : public CellController
	{
	public:
		TYPEINFO();

		ListBoxCellController(ListBoxControl* pParent);
		ListBoxControl& GetListBox() const {return (ListBoxControl &)GetWindow();}

		virtual sal_Bool IsModified() const;
		virtual void ClearModified();

	protected:
		virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
		virtual void SetModifyHdl(const Link& rLink);
	};

	class EditBrowserHeader : public BrowserHeader
	{
	public:
		EditBrowserHeader( BrowseBox* pParent, WinBits nWinBits = WB_BUTTONSTYLE )
			:BrowserHeader(pParent, nWinBits){}

	protected:
		virtual void DoubleClick();
	};

	class EditBrowseBox: public BrowseBox
	{
		friend class EditBrowserHeader;

		enum BrowseInfo
		{
			COLSELECT	=	1,
			ROWSELECT	=	2,
			ROWCHANGE	=	4,
			COLCHANGE	=	8
		};

	public:
		enum RowStatus
		{
			CLEAN				=	0,
			CURRENT				=	1,
			CURRENTNEW			=	2,
			MODIFIED			=	3,
			NEW					=	4,
			DELETED				=	5,
			PRIMARYKEY			=	6,
			CURRENT_PRIMARYKEY	=	7,
			FILTER				=	8
		};

	private:
		class BrowserMouseEventPtr
		{
			BrowserMouseEvent* pEvent;
			sal_Bool bDown;

		public:
			BrowserMouseEventPtr():pEvent(NULL){}
			~BrowserMouseEventPtr(){Clear();}
	sal_Bool Is() const {return pEvent != NULL;}
			sal_Bool IsDown() const {return bDown;}
			const BrowserMouseEvent* operator->() const {return pEvent;}
			const BrowserMouseEvent& operator*() const {return *pEvent;}
	void Clear();
			void Set(const BrowserMouseEvent* pEvt, sal_Bool bIsDown);
		} aMouseEvent;

		const BrowserMouseEvent* pMouseEvent;	
		CellControllerRef		 aController,
					 aOldController;

		sal_uInt32	nStartEvent, nEndEvent, nCellModifiedEvent;		
		Window*	m_pFocusWhileRequest;

		long	nPaintRow;	
		long	nEditRow, nOldEditRow;
		sal_uInt16	nEditCol, nOldEditCol;

		sal_Bool	bHasFocus : 1;
		sal_Bool	bPaintStatus : 1;	

		CheckBoxControl* pCheckBoxPaint;

		sal_Int32	m_nBrowserFlags;
		ImageList	m_aStatusImages;

	protected:
		BrowserHeader*  pHeader;

		sal_Bool isGetCellFocusPending() const { return nStartEvent != 0; }
		void cancelGetCellFocus() { if (nStartEvent) Application::RemoveUserEvent(nStartEvent); nStartEvent = 0; }
		void forceGetCellFocus() { cancelGetCellFocus(); LINK(this, EditBrowseBox, StartEditHdl).Call((void*)NULL); }

		BrowserMouseEventPtr& getMouseEvent() { return aMouseEvent; }

	protected:
		BrowserHeader*	GetHeaderBar() const {return pHeader;}

		virtual BrowserHeader* CreateHeaderBar(BrowseBox* pParent);

		virtual BrowserHeader* imp_CreateHeaderBar(BrowseBox* pParent);

		virtual void ColumnMoved(sal_uInt16 nId);
		virtual void ColumnResized(sal_uInt16 nColId);
		virtual void Resize();
		virtual void ArrangeControls(sal_uInt16& nX, sal_uInt16 nY);
		virtual sal_Bool SeekRow(long nRow);

		virtual void GetFocus();
		virtual void LoseFocus();
		virtual void KeyInput(const KeyEvent& rEvt);
		virtual void MouseButtonDown(const BrowserMouseEvent& rEvt);
		virtual void MouseButtonUp(const BrowserMouseEvent& rEvt);
		virtual void StateChanged( StateChangedType nType );
		virtual void DataChanged( const DataChangedEvent& rDCEvt );

		virtual long PreNotify(NotifyEvent& rNEvt );
		virtual long Notify(NotifyEvent& rNEvt);

		virtual void EndScroll();

		Rectangle GetCellRect(long nRow, sal_uInt16 nColId, sal_Bool bRelToBrowser = sal_True) const;
		virtual sal_uInt32 GetTotalCellWidth(long nRow, sal_uInt16 nColId);
		virtual sal_uInt32 GetAutoColumnWidth(sal_uInt16 nColId);

		virtual void PaintStatusCell(OutputDevice& rDev, const Rectangle& rRect) const;
		virtual void PaintCell(OutputDevice& rDev, const Rectangle& rRect, sal_uInt16 nColId) const = 0;

		virtual RowStatus GetRowStatus(long nRow) const;

		virtual sal_Bool CursorMoving(long nNewRow, sal_uInt16 nNewCol);

		virtual void CursorMoved();

		virtual void CellModified();		
		virtual sal_Bool SaveModified();	

		virtual sal_Bool SaveRow();			

		virtual sal_Bool IsModified() const {return aController.Is() && aController->IsModified();}

		virtual CellController* GetController(long nRow, sal_uInt16 nCol);
		virtual void InitController(CellControllerRef& rController, long nRow, sal_uInt16 nCol);
		virtual void ResizeController(CellControllerRef& rController, const Rectangle&);
		virtual void ReleaseController(CellControllerRef& pController, long nRow, sal_uInt16 nCol);
		virtual void DoubleClick(const BrowserMouseEvent&);

		void ActivateCell() { ActivateCell(GetCurRow(), GetCurColumnId()); }

		virtual Image GetImage(RowStatus) const;

		virtual void InsertHandleColumn(sal_uInt16 nWidth = 0);

		virtual sal_uInt16 AppendColumn(const String& rName, sal_uInt16 nWidth = 0, sal_uInt16 nPos = HEADERBAR_APPEND, sal_uInt16 nId = (sal_uInt16)-1);

		virtual sal_Bool IsTabAllowed(sal_Bool bForward) const;

		virtual sal_Bool IsCursorMoveAllowed(long nNewRow, sal_uInt16 nNewColId) const;

		void PaintTristate(OutputDevice& rDev, const Rectangle& rRect,const TriState& eState,sal_Bool _bEnabled=sal_True) const;

		void AsynchGetFocus();

	public:
		EditBrowseBox(Window* pParent, sal_Int32 nBrowserFlags = EBBF_NONE, WinBits nBits = WB_TABSTOP, BrowserMode nMode = 0 );
		EditBrowseBox(Window* pParent, const ResId& rId, sal_Int32 nBrowserFlags = EBBF_NONE, BrowserMode nMode = 0 );
		~EditBrowseBox();

		sal_Bool IsEditing() const {return aController.Is();}
		void InvalidateStatusCell(long nRow) {RowModified(nRow, 0);}
		void InvalidateHandleColumn();

		virtual sal_uInt16 GetDefaultColumnWidth(const String& rName) const;

		virtual void Init();
		virtual void RemoveRows();
		virtual void Dispatch(sal_uInt16 nId);

		CellControllerRef Controller() const { return aController; }
		sal_Int32	GetBrowserFlags() const { return m_nBrowserFlags; }
		void	SetBrowserFlags(sal_Int32 nFlags);

		virtual void ActivateCell(long nRow, sal_uInt16	nCol, sal_Bool bSetCellFocus = sal_True);
		virtual void DeactivateCell(sal_Bool bUpdate = sal_True);

	private:
		virtual void PaintField(OutputDevice& rDev, const Rectangle& rRect,
					sal_uInt16 nColumnId ) const;
		void ImplInitSettings( sal_Bool bFont, sal_Bool bForeground, sal_Bool bBackground );
		void DetermineFocus();
		inline void HideAndDisable(CellControllerRef& rController);
		inline void EnableAndShow() const;

		void implActivateCellOnMouseEvent(const BrowserMouseEvent& _rEvt, sal_Bool _bUp);

		DECL_LINK(ModifyHdl, void* );
		DECL_LINK(StartEditHdl, void* );
		DECL_LINK(EndEditHdl, void* );
		DECL_LINK(CellModifiedHdl, void* );
	};

}	

#endif 

