

#ifndef _FMTFIELD_HXX_
#define _FMTFIELD_HXX_

#ifndef _SV_SPINFLD_HXX 
#include <vcl/spinfld.hxx>
#endif

#ifndef _ZFORLIST_HXX 
#include <svtools/zforlist.hxx>
#endif

#ifndef _UNOTOOLS_TEXTSEARCH_HXX
#include <unotools/textsearch.hxx>
#endif

typedef USHORT FORMAT_CHANGE_TYPE;
#define FCT_KEYONLY			0x00		
#define FCT_FORMATTER		0x01		
#define FCT_PRECISION		0x02		
#define FCT_THOUSANDSSEP	0x03		

class FormattedField : public SpinField
{
private:

	class StaticFormatter
	{
		static SvNumberFormatter*	s_cFormatter;
		static ULONG				s_nReferences;
	public:
		StaticFormatter();
		~StaticFormatter();

		operator SvNumberFormatter* () { return GetFormatter(); }
		SvNumberFormatter* GetFormatter();
	};

protected:
	String		m_sLastValidText;

	Selection	m_aLastSelection;

	double				m_dMinValue;
	double				m_dMaxValue;
	BOOL				m_bHasMin : 1;
	BOOL				m_bHasMax : 1;

	BOOL				m_bStrictFormat : 1;

	BOOL				m_bValueDirty : 1;
	BOOL				m_bEnableEmptyField : 1;
	BOOL				m_bAutoColor : 1;
	double				m_dCurrentValue;
	double				m_dDefaultValue;

	ULONG				m_nFormatKey;
	SvNumberFormatter*	m_pFormatter;
	StaticFormatter		m_aStaticFormatter;

	double				m_dSpinSize;
	double				m_dSpinFirst;
	double				m_dSpinLast;

	BOOL				m_bTreatAsNumber;

	String				m_sCurrentTextValue;
	String				m_sDefaultText;

	Color*				m_pLastOutputColor;

public:
	FormattedField(Window* pParent, WinBits nStyle = 0, SvNumberFormatter* pInitialFormatter = NULL, INT32 nFormatKey = 0);
	FormattedField(Window* pParent, const ResId& rResId, SvNumberFormatter* pInitialFormatter = NULL, INT32 nFormatKey = 0);
	virtual ~FormattedField();

	BOOL	HasMinValue() const			{ return m_bHasMin; }
	void	ClearMinValue()				{ m_bHasMin = FALSE; }
	void	SetMinValue(double dMin);
	double	GetMinValue() const			{ return m_dMinValue; }

	BOOL	HasMaxValue() const			{ return m_bHasMax; }
	void	ClearMaxValue()				{ m_bHasMax = FALSE; }
	void	SetMaxValue(double dMax);
	double	GetMaxValue() const			{ return m_dMaxValue; }

	virtual void	SetValue(double dVal);
	virtual double	GetValue();

	void	GetColor() const;

	void	SetTextValue(const XubString& rText);

	BOOL	IsEmptyFieldEnabled() const			{ return m_bEnableEmptyField; }
	void	EnableEmptyField(BOOL bEnable);

	void	SetDefaultValue(double dDefault)	{ m_dDefaultValue = dDefault; m_bValueDirty = TRUE; }

	double	GetDefaultValue() const				{ return m_dDefaultValue; }

	ULONG	GetFormatKey() const				{ return m_nFormatKey; }
	void	SetFormatKey(ULONG nFormatKey);

	SvNumberFormatter*	GetFormatter() const	{ return m_pFormatter; }
	void	SetFormatter(SvNumberFormatter* pFormatter, BOOL bResetFormat = TRUE);

	BOOL	GetThousandsSep() const;
	void	SetThousandsSep(BOOL _bUseSeparator);

	USHORT	GetDecimalDigits() const;
	void	SetDecimalDigits(USHORT _nPrecision);

	SvNumberFormatter*	StandardFormatter() { return m_aStaticFormatter; }

	void		GetFormat(XubString& rFormatString, LanguageType& eLang) const;
	BOOL		SetFormat(const XubString& rFormatString, LanguageType eLang);

	BOOL	IsStrictFormat() const				{ return m_bStrictFormat; }
	void	SetStrictFormat(BOOL bEnable)		{ m_bStrictFormat = bEnable; }

	virtual void Up();
	virtual void Down();

	virtual void First();
	virtual void Last();

	void	SetSpinSize(double dStep)	{ m_dSpinSize = dStep; }
	double	GetSpinSize() const			{ return m_dSpinSize; }

	void	SetSpinFirst(double dFirst)	{ m_dSpinFirst = dFirst; }
	double	GetSpinFirst() const		{ return m_dSpinFirst; }

	void	SetSpinLast(double dLast)	{ m_dSpinLast = dLast; }
	double	GetSpinLast() const			{ return m_dSpinLast; }

	BOOL	TreatingAsNumber() const	{ return m_bTreatAsNumber; }
	void	TreatAsNumber(BOOL bDoSo) { m_bTreatAsNumber = bDoSo; }

public:
	virtual void SetText(const XubString& rStr);
	void	SetValidateText(const XubString& rText, const String* pErrorText = NULL);

	void SetTextFormatted(const XubString& rText);
	String	GetTextValue() const;

	void	SetDefaultText(const XubString& rDefault) { m_sDefaultText = rDefault; }
	String	GetDefaultText() const { return m_sDefaultText; }

	Color*	GetLastOutputColor() const { return m_pLastOutputColor; }

	void	SetAutoColor(BOOL _bAutomatic);
	BOOL	GetAutoColor() const { return m_bAutoColor; }

protected:
	virtual long Notify(NotifyEvent& rNEvt);
	virtual void Modify();

	virtual BOOL CheckText(const XubString& sText) const { return TRUE; }

	virtual void FormatChanged(FORMAT_CHANGE_TYPE nWhat);

	void ImplSetText(const XubString& rNew, Selection* pNewSel);
	void ImplSetValue(double dValue, BOOL bForce);
	BOOL ImplGetValue(double& dNewVal);

	void ImplSetFormatKey(ULONG nFormatKey);

	virtual SvNumberFormatter*	CreateFormatter() { SetFormatter(StandardFormatter()); return m_pFormatter; }
	SvNumberFormatter*	ImplGetFormatter() const { return m_pFormatter ? m_pFormatter : ((FormattedField*)this)->CreateFormatter(); }

	long PreNotify(NotifyEvent& rNEvt);

	virtual void ReFormat();
};

class DoubleNumericField : public FormattedField
{
protected:
	::utl::TextSearch*	m_pConformanceTester;

public:
	DoubleNumericField(Window* pParent, WinBits nStyle = 0) : FormattedField(pParent, nStyle), m_pConformanceTester(NULL) { }
	DoubleNumericField(Window* pParent, const ResId& rResId) : FormattedField(pParent, rResId), m_pConformanceTester(NULL) { }
	virtual ~DoubleNumericField();

protected:
	virtual BOOL CheckText(const XubString& sText) const;

	virtual void FormatChanged(FORMAT_CHANGE_TYPE nWhat);
	void ResetConformanceTester();
};

#define FCT_CURRENCY_SYMBOL		0x10
#define FCT_CURRSYM_POSITION	0x20

class DoubleCurrencyField : public FormattedField
{
	XubString	m_sCurrencySymbol;
	BOOL		m_bPrependCurrSym;
	BOOL		m_bChangingFormat;

public:
	DoubleCurrencyField(Window* pParent, WinBits nStyle = 0);
	DoubleCurrencyField(Window* pParent, const ResId& rResId);

	XubString	getCurrencySymbol() const { return m_sCurrencySymbol; }
	void		setCurrencySymbol(const XubString& _sSymbol);

	BOOL		getPrependCurrSym() const { return m_bPrependCurrSym; }
	void		setPrependCurrSym(BOOL _bPrepend);

protected:
	virtual void FormatChanged(FORMAT_CHANGE_TYPE nWhat);

	void UpdateCurrencyFormat();
};

#endif 

