

#ifndef _SVTOOLS_HASHTAB_HXX
#define _SVTOOLS_HASHTAB_HXX

#include <string.h>			

#ifndef HASH_NO_STRING_CLASS

#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif
#endif

typedef void* HASHNAMEPTR;

typedef void* HASHSYMPTR;

typedef size_t HASHTYPE;

typedef ULONG HASHSYMCOUNTTYPE;

const size_t nHashChainMaxLen = 128;

typedef struct HashElement
{
private:
	friend class HashTabBase;
#ifdef HASH_DEBUG
	friend int HashNameCmp( const void*, const void* );		
#endif
	HashElement*	pNext;		
	HashElement**	ppPrev;		
	HASHNAMEPTR		pName;		
	USHORT			nAccess;	
} HASHBUCKET;

struct HashStats
{
	HASHTYPE			nLengths[nHashChainMaxLen];	
	double				nMeanLen;		
	double				nVariance;		
	double				nMeanLenU;		
	double				nVarianceU;		
	HASHSYMCOUNTTYPE	nNumSyms;		
	HASHSYMCOUNTTYPE	nMinLen;		
	HASHSYMCOUNTTYPE	nMaxLen;		
	HASHTYPE			nLonger;		
	HASHTYPE			nSize;			
	HASHTYPE			nChainsU;		
};

class HashTabBase
{

public:

	static HASHTYPE		GetNearPrime( HASHTYPE nVal );
				HashTabBase( HASHTYPE nMaxSlots = 127,
				const Link& rDelLink = Link() );

	virtual				~HashTabBase() = 0;

	inline HASHTYPE		Is() const;

	virtual	HASHTYPE	Hash( const HASHNAMEPTR pName ) const = 0;

	virtual int			NameCmp( const HASHNAMEPTR pName1,
				const HASHNAMEPTR pName2 ) const = 0;

	virtual	HASHNAMEPTR	NameDup( const HASHNAMEPTR pName ) const = 0;

	virtual	void		NameDel( HASHNAMEPTR pName ) = 0;

	HASHSYMPTR			AddSym( const HASHNAMEPTR pName, size_t nDataSize,
					BOOL bUseLastFindSymHash = FALSE );

	HASHSYMPTR			AddPtrSym( const HASHNAMEPTR pName, void* pSymbol,
					BOOL bUseLastFindSymHash = FALSE );

	HASHSYMPTR			FindSym( const HASHNAMEPTR pName );

	inline void*		FindPtrSym( const HASHNAMEPTR pName );

	void				DelSym( HASHSYMPTR pSymbolData );

	inline void			SetDataDelLink( const Link& rLink );

	HASHSYMPTR			First();

	HASHSYMPTR			Next();

	HASHSYMPTR			NextInChain();

	const HASHNAMEPTR	GetName( const HASHSYMPTR pSymbolData ) const;

	USHORT				GetTimesAccessed( const HASHSYMPTR pSymbolData ) const;

	void				GetStats( HashStats* pStat ) const;

	static inline void*	Derefer( HASHSYMPTR pSymbolData );

	inline void			SetToTop( BOOL bFlg );

	inline HASHTYPE		GetSize();

	inline HASHSYMCOUNTTYPE Count();

#ifdef HASH_DEBUG

	int					PrintTab( void (*Print) ( const HASHNAMEPTR pName,
						const HASHSYMPTR pSymbolData ),
						BOOL bSort = FALSE ) const;

	void				dPrintTab();
#endif

protected:

	void				DtorCall();

private:

	Link				aDataDelLink;

	HASHBUCKET**		ppTable;

	HASHBUCKET*			pActBucket;

	HASHSYMCOUNTTYPE	nNumSyms;

	HASHTYPE			nSize;

	HASHTYPE			nLastFindSymHash;

	HASHTYPE			nActPos;

	BOOL				bActOnNext;

	BOOL				bDtorCalled;

	BOOL				bOwnDtorCall;

	BOOL				bToTop;
				HashTabBase( const HashTabBase& rTab );

	HashTabBase&		operator=( const HashTabBase& rTab );

	inline HASHSYMPTR	IncBucketPtr( HASHBUCKET* ) const;

	inline HASHBUCKET*	DecBucketPtr( HASHSYMPTR ) const;

	inline void			SetActBucket( HASHBUCKET* pB );

};

inline HASHTYPE HashTabBase::Is() const
{
	return ppTable ? nSize : 0;
}

inline void* HashTabBase::FindPtrSym(
		const HASHNAMEPTR pName		
	)
{
	return Derefer( FindSym( pName ) );
}

inline void HashTabBase::SetDataDelLink(
		const Link& rLink		
	)
{
	aDataDelLink = rLink;
}

inline void* HashTabBase::Derefer(
		HASHSYMPTR pSymbolData		
	)
{
	return (pSymbolData ? *(HASHSYMPTR*)pSymbolData : 0);
}

inline void HashTabBase::SetToTop(
		BOOL bFlg		
	)
{
	bToTop = bFlg;
}

inline HASHTYPE HashTabBase::GetSize()
{
	return nSize;
}

inline HASHSYMCOUNTTYPE HashTabBase::Count()
{
	return nNumSyms;
}

#define HASH_IMPL_DTOR( ClassName ) \
ClassName::~ClassName() \
{ \
	HashTabBase::DtorCall(); \
}

class CharHashTab : public HashTabBase
{

public:
				CharHashTab( HASHTYPE nMaxSlots = 127,
				const Link& rDelLink = Link() )
				: HashTabBase( nMaxSlots, rDelLink ) {}

	virtual				~CharHashTab();

	virtual	HASHTYPE	Hash( const HASHNAMEPTR pName ) const;

	virtual int			NameCmp( const HASHNAMEPTR pName1, const HASHNAMEPTR pName2 ) const;

	virtual	HASHNAMEPTR	NameDup( const HASHNAMEPTR pName ) const;

	virtual	void		NameDel( HASHNAMEPTR pName );
};

#ifndef HASH_NO_STRING_CLASS

class StringHashTab : public HashTabBase
{

public:
				StringHashTab( HASHTYPE nMaxSlots = 127,
				const Link& rDelLink = Link() )
				: HashTabBase( nMaxSlots, rDelLink ) {}

	virtual				~StringHashTab();

	virtual	HASHTYPE	Hash( const HASHNAMEPTR pName ) const;

	virtual int			NameCmp( const HASHNAMEPTR pName1, const HASHNAMEPTR pName2 ) const;

	virtual	HASHNAMEPTR	NameDup( const HASHNAMEPTR pName ) const;

	virtual	void		NameDel( HASHNAMEPTR pName );

	void				SetStringDataDelLink();

	DECL_STATIC_LINK( StringHashTab, StringDel, HASHSYMPTR );

	void				ClearDataDelLink();

};

#endif	

#endif
