

#ifndef _HEADBAR_HXX
#define _HEADBAR_HXX

#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _WINDOW_HXX
#include <vcl/window.hxx>
#endif

class Accelerator;
class ImplHeadItemList;

#define WB_BOTTOMBORDER 		((WinBits)0x0400)
#define WB_BUTTONSTYLE			((WinBits)0x0800)
#define WB_STDHEADERBAR 		(WB_BUTTONSTYLE | WB_BOTTOMBORDER)

typedef USHORT HeaderBarItemBits;

#define HIB_LEFT				((HeaderBarItemBits)0x0001)
#define HIB_CENTER				((HeaderBarItemBits)0x0002)
#define HIB_RIGHT				((HeaderBarItemBits)0x0004)
#define HIB_TOP 				((HeaderBarItemBits)0x0008)
#define HIB_VCENTER 			((HeaderBarItemBits)0x0010)
#define HIB_BOTTOM				((HeaderBarItemBits)0x0020)
#define HIB_LEFTIMAGE			((HeaderBarItemBits)0x0040)
#define HIB_RIGHTIMAGE			((HeaderBarItemBits)0x0080)
#define HIB_FIXED				((HeaderBarItemBits)0x0100)
#define HIB_FIXEDPOS			((HeaderBarItemBits)0x0200)
#define HIB_CLICKABLE			((HeaderBarItemBits)0x0400)
#define HIB_FLAT				((HeaderBarItemBits)0x0800)
#define HIB_DOWNARROW			((HeaderBarItemBits)0x1000)
#define HIB_UPARROW 			((HeaderBarItemBits)0x2000)
#define HIB_USERDRAW			((HeaderBarItemBits)0x4000)
#define HIB_STDSTYLE			(HIB_LEFT | HIB_LEFTIMAGE | HIB_VCENTER | HIB_CLICKABLE)

#define HEADERBAR_APPEND			((USHORT)0xFFFF)
#define HEADERBAR_ITEM_NOTFOUND 	((USHORT)0xFFFF)
#define HEADERBAR_FULLSIZE			((long)1000000000)

#define HEADERBAR_TEXTOFF			2

class HeaderBar : public Window
{
private:
	ImplHeadItemList*	mpItemList;
	long				mnBorderOff1;
	long				mnBorderOff2;
	long				mnOffset;
	long				mnDX;
	long				mnDY;
	long				mnDragSize;
	long				mnStartPos;
	long				mnDragPos;
	long				mnMouseOff;
	USHORT				mnCurItemId;
	USHORT				mnItemDragPos;
	BOOL				mbDragable;
	BOOL				mbDrag;
	BOOL				mbItemDrag;
	BOOL				mbOutDrag;
	BOOL				mbButtonStyle;
	BOOL				mbItemMode;
	Link				maStartDragHdl;
	Link				maDragHdl;
	Link				maEndDragHdl;
	Link				maSelectHdl;
	Link				maDoubleClickHdl;

#ifdef _SV_HEADBAR_CXX
	void				ImplInit( WinBits nWinStyle );
	void				ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	long				ImplGetItemPos( USHORT nPos ) const;
	Rectangle			ImplGetItemRect( USHORT nPos ) const;
	USHORT				ImplHitTest( const Point& rPos, long& nMouseOff, USHORT& nPos ) const;
	void				ImplInvertDrag( USHORT nStartPos, USHORT nEndPos );
	void				ImplDrawItem( OutputDevice* pDev,
						  USHORT nPos, BOOL bHigh, BOOL bDrag,
						  const Rectangle& rItemRect,
						  const Rectangle* pRect,
						  ULONG nFlags );
	void				ImplDrawItem( USHORT nPos, BOOL bHigh = FALSE,
						  BOOL bDrag = FALSE,
						  const Rectangle* pRect = NULL );
	void				ImplUpdate( USHORT nPos,
						BOOL bEnd = FALSE, BOOL bDirect = FALSE );
	void				ImplStartDrag( const Point& rPos, BOOL bCommand );
	void				ImplDrag( const Point& rPos );
	void				ImplEndDrag( BOOL bCancel );
#endif

public:
			HeaderBar( Window* pParent, WinBits nWinBits = WB_STDHEADERBAR );
			HeaderBar( Window* pParent, const ResId& rResId );
			~HeaderBar();

	virtual void		MouseButtonDown( const MouseEvent& rMEvt );
	virtual void		MouseMove( const MouseEvent& rMEvt );
	virtual void		Tracking( const TrackingEvent& rTEvt );
	virtual void		Paint( const Rectangle& rRect );
	virtual void		Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void		Resize();
	virtual void		Command( const CommandEvent& rCEvt );
	virtual void		RequestHelp( const HelpEvent& rHEvt );
	virtual void		StateChanged( StateChangedType nStateChange );
	virtual void		DataChanged( const DataChangedEvent& rDCEvt );

	virtual void		UserDraw( const UserDrawEvent& rUDEvt );
	virtual void		StartDrag();
	virtual void		Drag();
	virtual void		EndDrag();
	virtual void		Select();
	virtual void		DoubleClick();

	void				InsertItem( USHORT nItemId, const Image& rImage,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND );
	void				InsertItem( USHORT nItemId, const XubString& rText,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND );
	void				InsertItem( USHORT nItemId,
						const Image& rImage, const XubString& rText,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND );
	void				RemoveItem( USHORT nItemId );
	void				MoveItem( USHORT nItemId, USHORT nNewPos );
	void				Clear();

	void				SetOffset( long nNewOffset = 0 );
	long				GetOffset() const { return mnOffset; }
	void				SetDragSize( long nNewSize = 0 ) { mnDragSize = nNewSize; }
	long				GetDragSize() const { return mnDragSize; }

	USHORT				GetItemCount() const;
	USHORT				GetItemPos( USHORT nItemId ) const;
	USHORT				GetItemId( USHORT nPos ) const;
	USHORT				GetItemId( const Point& rPos ) const;
	Rectangle			GetItemRect( USHORT nItemId ) const;
	USHORT				GetCurItemId() const { return mnCurItemId; }
	long				GetDragPos() const { return mnDragPos; }
	USHORT				GetItemDragPos() const { return mnItemDragPos; }
	BOOL				IsItemMode() const { return mbItemMode; }
	BOOL				IsItemDrag() const { return mbItemDrag; }

	void				SetItemSize( USHORT nItemId, long nNewSize );
	long				GetItemSize( USHORT nItemId ) const;
	void				SetItemBits( USHORT nItemId, HeaderBarItemBits nNewBits );
	HeaderBarItemBits	GetItemBits( USHORT nItemId ) const;
	void				SetItemData( USHORT nItemId, void* pNewData );
	void*				GetItemData( USHORT nItemId ) const;

	void				SetItemImage( USHORT nItemId, const Image& rImage );
	Image				GetItemImage( USHORT nItemId ) const;
	void				SetItemText( USHORT nItemId, const XubString& rText );
	XubString			GetItemText( USHORT nItemId ) const;

	void				SetHelpText( USHORT nItemId, const XubString& rText );
	XubString			GetHelpText( USHORT nItemId ) const;
	void				SetHelpId( USHORT nItemId, ULONG nHelpId );
	ULONG				GetHelpId( USHORT nItemId ) const;

	Size				CalcWindowSizePixel() const;

	void				SetHelpText( const XubString& rText )
				{ Window::SetHelpText( rText ); }
	const XubString& 	GetHelpText() const
				{ return Window::GetHelpText(); }
	void				SetHelpId( ULONG nId )
				{ Window::SetHelpId( nId ); }
	ULONG				GetHelpId() const
				{ return Window::GetHelpId(); }

	void				SetStartDragHdl( const Link& rLink ) { maStartDragHdl = rLink; }
	const Link& 		GetStartDragHdl() const { return maStartDragHdl; }
	void				SetDragHdl( const Link& rLink ) { maDragHdl = rLink; }
	const Link& 		GetDragHdl() const { return maDragHdl; }
	void				SetEndDragHdl( const Link& rLink ) { maEndDragHdl = rLink; }
	const Link& 		GetEndDragHdl() const { return maEndDragHdl; }
	void				SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link& 		GetSelectHdl() const { return maSelectHdl; }
	void				SetDoubleClickHdl( const Link& rLink ) { maDoubleClickHdl = rLink; }
	const Link& 		GetDoubleClickHdl() const { return maDoubleClickHdl; }
};

#endif	
