

#ifndef _INETHIST_HXX
#define _INETHIST_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _SFXBRDCST_HXX
#include <svtools/brdcst.hxx>
#endif

#ifndef _SFXHINT_HXX
#include <svtools/hint.hxx>
#endif

class INetURLHistory_Impl;
class INetURLHistory : public SfxBroadcaster
{

	INetURLHistory_Impl *m_pImpl;
	String               m_aLocation;

	INetURLHistory (void);
	virtual ~INetURLHistory (void);

	void NormalizeUrl_Impl   (INetURLObject &rUrl);
	void PutUrl_Impl   (const INetURLObject &rUrl);
	BOOL QueryUrl_Impl (const INetURLObject &rUrl);

	INetURLHistory (const INetURLHistory&);
	INetURLHistory& operator= (const INetURLHistory&);

public:

	static INetURLHistory* GetOrCreate (void);
	static void            Delete      (void);

	const String& GetLocation (void) const { return m_aLocation; }
	void          SetLocation (const String &rLocation);

	BOOL QueryProtocol (INetProtocol eProto) const
	{
		return ((eProto == INET_PROT_FILE ) ||
	(eProto == INET_PROT_FTP  ) ||
	(eProto == INET_PROT_HTTP ) ||
	(eProto == INET_PROT_HTTPS)    );
	}

	BOOL QueryUrl (const INetURLObject &rUrl)
	{
		if (QueryProtocol (rUrl.GetProtocol()))
			return QueryUrl_Impl (rUrl);
		else
			return FALSE;
	}

	BOOL QueryUrl (const String &rUrl)
	{
		INetProtocol eProto =
			INetURLObject::CompareProtocolScheme (rUrl);
		if (QueryProtocol (eProto))
			return QueryUrl (INetURLObject (rUrl));
		else
			return FALSE;
	}

	void PutUrl (const INetURLObject &rUrl)
	{
		if (QueryProtocol (rUrl.GetProtocol()))
			PutUrl_Impl (rUrl);
	}

	void PutUrl (const String &rUrl)
	{
		INetProtocol eProto =
			INetURLObject::CompareProtocolScheme (rUrl);
		if (QueryProtocol (eProto))
			PutUrl (INetURLObject (rUrl));
	}
};

DECL_PTRHINT (INetURLHistoryHint, const INetURLObject);

#endif 

