
#ifndef _SFX_ITEMPROP_HXX
#define _SFX_ITEMPROP_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _SFXITEMSET_HXX
#include <svtools/itemset.hxx>
#endif

#include <cppuhelper/implbase1.hxx>

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSETINFO_HPP_
#include <com/sun/star/beans/XPropertySetInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYSTATE_HPP_
#include <com/sun/star/beans/PropertyState.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif

#define MAP_CHAR_LEN(cchar) cchar, sizeof(cchar) - 1
struct SfxItemPropertyMap
{
	const char* 						pName;
	USHORT 								nNameLen;
	USHORT								nWID;
	const com::sun::star::uno::Type*	pType;
	long								nFlags;
	BYTE								nMemberId;

	static const SfxItemPropertyMap*	GetByName( const SfxItemPropertyMap *pMap,
			const ::rtl::OUString &rName );
};

class SfxItemPropertySet
{
	const SfxItemPropertyMap* 	_pMap;
protected:
	virtual BOOL			FillItem(SfxItemSet& rSet, USHORT nWhich, BOOL bGetProperty) const;

public:
				SfxItemPropertySet( const SfxItemPropertyMap *pMap ) :
					_pMap(pMap) {}

	void getPropertyValue( const SfxItemPropertyMap& rMap,
								const SfxItemSet& rSet,
								com::sun::star::uno::Any& rAny) const
								throw(::com::sun::star::uno::RuntimeException);
	com::sun::star::uno::Any
		getPropertyValue( const SfxItemPropertyMap& rMap,
								const SfxItemSet& rSet ) const
								throw(::com::sun::star::uno::RuntimeException);
	void getPropertyValue( const ::rtl::OUString &rName,
								const SfxItemSet& rSet,
								com::sun::star::uno::Any& rAny) const
								throw(::com::sun::star::uno::RuntimeException,
										::com::sun::star::beans::UnknownPropertyException);
	com::sun::star::uno::Any
		getPropertyValue( const ::rtl::OUString &rName,
								const SfxItemSet& rSet ) const
								throw(::com::sun::star::uno::RuntimeException,
										::com::sun::star::beans::UnknownPropertyException);
	void					setPropertyValue( const SfxItemPropertyMap& rMap,
#if SUPD>633
								const com::sun::star::uno::Any& aVal,
#else
								const com::sun::star::uno::Any aVal,
#endif
								SfxItemSet& rSet ) const
								throw(::com::sun::star::uno::RuntimeException,
										com::sun::star::lang::IllegalArgumentException);
	void					setPropertyValue( const ::rtl::OUString& rPropertyName,
#if SUPD>633
								const com::sun::star::uno::Any& aVal,
#else
								const com::sun::star::uno::Any aVal,
#endif
								SfxItemSet& rSet ) const
								throw(::com::sun::star::uno::RuntimeException,
										com::sun::star::lang::IllegalArgumentException,
										::com::sun::star::beans::UnknownPropertyException);

	com::sun::star::beans::PropertyState
		getPropertyState(const ::rtl::OUString& rName, const SfxItemSet& rSet)
						throw(com::sun::star::beans::UnknownPropertyException);
	com::sun::star::beans::PropertyState
	    getPropertyState(const SfxItemPropertyMap& rMap, const SfxItemSet& rSet)
	                                throw();

	com::sun::star::uno::Reference<com::sun::star::beans::XPropertySetInfo>
		getPropertySetInfo() const;
	const SfxItemPropertyMap*
		getPropertyMap() const {return _pMap;}

};

class SfxItemPropertySetInfo : public
	cppu::WeakImplHelper1<com::sun::star::beans::XPropertySetInfo>
{
	const SfxItemPropertyMap* 	_pMap;

public:
	SfxItemPropertySetInfo(const SfxItemPropertyMap *pMap ) :
					_pMap(pMap) {}
	 virtual ::com::sun::star::uno::Sequence< ::com::sun::star::beans::Property > SAL_CALL
		getProperties(  )
			throw(::com::sun::star::uno::RuntimeException);

	virtual ::com::sun::star::beans::Property SAL_CALL
		getPropertyByName( const ::rtl::OUString& aName )
			throw(::com::sun::star::beans::UnknownPropertyException,
		::com::sun::star::uno::RuntimeException);

	virtual sal_Bool SAL_CALL
		hasPropertyByName( const ::rtl::OUString& Name )
			throw(::com::sun::star::uno::RuntimeException);
};

class SfxExtItemPropertySetInfo: public cppu::WeakImplHelper1<com::sun::star::beans::XPropertySetInfo >
{
	const SfxItemPropertyMap* 											_pExtMap;
	com::sun::star::uno::Sequence<com::sun::star::beans::Property>		aPropertySeq;
public:
				SfxExtItemPropertySetInfo(
					const SfxItemPropertyMap *pMap,
					const com::sun::star::uno::Sequence<com::sun::star::beans::Property>& rPropSeq );
	 virtual ::com::sun::star::uno::Sequence< ::com::sun::star::beans::Property > SAL_CALL
		getProperties(  )
			throw(::com::sun::star::uno::RuntimeException);

	virtual ::com::sun::star::beans::Property SAL_CALL
		getPropertyByName( const ::rtl::OUString& aName )
			throw(::com::sun::star::beans::UnknownPropertyException, ::com::sun::star::uno::RuntimeException);

	virtual sal_Bool SAL_CALL
		hasPropertyByName( const ::rtl::OUString& Name )
			throw(::com::sun::star::uno::RuntimeException);
};

#endif
