
#ifndef _SFXPOOLITEM_HXX
#define _SFXPOOLITEM_HXX

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#define TF_POOLABLE

#ifndef _SAL_CONFIG_H_ 
#include <sal/config.h>
#endif
#ifndef _RTTI_HXX 
#include <tools/rtti.hxx>
#endif
#include <limits.h>
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#include <svtools/svarray.hxx>
#include <svtools/hint.hxx>

typedef long SfxArgumentError;

class SbxVariable;
class SbxObject;
class SvStream;
class Color;
class IntlWrapper;

namespace com { namespace sun { namespace star { namespace uno { class Any; } } } }

#define SFX_ITEMS_DIRECT                    0xffff
#define SFX_ITEMS_NULL                      0xfff0  

#define SFX_ITEMS_POOLDEFAULT               0xffff
#define SFX_ITEMS_STATICDEFAULT             0xfffe
#define SFX_ITEMS_DELETEONIDLE              0xfffd
#define SFX_ITEMS_MAXREF                    0xffef

#define CONVERT_TWIPS 						0x80	

inline sal_Bool Any2Bool( const ::com::sun::star::uno::Any&rValue )
{
	sal_Bool nValue = sal_False;
	if( rValue.hasValue() )
	{
		if( rValue.getValueType() == ::getCppuBooleanType() )
		{
			nValue = *(sal_Bool*)rValue.getValue();
		}
		else
		{
			sal_Int32 nNum;
			if( rValue >>= nNum )
	nValue = nNum != 0;
		}
	}

	return nValue;
}

inline ::com::sun::star::uno::Any Bool2Any( sal_Bool bValue )
{
	return ::com::sun::star::uno::Any( &bValue, ::getCppuBooleanType() );
}

enum SfxFieldUnit
{
	SFX_FUNIT_NONE, SFX_FUNIT_MM, SFX_FUNIT_CM, SFX_FUNIT_M, SFX_FUNIT_KM,
	SFX_FUNIT_TWIP, SFX_FUNIT_POINT, SFX_FUNIT_PICA,
	SFX_FUNIT_INCH, SFX_FUNIT_FOOT, SFX_FUNIT_MILE, SFX_FUNIT_CUSTOM
};

enum SfxMapUnit
{
	SFX_MAPUNIT_100TH_MM,
	SFX_MAPUNIT_10TH_MM,
	SFX_MAPUNIT_MM,
	SFX_MAPUNIT_CM,
	SFX_MAPUNIT_1000TH_INCH,
	SFX_MAPUNIT_100TH_INCH,
	SFX_MAPUNIT_10TH_INCH,
	SFX_MAPUNIT_INCH,
	SFX_MAPUNIT_POINT,
	SFX_MAPUNIT_TWIP,
	SFX_MAPUNIT_PIXEL,
	SFX_MAPUNIT_SYSFONT,
	SFX_MAPUNIT_APPFONT,
	SFX_MAPUNIT_RELATIVE
};

enum SfxItemPresentation

{
	SFX_ITEM_PRESENTATION_NONE,
	SFX_ITEM_PRESENTATION_NAMEONLY,
	SFX_ITEM_PRESENTATION_NAMELESS,
	SFX_ITEM_PRESENTATION_COMPLETE
};

typedef USHORT SfxItemState;

#define	SFX_ITEM_UNKNOWN	0x0000

#define SFX_ITEM_DISABLED	0x0001
#define SFX_ITEM_READONLY	0x0002

#define SFX_ITEM_DONTCARE   0x0010
#define SFX_ITEM_DEFAULT	0x0020
#define SFX_ITEM_SET		0x0030

#define SFX_ITEM_AVAILABLE	SFX_ITEM_DEFAULT
#define SFX_ITEM_OFF      	SFX_ITEM_DEFAULT
#define SFX_ITEM_ON			SFX_ITEM_SET

DBG_NAMEEX(SfxPoolItem);
DBG_NAMEEX(SfxVoidItem);
DBG_NAMEEX(SfxItemHandle);

class SvXMLUnitConverter;

class String;
namespace rtl
{
	class OUString;
}

class SfxPoolItem
{
friend class SfxItemPool;
friend class SfxItemDesruptor_Impl;
friend class SfxItemPoolCache;
friend class SfxItemSet;
friend class SfxVoidItem;

	USHORT                   nRef;                    
	USHORT                   nWhich;

private:
	inline void              SetRef( USHORT n );
public:
	inline USHORT            AddRef( USHORT n = 1 ) const;
private:
	inline USHORT            ReleaseRef( USHORT n = 1 ) const;
	long                     Delete_Impl(void*);

	virtual void             Store( SvStream & ) const;
	virtual void             GetVersion() const;

protected:
				 SfxPoolItem( USHORT nWhich = 0 );
				 SfxPoolItem( const SfxPoolItem& );

public:
				 TYPEINFO();
	virtual                  ~SfxPoolItem();

	void                     SetWhich( USHORT nId ) {
					DBG_CHKTHIS(SfxPoolItem, 0);
					nWhich = nId; }
	USHORT                   Which() const {
					 DBG_CHKTHIS(SfxPoolItem, 0);
					 return nWhich; }
	virtual int              operator==( const SfxPoolItem& ) const = 0;
	int                      operator!=( const SfxPoolItem& rItem ) const
				 { return !(*this == rItem); }
	virtual int				 Compare( const SfxPoolItem &rWith ) const;
	virtual int              Compare( const SfxPoolItem &rWith, const IntlWrapper& rIntlWrapper ) const;

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePresentation,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresentationMetric,
						XubString &rText,
	                                const IntlWrapper * pIntlWrapper = 0 )
		const;
	virtual USHORT           GetVersion( USHORT nFileFormatVersion ) const;
	virtual int              ScaleMetrics( long lMult, long lDiv );
	virtual int              HasMetrics() const;

	virtual	BOOL        	 QueryValue( com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 ) const;
	virtual	BOOL			 PutValue( const com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 );

	virtual SfxPoolItem*     Create( SvStream &, USHORT nItemVersion ) const;
	virtual SvStream&        Store( SvStream &, USHORT nItemVersion ) const;
	virtual SfxPoolItem*     Clone( SfxItemPool *pPool = 0 ) const = 0;

	USHORT                   GetRef() const { return nRef; }

	static bool readByteString(SvStream & rStream, UniString & rString);

	static void writeByteString(SvStream & rStream,
					UniString const & rString);

	static bool readUnicodeString(SvStream & rStream, UniString & rString,
					  bool bUnicode);

	static void writeUnicodeString(SvStream & rStream,
					   UniString const & rString);

private:
	SfxPoolItem&             operator=( const SfxPoolItem& );    
};

inline void SfxPoolItem::SetRef( USHORT n )
{
	DBG_CHKTHIS( SfxPoolItem, 0 );
	nRef = n;
}

inline USHORT SfxPoolItem::AddRef( USHORT n ) const
{
	DBG_CHKTHIS( SfxPoolItem, 0 );
	DBG_ASSERT( nRef <= SFX_ITEMS_MAXREF, "AddRef mit nicht-Pool-Item" );
	DBG_ASSERT( (ULONG)nRef + n < (ULONG)SFX_ITEMS_MAXREF, "AddRef: Referenzzaehler ueberschlaegt sich" );
	return ( ((SfxPoolItem *)this)->nRef += n );
}

inline USHORT SfxPoolItem::ReleaseRef( USHORT n ) const
{
	DBG_CHKTHIS( SfxPoolItem, 0 );
	DBG_ASSERT( nRef <= SFX_ITEMS_MAXREF, "AddRef mit nicht-Pool-Item" );
	DBG_ASSERT( nRef >= n, "ReleaseRef: Referenzzaehler ueberschlaegt sich" );
	((SfxPoolItem *)this)->nRef -= n;
	return nRef;
}

inline int IsPoolDefaultItem(const SfxPoolItem *pItem )
{
	return pItem && pItem->GetRef() == SFX_ITEMS_POOLDEFAULT;
}

inline int IsStaticDefaultItem(const SfxPoolItem *pItem )
{
	return pItem && pItem->GetRef() == SFX_ITEMS_STATICDEFAULT;
}

inline int IsDefaultItem( const SfxPoolItem *pItem )
{
	return pItem && pItem->GetRef() >= SFX_ITEMS_STATICDEFAULT;
}

inline int IsPooledItem( const SfxPoolItem *pItem )
{
	return pItem && pItem->GetRef() > 0 && pItem->GetRef() <= SFX_ITEMS_MAXREF;
}

inline int IsInvalidItem(const SfxPoolItem *pItem)
{
	return pItem == (SfxPoolItem *)-1;
}

class SfxVoidItem: public SfxPoolItem
{
public:
				TYPEINFO();
				SfxVoidItem( USHORT nWhich );
				SfxVoidItem( USHORT nWhich, SvStream & );
				SfxVoidItem( const SfxVoidItem& );
				~SfxVoidItem();

	virtual int             operator==( const SfxPoolItem& ) const;

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
						XubString &rText,
	                                const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*    Clone( SfxItemPool *pPool = 0 ) const;
			void            SetWhich(USHORT nWh) { nWhich = nWh; }
};

class SfxSetItem: public SfxPoolItem
{
	SfxItemSet              *pSet;

public:
				TYPEINFO();
				SfxSetItem( USHORT nWhich, SfxItemSet *pSet );
				SfxSetItem( USHORT nWhich, const SfxItemSet &rSet );
				SfxSetItem( const SfxSetItem&, SfxItemPool *pPool = 0 );
				~SfxSetItem();

	virtual int             operator==( const SfxPoolItem& ) const;

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
						XubString &rText,
	                                const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*    Clone( SfxItemPool *pPool = 0 ) const = 0;
	virtual SfxPoolItem*    Create(SvStream &, USHORT nVersion) const = 0;
	virtual SvStream&       Store(SvStream &, USHORT nVer) const;

	const SfxItemSet&       GetItemSet() const
				{ return *pSet; }
	SfxItemSet&             GetItemSet()
				{ return *pSet; }
};

class SfxInvalidItem: public SfxPoolItem
{
friend class SfxItemSet;

	const SfxPoolItem*      pDefaultItem;

private:
				TYPEINFO();
				SfxInvalidItem( USHORT nWhich, const SfxPoolItem &rDefault );
				SfxInvalidItem( const SfxInvalidItem& );
	virtual                 ~SfxInvalidItem();

public:
	virtual int             operator==( const SfxPoolItem& ) const;

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
						XubString &rText,
	                                const IntlWrapper * = 0 ) const;
	const SfxPoolItem*      GetDefaultItem() const { return pDefaultItem; }

	virtual SfxPoolItem*    Clone( SfxItemPool *pPool = 0 ) const;
	virtual SfxPoolItem*    Create(SvStream &, USHORT nVersion) const;
	virtual SvStream&       Store(SvStream &, USHORT nVer ) const;
};

class SfxItemHandle
{
	USHORT      *pRef;
	SfxPoolItem *pItem;
public:
	SfxItemHandle( SfxPoolItem& );
	SfxItemHandle( const SfxItemHandle& );
	~SfxItemHandle();

	const SfxItemHandle &operator=(const SfxItemHandle &);
	const SfxPoolItem &GetItem() const { return *pItem; }
};

DECL_PTRHINT(SfxPoolItemHint,SfxPoolItem);

class SfxItemChangedHint: public SfxHint
{
	const SfxPoolItem& 	_rOld;
	const SfxPoolItem& 	_rNew;

public:
			TYPEINFO(); \
						SfxItemChangedHint( const SfxPoolItem &rOld,
								const SfxPoolItem &rNew )
			:	_rOld( rOld ),
				_rNew( rNew )
			{}

	const SfxPoolItem&	GetOldItem() const { return _rOld; }
	const SfxPoolItem&	GetNewItem() const { return _rNew; }
};

#endif 

