

#ifndef _RULER_HXX
#define _RULER_HXX

#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif

#ifndef _WINDOW_HXX
#include <vcl/window.hxx>
#endif
#ifndef _VIRDEV_HXX
#include <vcl/virdev.hxx>
#endif
#ifndef _SV_FIELD_HXX
#include <vcl/field.hxx>
#endif

class MouseEvent;
class TrackingEvent;
class DataChangedEvent;

#define WB_EXTRAFIELD		((WinBits)0x00004000)
#define WB_STDRULER 		WB_HORZ

struct ImplRulerHitTest;

enum RulerType { RULER_TYPE_DONTKNOW, RULER_TYPE_OUTSIDE,
	 RULER_TYPE_MARGIN1, RULER_TYPE_MARGIN2,
	 RULER_TYPE_BORDER, RULER_TYPE_INDENT, RULER_TYPE_TAB };

enum RulerExtra { RULER_EXTRA_DONTKNOW,
	  RULER_EXTRA_NULLOFFSET, RULER_EXTRA_TAB };

#define RULER_STYLE_HIGHLIGHT	((USHORT)0x8000)
#define RULER_STYLE_DONTKNOW	((USHORT)0x4000)
#define RULER_STYLE_INVISIBLE	((USHORT)0x2000)

#define RULER_DRAGSIZE_MOVE 	0
#define RULER_DRAGSIZE_1		1
#define RULER_DRAGSIZE_2		2

#define RULER_MOUSE_BORDERMOVE	5
#define RULER_MOUSE_BORDERWIDTH 5
#define RULER_MOUSE_TABLEWIDTH	1
#define RULER_MOUSE_MARGINWIDTH 3

#define RULER_SCROLL_NO 		0
#define RULER_SCROLL_1			1
#define RULER_SCROLL_2			2

#define RULER_MARGIN_SIZEABLE	((USHORT)0x0001)

#define RULER_BORDER_SIZEABLE	((USHORT)0x0001)
#define RULER_BORDER_MOVEABLE	((USHORT)0x0002)
#define RULER_BORDER_VARIABLE	((USHORT)0x0004)
#define RULER_BORDER_TABLE		((USHORT)0x0008)
#define RULER_BORDER_SNAP		((USHORT)0x0010)

struct RulerBorder
{
	long	nPos;
	long	nWidth;
	USHORT	nStyle;
};

#define RULER_INDENT_TOP		((USHORT)0x0000)
#define RULER_INDENT_BOTTOM 	((USHORT)0x0001)
#define RULER_INDENT_STYLE		((USHORT)0x000F)

struct RulerIndent
{
	long	nPos;
	USHORT	nStyle;
};

#define RULER_TAB_LEFT			((USHORT)0x0000)
#define RULER_TAB_RIGHT 		((USHORT)0x0001)
#define RULER_TAB_DECIMAL		((USHORT)0x0002)
#define RULER_TAB_CENTER		((USHORT)0x0003)
#define RULER_TAB_DEFAULT		((USHORT)0x0004)
#define RULER_TAB_STYLE 		((USHORT)0x000F)

struct RulerTab
{
	long	nPos;
	USHORT	nStyle;
};

#define RULER_TAB_WIDTH 		7
#define RULER_TAB_HEIGHT		6

struct RulerLine
{
	long	nPos;
	USHORT	nStyle;
};

struct RulerArrow
{
	long	nPos;
	long	nWidth;
	long	nLogWidth;
	USHORT	nStyle;
};

class ImplRulerData
{
	friend				class Ruler;

private:
	RulerLine*			pLines;
	RulerArrow* 		pArrows;
	RulerBorder*		pBorders;
	RulerIndent*		pIndents;
	RulerTab*			pTabs;
	long				nNullVirOff;
	long				nRulVirOff;
	long				nRulWidth;
	long				nPageOff;
	long				nPageWidth;
	long				nNullOff;
	long				nMargin1;
	long				nMargin2;
	USHORT				nLines;
	USHORT				nArrows;
	USHORT				nBorders;
	USHORT				nIndents;
	USHORT				nTabs;
	USHORT				nMargin1Style;
	USHORT				nMargin2Style;
	BOOL				bAutoPageWidth;
	BOOL				bDummyAlign;

#ifdef _SV_RULER_CXX
public:
			ImplRulerData();
			~ImplRulerData();
	ImplRulerData&		operator=( const ImplRulerData& rData );
#endif
};

class Ruler : public Window
{
private:
	VirtualDevice		maVirDev;
	MapMode 			maMapMode;
	long				mnBorderOff;
	long				mnWinOff;
	long				mnWinWidth;
	long				mnWidth;
	long				mnHeight;
	long				mnVirOff;
	long				mnVirWidth;
	long				mnVirHeight;
	long				mnBorderWidth;
	long				mnStartDragPos;
	long				mnDragPos;
	ULONG				mnUpdateEvtId;
	ImplRulerData*		mpData;
	ImplRulerData		maData;
	ImplRulerData		maDragData;
	Rectangle			maExtraRect;
	WinBits 			mnWinStyle;
	USHORT				mnUnitIndex;
	USHORT				mnDragAryPos;
	USHORT				mnDragSize;
	USHORT				mnDragScroll;
	USHORT				mnDragModifier;
	USHORT				mnExtraStyle;
	USHORT				mnExtraClicks;
	USHORT				mnExtraModifier;
	RulerExtra			meExtraType;
	RulerType			meDragType;
	MapUnit 			meSourceUnit;
	FieldUnit			meUnit;
	Fraction			maZoom;
	BOOL				mbCalc;
	BOOL				mbFormat;
	BOOL				mbDrag;
	BOOL				mbDragDelete;
	BOOL				mbDragCanceled;
	BOOL				mbAutoWinWidth;
	BOOL				mbActive;
	BYTE				mnUpdateFlags;
	Link				maStartDragHdl;
	Link				maDragHdl;
	Link				maEndDragHdl;
	Link				maClickHdl;
	Link				maDoubleClickHdl;
	Link				maExtraDownHdl;

#ifdef _SV_RULER_CXX
	void				ImplVDrawLine( long nX1, long nY1, long nX2, long nY2 );
	void				ImplVDrawRect( long nX1, long nY1, long nX2, long nY2 );
	void				ImplVDrawText( long nX, long nY, const String& rText );

	void				ImplDrawTicks( long nMin, long nMax, long nStart, long nCenter );
	void				ImplDrawArrows( long nCenter );
	void				ImplDrawBorders( long nMin, long nMax, long nVirTop, long nVirBottom );
	void				ImplDrawIndent( const Polygon& rPoly, USHORT nStyle );
	void				ImplDrawIndents( long nMin, long nMax, long nVirTop, long nVirBottom );
	void				ImplDrawTab( OutputDevice* pDevice, const Point& rPos, USHORT nStyle );
	void				ImplDrawTabs( long nMin, long nMax, long nVirBottom );
	void				ImplInit( WinBits nWinBits );
	void				ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	void				ImplCalc();
	void				ImplFormat();
	void				ImplInitExtraField( BOOL bUpdate );
	void				ImplInvertLines( BOOL bErase = FALSE );
	void				ImplDraw();
	void				ImplDrawExtra( BOOL bPaint = FALSE );
	void				ImplUpdate( BOOL bMustCalc = FALSE );
	BOOL				ImplHitTest( const Point& rPos, ImplRulerHitTest* pHitTest ) const;
	BOOL				ImplDocHitTest( const Point& rPos, RulerType eDragType, ImplRulerHitTest* pHitTest ) const;
	BOOL				ImplStartDrag( ImplRulerHitTest* pHitTest, USHORT nModifier );
	void				ImplDrag( const Point& rPos );
	void				ImplEndDrag();
			DECL_LINK( ImplUpdateHdl, void* );
#endif

public:
			Ruler( Window* pParent, WinBits nWinStyle = WB_STDRULER );
	virtual 			~Ruler();

	virtual void		MouseButtonDown( const MouseEvent& rMEvt );
	virtual void		MouseMove( const MouseEvent& rMEvt );
	virtual void		Tracking( const TrackingEvent& rTEvt );
	virtual void		Paint( const Rectangle& rRect );
	virtual void		Resize();
	virtual void		StateChanged( StateChangedType nStateChange );
	virtual void		DataChanged( const DataChangedEvent& rDCEvt );

	virtual long		StartDrag();
	virtual void		Drag();
	virtual void		EndDrag();
	virtual void		Click();
	virtual void		DoubleClick();
	virtual void		ExtraDown();

	void				Activate();
	void				Deactivate();
	BOOL				IsActive() const { return mbActive; }

	void				SetWinPos( long nOff = 0, long nWidth = 0 );
	long				GetWinOffset() const { return mnWinOff; }
	long				GetWinWidth() const { return mnWinWidth; }
	void				SetPagePos( long nOff = 0, long nWidth = 0 );
	long				GetPageOffset() const { return mpData->nPageOff; }
	long				GetPageWidth() const { return mpData->nPageWidth; }
	void				SetBorderPos( long nOff = 0 );
	long				GetBorderOffset() const { return mnBorderOff; }
	Rectangle			GetExtraRect() const { return maExtraRect; }

	void				SetUnit( FieldUnit eNewUnit );
	FieldUnit			GetUnit() const { return meUnit; }
	void				SetZoom( const Fraction& rNewZoom );
	Fraction			GetZoom() const { return maZoom; }

	void				SetSourceUnit( MapUnit eNewUnit ) { meSourceUnit = eNewUnit; }
	MapUnit 			GetSourceUnit() const { return meSourceUnit; }

	void				SetExtraType( RulerExtra eNewExtraType, USHORT nStyle = 0 );
	RulerExtra			GetExtraType() const { return meExtraType; }
	USHORT				GetExtraStyle()  const { return mnExtraStyle; }
	USHORT				GetExtraClicks() const { return mnExtraClicks; }
	USHORT				GetExtraModifier() const { return mnExtraModifier; }

	BOOL				StartDocDrag( const MouseEvent& rMEvt,
						  RulerType eDragType = RULER_TYPE_DONTKNOW );
	RulerType			GetDocType( const Point& rPos,
						RulerType eDragType = RULER_TYPE_DONTKNOW,
						USHORT* pAryPos = NULL ) const;
	RulerType			GetDragType() const { return meDragType; }
	long				GetDragPos() const { return mnDragPos; }
	USHORT				GetDragAryPos() const { return mnDragAryPos; }
	USHORT				GetDragSize() const { return mnDragSize; }
	BOOL				IsDragDelete() const { return mbDragDelete; }
	BOOL				IsDragCanceled() const { return mbDragCanceled; }
	USHORT				GetDragScroll() const { return mnDragScroll; }
	USHORT				GetDragModifier() const { return mnDragModifier; }
	BOOL				IsDrag() const { return mbDrag; }
	void				CancelDrag();
	long				GetClickPos() const { return mnDragPos; }
	RulerType			GetClickType() const { return meDragType; }
	USHORT				GetClickAryPos() const { return mnDragAryPos; }
	RulerType			GetType( const Point& rPos,
					 USHORT* pAryPos = NULL ) const;

	void				SetNullOffset( long nPos );
	long				GetNullOffset() const { return mpData->nNullOff; }
	void				SetMargin1() { SetMargin1( 0, RULER_STYLE_INVISIBLE ); }
	void				SetMargin1( long nPos, USHORT nMarginStyle = RULER_MARGIN_SIZEABLE );
	long				GetMargin1() const { return mpData->nMargin1; }
	USHORT				GetMargin1Style() const { return mpData->nMargin1Style; }
	void				SetMargin2() { SetMargin2( 0, RULER_STYLE_INVISIBLE ); }
	void				SetMargin2( long nPos, USHORT nMarginStyle = RULER_MARGIN_SIZEABLE );
	long				GetMargin2() const { return mpData->nMargin2; }
	USHORT				GetMargin2Style() const { return mpData->nMargin2Style; }

	void				SetLines( USHORT n = 0, const RulerLine* pLineAry = NULL );
	USHORT				GetLineCount() const { return mpData->nLines; }
	const RulerLine*	GetLines() const { return mpData->pLines; }

	void				SetArrows( USHORT n = 0, const RulerArrow* pArrowAry = NULL );
	USHORT				GetArrowCount() const { return mpData->nArrows; }
	const RulerArrow*	GetArrows() const { return mpData->pArrows; }

	void				SetBorders( USHORT n = 0, const RulerBorder* pBrdAry = NULL );
	USHORT				GetBorderCount() const { return mpData->nBorders; }
	const RulerBorder*	GetBorders() const { return mpData->pBorders; }

	void				SetIndents( USHORT n = 0, const RulerIndent* pIndentAry = NULL );
	USHORT				GetIndentCount() const { return mpData->nIndents; }
	const RulerIndent*	GetIndents() const { return mpData->pIndents; }

	void				SetTabs( USHORT n = 0, const RulerTab* pTabAry = NULL );
	USHORT				GetTabCount() const { return mpData->nTabs; }
	const RulerTab* 	GetTabs() const { return mpData->pTabs; }

	static void 		DrawTab( OutputDevice* pDevice,
					 const Point& rPos, USHORT nStyle );

	void				SetStyle( WinBits nStyle );
	WinBits 			GetStyle() const { return mnWinStyle; }

	void				SetStartDragHdl( const Link& rLink ) { maStartDragHdl = rLink; }
	const Link& 		GetStartDragHdl() const { return maStartDragHdl; }
	void				SetDragHdl( const Link& rLink ) { maDragHdl = rLink; }
	const Link& 		GetDragHdl() const { return maDragHdl; }
	void				SetEndDragHdl( const Link& rLink ) { maEndDragHdl = rLink; }
	const Link& 		GetEndDragHdl() const { return maEndDragHdl; }
	void				SetClickHdl( const Link& rLink ) { maClickHdl = rLink; }
	const Link& 		GetClickHdl() const { return maClickHdl; }
	void				SetDoubleClickHdl( const Link& rLink ) { maDoubleClickHdl = rLink; }
	const Link& 		GetDoubleClickHdl() const { return maDoubleClickHdl; }
	void				SetExtraDownHdl( const Link& rLink ) { maExtraDownHdl = rLink; }
	const Link& 		GetExtraDownHdl() const { return maExtraDownHdl; }
};

#endif	
