

#ifndef _SCRWIN_HXX
#define _SCRWIN_HXX

#ifndef _SCRBAR_HXX 
#include <vcl/scrbar.hxx>
#endif

class DataChangedEvent;

typedef USHORT ScrollableWindowFlags;

#define SCRWIN_THUMBDRAGGING 1
#define SCRWIN_VCENTER		 2
#define SCRWIN_HCENTER		 4
#define SCRWIN_DEFAULT		 (SCRWIN_THUMBDRAGGING | SCRWIN_VCENTER | SCRWIN_HCENTER)

class ScrollableWindow: public Window
{
private:
	Point			aPixOffset; 		
	Size			aTotPixSz;			
	long			nLinePixH;			
	long			nColumnPixW;

	ScrollBar		aVScroll;			
	ScrollBar		aHScroll;
	ScrollBarBox	aCornerWin; 		
	BOOL			bScrolling:1,		
		bHandleDragging:1,	
		bHCenter:1,
		bVCenter:1;

#ifdef _SVT_SCRWIN_CXX
	void			ImpInitialize( ScrollableWindowFlags nFlags );
	DECL_LINK( ScrollHdl, ScrollBar * );
	DECL_LINK( EndScrollHdl, ScrollBar * );
#endif

public:
		ScrollableWindow( Window* pParent, WinBits nBits = 0,
						  ScrollableWindowFlags = SCRWIN_DEFAULT );
		ScrollableWindow( Window* pParent, const ResId& rId,
						  ScrollableWindowFlags = SCRWIN_DEFAULT );

	virtual void	Resize();
	virtual void	Command( const CommandEvent& rCEvt );
	virtual void	DataChanged( const DataChangedEvent& rDEvt );

	virtual void	StartScroll();
	virtual void	EndScroll( long nDeltaX, long nDeltaY );

	void			SetMapMode( const MapMode& rNewMapMode );
	MapMode 		GetMapMode() const;

	void			SetTotalSize( const Size& rNewSize );
	Size			GetTotalSize() { return PixelToLogic( aTotPixSz ); }

	void			SetVisibleSize( const Size& rNewSize );
	BOOL			MakeVisible( const Rectangle& rTarget, BOOL bSloppy = FALSE );
	Rectangle		GetVisibleArea() const;

	void			SetLineSize( ULONG nHorz, ULONG nVert );
	void			Scroll( long nDeltaX, long nDeltaY );
	void			Scroll( long nDeltaX, long nDeltaY,
				const Rectangle& rRect );
	void			ScrollLines( long nLinesX, long nLinesY );
	void			ScrollPages( long nPagesX, ULONG nOverlapX,
					 long nPagesY, ULONG nOverlapY );

private:
	void			SetOutputSizePixel( const Size& rSize );
	Size			GetOutputSizePixel() const;
	Size			GetOutputSize() const;
};

#endif
