

#ifndef _SVT_SDBSQL_HXX
#define _SVT_SDBSQL_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

#ifdef MACOSX
#ifndef __SGI_STL_ALGORITHM
#include <algorithm>
#endif
#endif

#ifndef _DEBUG_HXX 
#include <tools/debug.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace sdbc
			{
	class XDatabaseMetaData;
			}
			namespace beans
			{
	class XPropertySet;
			}
			namespace data
			{
	class XDatabaseDescriptor;
			}
			namespace util
			{
	class XNumberFormatter;
			}
		}
	}
}

class International;

enum SdbSqlNodeType	{SDB_SQL_RULE, SDB_SQL_LISTRULE, SDB_SQL_COMMALISTRULE,
		 SDB_SQL_KEYWORD, SDB_SQL_COMPARISON, SDB_SQL_NAME,
		 SDB_SQL_STRING,	SDB_SQL_INTNUM, SDB_SQL_APPROXNUM,
		 SDB_SQL_EQUAL,SDB_SQL_LESS,SDB_SQL_GREAT,SDB_SQL_LESSEQ,SDB_SQL_GREATEQ,SDB_SQL_NOTEQUAL,
		 SDB_SQL_PUNCTUATION, SDB_SQL_AMMSC, SDB_SQL_ACCESS_DATE,SDB_SQL_DATE};

class SdbSqlParseNode
{
	friend class SdbSqlParser;

	::std::vector<SdbSqlParseNode*> m_aChilds;
	SdbSqlParseNode* 				m_pParent;		
	String 							m_aNodeValue;	

	SdbSqlNodeType 					m_eNodeType;	
	sal_uInt32						m_nNodeID; 		

protected:
	struct SQLParseNodeParameter
	{
		const International& rIntl;
		const String	aIdentifierQuote;
		const String	aCatalogSeparator;	
		::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > xFormatter;
		::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >  xField;		  		
		char				cDecSep ;
		sal_Bool			bQuote : 1;
		sal_Bool			bInternational : 1;
		sal_Bool			bPredicate : 1;

		SQLParseNodeParameter(const String& _rIdentifierQuote,
				  const String& _rCatalogSep,
				  const International& _rIntl,
				  sal_Bool _bIntl = sal_False, 
				  sal_Bool _bQuote= sal_True, 
				  char _cDecSep = '.',
				  const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField = ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > (),
				  const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & _xFormatter = ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > (),
				  sal_Bool _bPredicate = sal_False);
	};							

public:
	enum Rule
	{
		select_statement = 0,
		table_exp,
		table_ref_commalist,
		table_ref,
		table_name,
		opt_column_commalist,
		column_commalist,
		column_ref_commalist,
		column_ref,
		opt_order_by_clause,
		ordering_spec_commalist,
		ordering_spec,
		opt_asc_desc,
		where_clause,
		opt_where_clause,
		search_condition,
		comparison_predicate,
		between_predicate,
		like_predicate,
		opt_escape,
		test_for_null,
		scalar_exp_commalist,
		scalar_exp,
		parameter_ref,
		parameter,
		general_set_fct,
		range_variable,
		column,
		delete_statement_positioned,
		delete_statement_searched,
		update_statement_positioned,
		update_statement_searched,
		assignment_commalist,
		assignment,
		values_or_query_spec,
		insert_statement,
		insert_atom_commalist,
		insert_atom,
		predicate_check,
		from_clause,
		qualified_join,
		cross_union,
		select_sublist,
		derived_column,
		column_val,
		set_fct_spec,
		boolean_term,
		boolean_primary,
		num_value_exp,
		join_type,
		position_exp,
		extract_exp,
		length_exp,
		char_value_fct,
		odbc_call_spec,
		in_predicate,
		existence_test,
		unique_test,
		all_or_any_predicate,
		named_columns_join,
		join_condition,
		joined_table,
		boolean_factor,
		not,
		boolean_test,
		manipulative_statement,
		subquery,
		value_exp_commalist,
		odbc_fct_spec,
		union_statement,
		outer_join_type,
		char_value_exp,
		term,
		value_exp_primary,
		rule_count						
	};

	SdbSqlParseNode(const sal_Unicode* pNewValue,
		SdbSqlNodeType _eNodeType,
		sal_uInt32 _nNodeID = 0);
	SdbSqlParseNode(const String &_rNewValue,
					 SdbSqlNodeType eNewNodeType,
					 sal_uInt32 nNewNodeID=0);

	SdbSqlParseNode(const SdbSqlParseNode& rParseNode);
	SdbSqlParseNode & operator=(const SdbSqlParseNode& rParseNode);

	sal_Bool operator==(SdbSqlParseNode& rParseNode) const;

	virtual ~SdbSqlParseNode();

	SdbSqlParseNode * Parent() const {return m_pParent;};

	void SetParent(SdbSqlParseNode* pParseNode) {m_pParent = pParseNode;};

	sal_uInt32 Count() const {return m_aChilds.size();};
	inline SdbSqlParseNode* GetChild(sal_uInt32 nPos) const;		

	void Insert(SdbSqlParseNode* pNewSubTree, sal_uInt32 nPos);
	void Insert(SdbSqlParseNode* pNewSubTree, SdbSqlParseNode* pOldSubTree);

	SdbSqlParseNode* Remove(sal_uInt32 nPos);
	SdbSqlParseNode* Remove(SdbSqlParseNode* pSubTree);

	SdbSqlParseNode* Replace( SdbSqlParseNode* pNewSubTree, sal_uInt32 nPos);
	SdbSqlParseNode* Replace( SdbSqlParseNode* pNewSubTree, SdbSqlParseNode* pOldSubTree);

	void Replace(const String& rTableAlias,const String& rColumnName);

	void Append(SdbSqlParseNode* pNewSubTree);

	void ParseNodeToStr(String& rString, 
				const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,						
				sal_Bool _bIntl = sal_False, 
				sal_Bool _bQuote= sal_True) const;

	void ParseNodeToPredicateStr(String& rString, 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,														 
						 const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
						 const International& rIntl,
						 const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField = ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > ()) const;

	virtual void ParseNodeToStr(String& rString, 
			const ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > & xMeta,						
			sal_Bool _bIntl = sal_False, 
			sal_Bool _bQuote= sal_True) const;

	void ParseNodeToPredicateStr(String& rString, 
					 const ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > & xMeta,														 
					 const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
					 const International& rIntl,
					 const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField = ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > ()) const;

	SdbSqlParseNode* FindRule(SdbSqlParseNode::Rule eRule) const;

	void ShowParseTree(String& rString, sal_uInt32 nLevel=0);

	SdbSqlNodeType GetNodeType() const {return m_eNodeType;};

	sal_uInt32 GetRuleID() const {return m_nNodeID;}

	sal_uInt32 GetTokenID() const {return m_nNodeID;}

	sal_Bool IsRule() const 
		{ return (m_eNodeType == SDB_SQL_RULE) || (m_eNodeType == SDB_SQL_LISTRULE)
			|| (m_eNodeType == SDB_SQL_COMMALISTRULE);}

	sal_Bool IsToken() const {return !IsRule();} 

	const String& TokenValue() const {return m_aNodeValue;}

	void SetTokenValue(const String& rString) {	if (IsToken()) m_aNodeValue = rString;}

	sal_Bool IsLeaf() const {return m_aChilds.empty();}

	static void NegateSearchCondition(SdbSqlParseNode*& pSearchCondition,sal_Bool bNegate=sal_False);

	static void DisjunctiveNormalForm(SdbSqlParseNode*& pSearchCondition);

	static void Absorptions(SdbSqlParseNode*& pSearchCondition);

	static void EraseBraces(SdbSqlParseNode*& pSearchCondition);

	static void Compress(SdbSqlParseNode*& pSearchCondition);

protected:

	void ParseNodeToStr(String& rString, 
				const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData > & xMeta,						
				const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
				const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField,
				const International& rIntl,
				sal_Bool _bIntl, 
				sal_Bool _bQuote, 
				char _cDecSep,						
				sal_Bool bPredicate) const;

	virtual void ParseNodeToStr(String& rString, 
				const ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > & xMeta,						
				const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
				const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & _xField,
				const International& rIntl,
				sal_Bool _bIntl, 
				sal_Bool _bQuote, 
				char _cDecSep,						
				sal_Bool bPredicate) const;

	virtual void ParseNodeToStr(String& rString, 
			const SQLParseNodeParameter& rParam) const;				
private:
	void LikeNodeToStr(String& rString, 
		   const SQLParseNodeParameter& rParam) const;				
	void TableRangeNodeToStr(String& rString, 
				 const SQLParseNodeParameter& rParam) const;				
	sal_Bool AddDateValue(String& rString, const SQLParseNodeParameter& rParam) const;
	String ConvertDateTimeString(const SQLParseNodeParameter& rParam, const String& rString) const;
	String ConvertDateString(const SQLParseNodeParameter& rParam, const String& rString) const;
	String ConvertTimeString(const SQLParseNodeParameter& rParam, const String& rString) const;	
};

inline SdbSqlParseNode* SdbSqlParseNode::GetChild(sal_uInt32 nPos) const 
{
	DBG_ASSERT(nPos < m_aChilds.size(), "Invalid Position");
	return m_aChilds[nPos];
}	

#define SDB_ISRULE(pParseNode, eRule) 	((pParseNode)->IsRule() && (pParseNode)->GetRuleID() == SdbSqlParser::RuleID(SdbSqlParseNode::##eRule))
#define SDB_ISTOKEN(pParseNode, aTokenName) ((pParseNode)->IsToken() && (pParseNode)->GetTokenID() == SdbSqlParser::StrToTokenID(aTokenName))
#define SDB_ISPUNCTUATION(pParseNode, aString) ((pParseNode)->GetNodeType() == SDB_SQL_PUNCTUATION && (pParseNode)->TokenValue().EqualsAscii(aString))

#endif

