

#ifndef _SVPARSER_HXX
#define _SVPARSER_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif
#ifndef _RTL_TEXTENC_H
#include <rtl/textenc.h>
#endif

struct SvParser_Impl;
class SvStream;
class SvUShorts;

enum SvParserState
{
	SVPAR_ACCEPTED = 0,
	SVPAR_NOTSTARTED,
	SVPAR_WORKING,
	SVPAR_PENDING,
	SVPAR_WAITFORDATA,
	SVPAR_ERROR
};

class SvParser : public SvRefBase
{
	DECL_STATIC_LINK( SvParser, NewDataRead, void* );

protected:
	SvStream&		rInput;
	String 	  		aToken;				
	ULONG	  		nlLineNr;			
	ULONG	  		nlLinePos;			

	SvParser_Impl	*pImplData;			
	long           	nTokenValue;		
	SvParserState	eState;				

	rtl_TextEncoding	eSrcEnc;		

	sal_Unicode			nNextCh;    		

	BOOL			bDownloadingFile : 1;

	BOOL			bUCS2BSrcEnc : 1;	
	BOOL			bSwitchToUCS2 : 1;	

	BOOL			bRTF_InTextRead : 1;  

	struct TokenStackType
	{
		String	sToken;
		long 	nTokenValue;
		int 	nTokenId;

		inline TokenStackType() { nTokenId = 0; }
		inline ~TokenStackType() { }
	};

	int SkipToken( short nCnt = -1 );		
	TokenStackType* GetStackPtr( short nCnt );
	inline BYTE GetStackPos() const;

	int GetNextToken();
	virtual int _GetNextToken() = 0;

	virtual void NextToken( int nToken );

	virtual	~SvParser();

private:
	TokenStackType* pTokenStack;
	TokenStackType *pTokenStackPos;
	BYTE nTokenStackSize, nTokenStackPos;

public:

	SvParser( SvStream& rIn, BYTE nStackSize = 3 );

	virtual	 SvParserState CallParser() = 0;	

	inline SvParserState GetStatus() const	{ return eState; }	

	inline ULONG	GetLineNr() const		{ return nlLineNr; }
	inline ULONG	GetLinePos() const		{ return nlLinePos; }
	inline ULONG	IncLineNr() 			{ return ++nlLineNr; }
	inline ULONG	IncLinePos()			{ return ++nlLinePos; }
	inline ULONG	SetLineNr( ULONG nlNum );			
	inline ULONG	SetLinePos( ULONG nlPos );			

	sal_Unicode	GetNextChar();

	inline int 	IsParserWorking() const	{ return SVPAR_WORKING == eState; }

	Link GetAsynchCallLink() const
		{ return STATIC_LINK( this, SvParser, NewDataRead ); }

	long CallAsyncCallLink() { return NewDataRead( this, 0 ); }

	 void SaveState( int nToken );
	 void RestoreState();
	virtual void Continue( int nToken );

	inline void SetDownloadingFile( BOOL bSet ) { bDownloadingFile = bSet; }
	inline BOOL IsDownloadingFile() const { return bDownloadingFile; }

	void SetSrcEncoding( rtl_TextEncoding eSrcEnc );
	rtl_TextEncoding GetSrcEncoding() const { return eSrcEnc; }

	void SetSrcUCS2BEncoding( BOOL bSet ) { bUCS2BSrcEnc = bSet; }
	BOOL IsSrcUCS2BEncoding() const { return bUCS2BSrcEnc; }

	void SetSwitchToUCS2( BOOL bSet ) { bSwitchToUCS2 = bSet; }
	BOOL IsSwitchToUCS2() const { return bSwitchToUCS2; }

	inline USHORT GetCharSize() const;

	int GetSaveToken() const;

	static void BuildWhichTbl( SvUShorts &rWhichMap,
				   USHORT *pWhichIds,
				   USHORT nWhichIds );
};

#ifndef GOODIES_DECL_SVPARSER_DEFINED
#define GOODIES_DECL_SVPARSER_DEFINED
SV_DECL_REF(SvParser)
#endif
SV_IMPL_REF(SvParser)

inline ULONG SvParser::SetLineNr( ULONG nlNum )
{	ULONG nlOld = nlLineNr; nlLineNr = nlNum; return nlOld; }

inline ULONG SvParser::SetLinePos( ULONG nlPos )
{	ULONG nlOld = nlLinePos; nlLinePos = nlPos; return nlOld; }

inline BYTE SvParser::GetStackPos() const
{	return nTokenStackPos; }

inline USHORT SvParser::GetCharSize() const
{
	return (RTL_TEXTENCODING_UCS2 == eSrcEnc) ? 2 : 1;
}

#endif

