
#ifndef _SVTABBX_HXX
#define _SVTABBX_HXX

#include <svtools/svtreebx.hxx>

enum SvTabJustify
{
	AdjustRight = SV_LBOXTAB_ADJUST_RIGHT,
	AdjustLeft = SV_LBOXTAB_ADJUST_LEFT,
	AdjustCenter = SV_LBOXTAB_ADJUST_CENTER,
	AdjustNumeric = SV_LBOXTAB_ADJUST_NUMERIC
};

class SvTabListBox : public SvTreeListBox
{
private:
	SvLBoxTab*					pTabList;
	USHORT						nTabCount;
	XubString					aCurEntry;
	ULONG						nDummy1;
	ULONG						nDummy2;

protected:
	static const xub_Unicode*	GetToken( const xub_Unicode* pPtr, USHORT& rLen );
	virtual void				SetTabs();
	virtual void				InitEntry( SvLBoxEntry*, const XubString&, const Image&, const Image& );
	SvLBoxEntry*				pViewParent;

public:
	SvTabListBox( Window* pParent, WinBits = WB_BORDER );
	SvTabListBox( Window* pParent, const ResId& );
	~SvTabListBox();
	void			SetTabs( long* pTabs, MapUnit = MAP_APPFONT );
	USHORT			TabCount() const { return (USHORT)nTabCount; }
	long 			GetTab( USHORT nTab ) const;
	void			SetTab( USHORT nTab, long nValue, MapUnit = MAP_APPFONT );
	long			GetLogicTab( USHORT nTab );

	SvLBoxEntry*	InsertEntry( const XubString&, ULONG nPos = LIST_APPEND,
					 USHORT nCol = 0xffff, void* pUserData = NULL );
	SvLBoxEntry*	InsertEntry( const XubString&, SvLBoxEntry* pParent,
					 ULONG nPos, USHORT nCol, void* pUserData = NULL );
	SvLBoxEntry*	InsertEntry( const XubString&, const Image& rExpandedEntryBmp,
					 const Image& rCollapsedEntryBmp, SvLBoxEntry* pParent = NULL,
					 ULONG nPos = LIST_APPEND, USHORT nCol = 0xffff, void* pUserData = NULL );

	XubString		GetEntryText( ULONG nPos, USHORT nCol = 0xffff ) const;
	XubString		GetEntryText( SvLBoxEntry*, USHORT nCol = 0xffff ) const;
	void			SetEntryText( const XubString&, ULONG, USHORT nCol=0xffff );
	void			SetEntryText(const XubString&,SvLBoxEntry*,USHORT nCol=0xffff);
	ULONG			GetEntryPos( const XubString&, USHORT nCol = 0xffff );

	virtual void	Resize();
	void			SetTabJustify( USHORT nTab, SvTabJustify );
	SvTabJustify	GetTabJustify( USHORT nTab ) const;
};

inline long SvTabListBox::GetTab( USHORT nTab ) const
{
	DBG_ASSERT( nTab < nTabCount, "GetTabPos:Invalid Tab" );
	return pTabList[nTab].GetPos();
}

class HeaderBar;

class SvHeaderTabListBox : public SvTabListBox
{
private:
	sal_Bool			mbFirstPaint;
	HeaderBar*			mpHeaderBar;

	DECL_LINK( ScrollHdl_Impl, SvTabListBox* );

public:
	SvHeaderTabListBox( Window* pParent, WinBits nBits );
	~SvHeaderTabListBox();

	virtual void	Paint( const Rectangle& );

	void			InitHeaderBar( HeaderBar* pHeaderBar );
};

#endif 

