
#ifndef _SVTREEBOX_HXX
#define _SVTREEBOX_HXX

#ifndef _SVLBOX_HXX
#include <svtools/svlbox.hxx>
#endif
#ifndef _SVLBOXITM_HXX
#include <svtools/svlbitm.hxx>
#endif

class SvImpLBox;
class SvInplaceEdit;
class TabBar;

#define SV_TAB_BORDER 8

#define WB_HASBUTTONSATROOT       ((WinBits)0x0800)
#define WB_NOINITIALSELECTION		WB_DROPDOWN
#define WB_HIDESELECTION			WB_NOHIDESELECTION
#define WB_FORCE_MAKEVISIBLE		WB_SPIN

#define SV_LISTBOX_ID_TREEBOX 1   
#define SV_ENTRYHEIGHTOFFS_PIXEL 2

#define TREEFLAG_CHKBTN			0x0001
#define TREEFLAG_USESEL			0x0002
#define TREEFLAG_MANINS			0x0004
#define TREEFLAG_RECALCTABS		0x0008

class SvTreeListBox : public SvLBox
{
	friend class SvImpLBox;
	friend class ImpLBSelEng;

	SvImpLBox*		pImp;
	Image           aCollapsedEntryBmp;
	Image           aExpandedEntryBmp;
	Link			aCheckButtonHdl;
	Link			aScrolledHdl;
	Accelerator		aInpEditAcc;
	Image			aPrevInsertedExpBmp;
	Image			aPrevInsertedColBmp;
	Image			aCurInsertedExpBmp;
	Image			aCurInsertedColBmp;

	short			nContextBmpWidthMax;
	USHORT			nFirstSelTab, nLastSelTab;
	short			nEntryHeight;
	short			nEntryHeightOffs;
	short  			nIndent;
	short			nFocusWidth;
	USHORT			aContextBmpMode;

#ifdef _SVTREEBX_CXX
	DECL_LINK( CheckButtonClick, SvLBoxButtonData * );
	void 			InitAcc();
	DECL_LINK( InpEdActivateHdl, Accelerator * );

	short 			GetHeightOffset( const Image& rBmp, Size& rLogicSize);
	short 			GetHeightOffset( const Font& rFont, Size& rLogicSize);

	void			SetEntryHeight( SvLBoxEntry* pEntry );
	void			AdjustEntryHeight( const Image& rBmp );
	void 			AdjustEntryHeight( const Font& rFont );

	void			ImpEntryInserted( SvLBoxEntry* pEntry );
	long			PaintEntry1( SvLBoxEntry*, long nLine,
					USHORT nTabFlagMask=0xffff,
					BOOL bHasClipRegion=FALSE );

	void			InitTreeView( WinBits nWinStyle );
	SvLBoxItem*		GetItem_Impl( SvLBoxEntry*, long nX, SvLBoxTab** ppTab,
			USHORT nEmptyWidth );

#endif

protected:

	SvLBoxButtonData*	pCheckButtonData;
	USHORT			nTreeFlags;

	SvLBoxEntry*	pEdEntry;
	SvLBoxItem*		pEdItem;
	void			EditItemText( SvLBoxEntry* pEntry, SvLBoxString* pItem,
			const Selection& );
	void			EditedText( const XubString& );
	void			EditingRequest( SvLBoxEntry* pEntry, SvLBoxItem* pItem,
			const Point& rMousePos );

	SvPtrarr 		aTabs;

	virtual void	SetTabs();
	void			SetTabs_Impl();
	void 			AddTab( long nPos,USHORT nFlags=SV_LBOXTAB_ADJUST_LEFT,
			void* pUserData = 0 );
	USHORT			TabCount() const { return aTabs.Count(); }
	SvLBoxTab*		GetFirstDynamicTab() const;
	SvLBoxTab*		GetFirstDynamicTab( USHORT& rTabPos ) const;
	SvLBoxTab*		GetFirstTab( USHORT nFlagMask, USHORT& rTabPos );
	SvLBoxTab*		GetLastTab( USHORT nFlagMask, USHORT& rTabPos );
	SvLBoxTab*		GetTab( SvLBoxEntry*, SvLBoxItem* ) const;
	void			ClearTabList();

	virtual void InitEntry(SvLBoxEntry*,const XubString&,const Image&,const Image&);

	virtual void	NotifyBeginScroll();
	virtual void	NotifyEndScroll();

	virtual void	NotifyScrolling( long nLines );
	virtual	void	NotifyScrolled();
	void			SetScrolledHdl( const Link& rLink ) { aScrolledHdl = rLink; }
	const Link&		GetScrolledHdl() const { return aScrolledHdl; }
	long			GetXOffset() const { return GetMapMode().GetOrigin().X(); }

	virtual void	NotifyInvalidating();

	virtual ULONG	GetAscInsertionPos( SvLBoxEntry*, SvLBoxEntry* pParent );
	virtual ULONG	GetDescInsertionPos( SvLBoxEntry*, SvLBoxEntry* pParent );
	virtual void    Command( const CommandEvent& rCEvt );

	virtual void    RequestHelp( const HelpEvent& rHEvt );
	virtual void	CursorMoved( SvLBoxEntry* pNewCursor );
	virtual void	PreparePaint( SvLBoxEntry* );
#if SUPD > 500
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );
#endif
	void 			InitSettings(BOOL bFont,BOOL bForeground,BOOL bBackground);

public:

	SvTreeListBox( Window* pParent, WinBits nWinStyle=0 );
	SvTreeListBox( Window* pParent, const ResId& rResId );
	~SvTreeListBox();

	void			DisconnectFromModel();

	void 			EnableCheckButton( SvLBoxButtonData* );
	void 			SetNodeBitmaps( const Image& rCollapsedNodeBmp,
			const Image& rExpandedNodeBmp);
	void			SetDontKnowNodeBitmap( const Image& rCollapsedNodeBmp );

#if SUPD > 374

	SvLBoxEntry*	InsertEntry( const XubString& rText, SvLBoxEntry* pParent = 0,
			BOOL bChildsOnDemand = FALSE,
			ULONG nPos=LIST_APPEND, void* pUserData = 0 );

	SvLBoxEntry*	InsertEntry( const XubString& rText,
			const Image& rExpandedEntryBmp,
			const Image& rCollapsedEntryBmp,
			SvLBoxEntry* pParent = 0,
			BOOL bChildsOnDemand = FALSE,
			ULONG nPos = LIST_APPEND, void* pUserData = 0 );

#else
	SvLBoxEntry*	InsertEntry( const XubString& rText, SvLBoxEntry* pParent = 0,
			BOOL bChildsOnDemand = FALSE,
			ULONG nPos=LIST_APPEND );

	SvLBoxEntry*	InsertEntry( const XubString& rText,
			const Image& rExpandedEntryBmp,
			const Image& rCollapsedEntryBmp,
			SvLBoxEntry* pParent = 0,
			BOOL bChildsOnDemand = FALSE,
			ULONG nPos = LIST_APPEND );

	SvLBoxEntry*	InsertEntry( const XubString& rText, SvLBoxEntry* pParent,
			BOOL bChildsOnDemand,
			ULONG nPos, void* pUserData );

	SvLBoxEntry*	InsertEntry( const XubString& rText,
			const Image& rExpandedEntryBmp,
			const Image& rCollapsedEntryBmp,
			SvLBoxEntry* pParent,
			BOOL bChildsOnDemand,
			ULONG nPos, void* pUserData);
#endif

	const Image&    GetDefaultExpandedEntryBmp() const { return aExpandedEntryBmp; }
	const Image&	GetDefaultCollapsedEntryBmp() const { return aCollapsedEntryBmp; }
	void			SetDefaultExpandedEntryBmp( const Image& rBmp);
	void          	SetDefaultCollapsedEntryBmp( const Image& rBmp );

	void			SetCheckButtonState( SvLBoxEntry*, SvButtonState );
	SvButtonState 	GetCheckButtonState( SvLBoxEntry* ) const;

	void            ShowExpandBitmapOnCursor( BOOL );
	BOOL            IsExpandBitmapOnCursor() const 	{ return (BOOL)(aContextBmpMode & SVLISTENTRYFLAG_FOCUSED)!=0; }

	void			SetEntryText(SvLBoxEntry*, const XubString& );
	void			SetExpandedEntryBmp(SvLBoxEntry*, const Image& );
	void			SetCollapsedEntryBmp(SvLBoxEntry*, const Image& );

	XubString 		GetEntryText(SvLBoxEntry*) const;
	const Image&	GetExpandedEntryBmp(SvLBoxEntry*) const;
	const Image&	GetCollapsedEntryBmp(SvLBoxEntry*) const;

	void			SetCheckButtonHdl( const Link& rLink )	{ aCheckButtonHdl=rLink; }
	Link			GetCheckButtonHdl() const { return aCheckButtonHdl; }
	virtual void	CheckButtonHdl();

	virtual SvLBoxEntry*	CloneEntry( SvLBoxEntry* pSource );

	virtual USHORT	IsA();

	void			EnableInplaceEditing( BOOL bEnable );

	void			EditEntry( SvLBoxEntry* pEntry = 0 );
	void			CancelEditing();
	virtual BOOL	EditingEntry( SvLBoxEntry* pEntry, Selection& );
	virtual BOOL	EditedEntry( SvLBoxEntry* pEntry, const XubString& rNewText );
	BOOL			IsEditingActive() const { return SvLBox::IsEditingActive(); }
	void			EndEditing( BOOL bCancel = FALSE ) { SvLBox::EndEditing( bCancel ); }
	BOOL			EditingCanceled() const { return SvLBox::EditingCanceled(); }

	virtual void	RequestingChilds( SvLBoxEntry* pParent );

	virtual void    Paint( const Rectangle& rRect );
	virtual void    MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    MouseButtonUp( const MouseEvent& rMEvt );
	virtual void    MouseMove( const MouseEvent& rMEvt );
	virtual void    KeyInput( const KeyEvent& rKEvt );
	virtual void    Resize();
	virtual void    GetFocus();
	virtual void    LoseFocus();
	void 			SetUpdateMode( BOOL );

	void 			SetUpdateModeFast( BOOL );

	void			SetModel( SvLBoxTreeList* );
	virtual void 	ModelHasCleared();
	virtual void 	ModelHasInserted( SvListEntry* pEntry );
	virtual void 	ModelHasInsertedTree( SvListEntry* pEntry );
	virtual void 	ModelIsMoving(SvListEntry* pSource,
			SvListEntry* pTargetParent, ULONG nChildPos );
	virtual void 	ModelHasMoved(SvListEntry* pSource );
	virtual void 	ModelIsRemoving( SvListEntry* pEntry );
	virtual void 	ModelHasRemoved( SvListEntry* pEntry );
	virtual void 	ModelHasEntryInvalidated( SvListEntry* pEntry );

	void			ShowTargetEmphasis( SvLBoxEntry*, BOOL bShow );
	SvLBoxEntry*	GetDropTarget( const Point& );
	void			ScrollOutputArea( short nDeltaEntries );

	short			GetEntryHeight() const 	{ return nEntryHeight; }
	void			SetEntryHeight( short nHeight, BOOL bAlways = FALSE );
	Size			GetOutputSizePixel() const;
	short			GetIndent() const { return nIndent; }
	void			SetIndent( short nIndent );
	void			SetSpaceBetweenEntries( short nSpace );
	short			GetSpaceBetweenEntries() const {return nEntryHeightOffs;}
	Point			GetEntryPos( SvLBoxEntry* ) const;
	void			ShowEntry( SvLBoxEntry* );  
	virtual void	MakeVisible( SvLBoxEntry* );
	void			MakeVisible( SvLBoxEntry*, BOOL bMoveToTop );

	void          	SetCollapsedNodeBmp( const Image& );
	void          	SetExpandedNodeBmp( const Image& );
	Image          	GetCollapsedNodeBmp() const;
	Image          	GetExpandedNodeBmp() const;
	void			SetFont( const Font& rFont );

	void			SetCursor( SvLBoxEntry* pEntry, BOOL bForceNoSelect = FALSE );

	SvLBoxEntry*	GetCurEntry() const;
	void			SetCurEntry( SvLBoxEntry* );
	SvLBoxEntry*	GetEntry( const Point& rPos, BOOL bHit = FALSE ) const;
	SvLBoxEntry*	GetEntry( SvLBoxEntry* pParent, ULONG nPos ) const { return SvLBox::GetEntry(pParent,nPos); }
	SvLBoxEntry*	GetEntry( ULONG nRootPos ) const { return SvLBox::GetEntry(nRootPos);}

	void			SetWindowBits( WinBits nWinStyle );
	void			PaintEntry( SvLBoxEntry* );
	long			PaintEntry( SvLBoxEntry*, long nLine,
					USHORT nTabFlagMask=0xffff );
	virtual Rectangle GetFocusRect( SvLBoxEntry*, long nLine );

	virtual long	GetTabPos( SvLBoxEntry*, SvLBoxTab* );
	void			InvalidateEntry( SvLBoxEntry* );
	SvLBoxItem*		GetItem( SvLBoxEntry*, long nX, SvLBoxTab** ppTab);
	SvLBoxItem*		GetItem( SvLBoxEntry*, long nX );
	SvLBoxItem*		GetFirstDynamicItem( SvLBoxEntry* pEntry );

	void			SetDragDropMode( DragDropMode );
	void 			SetSelectionMode( SelectionMode );
	void			SetAddMode( BOOL bAdd );
	BOOL			IsAddMode() const;

	BOOL            Expand( SvLBoxEntry* pParent );
	BOOL            Collapse( SvLBoxEntry* pParent );
	BOOL            Select( SvLBoxEntry* pEntry, BOOL bSelect=TRUE );
	ULONG			SelectChilds( SvLBoxEntry* pParent, BOOL bSelect );
	void			SelectAll( BOOL bSelect, BOOL bPaint=TRUE );

	void			Invalidate();
	void			Invalidate( const Rectangle& );

	void			SetHighlightRange(USHORT nFirstTab=0,USHORT nLastTab=0xffff);
	void			RemoveHighlightRange();

	virtual Region	GetDragRegion() const;

	void			RemoveParentKeepChilds( SvLBoxEntry* pParent );

	DECL_LINK( DefaultCompare, SvSortData* );
	virtual void	ModelNotification( USHORT nActionId, SvListEntry* pEntry1,
			SvListEntry* pEntry2, ULONG nPos );

	long			GetTextOffset() const;
	void			EndSelection();
	BOOL			IsNodeButton( const Point& rPos ) const;
	void			RepaintScrollBars() const;
	ScrollBar*		GetVScroll();
	ScrollBar*		GetHScroll();
	void			EnableAsyncDrag( BOOL b );

	SvLBoxEntry*	GetFirstEntryInView() const;
	SvLBoxEntry*	GetNextEntryInView(SvLBoxEntry*) const;

	void			ShowFocusRect( const SvLBoxEntry* pEntry );
	void			SetTabBar( TabBar* pTabBar );
	void			InitStartEntry();
};

#endif

