

#ifndef _TEXTDATA_HXX
#define _TEXTDATA_HXX

#ifndef _SFXBRDCST_HXX
#include <svtools/brdcst.hxx>
#endif

#ifndef _SFXSMPLHINT_HXX
#include <svtools/smplhint.hxx>
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#define TEXTUNDO_START				100
#define TEXTUNDO_REMOVECHARS		100
#define TEXTUNDO_CONNECTPARAS		101
#define TEXTUNDO_SPLITPARA			102
#define TEXTUNDO_INSERTCHARS		103
#define TEXTUNDO_DELCONTENT			104
#define TEXTUNDO_DELETE				105
#define TEXTUNDO_CUT				106
#define TEXTUNDO_PASTE				107
#define TEXTUNDO_INSERT				108
#define TEXTUNDO_ATTRIBS			109
#define TEXTUNDO_DRAGANDDROP		110
#define TEXTUNDO_READ				111
#define TEXTUNDO_END				149

#define XTEXTUNDO_START				150
#define XTEXTUNDO_END				199

#define TEXTUNDO_USER				200

#define TEXT_PARA_ALL				0xFFFFFFFF

class TextPaM
{
private:
	ULONG			mnPara;
	USHORT			mnIndex;

public:
		TextPaM() 								{ mnPara = 0, mnIndex = 0; }
		TextPaM( ULONG nPara, USHORT nIndex ) 	{ mnPara = nPara, mnIndex = nIndex; }

	ULONG			GetPara() const 	{ return mnPara; }
	ULONG&			GetPara() 			{ return mnPara; }

	USHORT			GetIndex() const 	{ return mnIndex; }
	USHORT&			GetIndex() 			{ return mnIndex; }

	inline BOOL 	operator == ( const TextPaM& rPaM ) const;
	inline BOOL 	operator != ( const TextPaM& rPaM ) const;
	inline BOOL 	operator < ( const TextPaM& rPaM ) const;
	inline BOOL 	operator > ( const TextPaM& rPaM ) const;
};

inline BOOL TextPaM::operator == ( const TextPaM& rPaM ) const
{
	return ( ( mnPara == rPaM.mnPara ) && ( mnIndex == rPaM.mnIndex ) ) ? TRUE : FALSE;
}

inline BOOL TextPaM::operator != ( const TextPaM& rPaM ) const
{
	return !( *this == rPaM );
}

inline BOOL TextPaM::operator < ( const TextPaM& rPaM ) const
{
	return ( ( mnPara < rPaM.mnPara ) ||
	( ( mnPara == rPaM.mnPara ) && mnIndex < rPaM.mnIndex ) ) ? TRUE : FALSE;
}

inline BOOL TextPaM::operator > ( const TextPaM& rPaM ) const
{
	return ( ( mnPara > rPaM.mnPara ) ||
	( ( mnPara == rPaM.mnPara ) && mnIndex > rPaM.mnIndex ) ) ? TRUE : FALSE;
}

class TextSelection
{
private:
	TextPaM			maStartPaM;
	TextPaM			maEndPaM;

public:
		TextSelection();
		TextSelection( const TextPaM& rPaM );
		TextSelection( const TextPaM& rStart, const TextPaM& rEnd );

	const TextPaM&	GetStart() const	{ return maStartPaM; }
	TextPaM&		GetStart() 			{ return maStartPaM; }

	const TextPaM&	GetEnd() const		{ return maEndPaM; }
	TextPaM&		GetEnd() 			{ return maEndPaM; }

	void			Justify();

	BOOL			HasRange() const 	{ return maStartPaM != maEndPaM; }

	inline BOOL 	operator == ( const TextSelection& rSel ) const;
	inline BOOL 	operator != ( const TextSelection& rSel ) const;
};

inline BOOL TextSelection::operator == ( const TextSelection& rSel ) const
{
	return ( ( maStartPaM == rSel.maStartPaM ) && ( maEndPaM == rSel.maEndPaM ) );
}

inline BOOL TextSelection::operator != ( const TextSelection& rSel ) const
{
	return !( *this == rSel );
}

#define TEXT_HINT_PARAINSERTED		1
#define TEXT_HINT_PARAREMOVED		2
#define TEXT_HINT_TEXTHEIGHTCHANGED	3
#define TEXT_HINT_VIEWSCROLLED		4
#define TEXT_HINT_FORMATPARA		5
#define TEXT_HINT_TEXTFORMATTED		6
#define TEXT_HINT_MODIFIED			7

class TextHint : public SfxSimpleHint
{
private:
	ULONG 	mnValue;

public:
			TYPEINFO();
			TextHint( ULONG nId );
			TextHint( ULONG nId, ULONG nValue );

	ULONG 	GetValue() const 		{ return mnValue; }
	void	SetValue( ULONG n ) 	{ mnValue = n; }
};

struct TEIMEInfos
{
	String  aOldTextAfterStartPos;
	USHORT*	pAttribs;
	TextPaM	aPos;
	USHORT	nLen;
	BOOL	bCursor;
	BOOL	bWasCursorOverwrite;
	TEIMEInfos( const TextPaM& rPos, const String& rOldTextAfterStartPos );
			~TEIMEInfos();

	void	CopyAttribs( const USHORT* pA, USHORT nL );
	void	DestroyAttribs();
};

#endif 
