
#ifndef _SVTOOLS_UNOEVENT_HXX_
#define _SVTOOLS_UNOEVENT_HXX_

#ifndef _COM_SUN_STAR_CONTAINER_XNAMEREPLACE_HPP_
#include <com/sun/star/container/XNameReplace.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XSERVICEINFO_HPP_
#include <com/sun/star/lang/XServiceInfo.hpp>
#endif

#ifndef _COM_SUN_STAR_UNO_XINTERFACE_HPP_
#include <com/sun/star/uno/XInterface.hpp>
#endif

#ifndef _CPPUHELPER_IMPLBASE2_HXX_
#include <cppuhelper/implbase2.hxx>
#endif

class SvxMacroTableDtor;
class SvxMacroItem;
class SvxMacro;

struct SvEventDescription
{
	sal_uInt16 mnEvent;
	const sal_Char* mpEventName;
};

class SvBaseEventDescriptor : public cppu::WeakImplHelper2
<
	::com::sun::star::container::XNameReplace,
	::com::sun::star::lang::XServiceInfo
>
{
	const ::rtl::OUString sEventType;
	const ::rtl::OUString sMacroName;
	const ::rtl::OUString sLibrary;
	const ::rtl::OUString sStarBasic;
	const ::rtl::OUString sJavaScript;
	const ::rtl::OUString sScript;
	const ::rtl::OUString sNone;

	const ::rtl::OUString sServiceName;

protected:
	const ::rtl::OUString sEmpty;

	const SvEventDescription* mpSupportedMacroItems;
	sal_Int16 mnMacroItems;

public:

 	SvBaseEventDescriptor(const SvEventDescription* pSupportedMacroItems);

	virtual ~SvBaseEventDescriptor();
	 virtual void SAL_CALL replaceByName( 
		const ::rtl::OUString& rName,				 
		const ::com::sun::star::uno::Any& rElement ) 
			throw(
	::com::sun::star::lang::IllegalArgumentException, 
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);
	 virtual ::com::sun::star::uno::Any SAL_CALL getByName( 
		const ::rtl::OUString& rName ) 	
			throw(
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);
	 virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL 
											getElementNames() 
		throw(::com::sun::star::uno::RuntimeException);
	 virtual sal_Bool SAL_CALL hasByName( 
		const ::rtl::OUString& rName ) 
			throw(::com::sun::star::uno::RuntimeException);
	 virtual ::com::sun::star::uno::Type SAL_CALL getElementType() 
		throw(::com::sun::star::uno::RuntimeException);
	 virtual sal_Bool SAL_CALL hasElements() 
		throw(::com::sun::star::uno::RuntimeException);

	virtual rtl::OUString SAL_CALL getImplementationName(void) 
		throw( ::com::sun::star::uno::RuntimeException ) = 0;

	virtual sal_Bool SAL_CALL supportsService(const rtl::OUString& ServiceName)
		throw( ::com::sun::star::uno::RuntimeException );

	virtual ::com::sun::star::uno::Sequence< rtl::OUString > SAL_CALL 
		getSupportedServiceNames(void) 
			throw( ::com::sun::star::uno::RuntimeException );

protected:
	 virtual void replaceByName( 
		const sal_uInt16 nEvent,		
		const SvxMacro& rMacro)		
			throw(
	::com::sun::star::lang::IllegalArgumentException, 
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException) = 0;
	 virtual void getByName( 
		SvxMacro& rMacro,
		const sal_uInt16 nEvent ) 
			throw(
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException) = 0;

	sal_uInt16 mapNameToEventID(const ::rtl::OUString& rName) const;

	::rtl::OUString mapEventIDToName(sal_uInt16 nPoolID) const;

	sal_uInt16 getMacroID(const ::rtl::OUString& rName) const;

	void getAnyFromMacro(
		::com::sun::star::uno::Any& aAny,	
		const SvxMacro& rMacro);

	void getMacroFromAny(
		SvxMacro& aMacro,		
		const ::com::sun::star::uno::Any& rAny)
			throw (
	::com::sun::star::lang::IllegalArgumentException);

};

class SvEventDescriptor : public SvBaseEventDescriptor
{

	::com::sun::star::uno::Reference< 
		::com::sun::star::uno::XInterface > xParentRef;

public:

 	SvEventDescriptor(::com::sun::star::uno::XInterface& rParent,
		  const SvEventDescription* pSupportedMacroItems);

	virtual ~SvEventDescriptor();

protected:
	 virtual void replaceByName( 
		const sal_uInt16 nEvent,		
		const SvxMacro& rMacro)		
			throw(
	::com::sun::star::lang::IllegalArgumentException, 
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);
	 virtual void getByName( 
		SvxMacro& rMacros,			
		const sal_uInt16 nEvent ) 		
			throw(
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);

	virtual const SvxMacroItem& getMacroItem() = 0;

	virtual void setMacroItem(const SvxMacroItem& rItem) = 0;

	virtual sal_uInt16 getMacroItemWhich() const = 0;
};

class SvDetachedEventDescriptor : public SvBaseEventDescriptor
{

	SvxMacro** aMacros;

	const ::rtl::OUString sImplName;

public:

 	SvDetachedEventDescriptor(const SvEventDescription* pSupportedMacroItems);

	virtual ~SvDetachedEventDescriptor();

	virtual rtl::OUString SAL_CALL getImplementationName(void) 
		throw( ::com::sun::star::uno::RuntimeException );

protected:

	sal_Int16 getIndex(const sal_uInt16 nID);
	 virtual void replaceByName( 
		const sal_uInt16 nEvent,		
		const SvxMacro& rMacro)		
			throw(
	::com::sun::star::lang::IllegalArgumentException, 
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);
	 virtual void getByName( 
		SvxMacro& rMacro,			
		const sal_uInt16 nEvent ) 		
			throw(
	::com::sun::star::container::NoSuchElementException, 
	::com::sun::star::lang::WrappedTargetException, 
	::com::sun::star::uno::RuntimeException);
	 virtual const sal_Bool hasByName( 
		const sal_uInt16 nEvent )		
	throw(
	::com::sun::star::lang::IllegalArgumentException);

};

class SvMacroTableEventDescriptor : public SvDetachedEventDescriptor
{
public:

 	SvMacroTableEventDescriptor(const SvEventDescription* pSupportedMacroItems);
 	SvMacroTableEventDescriptor(const SvxMacroTableDtor& aFmt,
					const SvEventDescription* pSupportedMacroItems);

	virtual ~SvMacroTableEventDescriptor();

	void copyMacrosFromTable(const SvxMacroTableDtor& aFmt);
	void copyMacrosIntoTable(SvxMacroTableDtor& aFmt);
};

#endif
