

#ifndef _VALUESET_HXX
#define _VALUESET_HXX

#ifndef _CTRL_HXX
#include <vcl/ctrl.hxx>
#endif
#ifndef _VIRDEV_HXX
#include <vcl/virdev.hxx>
#endif
#ifndef _TIMER_HXX
#include <vcl/timer.hxx>
#endif

class MouseEvent;
class TrackingEvent;
class HelpEvent;
class KeyEvent;
class DataChangedEvent;
class ScrollBar;
class ValueItemList;
struct ValueSetItem;

#define VIB_NODOUBLECLICK		((USHORT)0x0080)

#define WB_RADIOSEL 			((WinBits)0x00008000)
#define WB_ITEMBORDER			((WinBits)0x00010000)
#define WB_DOUBLEBORDER 		((WinBits)0x00020000)
#define WB_NAMEFIELD			((WinBits)0x00040000)
#define WB_NONEFIELD			((WinBits)0x00080000)
#define WB_FLATVALUESET 		((WinBits)0x02000000)

#define VALUESET_APPEND 		((USHORT)0xFFFF)
#define VALUESET_ITEM_NOTFOUND	((USHORT)0xFFFF)

class ValueSet : public Control
{
private:
	VirtualDevice	maVirDev;
	Timer			maTimer;
	ValueItemList*	mpItemList;
	ValueSetItem*	mpNoneItem;
	ScrollBar*		mpScrBar;
	long			mnTextOffset;
	long			mnVisLines;
	long			mnLines;
	long			mnUserItemWidth;
	long			mnUserItemHeight;
	USHORT			mnOldItemId;
	USHORT			mnSelItemId;
	USHORT			mnHighItemId;
	USHORT			mnDropPos;
	USHORT			mnCols;
	USHORT			mnCurCol;
	USHORT			mnUserCols;
	USHORT			mnUserVisLines;
	USHORT			mnFirstLine;
	USHORT			mnSpacing;
	USHORT			mnFrameStyle;
	BOOL			mbFormat;
	BOOL			mbHighlight;
	BOOL			mbSelection;
	BOOL			mbNoSelection;
	BOOL			mbDrawSelection;
	BOOL			mbBlackSel;
	BOOL			mbDoubleSel;
	BOOL			mbScroll;
	BOOL			mbDropPos;
	BOOL			mbFullMode;
	Color			maColor;
	Link			maDoubleClickHdl;
	Link			maSelectHdl;

#ifdef _SV_VALUESET_CXX
	void			ImplInit( WinBits nWinStyle );
	void			ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	void			ImplInitScrollBar();
	void			ImplFormatItem( ValueSetItem* pItem );
	void			ImplDrawItemText( const XubString& rStr );
	void			ImplDrawSelect();
	void			ImplHideSelect( USHORT nItemId );
	void			ImplHighlightItem( USHORT nItemId, BOOL bIsSelection = TRUE );
	void			ImplDrawDropPos( BOOL bShow );
	void			ImplDraw();
	BOOL			ImplScroll( const Point& rPos );
	USHORT			ImplGetItem( const Point& rPoint, BOOL bMove = FALSE ) const;
	ValueSetItem*	ImplGetItem( USHORT nPos );
	ValueSetItem*	ImplGetFirstItem();
	void			ImplTracking( const Point& rPos, BOOL bRepeat );
	void			ImplEndTracking( const Point& rPos, BOOL bCancel );
		DECL_LINK( ImplScrollHdl, ScrollBar* );
		DECL_LINK( ImplTimerHdl, Timer* );
#endif

protected:
	BOOL			StartDrag( const CommandEvent& rCEvt, Region& rRegion );
	USHORT			ShowDropPos( const Point& rPos );
	void			HideDropPos();

public:
		ValueSet( Window* pParent,
				  WinBits nWinStyle = WB_ITEMBORDER );
		ValueSet( Window* pParent, const ResId& rResId );
		~ValueSet();

	virtual void	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void	MouseButtonUp( const MouseEvent& rMEvt );
	virtual void	MouseMove( const MouseEvent& rMEvt );
	virtual void	Tracking( const TrackingEvent& rMEvt );
	virtual void	KeyInput( const KeyEvent& rKEvt );
	virtual void	Command( const CommandEvent& rCEvt );
	virtual void	Paint( const Rectangle& rRect );
	virtual void	GetFocus();
	virtual void	LoseFocus();
	virtual void	Resize();
	virtual void	RequestHelp( const HelpEvent& rHEvt );
	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

	virtual void	Select();
	virtual void	DoubleClick();
	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

	void			InsertItem( USHORT nItemId, const Image& rImage,
					USHORT nPos = VALUESET_APPEND );
	void			InsertItem( USHORT nItemId, const Color& rColor,
					USHORT nPos = VALUESET_APPEND );
	void			InsertItem( USHORT nItemId,
					const Image& rImage, const XubString& rStr,
					USHORT nPos = VALUESET_APPEND );
	void			InsertItem( USHORT nItemId,
					const Color& rColor, const XubString& rStr,
					USHORT nPos = VALUESET_APPEND );
	void			InsertItem( USHORT nItemId,
					USHORT nPos = VALUESET_APPEND );
	void			InsertSpace( USHORT nItemId,
					 USHORT nPos = VALUESET_APPEND );
	void			RemoveItem( USHORT nItemId );

	void			CopyItems( const ValueSet& rValueSet );
	void			Clear();

	USHORT			GetItemCount() const;
	USHORT			GetItemPos( USHORT nItemId ) const;
	USHORT			GetItemId( USHORT nPos ) const;
	USHORT			GetItemId( const Point& rPos ) const;
	Rectangle		GetItemRect( USHORT nItemId ) const;

	void			EnableFullItemMode( BOOL bFullMode = TRUE );
	BOOL			IsFullItemModeEnabled() const { return mbFullMode; }
	void			SetColCount( USHORT nNewCols = 1 );
	USHORT			GetColCount() const { return mnUserCols; }
	void			SetLineCount( USHORT nNewLines = 0 );
	USHORT			GetLineCount() const { return mnUserVisLines; }
	void			SetItemWidth( long nItemWidth = 0 );
	long			GetItemWidth() const { return mnUserItemWidth; }
	void			SetItemHeight( long nLineHeight = 0 );
	long			GetItemHeight() const { return mnUserItemHeight; }
	void			SetFirstLine( USHORT nNewLine = 0 );
	USHORT			GetFirstLine() const { return mnFirstLine; }

	void			SelectItem( USHORT nItemId );
	USHORT			GetSelectItemId() const { return mnSelItemId; }
	BOOL			IsItemSelected( USHORT nItemId ) const
			{ return ((!mbNoSelection && (nItemId == mnSelItemId)) ? TRUE : FALSE); }
	void			SetNoSelection();
	BOOL			IsNoSelection() const { return mbNoSelection; }

	void			SetItemBits( USHORT nItemId, USHORT nBits );
	USHORT			GetItemBits( USHORT nItemId ) const;
	void			SetItemImage( USHORT nItemId, const Image& rImage );
	Image			GetItemImage( USHORT nItemId ) const;
	void			SetItemColor( USHORT nItemId, const Color& rColor );
	Color			GetItemColor( USHORT nItemId ) const;
	void			SetItemData( USHORT nItemId, void* pData );
	void*			GetItemData( USHORT nItemId ) const;
	void			SetItemText( USHORT nItemId, const XubString& rStr );
	XubString		GetItemText( USHORT nItemId ) const;
	void			SetColor( const Color& rColor );
	void			SetColor() { SetColor( Color( COL_TRANSPARENT ) ); }
	Color			GetColor() const { return maColor; }
	BOOL			IsColor() const { return maColor.GetTransparency() == 0; }

	void			SetExtraSpacing( USHORT nNewSpacing );
	USHORT			GetExtraSpacing() { return mnSpacing; }

	void			Format();

	void			StartSelection();
	void			EndSelection();

	Size			CalcWindowSizePixel( const Size& rItemSize,
							 USHORT nCalcCols = 0,
							 USHORT nCalcLines = 0 );
	Size			CalcItemSizePixel( const Size& rSize, BOOL bOut = TRUE ) const;
	long			GetScrollWidth() const;

	void			SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link& 	GetSelectHdl() const { return maSelectHdl; }
	void			SetDoubleClickHdl( const Link& rLink ) { maDoubleClickHdl = rLink; }
	const Link& 	GetDoubleClickHdl() const { return maDoubleClickHdl; }
};

#endif	
