
#ifndef _WALLITEM_HXX
#define _WALLITEM_HXX

#ifndef SHL_HXX
#include <tools/shl.hxx>
#endif

#ifndef _SFX_BRUSHITEMLINK_DECLARED
#define _SFX_BRUSHITEMLINK_DECLARED
#ifndef _SV_GRAPH_HXX
#include <vcl/graph.hxx>
#endif

#ifndef SHL_BRUSHITEM
#define SHL_BRUSHITEM SHL_SHL1
#endif

class Window;
class SfxItemSet;

typedef void* (*CreateSvxBrushTabPage)(Window *pParent, const SfxItemSet &rAttrSet);
typedef USHORT*	(*GetSvxBrushTabPageRanges)();

class Graphic;
class String;
class SfxBrushItemLink
{
public:
	virtual Graphic GetGraphic( const String& rLink, const String& rFilter) = 0;
	virtual CreateSvxBrushTabPage GetBackgroundTabpageCreateFunc() = 0;
	virtual GetSvxBrushTabPageRanges GetBackgroundTabpageRanges() = 0;
	static SfxBrushItemLink* Get() { return *(SfxBrushItemLink**)GetAppData(SHL_BRUSHITEM);	}
	static void Set( SfxBrushItemLink* pLink );
};
#endif	

#ifndef _SFX_BRUSHITEMLINK

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif
#ifndef _SV_WALL_HXX
#include <vcl/wall.hxx>
#endif

#include <svtools/poolitem.hxx>

class SvStream;
class Graphic;

DBG_NAMEEX(SfxWallpaperItem);

class SfxWallpaperItem : public SfxPoolItem
{
private:
	Wallpaper 				_aWallpaper;
	XubString				_aURL;
	XubString				_aFilter;
	USHORT					_nFlags;
	BOOL					IsDownloaded() const;
	void					Download();
public:
				TYPEINFO();
					SfxWallpaperItem( USHORT nWhich );
				SfxWallpaperItem( USHORT nWhich, SvStream& rStream, USHORT nVersion );
				SfxWallpaperItem( const SfxWallpaperItem& rCpy );
				SfxWallpaperItem( USHORT nWhich, const Wallpaper& );
				~SfxWallpaperItem();

	virtual	int				operator==( const SfxPoolItem& ) const;
	virtual SfxPoolItem*	Create( SvStream&, USHORT nItemVersion ) const;
	virtual SvStream&		Store( SvStream&, USHORT nItemVersion ) const;
	virtual SfxPoolItem*	Clone( SfxItemPool* pPool = 0 ) const;

	virtual	BOOL 			QueryValue( com::sun::star::uno::Any& rVal,
				 			BYTE nMemberId = 0 ) const;
	virtual	BOOL 			PutValue  ( const com::sun::star::uno::Any& rVal,
			    			BYTE nMemberId = 0 );

	void					SetBitmapURL( const XubString& rURL, const XubString& rFilter );
	void					SetGraphic( const Graphic& rGraphic, BOOL bKeepLink=FALSE );
	void					SetColor( const Color& rColor ) { _aWallpaper.SetColor(rColor); }
	void					SetStyle( WallpaperStyle eStyle ) { _aWallpaper.SetStyle(eStyle); }

	const Wallpaper&		GetWallpaper( BOOL bNeedsBitmap = TRUE ) const;
	const XubString&		GetBitmapURL() const { return _aURL; }
	const XubString&		GetBitmapFilter() const { return _aFilter; }
};

class WallpaperLoader_Impl;
class CntWallpaperItem;

class WallpaperLoader : public SvRefBase
{
	WallpaperLoader_Impl	*pImp;

public:
				WallpaperLoader( const CntWallpaperItem& rItem);
			~WallpaperLoader();

	void				RequestData( const Link& rLink );
	const Wallpaper&	GetWallpaper() const;
	void				Done();
	void				SetBitmap( SvStream* pStream );
};

SV_DECL_REF( WallpaperLoader );
SV_IMPL_REF( WallpaperLoader );

#endif 

#endif 

