

#ifndef _SVTOOLS_WIZARDMACHINE_HXX_
#define _SVTOOLS_WIZARDMACHINE_HXX_

#ifndef _SVT_WIZDLG_HXX
#include <svtools/wizdlg.hxx>
#endif
#ifndef _SV_BUTTON_HXX 
#include <vcl/button.hxx>
#endif
#ifndef _SV_TABPAGE_HXX 
#include <vcl/tabpage.hxx>
#endif
#ifndef _COMPHELPER_STLTYPES_HXX_
#include <comphelper/stl_types.hxx>
#endif

class Bitmap;

namespace svt
{

#define WZB_NEXT				0x0001
#define WZB_PREVIOUS			0x0002
#define WZB_FINISH				0x0004
#define WZB_CANCEL				0x0008
#define WZB_HELP				0x0010

#define WZS_INVALID_STATE		((sal_uInt16)-1)

	class OWizardMachine;
	struct WizardPageImplData;
	class OWizardPage : public TabPage
	{
	private:
		WizardPageImplData*		m_pImpl;

	public:

		struct GrantAccess
		{
			friend class OWizardMachine;
		private:
			GrantAccess() { }
		};

	public:
		OWizardPage( OWizardMachine* _pParent, WinBits _nStyle = 0 );
		OWizardPage( OWizardMachine* _pParent, const ResId& _rResId );
		~OWizardPage();

		enum COMMIT_REASON
		{
			CR_TRAVEL_NEXT,			
			CR_TRAVEL_PREVIOUS,		
			CR_FINISH,				
			CR_VALIDATE,			
			CR_VALIDATE_NOUI		
		};

		virtual void		initializePage();
		virtual sal_Bool	commitPage(COMMIT_REASON _eReason);

		void				enableHeader( const Bitmap& _rBitmap, sal_Int32 _nPixelHeight, GrantAccess );

	protected:

		virtual void	ActivatePage();

		sal_Bool		isHeaderEnabled( ) const;

		void			setHeaderText( const String& _rHeaderText );

	protected:

		void implCheckNextButton();

		virtual sal_Bool determineNextButtonState();
	};

	struct WizardMachineImplData;

	class OWizardMachine : public WizardDialog
	{
	private:

		void				AddPage( TabPage* pPage ) { WizardDialog::AddPage(pPage); }
		void				RemovePage( TabPage* pPage ) { WizardDialog::RemovePage(pPage); }
		void				SetPage( USHORT nLevel, TabPage* pPage ) { WizardDialog::SetPage(nLevel, pPage); }
		TabPage*			GetPage( USHORT nLevel ) const { return WizardDialog::GetPage(nLevel); }

	protected:
		OKButton*		m_pFinish;
		CancelButton*	m_pCancel;
		PushButton*		m_pNextPage;
		PushButton*		m_pPrevPage;
		HelpButton*		m_pHelp;

	private:
		WizardMachineImplData*
			m_pImpl;

	public:

		OWizardMachine(Window* _pParent, const ResId& _rRes, sal_uInt32 _nButtonFlags);
		~OWizardMachine();

		void	enableButtons(sal_uInt32 _nWizardButtonFlags, sal_Bool _bEnable);

		void	defaultButton(sal_uInt32 _nWizardButtonFlags);

		void	defaultButton(PushButton* _pNewDefButton);

		void			setTitleBase(const String& _rTitleBase);
		const String&	getTitleBase() const;

	protected:

		virtual void		ActivatePage();
		virtual long		DeactivatePage();

		virtual OWizardPage*	createPage(sal_uInt16 _nState) = 0;

		virtual	void			enterState(sal_uInt16 _nState);

		virtual	sal_Bool		leaveState(sal_uInt16 _nState);

		virtual sal_uInt16		determineNextState(sal_uInt16 _nCurrentState);

		virtual sal_Bool onFinish(sal_Int32 _nResult);

		void	enableHeader( const Bitmap& _rBitmap, sal_Int32 _nPixelHeight = -1 );

		sal_Bool	travelNext();

		sal_Bool	travelPrevious();

		sal_Bool	skip(sal_Int32 _nSteps = 1);

		sal_uInt16		getCurrentState() const { return WizardDialog::GetCurLevel(); }

		OWizardPage*	getPage(sal_uInt16 _nState) const { return static_cast<OWizardPage*>(WizardDialog::GetPage(_nState)); }

		OWizardPage*	getCurrentPage() const { return getPage(getCurrentState()); }

	private:
		DECL_LINK(OnNextPage, PushButton*);
		DECL_LINK(OnPrevPage, PushButton*);
		DECL_LINK(OnFinish, PushButton*);

		void		implResetDefault(Window* _pWindow);
		void		implUpdateTitle();
		sal_Bool	implCommitCurrentPage(OWizardPage::COMMIT_REASON _eReason);
	};

}	

#endif 

