
#ifndef _SVX_CHARDLG_HXX
#define _SVX_CHARDLG_HXX

#ifndef _CTRLBOX_HXX 
#include <svtools/ctrlbox.hxx>
#endif
#ifndef _STDCTRL_HXX 
#include <svtools/stdctrl.hxx>
#endif
#ifndef _SFXTABDLG_HXX 
#include <sfx2/tabdlg.hxx>
#endif
#ifndef _SVX_FNTCTRL_HXX
#include <svx/fntctrl.hxx>
#endif
#include <svx/checklbx.hxx>
#include <svx/langbox.hxx>

class SvxFontListItem;
class FontList;

#define DISABLE_CASEMAP				((USHORT)0x0001)
#define DISABLE_WORDLINE			((USHORT)0x0002)
#define DISABLE_BLINK				((USHORT)0x0004)
#define DISABLE_UNDERLINE_COLOR		((USHORT)0x0008)

struct SvxCharNamePage_Impl;
class SvxCharNamePage : public SfxTabPage
{
private:
	FixedLine*          m_pWestLine;
	FixedText*          m_pWestFontNameFT;
	FontNameBox*        m_pWestFontNameLB;
	FixedText*          m_pWestFontStyleFT;
	FontStyleBox*       m_pWestFontStyleLB;
	FixedText*          m_pWestFontSizeFT;
	FontSizeBox*        m_pWestFontSizeLB;
	FixedText*          m_pWestFontLanguageFT;
	SvxLanguageBox*     m_pWestFontLanguageLB;
	 FixedLine*          m_pEastLine;
	FixedText*          m_pEastFontNameFT;
	FontNameBox*        m_pEastFontNameLB;
	FixedText*          m_pEastFontStyleFT;
	FontStyleBox*       m_pEastFontStyleLB;
	FixedText*          m_pEastFontSizeFT;
	FontSizeBox*        m_pEastFontSizeLB;
	FixedText*          m_pEastFontLanguageFT;
	SvxLanguageBox*     m_pEastFontLanguageLB;
	 FixedLine*          m_pColorFL;
	FixedText*          m_pColorFT;
	ColorListBox*       m_pColorLB;

	SvxFontPrevWindow	m_aPreviewWin;
	FixedInfo			m_aFontTypeFT;

	SvxCharNamePage_Impl*	m_pImpl;
				SvxCharNamePage( Window* pParent, const SfxItemSet& rSet );

	void				Initialize();
	const FontList*		GetFontList() const;
	void                UpdatePreview_Impl();
	void                FillStyleBox_Impl( const FontNameBox* rBox );
	void                FillSizeBox_Impl( const FontNameBox* rBox );
	void				ResetWestOrEast_Impl( const SfxItemSet& rSet, BOOL bWest );
	BOOL				FillItemSetWestOrEast_Impl( SfxItemSet& rSet, BOOL bWest );
	void				ResetColor_Impl( const SfxItemSet& rSet );
	BOOL				FillItemSetColor_Impl( SfxItemSet& rSet );

	DECL_LINK( 			UpdateHdl_Impl, Timer* );
	DECL_LINK(			FontModifyHdl_Impl, void* );
	DECL_LINK( 			ColorBoxSelectHdl_Impl, ColorListBox* );

protected:
	virtual void        ActivatePage( const SfxItemSet& rSet );
	virtual int         DeactivatePage( SfxItemSet* pSet = 0 );

public:
			~SvxCharNamePage();

	static SfxTabPage*  Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*      GetRanges();

	virtual void        Reset( const SfxItemSet& rSet );
	virtual BOOL        FillItemSet( SfxItemSet& rSet );

	void                SetFontList( const SvxFontListItem& rItem );
	void                EnableRelativeMode();
	void				EnableSearchMode();
	 void                SetPreviewBackgroundToCharacter();
};

class SvxCharEffectsPage : public SfxTabPage
{
private:
	FixedText			m_aUnderlineFT;
	ListBox				m_aUnderlineLB;
	FixedText			m_aColorFT;
	ColorListBox		m_aColorLB;
	FixedText			m_aStrikeoutFT;
	ListBox				m_aStrikeoutLB;
	CheckBox			m_aIndividualWordsBtn;
	FixedText			m_aEmphasisFT;
	ListBox				m_aEmphasisLB;
	FixedText			m_aPositionFT;
	ListBox				m_aPositionLB;

	FixedText			m_aEffectsFT;
	SvxCheckListBox		m_aEffectsLB;

	ListBox				m_aEffects2LB;
	 FixedText           m_aReliefFT;
	ListBox             m_aReliefLB;
	 TriStateBox         m_aOutlineBtn;
	TriStateBox			m_aShadowBtn;
	CheckBox			m_aBlinkingBtn;

	SvxFontPrevWindow	m_aPreviewWin;
	FixedInfo			m_aFontTypeFT;

	USHORT				m_nHtmlMode;
	BOOL                m_bPreviewBackgroundToCharacter;

	String				m_aTransparentColorName;
				SvxCharEffectsPage( Window* pParent, const SfxItemSet& rSet );

	void				Initialize();
	void                UpdatePreview_Impl();
	void                SetCaseMap_Impl( SvxCaseMap eCaseMap );

	DECL_LINK(			SelectHdl_Impl, ListBox* );
	DECL_LINK(          CbClickHdl_Impl, CheckBox* );
	DECL_LINK(          TristClickHdl_Impl, TriStateBox* );

	DECL_LINK(			UpdatePreview_Impl, ListBox* );

protected:
	virtual void        ActivatePage( const SfxItemSet& rSet );
	virtual int         DeactivatePage( SfxItemSet* pSet = 0 );

public:
	static SfxTabPage*  Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*      GetRanges();

	virtual void        Reset( const SfxItemSet& rSet );
	virtual BOOL        FillItemSet( SfxItemSet& rSet );

	void                DisableControls( USHORT nDisable );
	void                EnableFlash();
	 void                SetPreviewBackgroundToCharacter();
};

class SvxCharPositionPage : public SfxTabPage
{
private:
	FixedLine			m_aPositionLine;
	RadioButton			m_aHighPosBtn;
	RadioButton			m_aNormalPosBtn;
	RadioButton			m_aLowPosBtn;
	FixedText			m_aHighLowFT;
	MetricField			m_aHighLowEdit;
	CheckBox			m_aHighLowRB;
	FixedText			m_aFontSizeFT;
	MetricField			m_aFontSizeEdit;
	FixedLine			m_aRotationScalingFL;
	FixedLine			m_aScalingFL;
	RadioButton			m_a0degRB;
	RadioButton			m_a90degRB;
	RadioButton			m_a270degRB;
	CheckBox			m_aFitToLineCB;
	FixedText			m_aScaleWidthFT;
	MetricField			m_aScaleWidthMF;

	FixedLine			m_aKerningLine;
	ListBox				m_aKerningLB;
	FixedText			m_aKerningFT;
	MetricField			m_aKerningEdit;
	CheckBox			m_aPairKerningBtn;
	 SvxFontPrevWindow   m_aPreviewWin;
	FixedInfo			m_aFontTypeFT;

	short               m_nSuperEsc;
	short               m_nSubEsc;
	UINT16				m_nScaleWidthItemSetVal;
	UINT16				m_nScaleWidthInitialVal;
	BYTE                m_nSuperProp;
	BYTE                m_nSubProp;
	BOOL                m_bPreviewBackgroundToCharacter;
				SvxCharPositionPage( Window* pParent, const SfxItemSet& rSet );

	void				Initialize();
	void                UpdatePreview_Impl( BYTE nProp, BYTE nEscProp, short nEsc );
	void                SetEscapement_Impl( USHORT nEsc );

	DECL_LINK(			PositionHdl_Impl, RadioButton* );
	DECL_LINK(			RotationHdl_Impl, RadioButton* );
	DECL_LINK(			FontModifyHdl_Impl, MetricField* );
	DECL_LINK(			AutoPositionHdl_Impl, CheckBox* );
	DECL_LINK(			FitToLineHdl_Impl, CheckBox* );
	DECL_LINK(			KerningSelectHdl_Impl, ListBox* );
	DECL_LINK(			KerningModifyHdl_Impl, MetricField* );
	DECL_LINK( 			PairKerningHdl_Impl, CheckBox* );
	DECL_LINK(			LoseFocusHdl_Impl, MetricField* );

protected:
	virtual void        ActivatePage( const SfxItemSet& rSet );
	virtual int         DeactivatePage( SfxItemSet* pSet = 0 );

public:
	static SfxTabPage*  Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*      GetRanges();

	virtual void        Reset( const SfxItemSet& rSet );
	virtual BOOL        FillItemSet( SfxItemSet& rSet );
	virtual void        FillUserData();
	 void                SetPreviewBackgroundToCharacter();
};

class SvxCharTwoLinesPage : public SfxTabPage
{
private:
	FixedLine			m_aSwitchOnLine;
	CheckBox			m_aTwoLinesBtn;

	FixedLine			m_aEncloseLine;
	FixedText			m_aStartBracketFT;
	ListBox				m_aStartBracketLB;
	FixedText			m_aEndBracketFT;
	ListBox				m_aEndBracketLB;

	SvxFontPrevWindow	m_aPreviewWin;
	FixedInfo			m_aFontTypeFT;
	 BOOL                m_bPreviewBackgroundToCharacter;
				SvxCharTwoLinesPage( Window* pParent, const SfxItemSet& rSet );

	void				UpdatePreview_Impl();
	void				Initialize();
	void				SelectCharacter( ListBox* pBox );
	void				SetBracket( sal_Unicode cBracket, BOOL bStart );

	DECL_LINK( 			TwoLinesHdl_Impl, CheckBox* );
	DECL_LINK(			CharacterMapHdl_Impl, ListBox* );

protected:
	virtual void        ActivatePage( const SfxItemSet& rSet );
	virtual int         DeactivatePage( SfxItemSet* pSet = 0 );

public:
	static SfxTabPage*  Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*      GetRanges();

	virtual void        Reset( const SfxItemSet& rSet );
	virtual BOOL        FillItemSet( SfxItemSet& rSet );
	 void                SetPreviewBackgroundToCharacter();
};

#endif 

