
#ifndef _SVX_CHARMAP_HXX
#define _SVX_CHARMAP_HXX

#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif
#ifndef _SV_METRIC_HXX
#include <vcl/metric.hxx>
#endif
#ifndef _SV_SCRBAR_HXX
#include <vcl/scrbar.hxx>
#endif

#ifndef _BASEDLGS_HXX
#include <sfx2/basedlgs.hxx>
#endif

class SubsetMap;
class SvxCharMapData;

#ifdef MAC
#define CHARMAP_MAXLEN  28
#define COLUMN_COUNT    28
#define ROW_COUNT        8
#else
#define CHARMAP_MAXLEN  32
#define COLUMN_COUNT    16
#define ROW_COUNT        7
#endif

class SvxShowCharSet : public Control
{
public:
		SvxShowCharSet( Window* pParent, const ResId& rResId );
		~SvxShowCharSet();

	void            SetFont( const Font& rFont );

	void            SelectCharacter( sal_Unicode cNew, BOOL bFocus = FALSE );
	sal_Unicode     GetSelectCharacter() const;

	Link            GetDoubleClickHdl() const { return aDoubleClkHdl; }
	void			SetDoubleClickHdl( const Link& rLink ) { aDoubleClkHdl = rLink; }
	Link            GetSelectHdl() const { return aSelectHdl; }
	void            SetSelectHdl( const Link& rHdl ) { aSelectHdl = rHdl; }
	Link            GetHighlightHdl() const { return aHighHdl; }
	void            SetHighlightHdl( const Link& rHdl )	{ aHighHdl = rHdl; }
	Link            GetPreSelectHdl() const { return aHighHdl; }
	void            SetPreSelectHdl( const Link& rHdl )	{ aPreSelectHdl = rHdl; }
protected:
	virtual void    Paint( const Rectangle& );
	virtual void    MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    MouseButtonUp( const MouseEvent& rMEvt );
	virtual void    MouseMove( const MouseEvent& rMEvt );
	virtual void    Command( const CommandEvent& rCEvt );
	virtual void    KeyInput( const KeyEvent& rKEvt );
	virtual void    GetFocus();
	virtual void    LoseFocus();
	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

private:
	Link            aDoubleClkHdl;
	Link            aSelectHdl;
	Link            aHighHdl;
	Link			aPreSelectHdl;
	long	        nX;
	long			nY;
	BOOL            bDrag;
	 static long     nSelectedIndex; 
	 FontCharMap     maFontCharMap;
	ScrollBar       aVscrollSB;
	Size            aOrigSize;
	Point           aOrigPos;

private:
	void            DrawChars_Impl( int n1, int n2);
	void            InitSettings( BOOL bForeground, BOOL bBackground);
	 void            SelectIndex( int index, BOOL bFocus = FALSE );
	 Point           MapIndexToPixel( int) const;
	int             PixelToMapIndex( const Point) const;
	 int             FirstInView( void) const;
	int             LastInView( void) const;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( VscrollHdl, ScrollBar* );
#endif
};

class SvxShowText : public Control
{
public:
		SvxShowText( Window* pParent,
					 const ResId& rResId,
					 BOOL bCenter = FALSE );
		~SvxShowText();

	void            SetFont( const Font& rFont );
	void            SetText( const String& rText );

protected:
	virtual void    Paint( const Rectangle& );

private:
	long            nY;
	BOOL            bCenter;

};

class SvxCharacterMap : public SfxModalDialog
{
private:
	SvxCharMapData* const mpCharMapData;

public:
	                SvxCharacterMap( Window* pParent, BOOL bOne = TRUE );
	                ~SvxCharacterMap();
	 void            DisableFontSelection();
	 const Font&     GetCharFont() const;
	void            SetCharFont( const Font& rFont );
	void            SetFont( const Font& rFont ) { SetCharFont( rFont ); } 
	 void            SetChar( sal_Unicode c );
	sal_Unicode     GetChar() const;
	 String          GetCharacters() const;
};

#endif

