

#ifndef _EDITOBJ_HXX
#define _EDITOBJ_HXX

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _RSCSFX_HXX
#include <rsc/rscsfx.hxx>
#endif
#ifndef _SFXITEMSET_HXX
#include <svtools/itemset.hxx>
#endif
#ifndef _EEITEM_HXX 
#include <svx/eeitem.hxx>
#endif

DBG_NAMEEX( EE_EditTextObject );

class SfxItemPool;
class SfxStyleSheetPool; 
class SvxFieldItem;
class EECharAttribArray; 

#define EDTOBJ_SETTINGS_ULITEMSUMMATION		0x00000001
#define EDTOBJ_SETTINGS_ULITEMFIRSTPARA		0x00000002

class EditTextObject
{
private:
	USHORT				nWhich;
	EditTextObject&		operator=( const EditTextObject& );

protected:
			EditTextObject( USHORT nWhich );
			EditTextObject( const EditTextObject& r );

	virtual void		StoreData( SvStream& rOStream ) const;
	virtual void		CreateData( SvStream& rIStream );

public:
	virtual				~EditTextObject();

	USHORT				Which() const { return nWhich; }

	virtual USHORT		GetUserType() const;	
	virtual void		SetUserType( USHORT n );

	virtual ULONG		GetObjectSettings() const;
	virtual void		SetObjectSettings( ULONG n );

	virtual	BOOL		IsVertical() const;
#if SUPD >= 615
	virtual
#endif
	void				SetVertical( BOOL bVertical );

#if SUPD >= 615
	virtual
#endif
	USHORT				GetScriptType() const;

	virtual USHORT		GetVersion() const;	

	virtual EditTextObject*	Clone() const = 0;

	BOOL					Store( SvStream& rOStream ) const;
	static EditTextObject*	Create( SvStream& rIStream,
					SfxItemPool* pGlobalTextObjectPool = 0 );
	void					Skip( SvStream& rIStream );

	virtual void			SetLRSpaceItemFlags( BOOL bOutlineMode );
	virtual void			AdjustImportedLRSpaceItems( BOOL bTurnOfBullets );
	virtual	void 			PrepareStore( SfxStyleSheetPool* pStyleSheetPool );
	virtual	void 			FinishStore();
	virtual	void 			FinishLoad( SfxStyleSheetPool* pStyleSheetPool );

	virtual USHORT		GetParagraphCount() const;

	virtual XubString	GetText( USHORT nParagraph ) const;
	virtual void		Insert( const EditTextObject& rObj, USHORT nPara );
	virtual void		RemoveParagraph( USHORT nPara );
	virtual EditTextObject*	CreateTextObject( USHORT nPara, USHORT nParas = 1 ) const;

	virtual BOOL		HasPortionInfo() const;
	virtual void		ClearPortionInfo();

	virtual BOOL		HasOnlineSpellErrors() const;

	virtual BOOL		HasCharAttribs( USHORT nWhich = 0 ) const;
	virtual	void		GetCharAttribs( USHORT nPara, EECharAttribArray& rLst ) const;

	virtual BOOL		RemoveCharAttribs( USHORT nWhich = 0 );
	virtual BOOL		RemoveParaAttribs( USHORT nWhich = 0 );

	virtual	void		MergeParaAttribs( const SfxItemSet& rAttribs, USHORT nStart = EE_CHAR_START, USHORT nEnd = EE_CHAR_END );

	virtual BOOL		IsFieldObject() const;
	virtual const SvxFieldItem*	GetField() const;
	virtual BOOL		HasField( TypeId aType = NULL ) const;

	virtual SfxItemSet	GetParaAttribs( USHORT nPara ) const;
	virtual void		SetParaAttribs( USHORT nPara, const SfxItemSet& rAttribs );

	virtual BOOL		HasStyleSheet( const XubString& rName, SfxStyleFamily eFamily ) const;
	virtual void		GetStyleSheet( USHORT nPara, XubString& rName, SfxStyleFamily& eFamily ) const;
	virtual void		SetStyleSheet( USHORT nPara, const XubString& rName, const SfxStyleFamily& eFamily );
	virtual BOOL		ChangeStyleSheets( 	const XubString& rOldName, SfxStyleFamily eOldFamily,
								const XubString& rNewName, SfxStyleFamily eNewFamily );
	virtual void		ChangeStyleSheetName( SfxStyleFamily eFamily, const XubString& rOldName, const XubString& rNewName );
};

#endif	
