
#ifndef _SVX_FLDITEM_HXX
#define _SVX_FLDITEM_HXX

#ifndef _LANG_HXX 
#include <tools/lang.hxx>
#endif
#ifndef _SVX_ITEMDATA_HXX
#include <svx/itemdata.hxx>
#endif
#ifndef _TIME_HXX 
#include <tools/time.hxx>
#endif
#ifndef _DATE_HXX 
#include <tools/date.hxx>
#endif
#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif
#ifndef _PSTM_HXX 
#include <tools/pstm.hxx>
#endif

class SvxAddressItem;
class SvNumberFormatter;

#ifdef ITEMID_FIELD

class SvxFieldData : public SvPersistBase
{
public:
				SV_DECL_PERSIST1( SvxFieldData, SvPersistBase, 1 )
					SvxFieldData();
	virtual 				~SvxFieldData();

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxFieldItem : public SfxPoolItem
{
private:
	SvxFieldData*			pField;
			SvxFieldItem( SvxFieldData* pField, const USHORT nId );

public:
			TYPEINFO();
	SvxFieldItem( const SvxFieldData& rField, const USHORT nId = ITEMID_FIELD );
			SvxFieldItem( const SvxFieldItem& rItem );
			~SvxFieldItem();

	virtual int 			operator==( const SfxPoolItem& ) const;
	virtual SfxPoolItem*	Clone( SfxItemPool *pPool = 0 ) const;
	virtual SfxPoolItem*	Create( SvStream&, USHORT nVer ) const;
	virtual SvStream&		Store( SvStream& , USHORT nItemVersion ) const;

	const SvxFieldData*		GetField() const 	{ return pField; }
	static SvClassManager&	GetClassManager() 	{
					DBG_ASSERT( ITEMDATA(), "GlobalItemData nicht initialisiert" );
					return ITEMDATA()->GetClassManager(); }
};

#define SVX_DATEFIELD		2
#define SVX_URLFIELD		3

#define SVX_PAGEFIELD		100		
#define SVX_PAGESFIELD		101
#define SVX_TIMEFIELD		102
#define SVX_FILEFIELD		103
#define SVX_TABLEFIELD		104
#define SVX_EXT_TIMEFIELD	105
#define SVX_EXT_FILEFIELD	106
#define SVX_AUTHORFIELD		107

#define SVX_USERFIELD		200	

enum SvxDateType { SVXDATETYPE_FIX, SVXDATETYPE_VAR };
enum SvxDateFormat {	SVXDATEFORMAT_APPDEFAULT, 	
			SVXDATEFORMAT_SYSTEM, 		
			SVXDATEFORMAT_STDSMALL,
			SVXDATEFORMAT_STDBIG,
			SVXDATEFORMAT_A,	
			SVXDATEFORMAT_B,	
			SVXDATEFORMAT_C,	
			SVXDATEFORMAT_D,	
			SVXDATEFORMAT_E,	
			SVXDATEFORMAT_F		
		};

class SvxDateField : public SvxFieldData
{
	ULONG					nFixDate;
	SvxDateType				eType;
	SvxDateFormat			eFormat;

public:
				SV_DECL_PERSIST1( SvxDateField, SvxFieldData, SVX_DATEFIELD )
					SvxDateField();
				SvxDateField( const Date& rDate,
					SvxDateType eType = SVXDATETYPE_VAR,
					SvxDateFormat eFormat = SVXDATEFORMAT_STDSMALL );

	ULONG					GetFixDate() const { return nFixDate; }
	void					SetFixDate( const Date& rDate ) { nFixDate = rDate.GetDate(); }

	SvxDateType				GetType() const { return eType; }
	void					SetType( SvxDateType eTp ) { eType = eTp; }

	SvxDateFormat			GetFormat() const { return eFormat; }
	void					SetFormat( SvxDateFormat eFmt ) { eFormat = eFmt; }

	String					GetFormatted( LanguageType eLanguage, LanguageType eFormat ) const;
	 String                  GetFormatted( SvNumberFormatter& rFormatter,
	                            LanguageType eLanguage ) const;

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

enum SvxURLFormat	{	SVXURLFORMAT_APPDEFAULT, 	
			SVXURLFORMAT_URL,			
			SVXURLFORMAT_REPR			
		};

class SvxURLField : public SvxFieldData
{
private:
	SvxURLFormat			eFormat;
	XubString				aURL;      			
	XubString				aRepresentation;	
	XubString				aTargetFrame;		

public:
				SV_DECL_PERSIST1( SvxURLField, SvxFieldData, SVX_URLFIELD )
					SvxURLField();
				SvxURLField( const XubString& rURL, const XubString& rRepres, SvxURLFormat eFmt = SVXURLFORMAT_URL );

	const XubString&			GetURL() const { return aURL; }
	void					SetURL( const XubString& rURL ) { aURL = rURL; }

	const XubString&			GetRepresentation() const { return aRepresentation; }
	void					SetRepresentation( const XubString& rRep ) { aRepresentation= rRep; }

	const XubString&			GetTargetFrame() const { return aTargetFrame; }
	void					SetTargetFrame( const XubString& rFrm ) { aTargetFrame = rFrm; }

	SvxURLFormat			GetFormat() const { return eFormat; }
	void					SetFormat( SvxURLFormat eFmt ) { eFormat = eFmt; }

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxPageField : public SvxFieldData
{
public:
				SV_DECL_PERSIST1( SvxPageField, SvxFieldData, SVX_PAGEFIELD )
				SvxPageField() {}
	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxPagesField : public SvxFieldData
{
public:
				SV_DECL_PERSIST1( SvxPagesField, SvxFieldData, SVX_PAGESFIELD )
				SvxPagesField() {}
	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxTimeField : public SvxFieldData
{
public:
				SV_DECL_PERSIST1( SvxTimeField, SvxFieldData, SVX_TIMEFIELD )
				SvxTimeField() {}
	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxFileField : public SvxFieldData
{
public:
				SV_DECL_PERSIST1( SvxFileField, SvxFieldData, SVX_FILEFIELD )
				SvxFileField() {}
	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

class SvxTableField : public SvxFieldData
{
public:
				SV_DECL_PERSIST1( SvxTableField, SvxFieldData, SVX_TABLEFIELD )
				SvxTableField() {}
	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

enum SvxTimeType { SVXTIMETYPE_FIX, SVXTIMETYPE_VAR };
enum SvxTimeFormat {	SVXTIMEFORMAT_APPDEFAULT, 	
			SVXTIMEFORMAT_SYSTEM, 		
			SVXTIMEFORMAT_STANDARD,
			SVXTIMEFORMAT_24_HM,	
			SVXTIMEFORMAT_24_HMS,	
			SVXTIMEFORMAT_24_HMSH,	
			SVXTIMEFORMAT_12_HM,	
			SVXTIMEFORMAT_12_HMS,	
			SVXTIMEFORMAT_12_HMSH,	
			SVXTIMEFORMAT_AM_HM,	
			SVXTIMEFORMAT_AM_HMS,	
			SVXTIMEFORMAT_AM_HMSH	
		};

class SvxExtTimeField : public SvxFieldData
{
private:
	ULONG					nFixTime;
	SvxTimeType				eType;
	SvxTimeFormat			eFormat;

public:
				SV_DECL_PERSIST1( SvxExtTimeField, SvxFieldData, SVX_EXT_TIMEFIELD )
				SvxExtTimeField();
				SvxExtTimeField( const Time& rTime,
					SvxTimeType eType = SVXTIMETYPE_VAR,
					SvxTimeFormat eFormat = SVXTIMEFORMAT_STANDARD );

	ULONG					GetFixTime() const { return nFixTime; }
	void					SetFixTime( const Time& rTime ) { nFixTime = rTime.GetTime(); }

	SvxTimeType				GetType() const { return eType; }
	void					SetType( SvxTimeType eTp ) { eType = eTp; }

	SvxTimeFormat			GetFormat() const { return eFormat; }
	void					SetFormat( SvxTimeFormat eFmt ) { eFormat = eFmt; }

	XubString				GetFormatted( LanguageType eLanguage, LanguageType eFormat ) const;
	 String                  GetFormatted( SvNumberFormatter& rFormatter,
	                            LanguageType eLanguage ) const;

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

enum SvxFileType { SVXFILETYPE_FIX, SVXFILETYPE_VAR };
enum SvxFileFormat {	SVXFILEFORMAT_NAME_EXT,	
			SVXFILEFORMAT_FULLPATH,	
			SVXFILEFORMAT_PATH,		
			SVXFILEFORMAT_NAME		
	   };

class SvxExtFileField : public SvxFieldData
{
private:
	XubString  				aFile;
	SvxFileType				eType;
	SvxFileFormat			eFormat;

public:
				SV_DECL_PERSIST1( SvxExtFileField, SvxFieldData, SVX_EXT_FILEFIELD )
				SvxExtFileField();
				SvxExtFileField( const XubString& rString,
					SvxFileType eType = SVXFILETYPE_VAR,
					SvxFileFormat eFormat = SVXFILEFORMAT_FULLPATH );

	String  				GetFile() const { return aFile; }
	void					SetFile( const XubString& rString ) { aFile = rString; }

	SvxFileType				GetType() const { return eType; }
	void					SetType( SvxFileType eTp ) { eType = eTp; }

	SvxFileFormat			GetFormat() const { return eFormat; }
	void					SetFormat( SvxFileFormat eFmt ) { eFormat = eFmt; }

	XubString				GetFormatted() const;

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

enum SvxAuthorType { SVXAUTHORTYPE_FIX, SVXAUTHORTYPE_VAR };
enum SvxAuthorFormat {	SVXAUTHORFORMAT_FULLNAME,	
			SVXAUTHORFORMAT_NAME,		
			SVXAUTHORFORMAT_FIRSTNAME,	
			SVXAUTHORFORMAT_SHORTNAME	
		};

class SvxAuthorField : public SvxFieldData
{
private:
	XubString		aName;
	XubString		aFirstName;
	XubString		aShortName;
	SvxAuthorType	eType;
	SvxAuthorFormat	eFormat;

public:
				SV_DECL_PERSIST1( SvxAuthorField, SvxFieldData, SVX_AUTHORFIELD )
				SvxAuthorField();
				SvxAuthorField( const SvxAddressItem& rAdrItem,
					SvxAuthorType eType = SVXAUTHORTYPE_VAR,
					SvxAuthorFormat eFormat = SVXAUTHORFORMAT_FULLNAME );

	XubString  				GetName() const { return aName; }
	void					SetName( const XubString& rString ) { aName = rString; }

	XubString  				GetFirstName() const { return aFirstName; }
	void					SetFirstName( const XubString& rString ) { aFirstName = rString; }

	XubString  				GetShortName() const { return aShortName; }
	void					SetShortName( const XubString& rString ) { aShortName = rString; }

	SvxAuthorType			GetType() const { return eType; }
	void					SetType( SvxAuthorType eTp ) { eType = eTp; }

	SvxAuthorFormat			GetFormat() const { return eFormat; }
	void					SetFormat( SvxAuthorFormat eFmt ) { eFormat = eFmt; }

	XubString				GetFormatted() const;

	virtual SvxFieldData*	Clone() const;
	virtual int 			operator==( const SvxFieldData& ) const;
};

#endif

#endif

