

#ifndef _FMSEARCH_HXX
#define _FMSEARCH_HXX

#ifndef _COM_SUN_STAR_SDBC_XRESULTSET_HPP_
#include <com/sun/star/sdbc/XResultSet.hpp>
#endif

#define _SVSTDARR_STRINGSDTOR
#include <svtools/svstdarr.hxx>

#ifndef _DIALOG_HXX 
#include <vcl/dialog.hxx>
#endif

#ifndef _BUTTON_HXX 
#include <vcl/button.hxx>
#endif

#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif

#ifndef _EDIT_HXX 
#include <vcl/edit.hxx>
#endif

#ifndef _SV_COMBOBOX_HXX 
#include <vcl/combobox.hxx>
#endif

#ifndef _SV_LSTBOX_HXX 
#include <vcl/lstbox.hxx>
#endif

#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif

#ifndef _SFXCFGITEM_HXX 
#include <sfx2/cfgitem.hxx>
#endif

#ifndef _COMPHELPER_UNO3_HXX_
#include <comphelper/uno3.hxx>
#endif
#ifndef _COMPHELPER_STLTYPES_HXX_
#include <comphelper/stl_types.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

FORWARD_DECLARE_INTERFACE(util,XNumberFormatsSupplier)

#define	FM_SEARCH_GETFOCUS_ASYNC	0x0001

#define MAX_HISTORY_ENTRIES		50

sal_Bool IsSearchableControl(const ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface>& _xControl, ::rtl::OUString* pCurrentText = NULL);

struct FmFoundRecordInformation
{
	::com::sun::star::uno::Any		aPosition;	
	sal_Int16		nFieldPos;	
	sal_Int16		nContext;	
};

DECLARE_STL_VECTOR( ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface>, InterfaceArray);

struct FmSearchContext
{

	sal_Int16					nContext;		

	::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XResultSet>			xCursor;		
	String					strUsedFields;	
	InterfaceArray			arrFields;		
	String					sFieldDisplayNames;		
};

class FmSearchEngine;
struct FmSearchProgress;
class FmSearchConfigAdmin;
class FmSearchDialog : public ModalDialog
{
	friend class FmSearchEngine;
	 FixedLine       m_flSearchFor;
	RadioButton		m_rbSearchForText;
	RadioButton		m_rbSearchForNull;
	RadioButton		m_rbSearchForNotNull;
	ComboBox		m_cmbSearchText;
	FixedLine       m_flWhere;
	FixedText		m_ftForm;
	ListBox			m_lbForm;
	RadioButton		m_rbAllFields;
	RadioButton		m_rbSingleField;
	ListBox			m_lbField;
	FixedLine       m_flOptions;
	FixedText		m_ftPosition;
	ListBox			m_lbPosition;
	CheckBox		m_cbUseFormat;
	CheckBox		m_cbCase;
	CheckBox		m_cbBackwards;
	CheckBox		m_cbStartOver;
	CheckBox		m_cbWildCard;
	CheckBox		m_cbRegular;
	CheckBox		m_cbApprox;
	PushButton		m_pbApproxSettings;
	CheckBox		m_aHalfFullFormsCJK;
	CheckBox		m_aSoundsLikeCJK;
	PushButton		m_aSoundsLikeCJKSettings;
	FixedLine       m_flState;
	FixedText		m_ftRecordLabel;
	FixedText		m_ftRecord;
	FixedText		m_ftHint;
	PushButton		m_pbSearchAgain;
	CancelButton	m_pbClose;
	HelpButton		m_pbHelp;

	Link	m_lnkFoundHandler;			
	Link	m_lnkCanceledNotFoundHdl;	

	Link	m_lnkContextSupplier;		

	::std::vector<String> m_arrContextFields;

	FmSearchEngine*	m_pSearchEngine;

	Timer			m_aDelayedPaint;

	FmSearchConfigAdmin*	m_pConfig;
public:

	enum SEARCH_MODE { SM_BRUTE, SM_ALLOWSCHEDULE, SM_USETHREAD };

	FmSearchDialog(Window* pParent, const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XResultSet>& xCursor, const String& strVisibleFields, const String& strInitialText,
		const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatsSupplier>& xFormatSupplier, SEARCH_MODE eMode = SM_ALLOWSCHEDULE);

	FmSearchDialog(Window* pParent, const String& strInitialText, const String& strContexts, sal_Int16 nInitialContext,
		const Link& lnkContextSupplier, SEARCH_MODE eMode = SM_ALLOWSCHEDULE);

	virtual ~FmSearchDialog();

	void SetFoundHandler(const Link& lnk) { m_lnkFoundHandler = lnk; }

	void SetCanceledNotFoundHdl(const Link& lnk) { m_lnkCanceledNotFoundHdl = lnk; }

	inline void SetActiveField(const String& strField);

protected:
	virtual sal_Bool Close();

	void Init(const String& strVisibleFields, const String& strInitialText);

	void OnFound(const ::com::sun::star::uno::Any& aCursorPos, sal_Int16 nFieldPos);

	void EnableSearchUI(sal_Bool bEnable);

	void EnableSearchForDependees(sal_Bool bEnable);

	void EnableControlPaint(sal_Bool bEnable);

	void InitContext(sal_Int16 nContext);

	void LoadParams();
	void SaveParams() const;

private:

	DECL_LINK( OnClickedFieldRadios, Button* );
	DECL_LINK( OnClickedSearchAgain, Button* );
	DECL_LINK( OnClickedSpecialSettings, Button* );

	DECL_LINK( OnSearchTextModified, ComboBox* );

	DECL_LINK( OnPositionSelected, ListBox* );
	DECL_LINK( OnFieldSelected, ListBox* );

	DECL_LINK( OnCheckBoxToggled, CheckBox* );

	DECL_LINK( OnContextSelection, ListBox* );

	DECL_LINK( AsyncGrabFocus, void* );

	DECL_LINK( OnSearchProgress, FmSearchProgress* );

	DECL_LINK( OnDelayedPaint, void* );

	void implMoveControls(Control** _ppControls, sal_Int32 _nControls, sal_Int32 _nUp, Control* _pToResize);
};

inline void FmSearchDialog::SetActiveField(const String& strField)
{
	sal_uInt16 nInitialField = m_lbField.GetEntryPos(strField);
	if (nInitialField == COMBOBOX_ENTRY_NOTFOUND)
		nInitialField = 0;
	m_lbField.SelectEntryPos(nInitialField);
	LINK(this, FmSearchDialog, OnFieldSelected).Call(&m_lbField);
}

#endif 
