
#ifndef _SVX_FRMSEL_HXX
#define _SVX_FRMSEL_HXX

#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif
#ifndef _WINDOW_HXX 
#include <vcl/window.hxx>
#endif
#ifndef _COLOR_HXX 
#include <vcl/color.hxx>
#endif
#ifndef _SV_BITMAP_HXX
#include <vcl/bitmap.hxx>
#endif
#ifndef SVX_LINKLINE_HXX
#include <svx/linelink.hxx>
#endif

enum SvxFrameSelectorType
{
	SVX_FRMSELTYPE_TABLE = 0,
	SVX_FRMSELTYPE_PARAGRAPH
};
enum SvxFrameSelectorLine
{
	SVX_FRMSELLINE_NONE = 0,
	SVX_FRMSELLINE_LEFT,
	SVX_FRMSELLINE_RIGHT,
	SVX_FRMSELLINE_TOP,
	SVX_FRMSELLINE_BOTTOM,
	SVX_FRMSELLINE_HOR,
	SVX_FRMSELLINE_VER
};
enum SvxFrameLineState
{
	SVX_FRMLINESTATE_SHOW = 0,
	SVX_FRMLINESTATE_HIDE,
	SVX_FRMLINESTATE_DONT_CARE
};
enum SvxFrameShadow
{
	SVX_FRMSHADOW_NONE = 0,
	SVX_FRMSHADOW_BOT_RIGHT,
	SVX_FRMSHADOW_TOP_RIGHT,
	SVX_FRMSHADOW_BOT_LEFT,
	SVX_FRMSHADOW_TOP_LEFT
};
enum SvxSelArrowType
{
	SVX_SELARROWTYPE_LEFT = 0,
	SVX_SELARROWTYPE_RIGHT,
	SVX_SELARROWTYPE_UP,
	SVX_SELARROWTYPE_DOWN
};

class SvxFrameLine
{
			SvxFrameLine();
public:
	SvxLineStruct		GetStyle() const { return theCoreStyle; }
	void				SetStyle( const SvxLineStruct& aStyle );
	Color				GetColor() const { return aColor; }
	void				SetColor( const Color& rColor )  { aColor = rColor; }
	SvxFrameLineState	GetState() const { return theState; }
	void				SetState( SvxFrameLineState eState );

private:
	friend class SvxFrameSelector;

	static struct SvxLineStruct NO_LINE;		
	static struct SvxLineStruct THIN_LINE;
	static struct SvxLineStruct THICK_LINE;
	static struct SvxLineStruct THIN_DOUBLE_LINE;
	static struct SvxLineStruct THICK_DOUBLE_LINE;

	Point				aStartPos;		
	Point				aMidPos;
	Point				aEndPos;
	Color				aColor;			
	SvxLineStruct		theStyle;		
	SvxLineStruct		theCoreStyle;	
	SvxFrameLineState	theState;	 	
	BOOL				bIsSelected;	
};

class SvxFrameSelector : public Window
{
public:
			SvxFrameSelector( Window* pParent,
							  SvxFrameSelectorType eType,
							  BOOL bDontCare );

	SvxFrameLine&		GetTop() 	{ return theTopLine; }
	SvxFrameLine&		GetBottom()	{ return theBottomLine; }
	SvxFrameLine&		GetLeft() 	{ return theLeftLine; }
	SvxFrameLine&		GetRight() 	{ return theRightLine; }
	SvxFrameLine&		GetHor() 	{ return theHorLine; }
	SvxFrameLine&		GetVer() 	{ return theVerLine; }

	void 				ShowLines();
	void				HideLines();
	void				SelectLine( SvxFrameSelectorLine eNewLine,
						BOOL bSet = TRUE );
	 SvxFrameShadow      GetShadowPos() const;
	void                SetShadowPos( SvxFrameShadow eShadowPos );
	 Color               GetShadowColor() const;
	void                SetShadowColor( const Color& aColor );

	void				SetCurLineColor( const Color& aColor );
	void				SetCurLineStyle( const SvxLineStruct& aStyle );

	void				SetSelectLink(const Link& rLnk) {aSelectLink = rLnk;}

	BOOL				IsDirty() const { return bIsClicked; }
	BOOL				IsAnyLineSet() const;

protected:
	virtual void 		Paint( const Rectangle& rRect );
	virtual void    	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    	MouseButtonUp( const MouseEvent& rMEvt );

private:
	const SvxFrameSelectorType	eSel; 

	Color				theCurLineCol;	
	SvxLineStruct		theCurLineStyle;
	SvxFrameShadow      eShadow;        
	Color               theShadowCol;   
	Bitmap				theBmp;			
	Rectangle			aRectFrame;		
	Rectangle			theBoundingRect;
	SvxFrameLine		theLeftLine;	
	SvxFrameLine		theRightLine;
	SvxFrameLine		theTopLine;
	SvxFrameLine		theBottomLine;
	SvxFrameLine		theHorLine;
	SvxFrameLine		theVerLine;
	Link				aSelectLink;
	Rectangle			aSpotLeft;		
	Rectangle			aSpotRight;
	Rectangle			aSpotTop;
	Rectangle			aSpotBottom;
	Rectangle			aSpotHor;
	Rectangle			aSpotVer;
	BOOL				bIsDontCare;
	BOOL				bIsClicked;

#ifdef _SVX_FRMSEL_CXX
	void				InitBitmap_Impl( const Size& rSize );
	void				DrawSelArrow_Impl( OutputDevice& rDev,
							   Point aPos,
							   Color aCol,
							   SvxSelArrowType eType );
	void				DrawContents_Impl( OutputDevice& rVirDev );
	void				DrawFrameLine_Impl( OutputDevice& rVirDev,
								Point from,
								Point to,
								BOOL bHor,
								SvxLineStruct& dline,
								SvxLineStruct& lt,
								SvxLineStruct& lLine,
								SvxLineStruct& lb,
								SvxLineStruct& rt,
								SvxLineStruct& rLine,
								SvxLineStruct& rb );
	void				LineClicked_Impl( SvxFrameLine& aLine,
							  BOOL bShiftPressed,
							  BOOL bDoubleClick );
#endif
};

#endif

