
#ifndef _SVX_NUMFMT_HXX
#define _SVX_NUMFMT_HXX

#ifndef _WINDOW_HXX 
#include <vcl/window.hxx>
#endif

#ifndef _COLOR_HXX 
#include <vcl/color.hxx>
#endif

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef _SFXTABDLG_HXX 
#include <sfx2/tabdlg.hxx>
#endif

#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif

#ifndef _LSTBOX_HXX 
#include <vcl/lstbox.hxx>
#endif

#ifndef _EDIT_HXX 
#include <vcl/edit.hxx>
#endif

#ifndef _TOOLBOX_HXX 
#include <vcl/toolbox.hxx>
#endif

#ifndef _FIELD_HXX 
#include <vcl/field.hxx>
#endif

#ifndef _SVX_LANGBOX_HXX 
#include <svx/langbox.hxx>
#endif

#ifndef SVX_FONTLB_HXX 
#include <svx/fontlb.hxx>
#endif

#define SVX_NUMVAL_STANDARD		-1234.12345678901234
#define SVX_NUMVAL_CURRENCY		-1234
#define SVX_NUMVAL_PERCENT		-0.1295
#define SVX_NUMVAL_TIME 		36525.5678935185
#define SVX_NUMVAL_DATE 		36525.5678935185
#define SVX_NUMVAL_BOOLEAN 		1

class SvxNumberFormatShell;
class SvxNumberInfoItem;

class SvxNumberPreviewImpl : public Window
{
private:
	String			aPrevStr;
	Color			aPrevCol;

	void			InitSettings( BOOL bForeground, BOOL bBackground );

protected:
	virtual void	Paint( const Rectangle& rRect );
	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

public:
	SvxNumberPreviewImpl( Window* pParent, const ResId& rResId );
	~SvxNumberPreviewImpl();

	void			NotifyChange( const String& rPrevStr, const Color& rColor );
};

class SvxNumberFormatTabPage : public SfxTabPage

{
public:
	~SvxNumberFormatTabPage();

	static SfxTabPage*		Create( Window* pParent,
						const SfxItemSet& rAttrSet );
	static USHORT*			GetRanges();

	virtual	BOOL 			FillItemSet( SfxItemSet& rSet );
	virtual	void 			Reset( const SfxItemSet& rSet );
	virtual int 			DeactivatePage	( SfxItemSet* pSet = NULL );

	void					SetInfoItem( const SvxNumberInfoItem& rItem );
	void					SetNumberFormatList( const SvxNumberInfoItem& rItem )
					{ SetInfoItem( rItem ); }

	void					SetOkHdl( const Link& rOkHandler );
	void					HideLanguage(BOOL nFlag=TRUE);
	virtual long			PreNotify( NotifyEvent& rNEvt );

private:
	SvxNumberFormatTabPage( Window*	pParent,
				const SfxItemSet& rCoreAttrs );
	FixedText				aFtCategory;
	ListBox					aLbCategory;
	FixedText				aFtFormat;
	ListBox					aLbCurrency;
	SvxFontListBox			aLbFormat;
	FixedText				aFtLanguage;
	SvxLanguageBox			aLbLanguage;
	CheckBox                aCbSourceFormat;

	FixedText				aFtDecimals;
	NumericField			aEdDecimals;
	FixedText				aFtLeadZeroes;
	NumericField			aEdLeadZeroes;
	CheckBox				aBtnNegRed;
	CheckBox				aBtnThousand;
	FixedLine               aFlOptions;

	FixedText				aFtEdFormat;
	Edit					aEdFormat;
	ImageButton				aIbAdd;
	ImageButton				aIbInfo;
	ImageButton				aIbRemove;
	ImageList				aIconList;

	FixedText				aFtComment;
	Edit					aEdComment;
	Timer					aResetWinTimer;

	SvxNumberPreviewImpl	aWndPreview;

	SvxNumberInfoItem*		pNumItem;
	SvxNumberFormatShell* 	pNumFmtShell;
	ULONG					nInitFormat;
	Link					fnOkHdl;

	BOOL					bNumItemFlag; 
	BOOL					bOneAreaFlag;
	short					nFixedCategory;

	long					nCatHeight;

	long					nCurFormatY;
	long					nCurFormatHeight;
	long					nStdFormatY;
	long					nStdFormatHeight;
	String					aStrEurope;

	Window*					pLastActivWindow;

#ifdef _SVX_NUMFMT_CXX
	void 	Init_Impl();
	void	FillCurrencyBox();
	void 	FillFormatListBox_Impl( SvxDelStrgs& rEntries );
	void 	UpdateOptions_Impl( BOOL bCheckCatChange );
	void	UpdateFormatListBox_Impl( USHORT bCat, BOOL bUpdateEdit );
	void	DeleteEntryList_Impl( SvxDelStrgs& rEntries );
	void	Obstructing();
	void    EnableBySourceFormat_Impl();
	void	SetCategory(USHORT nPos);
	String	GetExpColorString(Color *aPreviewColor,String aFormatStr,
					short nTmpCatPos);

	DECL_LINK( LostFocusHdl_Impl, Edit* pEd );
	DECL_LINK( DoubleClickHdl_Impl, SvxFontListBox* pLb );
	DECL_LINK( SelFormatHdl_Impl, void * );
	DECL_LINK( ClickHdl_Impl, ImageButton* pIB );
	DECL_LINK( EditHdl_Impl, Edit* pEdFormat );
	DECL_LINK( OptHdl_Impl, void * );
	DECL_LINK( TimeHdl_Impl, Timer * );

#endif
};

#endif

