
#ifndef _SVX_NUMVSET_HXX
#define _SVX_NUMVSET_HXX

#ifndef _VALUESET_HXX 
#include <svtools/valueset.hxx>
#endif
#include <limits.h>
#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif
#ifndef _COM_SUN_STAR_LANG_LOCALE_HPP_
#include <com/sun/star/lang/Locale.hpp>
#endif

namespace com{namespace sun{ namespace star{
	namespace container{
		class XIndexAccess;
	}
	namespace beans{
		struct PropertyValue;
	}
	namespace text{
		class XNumberingFormatter;
	}
}}}

class SvxBrushItem;
class SvxNumRule;
struct SvxBmpItemInfo
{
	SvxBrushItem* 	pBrushItem;
	USHORT 			nItemId;
};

class SvxNumValueSet : public ValueSet
{
	Color 			aLineColor;
	USHORT 			nPageType;
	BOOL			bHTMLMode;
	Rectangle		aOrgRect;
	VirtualDevice* 	pVDev;

	com::sun::star::uno::Reference<com::sun::star::text::XNumberingFormatter> xFormatter;
	com::sun::star::lang::Locale aLocale;

	com::sun::star::uno::Sequence<
		com::sun::star::uno::Sequence<
			com::sun::star::beans::PropertyValue> > aNumSettings;

	com::sun::star::uno::Sequence<
		com::sun::star::uno::Reference<
			com::sun::star::container::XIndexAccess> > aOutlineSettings;
	public:
		SvxNumValueSet( Window* pParent, const ResId& rResId, USHORT nType );
		~SvxNumValueSet();

	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

	void			SetHTMLMode(BOOL bSet) {bHTMLMode = bSet;}
	void			SetNumberingSettings(
		const com::sun::star::uno::Sequence<
			com::sun::star::uno::Sequence<
	com::sun::star::beans::PropertyValue> >& aNum,
		com::sun::star::uno::Reference<com::sun::star::text::XNumberingFormatter>& xFormatter,
		const com::sun::star::lang::Locale& rLocale	);

	void			SetOutlineNumberingSettings(
			com::sun::star::uno::Sequence<
	com::sun::star::uno::Reference<
		com::sun::star::container::XIndexAccess> >& rOutline,
			com::sun::star::uno::Reference<com::sun::star::text::XNumberingFormatter>& xFormatter,
			const com::sun::star::lang::Locale& rLocale);
};

class SvxBmpNumValueSet : public SvxNumValueSet
{
	const List& rStrList;
	List		aGrfBrushItems;
	Timer 		aFormatTimer;
	BOOL		bGrfNotFound;

	protected:
		DECL_STATIC_LINK(SvxBmpNumValueSet, GraphicArrivedHdl_Impl, SvxBrushItem*);
		DECL_LINK(FormatHdl_Impl, Timer*);

	void			SetGrfNotFound(BOOL bSet) {bGrfNotFound = bSet;}
	BOOL			IsGrfNotFound()const {return bGrfNotFound;}

	const List&		GetStringList(){return rStrList;}
	List& 			GetGrfBrushItems() {return aGrfBrushItems;}

	Timer&			GetFormatTimer() {return aFormatTimer;}

	public:
		SvxBmpNumValueSet( Window* pParent, const ResId& rResId, const List& rStrNames);
		~SvxBmpNumValueSet();

	SvxBmpItemInfo* FindInfo(USHORT nInfo);
	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

};

class SvxNumberingPreview : public Window
{
	const SvxNumRule* 	pActNum;
	Font 				aStdFont;
	long				nPageWidth;
	const String* 		pOutlineNames;
	BOOL				bPosition;
	UINT16 				nActLevel;

	protected:
		virtual void		Paint( const Rectangle& rRect );

	public:
		SvxNumberingPreview(Window* pParent, const ResId& rResId ) :
			Window(pParent, rResId),
	        pActNum(0),nPageWidth(0), pOutlineNames(0), bPosition(FALSE), nActLevel(USHRT_MAX)
	        { SetBorderStyle( WINDOW_BORDER_MONO ); }

		void	SetNumRule(const SvxNumRule* pNum)
		{pActNum = pNum; Invalidate();};
		void	SetPageWidth(long nPgWidth)
					{nPageWidth = nPgWidth;}
		void	SetOutlineNames(const String* pNames)
			{pOutlineNames = pNames;}
		void	SetPositionMode()
			{ bPosition = TRUE;}
		void	SetLevel(USHORT nSet) {nActLevel = nSet;}

};

#endif

