
#ifndef _SVX_PAGE_HXX
#define _SVX_PAGE_HXX

#ifndef _SFXTABDLG_HXX 
#include <sfx2/tabdlg.hxx>
#endif

#ifndef _FIELD_HXX 
#include <vcl/field.hxx>
#endif

#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif

#ifndef _GROUP_HXX 
#include <vcl/group.hxx>
#endif

#ifndef _LSTBOX_HXX 
#include <vcl/lstbox.hxx>
#endif

#ifndef _STDCTRL_HXX 
#include <svtools/stdctrl.hxx>
#endif

#ifndef _SVX_PAGECTRL_HXX 
#include <svx/pagectrl.hxx>
#endif

#include <svx/svxenum.hxx>

enum SvxModeType
{
	SVX_PAGE_MODE_STANDARD,
	SVX_PAGE_MODE_CENTER,
	SVX_PAGE_MODE_PRESENTATION
};

class SvxPageDescPage : public SfxTabPage
{
	 FixedLine           aPaperSizeFl;
	FixedText           aPaperFormatText;
	ListBox             aPaperSizeBox;
	 FixedText           aPaperWidthText;
	MetricField         aPaperWidthEdit;
	FixedText           aPaperHeightText;
	MetricField         aPaperHeightEdit;
	 FixedText           aOrientationFT;
	RadioButton         aPortraitBtn;
	RadioButton         aLandscapeBtn;
	 SvxPageWindow       aBspWin;
	 FixedText           aPaperTrayLbl;
	ListBox             aPaperTrayBox;
	 FixedLine           aMarginFl;
	FixedText           aLeftMarginLbl;
	MetricField         aLeftMarginEdit;
	FixedText           aRightMarginLbl;
	MetricField         aRightMarginEdit;
	FixedText           aTopMarginLbl;
	MetricField         aTopMarginEdit;
	FixedText           aBottomMarginLbl;
	MetricField         aBottomMarginEdit;
	 FixedLine           aLayoutFL;
	ListBox             aLayoutBox;
	FixedText           aPageText;
	ListBox             aNumberFormatBox;
	FixedText			aNumberFormatText;
	 FixedLine           aBottomSeparatorFl;
	 FixedText           aTblAlignFT;
	CheckBox            aHorzBox;
	CheckBox            aVertBox;

	CheckBox            aAdaptBox;

	CheckBox			aRegisterCB;
	FixedText			aRegisterFT;
	ListBox				aRegisterLB;

	String				sStandardRegister;

	String              aInsideText;
	String              aOutsideText;
	String              aLeftText;
	String              aRightText;
	String				aPrintRangeQueryText;

	long				nFirstLeftMargin;
	long				nFirstRightMargin;
	long				nFirstTopMargin;
	long				nFirstBottomMargin;
	long				nLastLeftMargin;
	long				nLastRightMargin;
	long				nLastTopMargin;
	long				nLastBottomMargin;

	Size                aMaxSize;
	BOOL                bLandscape;
	FASTBOOL            bBorderModified;
	SvxModeType         eMode;
	SvxPaper            ePaperStart;
	SvxPaper            ePaperEnd;

#ifdef _SVX_PAGE_CXX
	void                Init_Impl();
	DECL_LINK( LayoutHdl_Impl, ListBox* );
	DECL_LINK( PaperBinHdl_Impl, ListBox* );
	DECL_LINK( SwapOrientation_Impl, RadioButton* );
	void                SwapFirstValues_Impl( FASTBOOL bSet );
	DECL_LINK( BorderModify_Impl, MetricField* );
	void                InitHeadFoot_Impl( const SfxItemSet& rSet );
	DECL_LINK( CenterHdl_Impl, CheckBox* );
	void                UpdateExample_Impl();

	DECL_LINK( PaperSizeSelect_Impl, ListBox* );
	DECL_LINK( PaperSizeModify_Impl, Edit* );

	void                ResetBackground_Impl( const SfxItemSet& rSet );

	DECL_LINK( RangeHdl_Impl, Edit* );
	void                CalcMargin_Impl();

	DECL_LINK( RegisterModify, CheckBox * );
#endif
	SvxPageDescPage( Window* pParent, const SfxItemSet& rSet );

protected:
	virtual void        ActivatePage( const SfxItemSet& rSet );
	virtual int         DeactivatePage( SfxItemSet* pSet = 0 );

public:
	static SfxTabPage*  Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*      GetRanges();

	virtual BOOL        FillItemSet( SfxItemSet& rOutSet );
	virtual void        Reset( const SfxItemSet& rSet );
	virtual void        FillUserData();

	virtual ~SvxPageDescPage();

	void                SetMode( SvxModeType eMType ) { eMode = eMType; }
	void                SetPaperFormatRanges( SvxPaper eStart, SvxPaper eEnd )
				{ ePaperStart = eStart, ePaperEnd = eEnd; }

	void				SetCollectionList(const List* pList);

};

#endif

