
#ifndef _SVX_PARAGRPH_HXX
#define _SVX_PARAGRPH_HXX

#ifndef _BUTTON_HXX 
#include <vcl/button.hxx>
#endif
#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif
#ifndef _SFXTABDLG_HXX 
#include <sfx2/tabdlg.hxx>
#endif
#ifndef _SVX_RELFLD_HXX
#include <svx/relfld.hxx>		
#endif
#ifndef _SVX_PARAPREV_HXX
#include <svx/paraprev.hxx>		
#endif

#ifndef _LSTBOX_HXX 
#include <vcl/lstbox.hxx>
#endif

#define MM50   283

class SvxLineSpacingItem;
class SvxParagraphControllerItem;

class SvxStdParagraphTabPage: public SfxTabPage
{
private:
	SvxStdParagraphTabPage( Window* pParent, const SfxItemSet& rSet );

	FixedText				aLeftLabel;
	SvxRelativeField		aLeftIndent;

	FixedText				aRightLabel;
	SvxRelativeField		aRightIndent;

	FixedText				aFLineLabel;
	SvxRelativeField		aFLineIndent;
	CheckBox				aAutoCB;
	 FixedLine               aIndentFrm;

	FixedText				aTopLabel;
	SvxRelativeField		aTopDist;
	FixedText				aBottomLabel;
	SvxRelativeField		aBottomDist;
	FixedLine               aDistFrm;

	ListBox					aLineDist;
	FixedText				aLineDistAtLabel;
	MetricField				aLineDistAtPercentBox;
	MetricField				aLineDistAtMetricBox;
	FixedLine               aLineDistFrm;
	String 					sAbsDist;
	SvxParaPrevWindow		aExampleWin;

	CheckBox				aRegisterCB;
	FixedLine               aRegisterFL;

	Edit*					pActLineDistFld;
	long					nAbst;
	long					nWidth;
	long 					nMinFixDist;
	BOOL					bRelativeMode;
	BOOL                    bNegativeIndents;

#ifdef _SVX_PARAGRPH_CXX
	void					SetLineSpacing_Impl( const SvxLineSpacingItem& rAttr );
	void					Init_Impl();
	void					UpdateExample_Impl( BOOL bAll = FALSE );

	DECL_LINK( LineDistHdl_Impl, ListBox* );
	DECL_LINK( ModifyHdl_Impl, SvxRelativeField* );
	DECL_LINK( AutoHdl_Impl, CheckBox* );
#endif

protected:
	virtual int				DeactivatePage( SfxItemSet* pSet = 0 );

public:
	DECL_LINK( ELRLoseFocusHdl, Edit* );

	static SfxTabPage*		Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*			GetRanges();

	virtual BOOL			FillItemSet( SfxItemSet& rSet );
	virtual void			Reset( const SfxItemSet& rSet );

	void					SetPageWidth( USHORT nPageWidth );
	void					SetMaxDistance( USHORT nMaxDist );
	void					EnableRelativeMode();
	void 					EnableRegisterMode();
	void					EnableAutoFirstLine();
	void					EnableAbsLineDist(long nMinTwip);
	void					EnableNegativeMode();
};

class SvxParaAlignTabPage : public SfxTabPage
{
	 FixedLine               aAlignFrm;
	RadioButton             aLeft;
	RadioButton				aRight;
	RadioButton				aCenter;
	RadioButton				aJustify;
	FixedText				aLastLineFT;
	ListBox				    aLastLineLB;
	CheckBox				aExpandCB;

	SvxParaPrevWindow		aExampleWin;
	 FixedLine               aVertAlignFL;
	ListBox                 aVertAlignLB;

#ifdef _SVX_PARAGRPH_CXX
	DECL_LINK( AlignHdl_Impl, RadioButton* );
	DECL_LINK( LastLineHdl_Impl, ListBox* );

	void					UpdateExample_Impl( BOOL bAll = FALSE );
#endif

	SvxParaAlignTabPage( Window* pParent, const SfxItemSet& rSet );
	~SvxParaAlignTabPage();

protected:
	virtual int				DeactivatePage( SfxItemSet* pSet = 0 );

public:
	static SfxTabPage*		Create( Window* pParent, const SfxItemSet& rSet );
	static USHORT*			GetRanges();

	virtual BOOL			FillItemSet( SfxItemSet& rSet );
	virtual void			Reset( const SfxItemSet& rSet );

	void					EnableJustifyExt();
};

class SvxExtParagraphTabPage: public SfxTabPage
{
public:
	~SvxExtParagraphTabPage();

	static SfxTabPage*	Create(	Window* pParent,
					const SfxItemSet& rSet );
	static USHORT*		GetRanges();

	virtual BOOL		FillItemSet( SfxItemSet& rSet );
	virtual void		Reset( const SfxItemSet& rSet );

	void				DisablePageBreak();

protected:
	virtual int			DeactivatePage( SfxItemSet* pSet = 0 );

private:
	SvxExtParagraphTabPage( Window* pParent, const SfxItemSet& rSet );

	TriStateBox			aHyphenBox;
	FixedText			aBeforeText;
	NumericField		aExtHyphenBeforeBox;
	FixedText			aHyphenBeforeLabel;
	FixedText			aAfterText;
	NumericField		aExtHyphenAfterBox;
	FixedText			aHyphenAfterLabel;
	FixedText     		aMaxHyphenLabel;
	NumericField        aMaxHyphenEdit;
	FixedLine           aExtFL;

	TriStateBox			aPageBreakBox;
	RadioButton			aPageBox;
	RadioButton			aColumnBox;
	RadioButton			aBeforeBox;
	RadioButton			aAfterBox;
	TriStateBox			aApplyCollBtn;
	ListBox				aApplyCollBox;
	FixedText           aPagenumText;
	NumericField       	aPagenumEdit;

	TriStateBox			aKeepTogetherBox;
	TriStateBox			aKeepParaBox;

	TriStateBox			aOrphanBox;
	NumericField       	aOrphanRowNo;
	FixedText			aOrphanRowLabel;

	TriStateBox			aWidowBox;
	NumericField       	aWidowRowNo;
	FixedText			aWidowRowLabel;
	 FixedLine           aExtendFL;

	BOOL				bPageBreak;
	BOOL				bHtmlMode;
	USHORT				nStdPos;

#ifdef _SVX_PARAGRPH_CXX
	DECL_LINK( PageBreakHdl_Impl, TriStateBox* );
	DECL_LINK( KeepTogetherHdl_Impl, TriStateBox* );
	DECL_LINK( WidowHdl_Impl, TriStateBox* );
	DECL_LINK( OrphanHdl_Impl, TriStateBox* );
	DECL_LINK( HyphenClickHdl_Impl, TriStateBox* );
	DECL_LINK( ApplyCollClickHdl_Impl, TriStateBox* );
	DECL_LINK( PageBreakPosHdl_Impl, RadioButton* );
	DECL_LINK( PageBreakTypeHdl_Impl, RadioButton* );
#endif
};

class SvxAsianTabPage : public SfxTabPage
{
	FixedLine       aOptionsFL;

	TriStateBox		aForbiddenRulesCB;
	TriStateBox		aAllowWordBreakCB;
	TriStateBox		aHangingPunctCB;
	 FixedLine       aCharDistFL;
	TriStateBox		aPuntuationCB;
	TriStateBox		aScriptSpaceCB;
	TriStateBox		aAdjustNumbersCB;

	FixedText		aTextAlignFT;
	ListBox			aTextAlignLB;

	SvxAsianTabPage( Window* pParent, const SfxItemSet& rSet );

#ifdef _SVX_PARAGRPH_CXX
	DECL_LINK( ClickHdl_Impl, TriStateBox* );
#endif
public:
	~SvxAsianTabPage();

	static SfxTabPage*	Create(	Window* pParent,
					const SfxItemSet& rSet );
	static USHORT*		GetRanges();

	virtual BOOL		FillItemSet( SfxItemSet& rSet );
	virtual void		Reset( const SfxItemSet& rSet );

};

#endif 

