
#ifndef _SVX_RULER_HXX
#define _SVX_RULER_HXX

#ifndef _SV_MENU_HXX 
#include <vcl/menu.hxx>
#endif

#ifndef _RULER_HXX 
#include <svtools/ruler.hxx>
#endif

#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif

class SvxProtectItem;
class SvxRulerItem;
class SfxBindings;
class SvxLongLRSpaceItem;
class SvxLongULSpaceItem;
class SvxTabStopItem;
class SvxLRSpaceItem;
class SvxPagePosSizeItem;
class SvxColumnItem;
class SfxRectangleItem;
class SvxObjectItem;
struct SvxRuler_Impl;

class SvxRuler: public Ruler, public SfxListener
{
friend class SvxRulerItem;
	SvxRulerItem **pCtrlItem;
	SvxLongLRSpaceItem *pLRSpaceItem;  
	SfxRectangleItem *pMinMaxItem; 
	SvxLongULSpaceItem *pULSpaceItem;  
	SvxTabStopItem *pTabStopItem;  
	SvxLRSpaceItem *pParaItem;     
	SvxPagePosSizeItem *pPagePosItem; 
	SvxColumnItem *pColumnItem;    
	SvxObjectItem *pObjectItem;    
	Window *pEditWin;
	SvxRuler_Impl *pRuler_Imp;
	BOOL bAppSetNullOffset :1;
	BOOL bHorz :1;
	long lLogicNullOffset;      
	long lAppNullOffset;        
	long lMinFrame;             
	long lInitialDragPos;
	USHORT nFlags;
	enum {
		NONE = 0x0000,
		DRAG_OBJECT =  0x0001,

		DRAG_OBJECT_SIZE_LINEAR = 0x0002,
		DRAG_OBJECT_SIZE_PROPORTIONAL = 0x0004, 

		DRAG_OBJECT_ACTLINE_ONLY = 0x0008,

		DRAG_OBJECT_LEFT_INDENT_ONLY = DRAG_OBJECT_SIZE_PROPORTIONAL
		}
	nDragType;
	USHORT nDefTabType;
	USHORT nTabCount;
	USHORT nTabBufSize;
	long lDefTabDist;
	long lTabPos;
	RulerTab *pTabs;            
	RulerIndent *pIndents;      
	RulerBorder *pBorders;
	USHORT nBorderCount;
	RulerBorder *pObjectBorders;
	SfxBindings *pBindings;
	long nDragOffset;
	long nMaxLeft;
	long nMaxRight;
	BOOL bValid;
	BOOL bListening;
	BOOL bActive;
#if 0 // _SOLAR__PRIVATE
	long GetCorrectedDragPos( BOOL bLeft = TRUE, BOOL bRight = TRUE );
	void DrawLine_Impl(long &lTabPos, int, BOOL Hori=TRUE);
	USHORT GetObjectBordersOff(USHORT nIdx) const;

	void UpdateFrame(const SvxLongLRSpaceItem *);
	void UpdateFrame(const SvxLongULSpaceItem *);
	void UpdateFrameMinMax(const SfxRectangleItem *);

	void UpdatePara(const SvxLRSpaceItem *);

	void Update(const SvxTabStopItem *);

	void Update(const SvxPagePosSizeItem *);

	void Update(const SvxColumnItem *);

	void Update(const SvxObjectItem *);

	void Update( const SvxProtectItem* );

	void UpdatePara();
	void UpdateTabs();
	void UpdatePage();
	void UpdateFrame();
	void UpdateColumns();
	void UpdateObject();

	long PixelHAdjust(long lPos, long lPos2) const;
	long PixelVAdjust(long lPos, long lPos2) const;
	long PixelAdjust(long lPos, long lPos2) const;

	long ConvertHPosPixel(long lPos) const;
	long ConvertVPosPixel(long lPos) const;
	long ConvertHSizePixel(long lSize) const;
	long ConvertVSizePixel(long lSize) const;

	long ConvertPosPixel(long lPos) const;
	long ConvertSizePixel(long lSize) const;

	long ConvertHPosLogic(long lPos) const;
	long ConvertVPosLogic(long lPos) const;
	long ConvertHSizeLogic(long lSize) const;
	long ConvertVSizeLogic(long lSize) const;

	long ConvertPosLogic(long lPos) const;
	long ConvertSizeLogic(long lSize) const;

	long GetFirstLineIndent() const;

	long GetLeftIndent() const;

	long GetRightIndent() const;
	long GetLogicRightIndent() const;
	long GetPageWidth() const;

	inline long GetLeftFrameMargin() const;
	long GetRightFrameMargin() const;

	void CalcMinMax();

	void EvalModifier();
	void DragMargin1();
	void DragMargin2();
	void DragIndents();
	void DragTabs();
	void DragBorders();
	void DragObjectBorder();

	void ApplyMargins();
	void ApplyIndents();
	void ApplyTabs();
	void ApplyBorders();
	void ApplyObject();

	long GetFrameLeft() const;
	void SetFrameLeft(long);

	long GetLeftMin() const;
	long GetRightMax() const;

	DECL_LINK( TabMenuSelect, Menu * );
	DECL_LINK( MenuSelect, Menu * );
	void PrepareProportional_Impl(RulerType);

	USHORT GetNextVisible(USHORT nColumn);
	USHORT GetPrevVisible(USHORT nColumn);
#endif

	void Update();

enum UpdateType {
	MOVE_ALL,
	MOVE_LEFT,
	MOVE_RIGHT
	};
	void UpdateParaContents_Impl(long lDiff, UpdateType = MOVE_ALL);
protected:
	virtual void    Command( const CommandEvent& rCEvt );
	virtual void    Click();
	virtual long    StartDrag();
	virtual void    Drag();
	virtual void    EndDrag();
	virtual void    ExtraDown();
	virtual void	MouseMove( const MouseEvent& rMEvt );

	virtual void SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
			 const SfxHint& rHint, const TypeId& rHintType );

	virtual BOOL    CalcLimits(long &nMax1, long &nMax2, BOOL bFirst) const;
	BOOL IsActLastColumn(
		BOOL bForceDontConsiderHidden = FALSE, USHORT nAct=USHRT_MAX) const ;
	BOOL IsActFirstColumn(
		BOOL bForceDontConsiderHidden = FALSE, USHORT nAct=USHRT_MAX) const;
	USHORT GetActLeftColumn(
		BOOL bForceDontConsiderHidden = FALSE, USHORT nAct=USHRT_MAX ) const;
	USHORT GetActRightColumn (
		BOOL bForceDontConsiderHidden = FALSE, USHORT nAct=USHRT_MAX ) const;
	long CalcPropMaxRight(USHORT nCol=USHRT_MAX) const;

public:
#define     SVXRULER_SUPPORT_TABS                   0x0001
#define     SVXRULER_SUPPORT_PARAGRAPH_MARGINS      0x0002
#define     SVXRULER_SUPPORT_BORDERS                0x0004
#define     SVXRULER_SUPPORT_OBJECT                 0x0008
#define     SVXRULER_SUPPORT_SET_NULLOFFSET         0x0010
#define     SVXRULER_SUPPORT_NEGATIVE_MARGINS       0x0020

	SvxRuler(Window* pParent,
	Window *pEditWin,
	USHORT nRulerFlags,
	SfxBindings &rBindings,
	WinBits nWinStyle = WB_STDRULER);
	~SvxRuler();

	void SetMinFrameSize(long lSize);
	long GetMinFrameSize() const ;

	USHORT GetRulerFlags() const { return nFlags; }

	void SetDefTabDist(long);
	long GetDefTabDist() const;

	void SetNullOffsetLogic(long lOff = 0);
	long GetNullOffsetLogic() const { return lAppNullOffset; }

	void SetActive(BOOL bOn = TRUE);
	BOOL IsActive() const { return bActive; }

	void ForceUpdate() { Update(); }
};

#endif

