
#ifndef _SVX_SRCHDLG_HXX
#define _SVX_SRCHDLG_HXX

#ifndef _STDCTRL_HXX 
#include <svtools/stdctrl.hxx>
#endif
#ifndef _COMBOBOX_HXX 
#include <vcl/combobox.hxx>
#endif
#ifndef _EDIT_HXX 
#include <vcl/edit.hxx>
#endif
#ifndef _LSTBOX_HXX 
#include <vcl/lstbox.hxx>
#endif
#ifndef _BUTTON_HXX 
#include <vcl/button.hxx>
#endif
#ifndef _GROUP_HXX 
#include <vcl/group.hxx>
#endif
#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif
#ifndef _DIALOG_HXX 
#include <vcl/dialog.hxx>
#endif
#ifndef _SFX_CHILDWIN_HXX 
#include <sfx2/childwin.hxx>
#endif
#ifndef _BASEDLGS_HXX
#include <sfx2/basedlgs.hxx>
#endif
#ifndef _SVARRAY_HXX 
#include <svtools/svarray.hxx>
#endif
#define _SVSTDARR_STRINGSDTOR
#include <svtools/svstdarr.hxx>

#ifndef _SVEDIT_HXX 
#include <svtools/svmedit.hxx>
#endif

#ifndef _SFX_SRCHDEFS_HXX_
#include <sfx2/srchdefs.hxx>
#endif

class SvxSearchItem;
class MoreButton;
class SfxStyleSheetBasePool;
class SvxJSearchOptionsPage;

struct SearchDlg_Impl;

#ifndef NO_SVX_SEARCH

struct SearchAttrItem
{
	USHORT 			nSlot;
	SfxPoolItem* 	pItem;
};

SV_DECL_VARARR(SrchAttrItemList, SearchAttrItem, 8, 8);

class SearchAttrItemList : private SrchAttrItemList
{
public:
	SearchAttrItemList() {}
	SearchAttrItemList( const SearchAttrItemList& rList );
	~SearchAttrItemList();

	void            Put( const SfxItemSet& rSet );
	SfxItemSet&     Get( SfxItemSet& rSet );
	void            Clear();
	USHORT          Count() const { return SrchAttrItemList::Count(); }
	SearchAttrItem& operator[](USHORT nPos) const
			{ return SrchAttrItemList::operator[]( nPos ); }
	SearchAttrItem& GetObject( USHORT nPos ) const
			{ return SrchAttrItemList::GetObject( nPos ); }

	void Insert( const SearchAttrItem& rItem )
		{ SrchAttrItemList::Insert( rItem, SrchAttrItemList::Count() ); }

	void Remove( USHORT nPos, USHORT nLen = 1 );
};

#ifndef SV_NODIALOG

class SvxSearchDialogWrapper : public SfxChildWindow
{
public:
	SvxSearchDialogWrapper( Window*pParent, USHORT nId,
				SfxBindings* pBindings, SfxChildWinInfo* pInfo );

	SFX_DECL_CHILDWINDOW(SvxSearchDialogWrapper);
};

class SvxSearchDialog : public ModelessDialog
{
	friend class SvxSearchController;
	friend class SvxSearchDialogWrapper;
	friend class SvxJSearchOptionsDialog;

public:
	SvxSearchDialog( Window* pParent, SfxBindings& rBind );
	~SvxSearchDialog();

	virtual BOOL    Close();

	virtual void	Activate();

	void            GetSearchItems( SfxItemSet& rSet );
	void            GetReplaceItems( SfxItemSet& rSet );

	const SearchAttrItemList*   GetSearchItemList() const
						{ return pSearchList; }
	const SearchAttrItemList*   GetReplaceItemList() const
						{ return pReplaceList; }

	inline BOOL 	HasSearchAttributes() const;
	inline BOOL 	HasReplaceAttributes() const;

	PushButton&     GetReplaceBtn() { return aReplaceBtn; }

	INT32			GetTransliterationFlags() const;

private:
	FixedText       aSearchText;
	ComboBox        aSearchLB;
	ListBox         aSearchTmplLB;
	FixedInfo       aSearchAttrText;
	MultiLineEdit 	aSearchFormatsED;

	FixedText       aReplaceText;
	ComboBox        aReplaceLB;
	ListBox         aReplaceTmplLB;
	FixedInfo       aReplaceAttrText;
	MultiLineEdit 	aReplaceFormatsED;

	PushButton      aSearchAllBtn;
	PushButton      aSearchBtn;
	PushButton      aReplaceAllBtn;
	PushButton      aReplaceBtn;
	PushButton      aAttributeBtn;
	CancelButton    aCloseBtn;
	PushButton      aFormatBtn;
	HelpButton      aHelpBtn;
	PushButton      aNoFormatBtn;
	MoreButton*     pMoreBtn;

	CheckBox        aWordBtn;
	CheckBox        aMatchCaseCB;
	CheckBox        aBackwardsBtn;
	CheckBox        aSelectionBtn;
	CheckBox        aRegExpBtn;
	CheckBox        aLayoutBtn;

	CheckBox        aSimilarityBox;
	PushButton      aSimilarityBtn;
	CheckBox		aJapMatchFullHalfWidthCB;
	CheckBox		aJapOptionsCB;
	PushButton      aJapOptionsBtn;
	FixedLine       aOptionsFL;

	RadioButton     aFormulasBtn;
	RadioButton     aValuesBtn;
	RadioButton     aNotesBtn;
	FixedLine       aSearchFL;
	FixedLine       aSearchVertFL;

	RadioButton     aRowsBtn;
	RadioButton     aColumnsBtn;
	FixedLine       aSearchDirFL;
	FixedLine       aSearchDirVertFL;

	CheckBox        aAllTablesCB;
	FixedLine       aCalcExtrasFL;

	SfxBindings&    rBindings;
	BOOL            bWriter;
	BOOL            bSearch;
	BOOL            bFormat;
	USHORT          nOptions;
	FASTBOOL		bSet;
	FASTBOOL		bReadOnly;
	FASTBOOL		bConstruct;
	ULONG			nModifyFlag;
	String          aStylesStr;
	String			aLayoutStr;
	String			aCalcStr;

	SvStringsDtor   aSearchStrings;
	SvStringsDtor   aReplaceStrings;

	SearchDlg_Impl*			pImpl;
	SearchAttrItemList*     pSearchList;
	SearchAttrItemList*     pReplaceList;
	SvxSearchItem*          pSearchItem;

	SvxSearchController*    pSearchController;
	SvxSearchController*    pOptionsController;
	SvxSearchController*    pFamilyController;
	SvxSearchController*    pSearchSetController;
	SvxSearchController*    pReplaceSetController;

	INT32			nTransliterationFlags;

#ifdef _SVX_SRCHDLG_CXX
	DECL_LINK( ModifyHdl_Impl, ComboBox* pEdit );
	DECL_LINK( FlagHdl_Impl, Button* pBtn );
	DECL_LINK( CommandHdl_Impl, Button* pBtn );
	DECL_LINK( TemplateHdl_Impl, Button* );
	DECL_LINK( FocusHdl_Impl, Control* );
	DECL_LINK( LoseFocusHdl_Impl, Control* );
	DECL_LINK( FormatHdl_Impl, Button* );
	DECL_LINK( NoFormatHdl_Impl, Button* );
	DECL_LINK( AttributeHdl_Impl, Button* );
	DECL_LINK( TimeoutHdl_Impl, Timer* );

	void            InitControls_Impl();
	void            Init_Impl( int bHasItemSet );
	void            InitAttrList_Impl( const SfxItemSet* pSSet,
						   const SfxItemSet* pRSet );
	void            Remember_Impl( const String &rStr,BOOL bSearch );
	void            PaintAttrText_Impl();
	String&         BuildAttrText_Impl( String& rStr, BOOL bSrchFlag ) const;

	void            TemplatesChanged_Impl( SfxStyleSheetBasePool& rPool );
	void            EnableControls_Impl( const USHORT nFlags );
	void            EnableControl_Impl( Control* pCtrl );
	void            SetItem_Impl( const SvxSearchItem* pItem );

	void			SetModifyFlag_Impl( const Control* pCtrl );
	void			SaveToModule_Impl();

	void			ApplyTransliterationFlags_Impl( INT32 nSettings );
#endif
};

inline BOOL SvxSearchDialog::HasSearchAttributes() const
{
	int bLen = aSearchAttrText.GetText().Len();
	return ( aSearchAttrText.IsEnabled() && bLen );
}

inline BOOL SvxSearchDialog::HasReplaceAttributes() const
{
	int bLen = aReplaceAttrText.GetText().Len();
	return ( aReplaceAttrText.IsEnabled() && bLen );
}

class SvxJSearchOptionsDialog : public SfxSingleTabDialog
{
	INT32					nInitialTlFlags;
	SvxJSearchOptionsPage  *pPage;

	SvxJSearchOptionsDialog( const SvxJSearchOptionsDialog & );
	SvxJSearchOptionsDialog & operator == ( const SvxJSearchOptionsDialog & );

public:
	SvxJSearchOptionsDialog( Window *pParent,
				const SfxItemSet& rOptionsSet, USHORT nUniqueId,
				INT32 nInitialFlags  );
	virtual ~SvxJSearchOptionsDialog();

	virtual void	Activate();

	INT32			GetTransliterationFlags() const;
	void 			SetTransliterationFlags( INT32 nSettings );
};

#endif  
#endif  

#endif

