

#ifndef _SVDDRGV_HXX
#define _SVDDRGV_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdxcgv.hxx>

class SdrUndoGeoObj;

class SdrDragView: public SdrExchangeView
{
	friend class				SdrPageView;
	friend class				SdrDragMethod;

protected:
	SdrHdl*						pDragHdl;
	SdrDragMethod*				pDragBla;
	SdrUndoGeoObj*				pInsPointUndo;

	Rectangle					aDragLimit;
	XubString					aInsPointUndoStr;
	SdrMarkList					aFollowingEdges; 

	SdrHdlKind					eDragHdl;

	ULONG						nDragXorPolyLimit;
	ULONG						nDragXorPointLimit;
	USHORT						nRubberEdgeDraggingLimit;
	USHORT						nDetailedEdgeDraggingLimit;

	unsigned					bFramDrag : 1;        
	unsigned					bDragSpecial : 1;     
	unsigned					bMarkedHitMovesAlways : 1; 
	unsigned					bDragLimit : 1;      
	unsigned					bDragHdl : 1;        
	unsigned					bDragStripes : 1;    
	unsigned					bNoDragHdl : 1;      
	unsigned					bMirrRefDragObj : 1; 
	unsigned					bSolidDragging : 1;  
	unsigned					bSolidDrgNow : 1;    
	unsigned					bSolidDrgChk : 1;    
	unsigned					bMouseHideWhileDraggingPoints : 1;
	unsigned					bResizeAtCenter : 1;
	unsigned					bCrookAtCenter : 1;
	unsigned					bDragWithCopy : 1;
	unsigned					bInsAfter : 1;       
	unsigned					bInsGluePoint : 1;
	unsigned					bInsObjPointMode : 1;
	unsigned					bInsGluePointMode : 1;
	unsigned					bNoDragXorPolys : 1;
	unsigned					bAutoVertexCon : 1;  
	unsigned					bAutoCornerCon : 1;  
	unsigned					bRubberEdgeDragging : 1;
	unsigned					bDetailedEdgeDragging : 1;

private:
	void ImpClearVars();
	void ImpMakeDragAttr();
	void ImpDelDragAttr();

protected:

	void ImpDrawEdgeXor(ExtOutputDevice& rXOut, BOOL bFull) const;
	void SetDragPolys(BOOL bReset=FALSE, BOOL bSeparate=FALSE);
	virtual void ShowDragObj(OutputDevice* pOut);
	virtual void HideDragObj(OutputDevice* pOut);
	virtual void DrawDragObj(OutputDevice* pOut, BOOL bFull) const;
	BOOL BegInsObjPoint(BOOL bIdxZwang, USHORT nIdx, const Point& rPnt, BOOL bNewObj, OutputDevice* pOut, short nMinMov);
	virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead, SvStream& rIn);

public:
	SdrDragView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrDragView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrDragView();

	virtual void ToggleShownXor(OutputDevice* pOut, const Region* pRegion) const;
	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BckAction();
	virtual void BrkAction();
	virtual void TakeActionRect(Rectangle& rRect) const;

	BOOL TakeDragObjAnchorPos(Point& rPos) const;

	virtual BOOL BegDragObj(const Point& rPnt, OutputDevice* pOut=NULL, SdrHdl* pHdl=NULL, short nMinMov=-3, SdrDragMethod* pForcedMeth=NULL);
	void MovDragObj(const Point& rPnt);
	BOOL EndDragObj(BOOL bCopy=FALSE);
	void BrkDragObj();
	BOOL IsDragObj() const { return pDragBla!=NULL && !bInsPolyPoint && !bInsGluePoint; }
	SdrHdl* GetDragHdl() const { return pDragHdl; }
	SdrDragMethod* GetDragMethod() const { return pDragBla; }
	BOOL IsMoveOnlyDragObj(BOOL bAskRTTI=FALSE) const;
	BOOL IsDraggingPoints() const { return eDragHdl==HDL_POLY; }
	BOOL IsDraggingGluePoints() const { return eDragHdl==HDL_GLUE; }

	void SetDragWithCopy(BOOL bOn) { bDragWithCopy = bOn; }
	BOOL IsDragWithCopy() const { return bDragWithCopy; }

	void SetInsertAfter(BOOL bOn) { bInsAfter = bOn; }
	BOOL IsInsertAfter() const { return bInsAfter; }

	void SetInsertGluePoint(BOOL bOn) { bInsGluePoint = bOn; }
	BOOL IsInsertGluePoint() const { return bInsGluePoint; }

	BOOL IsInsObjPointPossible() const;
	BOOL IsInsPointPossible() const { return IsInsObjPointPossible(); }
	BOOL BegInsObjPoint(USHORT nIdx, const Point& rPnt, BOOL bNewObj, OutputDevice* pOut=NULL, short nMinMov=-3) { return BegInsObjPoint(TRUE,nIdx,rPnt,bNewObj,pOut,nMinMov); }
	BOOL BegInsObjPoint(const Point& rPnt, BOOL bNewObj, OutputDevice* pOut=NULL, short nMinMov=-3) { return BegInsObjPoint(FALSE,0,rPnt,bNewObj,pOut,nMinMov); }
	void MovInsObjPoint(const Point& rPnt) { MovDragObj(rPnt); }
	BOOL EndInsObjPoint(SdrCreateCmd eCmd);
	void BrkInsObjPoint() { BrkDragObj(); }
	BOOL IsInsObjPoint() const { return pDragBla!=NULL && bInsPolyPoint; }

	void SetInsObjPointMode(BOOL bOn) { bInsObjPointMode = bOn; }
	BOOL IsInsObjPointMode() const { return bInsObjPointMode; }

	BOOL IsInsGluePointPossible() const;
	BOOL BegInsGluePoint(const Point& rPnt);
	void MovInsGluePoint(const Point& rPnt) { MovDragObj(rPnt); }
	BOOL EndInsGluePoint() { return EndDragObj(); }
	void BrkInsGluePoint() { BrkDragObj(); }
	BOOL IsInsGluePoint() const { return pDragBla!=NULL && bInsGluePoint; }

	void SetInsGluePointMode(BOOL bOn) { bInsGluePointMode = bOn; }
	BOOL IsInsGluePointMode() const { return bInsGluePointMode; }

	void SetDragStripes(BOOL bOn);
	BOOL IsDragStripes() const { return bDragStripes; }

	void SetDragHdlHide(BOOL bOn);
	BOOL IsDragHdlHide() const { return bNoDragHdl; }

	void SetMouseHideWhileDraggingPoints(BOOL bOn) { bMouseHideWhileDraggingPoints = bOn; }
	BOOL IsMouseHideWhileDraggingPoints() const { return bMouseHideWhileDraggingPoints; }

	void SetNoDragXorPolys(BOOL bOn);
	BOOL IsNoDragXorPolys() const { return bNoDragXorPolys; }

	void  SetDragXorPolyLimit(ULONG nObjAnz) { nDragXorPolyLimit=nObjAnz; }
	ULONG GetDragXorPolyLimit() const { return nDragXorPolyLimit; }

	void  SetDragXorPointLimit(ULONG nPntAnz) { nDragXorPointLimit=nPntAnz; }
	ULONG GetDragXorPointLimit() const { return nDragXorPointLimit; }

	void SetSolidDragging(BOOL bOn) { bSolidDragging = bOn; }
	BOOL IsSolidDragging() const { return bSolidDragging; }

	void SetSolidDraggingNow(BOOL bOn) { bSolidDrgNow = bOn; }
	BOOL IsSolidDraggingNow() const { return bSolidDrgNow; }

	void SetSolidDraggingCheck(BOOL bOn) { bSolidDrgChk = bOn; }
	BOOL IsSolidDraggingCheck() const { return bSolidDrgChk; }

	void SetAutoVertexConnectors(BOOL bOn) { bAutoVertexCon = bOn; }
	BOOL IsAutoVertexConnectors() const { return bAutoVertexCon; }

	void SetAutoCornerConnectors(BOOL bOn) { bAutoCornerCon = bOn; }
	BOOL IsAutoCornerConnectors() const { return bAutoCornerCon; }

	void SetDetailedEdgeDragging(BOOL bOn); 
	BOOL IsDetailedEdgeDragging() const { return bDetailedEdgeDragging; }

	void SetDetailedEdgeDraggingLimit(USHORT nEdgeObjAnz);
	USHORT GetDetailedEdgeDraggingLimit() const { return nDetailedEdgeDraggingLimit; }

	void SetRubberEdgeDragging(BOOL bOn);  
	BOOL IsRubberEdgeDragging() const { return bRubberEdgeDragging; }

	void SetRubberEdgeDraggingLimit(USHORT nEdgeObjAnz);
	USHORT GetRubberEdgeDraggingLimit() const { return nRubberEdgeDraggingLimit; }

	void SetMarkedHitMovesAlways(BOOL bOn) { bMarkedHitMovesAlways = bOn; }
	BOOL IsMarkedHitMovesAlways() const { return bMarkedHitMovesAlways; }

	void SetMirrRefDragObj(BOOL bOn) { bMirrRefDragObj = bOn; }
	BOOL IsMirrRefDragObj() const { return bMirrRefDragObj; }

	BOOL IsOrthoDesired() const;

	BOOL IsResizeAtCenter() const { return bResizeAtCenter; }
	void SetResizeAtCenter(BOOL bOn) { bResizeAtCenter = bOn; SnapMove(); }

	BOOL IsCrookAtCenter() const { return bCrookAtCenter; }
	void SetCrookAtCenter(BOOL bOn) { bCrookAtCenter = bOn; SnapMove(); }

	void SetWorkArea(const Rectangle& rRect) { aMaxWorkArea=rRect; }
	const Rectangle& GetWorkArea() const { return aMaxWorkArea; }

	virtual BOOL TakeDragLimit(SdrDragMode eMode, Rectangle& rRect) const;
};

#endif 

