

#ifndef _SVDGLUE_HXX
#define _SVDGLUE_HXX

class Window;
class OutputDevice;
class SvStream;
class SdrObject;

#ifndef _CONTNR_HXX 
#include <tools/contnr.hxx>
#endif
#ifndef _GEN_HXX 
#include <tools/gen.hxx>
#endif

#define SDRESC_SMART  0x0000
#define SDRESC_LEFT   0x0001
#define SDRESC_RIGHT  0x0002
#define SDRESC_TOP    0x0004
#define SDRESC_BOTTOM 0x0008
#define SDRESC_LO    0x0010 
#define SDRESC_LU    0x0020 
#define SDRESC_RO    0x0040 
#define SDRESC_RU    0x0080 
#define SDRESC_HORZ  (SDRESC_LEFT|SDRESC_RIGHT)
#define SDRESC_VERT  (SDRESC_TOP|SDRESC_BOTTOM)
#define SDRESC_ALL   0x00FF

#define SDRHORZALIGN_CENTER   0x0000
#define SDRHORZALIGN_LEFT     0x0001
#define SDRHORZALIGN_RIGHT    0x0002
#define SDRHORZALIGN_DONTCARE 0x0010
#define SDRVERTALIGN_CENTER   0x0000
#define SDRVERTALIGN_TOP      0x0100
#define SDRVERTALIGN_BOTTOM   0x0200
#define SDRVERTALIGN_DONTCARE 0x1000

class SdrGluePoint {

	Point    aPos;
	USHORT   nEscDir;
	USHORT   nId;
	USHORT   nAlign;
	FASTBOOL bNoPercent:1;
	FASTBOOL bReallyAbsolute:1; 
public:
	SdrGluePoint(): nEscDir(SDRESC_SMART),nId(0),nAlign(0) { bNoPercent=FALSE; bReallyAbsolute=FALSE; }
	SdrGluePoint(const Point& rNewPos, FASTBOOL bNewPercent=TRUE, USHORT nNewAlign=0): aPos(rNewPos),nEscDir(SDRESC_SMART),nId(0),nAlign(nNewAlign) { bNoPercent=!bNewPercent; bReallyAbsolute=FALSE; }
	FASTBOOL operator==(const SdrGluePoint& rCmpGP) const   { return aPos==rCmpGP.aPos && nEscDir==rCmpGP.nEscDir && nId==rCmpGP.nId && nAlign==rCmpGP.nAlign && bNoPercent==rCmpGP.bNoPercent && bReallyAbsolute==rCmpGP.bReallyAbsolute; }
	FASTBOOL operator!=(const SdrGluePoint& rCmpGP) const   { return !operator==(rCmpGP); }
	const Point& GetPos() const                             { return aPos; }
	void         SetPos(const Point& rNewPos)               { aPos=rNewPos; }
	USHORT       GetEscDir() const                          { return nEscDir; }
	void         SetEscDir(USHORT nNewEsc)                  { nEscDir=nNewEsc; }

	USHORT       GetId() const                              { return nId; }
	void         SetId(USHORT nNewId)                       { nId=nNewId; }
	FASTBOOL     IsPercent() const                          { return !bNoPercent; }
	void         SetPercent(FASTBOOL bOn)                   { bNoPercent=!bOn; }

	FASTBOOL     IsReallyAbsolute() const                   { return bReallyAbsolute; }
	void         SetReallyAbsolute(FASTBOOL bOn, const SdrObject& rObj);
	USHORT       GetAlign() const                           { return nAlign; }
	void         SetAlign(USHORT nAlg)                      { nAlign=nAlg; }
	USHORT       GetHorzAlign() const                       { return nAlign&0x00FF; }
	void         SetHorzAlign(USHORT nAlg)                  { nAlign=(nAlign&0xFF00)|(nAlg&0x00FF); }
	USHORT       GetVertAlign() const                       { return nAlign&0xFF00; }
	void         SetVertAlign(USHORT nAlg)                  { nAlign=(nAlign&0x00FF)|(nAlg&0xFF00); }
	void         Draw(OutputDevice& rOut, const SdrObject* pObj) const;
	FASTBOOL     IsHit(const Point& rPnt, const OutputDevice& rOut, const SdrObject* pObj) const;
	void         Invalidate(Window& rWin, const SdrObject* pObj) const;
	Point        GetAbsolutePos(const SdrObject& rObj) const;
	void         SetAbsolutePos(const Point& rNewPos, const SdrObject& rObj);
	long         GetAlignAngle() const;
	void         SetAlignAngle(long nWink);
	long         EscDirToAngle(USHORT nEsc) const;
	USHORT       EscAngleToDir(long nWink) const;
	void         Rotate(const Point& rRef, long nWink, double sn, double cs, const SdrObject* pObj);
	void         Mirror(const Point& rRef1, const Point& rRef2, const SdrObject* pObj);
	void         Mirror(const Point& rRef1, const Point& rRef2, long nWink, const SdrObject* pObj);
	void         Shear (const Point& rRef, long nWink, double tn, FASTBOOL bVShear, const SdrObject* pObj);
	friend SvStream& operator<<(SvStream& rOut, const SdrGluePoint& rGP);
	friend SvStream& operator>>(SvStream& rIn, SdrGluePoint& rGP);
};

#define SDRGLUEPOINT_NOTFOUND 0xFFFF

class SdrGluePointList {
	Container aList;
#if SUPD > 322

#endif
protected:
	SdrGluePoint* GetObject(USHORT i) const { return (SdrGluePoint*)(aList.GetObject(i)); }
public:
	SdrGluePointList(): aList(1024,4,4) {}
	SdrGluePointList(const SdrGluePointList& rSrcList): aList(1024,4,4)     { *this=rSrcList; }
	~SdrGluePointList()                                                     { Clear(); }
	void                Clear();
	void                operator=(const SdrGluePointList& rSrcList);
	USHORT              GetCount() const                                    { return USHORT(aList.Count()); }

	USHORT              Insert(const SdrGluePoint& rGP);
	void                Delete(USHORT nPos)                                 { delete (SdrGluePoint*)aList.Remove(nPos); }
	SdrGluePoint&       operator[](USHORT nPos)                             { return *GetObject(nPos); }
	const SdrGluePoint& operator[](USHORT nPos) const                       { return *GetObject(nPos); }
	USHORT              FindGluePoint(USHORT nId) const;
	USHORT              HitTest(const Point& rPnt, const OutputDevice& rOut, const SdrObject* pObj, FASTBOOL bBack=FALSE, FASTBOOL bNext=FALSE, USHORT nId0=0) const;
	void                DrawAll(OutputDevice& rOut, const SdrObject* pObj) const;
	void                Invalidate(Window& rWin, const SdrObject* pObj) const;

	void                SetReallyAbsolute(FASTBOOL bOn, const SdrObject& rObj);
	void                Rotate(const Point& rRef, long nWink, double sn, double cs, const SdrObject* pObj);
	void                Mirror(const Point& rRef1, const Point& rRef2, const SdrObject* pObj);
	void                Mirror(const Point& rRef1, const Point& rRef2, long nWink, const SdrObject* pObj);
	void                Shear (const Point& rRef, long nWink, double tn, FASTBOOL bVShear, const SdrObject* pObj);
	friend SvStream& operator<<(SvStream& rOut, const SdrGluePointList& rGPL);
	friend SvStream& operator>>(SvStream& rIn, SdrGluePointList& rGPL);
};

#endif 

