

#ifndef _SVDHDL_HXX
#define _SVDHDL_HXX

#ifndef _GEN_HXX 
#include <tools/gen.hxx>
#endif

#ifndef _POINTR_HXX 
#include <vcl/pointr.hxx>
#endif

#ifndef _CONTNR_HXX 
#include <tools/contnr.hxx>
#endif

#ifndef _SOLAR_HRC
#include <svtools/solar.hrc>
#endif

#ifndef _B2D_MPNT_HXX
#include <goodies/b2dmpnt.hxx>
#endif

#ifndef _B2D_MLINE_HXX
#include <goodies/b2dmline.hxx>
#endif

#ifndef _B2D_MBMP_HXX
#include <goodies/b2dmbmp.hxx>
#endif

#include <svx/xpoly.hxx>
#include <svx/svdoedge.hxx>

class VirtualDevice;
class OutputDevice;
class Region;
class Window;
class SdrHdlList;
class SdrObject;
class SdrPageView;
class SdrHdlBitmapSet;

enum SdrHdlKind 
{
	HDL_MOVE,		
	HDL_UPLFT,		
	HDL_UPPER,		
	HDL_UPRGT,		
	HDL_LEFT,		
	HDL_RIGHT,		
	HDL_LWLFT,		
	HDL_LOWER,		
	HDL_LWRGT,		
	HDL_POLY,		
	HDL_BWGT,		
	HDL_CIRC,		
	HDL_REF1,		
	HDL_REF2,		
	HDL_MIRX,		
	HDL_GLUE,		
	HDL_ANCHOR,		
	HDL_TRNS,		
	HDL_GRAD,		
	HDL_COLR,		
	HDL_USER
};

enum BitmapColorIndex
{
	LightGreen,
	Cyan,
	LightCyan,
	Red,
	LightRed
};

enum BitmapMarkerKind
{
	Rect_7x7,
	Rect_9x9,
	Rect_11x11,
	Rect_13x13,
	Circ_7x7,
	Circ_9x9,
	Circ_11x11,
	Elli_7x9,
	Elli_9x11,
	Elli_9x7,
	Elli_11x9,
	RectPlus_7x7,
	RectPlus_9x9,
	RectPlus_11x11,
	Crosshair,
	Glue,
	Anchor
};

class SdrHdl
{
	friend class				SdrMarkView; 
	friend class				SdrHdlList;

	static SdrHdlBitmapSet*		pSimpleSet;
	static SdrHdlBitmapSet*		pModernSet;

protected:
	SdrObject*					pObj;      
	SdrPageView*				pPV;       
	SdrHdlList*					pHdlList;  

	B2dIAOGroup					aIAOGroup;

	Point						aPos;

	SdrHdlKind					eKind;

	long						nDrehWink; 
	USHORT						nObjHdlNum; 
	USHORT						nPolyNum;  
	USHORT						nPPntNum;  
	USHORT						nSourceHdlNum; 

	unsigned					bSelect : 1;   
	unsigned					b1PixMore : 1; 
	unsigned					bPlusHdl : 1;  

	virtual void CreateB2dIAObject();

	void GetRidOfIAObject();

protected:
	B2dIAObject* CreateMarkerObject(B2dIAOManager* pMan, Point aPos, BitmapColorIndex eColIndex, BitmapMarkerKind eKindOfMarker);

public:
	SdrHdl();
	SdrHdl(const Point& rPnt, SdrHdlKind eNewKind=HDL_MOVE);
	virtual ~SdrHdl();

	const B2dIAOGroup& GetIAOGroup() const { return aIAOGroup; }

	void SetHdlList(SdrHdlList* pList);
	SdrHdlKind GetKind() const { return eKind; }
	void Touch();

	const Point& GetPos() const { return aPos; }
	void SetPos(const Point& rPnt);

	SdrPageView* GetPageView() const { return pPV; }
	void SetPageView(SdrPageView* pNewPV) { pPV=pNewPV; }

	SdrObject* GetObj() const { return pObj;  }
	void SetObj(SdrObject* pNewObj);

	BOOL IsSelected() const { return bSelect; }
	void SetSelected(BOOL bJa=TRUE);

	void Set1PixMore(BOOL bJa=TRUE);
	void SetDrehWink(long n);

	BOOL IsCornerHdl() const { return eKind==HDL_UPLFT || eKind==HDL_UPRGT || eKind==HDL_LWLFT || eKind==HDL_LWRGT; }
	BOOL IsVertexHdl() const { return eKind==HDL_UPPER || eKind==HDL_LOWER || eKind==HDL_LEFT  || eKind==HDL_RIGHT; }

	void SetObjHdlNum(USHORT nNum) { nObjHdlNum=nNum; }
	USHORT GetObjHdlNum() const { return nObjHdlNum; }

	void SetPolyNum(USHORT nNum) { nPolyNum=nNum; }
	USHORT GetPolyNum() const { return nPolyNum; }

	void SetPointNum(USHORT nNum) { nPPntNum=nNum; }
	USHORT GetPointNum() const { return nPPntNum; }

	void SetPlusHdl(BOOL bOn) { bPlusHdl=bOn; }
	BOOL IsPlusHdl() const { return bPlusHdl; }

	void SetSourceHdlNum(USHORT nNum) { nSourceHdlNum=nNum; }
	USHORT GetSourceHdlNum() const { return nSourceHdlNum; }

	virtual Pointer GetPointer() const;
	virtual BOOL IsHit(const Point& rPnt, const OutputDevice& rOut) const;
};

#define	SDR_HANDLE_COLOR_SIZE_NORMAL			Size(13, 13)
#define	SDR_HANDLE_COLOR_SIZE_SELECTED			Size(17, 17)

class SdrHdlColor : public SdrHdl
{
private:

	Size						aMarkerSize;

	Color						aMarkerColor;
	 Link						aColorChangeHdl;

	unsigned					bUseLuminance : 1;

	virtual void CreateB2dIAObject();

	Bitmap CreateColorDropper(Color aCol);
	Color GetLuminance(const Color& rCol);
	void CallColorChangeLink();

public:
	SdrHdlColor(const Point& rRef, Color aCol = Color(COL_BLACK), const Size& rSize = Size(11, 11), BOOL bLum = FALSE);
	virtual ~SdrHdlColor();

	BOOL IsUseLuminance() const { return bUseLuminance; }

	Color GetColor() const { return aMarkerColor; }
	void SetColor(Color aNew, BOOL bCallLink = FALSE);

	const Size& GetSize() const { return aMarkerSize; }
	void SetSize(const Size& rNew);
	 void SetColorChangeHdl(const Link& rLink) { aColorChangeHdl = rLink; }
	const Link& GetColorChangeHdl() const { return aColorChangeHdl; }
};

class SdrHdlGradient : public SdrHdl
{
private:

	SdrHdlColor*				pColHdl1;
	SdrHdlColor*				pColHdl2;

	Point						a2ndPos;

	unsigned					bGradient : 1;

	unsigned					bMoveSingleHandle : 1;
	unsigned					bMoveFirstHandle : 1;

	virtual void CreateB2dIAObject();

public:
	SdrHdlGradient(const Point& rRef1, const Point& rRef2, BOOL bGrad = TRUE);
	virtual ~SdrHdlGradient();

	BOOL IsGradient() const { return bGradient; }

	void SetColorHandles(SdrHdlColor* pL1, SdrHdlColor* pL2) { pColHdl1 = pL1; pColHdl2 = pL2; }
	SdrHdlColor* GetColorHdl1() const { return pColHdl1; }
	SdrHdlColor* GetColorHdl2() const { return pColHdl2; }

	const Point& Get2ndPos() const { return a2ndPos; }
	void Set2ndPos(const Point& rPnt);

	DECL_LINK(ColorChangeHdl, SdrHdl*);

	void FromIAOToItem(SdrObject* pObj, BOOL bSetItemOnObject, BOOL bUndo);

	BOOL IsMoveSingleHandle() const { return bMoveSingleHandle; }
	void SetMoveSingleHandle(BOOL bNew) { bMoveSingleHandle = bNew; }
	BOOL IsMoveFirstHandle() const { return bMoveFirstHandle; }
	void SetMoveFirstHandle(BOOL bNew) { bMoveFirstHandle = bNew; }
};

class SdrHdlLine: public SdrHdl
{

	virtual void CreateB2dIAObject();

protected:
	SdrHdl*						pHdl1;
	SdrHdl*						pHdl2;

public:
	SdrHdlLine(SdrHdl& rHdl1, SdrHdl& rHdl2, SdrHdlKind eNewKind=HDL_MIRX) { eKind=eNewKind; pHdl1=&rHdl1; pHdl2=&rHdl2; }
	virtual ~SdrHdlLine();

	virtual Pointer GetPointer() const;
	virtual BOOL IsHit(const Point& rPnt, const OutputDevice& rOut) const;
};

class SdrHdlBezWgt: public SdrHdl
{

	virtual void CreateB2dIAObject();

protected:
	const SdrHdl* pHdl1;

public:

	SdrHdlBezWgt(const SdrHdl* pRefHdl1, SdrHdlKind eNewKind=HDL_BWGT) { eKind=eNewKind; pHdl1=pRefHdl1; }
	virtual ~SdrHdlBezWgt();
};

class E3dVolumeMarker : public SdrHdl
{
	XPolyPolygon	aWireframePoly;

	virtual void CreateB2dIAObject();

public:
	E3dVolumeMarker(const XPolyPolygon& rXPP);
};

class ImpEdgeHdl: public SdrHdl
{
	SdrEdgeLineCode eLineCode;

	virtual void CreateB2dIAObject();

public:
	ImpEdgeHdl(const Point& rPnt, SdrHdlKind eNewKind): SdrHdl(rPnt,eNewKind),eLineCode(MIDDLELINE) {}
	virtual ~ImpEdgeHdl();

	void SetLineCode(SdrEdgeLineCode eCode);
	SdrEdgeLineCode GetLineCode() const     { return eLineCode; }
	BOOL IsHorzDrag() const;
	virtual Pointer GetPointer() const;
};

class ImpMeasureHdl: public SdrHdl
{

	virtual void CreateB2dIAObject();

public:
	ImpMeasureHdl(const Point& rPnt, SdrHdlKind eNewKind): SdrHdl(rPnt,eNewKind) {}
	virtual ~ImpMeasureHdl();

	virtual Pointer GetPointer() const;
};

class SdrHdlList
{
protected:
	SdrMarkView*				pView;

	Container					aList;
	USHORT						nHdlSize;

	unsigned					bRotateShear : 1;
	unsigned					bDistortShear : 1;
	unsigned					bMoveOutside : 1;      
	unsigned					bFineHandles : 1;

private:
	SdrHdlList(const SdrHdlList&): aList(1024,64,64)  {}
	void operator=(const SdrHdlList&)                 {}
	BOOL operator==(const SdrHdlList&) const      { return FALSE; }
	BOOL operator!=(const SdrHdlList&) const      { return FALSE; }

public:
	SdrHdlList(SdrMarkView* pV);
	~SdrHdlList();
	void Clear();

	SdrMarkView* GetView() const { return pView; }

	void     Sort();
	ULONG    GetHdlCount() const                       { return aList.Count(); }
	SdrHdl*  GetHdl(ULONG nNum) const                  { return (SdrHdl*)(aList.GetObject(nNum)); }
	ULONG    GetHdlNum(const SdrHdl* pHdl) const;
	void     SetHdlSize(USHORT nSiz);
	USHORT   GetHdlSize() const                        { return nHdlSize; }
	void     SetMoveOutside(BOOL bOn);
	BOOL IsMoveOutside() const                     { return bMoveOutside; }
	void     SetRotateShear(BOOL bOn);
	BOOL IsRotateShear() const                     { return bRotateShear; }
	void     SetDistortShear(BOOL bOn);
	BOOL IsDistortShear() const                    { return bDistortShear; }
	void     SetFineHdl(BOOL bOn);
	BOOL IsFineHdl() const                        { return bFineHandles; }

	void    AddHdl(SdrHdl* pHdl, BOOL bAtBegin=FALSE);
	SdrHdl* RemoveHdl(ULONG nNum);

	SdrHdl* HitTest(const Point& rPnt, const OutputDevice& rOut, BOOL bBack=FALSE, BOOL bNext=FALSE, SdrHdl* pHdl0=NULL) const;
	SdrHdl* GetHdl(SdrHdlKind eKind1) const;
};

#endif 

