

#ifndef _SVDIO_HXX
#define _SVDIO_HXX

#ifndef _STREAM_HXX 
#include <tools/stream.hxx>
#endif

const char SdrIOMagic [2]={'D','r'};
const char SdrIOModlID[2]={'M','d'}; 
const char SdrIOItPlID[2]={'I','P'}; 
const char SdrIOLayrID[2]={'L','y'}; 
const char SdrIOLSetID[2]={'L','S'}; 
const char SdrIOPageID[2]={'P','g'}; 
const char SdrIOMaPgID[2]={'M','P'}; 
const char SdrIOMPgDID[2]={'M','D'}; 
const char SdrIOMPDLID[2]={'M','L'}; 
const char SdrIOHlpLID[2]={'H','l'}; 
const char SdrIOHLstID[2]={'H','L'}; 
const char SdrIODObjID[2]={'O','b'}; 
const char SdrIOConnID[2]={'C','n'}; 
const char SdrIOViewID[2]={'V','w'}; 
const char SdrIOPgVwID[2]={'P','V'}; 
const char SdrIOEndeID[2]={'X','X'}; 
const char SdrIOJoeMagic[4]={'J','o','e','M'}; 

const nAktSdrFileMajorVersion =  0;

const nAktSdrFileMinorVersion = 17; 

const nAktSdrFileVersion = (nAktSdrFileMajorVersion <<8) + (nAktSdrFileMinorVersion);
const nSdrFileVersion31 = 12; 
const nSdrFileVersion40 = 13; 
const nSdrFileVersion50 = 14; 
const nSdrFileVersion51 = 15; 

class SvStream;

class SdrIOHeader 
{
protected:

	char						cMagic[2];
	char						cHdrID[2];

	UINT16						nVersion;    

	UINT32						nBlkSize;    

	SvStream&					rStream;
	UINT32						nFilePos;
	UINT16						nMode;
	BOOL						bOpen;
	BOOL						bClosed;

	BOOL						bLookAhead; 

	virtual void Write();
	virtual void Read();

#if 0 // _SOLAR__PRIVATE
#ifdef DBG_UTIL
	void ImpCheckRecordIntegrity();
	void ImpGetRecordName(ByteString& rStr, INT32 nSubRecCount=-1, INT32 nSubRecReadCount=-1) const;
	BOOL ImpHasSubRecords() const;
#endif
#endif 

public:
	SdrIOHeader(SvStream& rNewStream, UINT16 nNewMode, const char cID[2]=SdrIOEndeID, BOOL bAutoOpen=TRUE);
	virtual ~SdrIOHeader();

	BOOL IsMagic() const { return cMagic[0] == SdrIOMagic[0] && cMagic[1] == SdrIOMagic[1];  }
	void SetMagic() { cMagic[0] = SdrIOMagic[0]; cMagic[1] = SdrIOMagic[1]; }
	BOOL IsEnde() const { return cHdrID[0] == SdrIOEndeID[0] && cHdrID[1] == SdrIOEndeID[1]; }
	void SetEnde() { cHdrID[0] = SdrIOEndeID[0]; cHdrID[1] = SdrIOEndeID[1]; nBlkSize = 0; SetMagic(); }
	BOOL IsID(const char cID[2]) const { return cHdrID[0] == cID[0] && cHdrID[1] == cID[1]; }
	void SetID(const char cID[2]) { cHdrID[0] = cID[0]; cHdrID[1] = cID[1]; SetMagic(); }
	UINT16 GetVersion() const { return nVersion; }
	UINT16 GetMinorVersion() const { return nVersion | 0xFF; }
	UINT16 GetMajorVersion() const { return nVersion >>8; }
	UINT32 GetFilePos() const { return nFilePos; }
	UINT32 GetBlockSize() const { return nBlkSize; }
	void OpenRecord();
	void CloseRecord();
	UINT32 GetBytesLeft() const;
};

class SdrIOHeaderLookAhead: public SdrIOHeader 
{
public:
	SdrIOHeaderLookAhead(SvStream& rNewStream, BOOL bAutoOpen = TRUE);
	void SkipRecord() const { rStream.Seek(nFilePos+nBlkSize); }
};

class SdrObject;

class SdrObjIOHeader: public SdrIOHeader 
{
public:

	UINT32						nInventor;
	UINT16						nIdentifier;

	const SdrObject*			pObj;

protected:
	virtual void Write();
	virtual void Read();

public:
	SdrObjIOHeader(SvStream& rNewStream, UINT16 nNewMode, const SdrObject* pNewObj = NULL, BOOL bAutoOpen = TRUE);

	virtual ~SdrObjIOHeader();
};

class SdrObjIOHeaderLookAhead: public SdrObjIOHeader 
{
public:
	SdrObjIOHeaderLookAhead(SvStream& rNewStream, BOOL bAutoOpen = TRUE);
	void SkipRecord() const { rStream.Seek(nFilePos+nBlkSize); }
};

class SdrDownCompat 
{
protected:
	SvStream&					rStream;
	UINT32						nSubRecSiz;
	UINT32						nSubRecPos;
	UINT16						nMode;
	BOOL						bOpen;
	BOOL						bClosed;

	const char*					pRecId; 

protected:
	virtual void Read();
	virtual void Write();

public:

	SdrDownCompat(SvStream& rNewStream, UINT16 nNewMode, BOOL bAutoOpen = TRUE);
	~SdrDownCompat();

#ifdef DBG_UTIL

	void SetID(const char* pId) { pRecId=pId; }
#endif

	UINT32 GetBytesLeft() const;
	BOOL IsOpen() const { return bOpen; }

	UINT32 GetSubRecordPos() const { return nSubRecPos; }

	UINT32 GetSubRecordSize() const { return nSubRecSiz; }
	void  OpenSubRecord();
	void  CloseSubRecord();
};

enum {SDRIORECNAME_VIEWPAGEVIEWS   =0x0000,  
	  SDRIORECNAME_VIEWVISIELEM    =0x0001,  
	SDRIORECNAME_VIEWSNAP        =0x0010,  
	  SDRIORECNAME_VIEWORTHO       =0x0011,  
	SDRIORECNAME_VIEWDRAGMODE    =0x0020,  
	  SDRIORECNAME_VIEWMARKEDOBJ   =0x0021,  
	  SDRIORECNAME_VIEWMARKEDPNT   =0x0022,  
	  SDRIORECNAME_VIEWCROOKCENTER =0x0023,  
	  SDRIORECNAME_VIEWMARKEDGLUE  =0x0024,  
	SDRIORECNAME_VIEWDRAGSTRIPES =0x0030,  
	  SDRIORECNAME_VIEWDRAGHIDEHDL =0x0031,  
	  SDRIORECNAME_VIEWDRAGEDGES   =0x0032,  
	  SDRIORECNAME_VIEWOBJHITMOVES =0x0033,  
	  SDRIORECNAME_VIEWMIRRDRAGOBJ =0x0034,  
	SDRIORECNAME_VIEWCROBJECT    =0x0040,  
	  SDRIORECNAME_VIEWAKTLAYER    =0x0041,  
	  SDRIORECNAME_VIEWCRATTRIBUTES=0x0042,  
	  SDRIORECNAME_VIEWCRSTYLESHEET=0x0043,  
	  SDRIORECNAME_VIEWCRFLAGS     =0x0044,  
	SDRIORECNAME_VIEWTEXTEDIT    =0x0050,  
	  SDRIORECNAME_VIEWMACRO       =0x0051,  
	SDRIORECNAME_VIEWCHARSET     =0x0080,  
	SDRIORECNAME_PAGVIEW         =0x0100,  
	  SDRIORECNAME_PAGVLAYER       =0x0101,  
	  SDRIORECNAME_PAGVHELPLINES   =0x0102,  
	  SDRIORECNAME_PAGVAKTGROUP    =0x0103,  
	SDRIORECNAME_MARK            =0x0200,  
	SDRIORECNAME_USERDEFINEDFIRST=0x7000,
	  SDRIORECNAME_USERDEFINEDLAST =0x7FFF};

class SdrNamedSubRecord: public SdrDownCompat 
{
	UINT32						nInventor;
	UINT16						nIdentifier;

protected:
	virtual void Read();
	virtual void Write();

public:
	SdrNamedSubRecord(SvStream& rNewStream, UINT16 nNewMode,
		UINT32 nNewInventor = 0xFFFF, UINT16 nNewIdentifier = 0xFFFF,
		BOOL bAutoOpen = TRUE);
	UINT32 GetInventor() const { return nInventor; }
	UINT16 GetIdentifier() const { return nIdentifier; }
};

BOOL SdrIsModel(SvStream& rIn);
BOOL SdrIsView(SvStream& rIn);

#endif 

