

#ifndef _SVDLAYER_HXX
#define _SVDLAYER_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef _STREAM_HXX 
#include <tools/stream.hxx>
#endif

#ifndef _SVDSOB_HXX 
#include <svx/svdsob.hxx>
#endif

#ifndef _SVDTYPES_HXX
#include <svx/svdtypes.hxx> 
#endif

class SdrModel;
class SdrLayerSet;

class SdrLayer {
friend class SdrLayerSet;
friend class SdrLayerAdmin;
protected:
	String     aName;
	SdrModel*  pModel; 
	UINT16     nType;  
	SdrLayerID nID;
protected:
	SdrLayer(SdrLayerID nNewID, const String& rNewName)       { nID=nNewID; aName=rNewName; nType=0; pModel=NULL; }
	void SetID(SdrLayerID nNewID)                             { nID=nNewID; }
public:
	SdrLayer(): pModel(NULL),nType(0),nID(0)                  {}
	FASTBOOL      operator==(const SdrLayer& rCmpLayer) const;
	FASTBOOL      operator!=(const SdrLayer& rCmpLayer) const { return !operator==(rCmpLayer); }
	void          SetName(const String& rNewName);
	const String& GetName() const                             { return aName; }
	SdrLayerID    GetID() const                               { return nID; }
	void          SetModel(SdrModel* pNewModel)               { pModel=pNewModel; }
	SdrModel*     GetModel() const                            { return pModel; }

	void          SetStandardLayer(FASTBOOL bStd=TRUE);
	FASTBOOL      IsStandardLayer() const                     { return nType==1; }
	friend SvStream& operator>>(SvStream& rIn, SdrLayer& rLayer);
	friend SvStream& operator<<(SvStream& rOut, const SdrLayer& rLayer);
};

class SdrLayerSet {
friend class SdrLayerAdmin;
friend class SdrView;
protected:

	String    aName;
	SetOfByte aMember;
	SetOfByte aExclude;
	SdrModel* pModel; 
protected:

	void            Broadcast() const;
public:
	SdrLayerSet(): pModel(NULL) {}
	SdrLayerSet(const String& rNewName): aName(rNewName), pModel(NULL) {}
	FASTBOOL        operator==(const SdrLayerSet& rCmpLayerSet) const;
	FASTBOOL        operator!=(const SdrLayerSet& rCmpLayerSet) const { return !operator==(rCmpLayerSet); }
	void            SetName(const String& rNewName)        { aName=rNewName; Broadcast(); }
	const String&   GetName() const                        { return aName; }
	void            SetModel(SdrModel* pNewModel)          { pModel=pNewModel; }
	SdrModel*       GetModel() const                       { return pModel; }
	FASTBOOL        IsMember(SdrLayerID nLayID) const      { return aMember.IsSet(nLayID); }
	void            AddMember(SdrLayerID nLayID)           { if (nLayID!=SDRLAYER_NOTFOUND) aMember.Set(nLayID); Broadcast(); }
	void            DelMember(SdrLayerID nLayID)           { aMember.Clear(nLayID); Broadcast(); }

	void            DelAll()                               { aMember.ClearAll(); Broadcast(); }
	USHORT          GetMemberCount() const                 { return aMember.GetSetCount(); }
	SdrLayerID      GetMemberID(USHORT i) const            { return aMember.GetSetBit(i); }
	FASTBOOL        IsExcluded(SdrLayerID nLayID) const    { return aExclude.IsSet(nLayID); }
	void            AddExcluded(SdrLayerID nLayID)         { if (nLayID!=SDRLAYER_NOTFOUND) aExclude.Set(nLayID); Broadcast(); }
	void            DelExcluded(SdrLayerID nLayID)         { aExclude.Clear(nLayID); Broadcast(); }

	void            UnExcludeAll()                         { aExclude.ClearAll(); Broadcast(); }
	USHORT          GetExcludedCount() const               { return aExclude.GetSetCount(); }
	SdrLayerID      GetExcludedID(USHORT i) const          { return aExclude.GetSetBit(i); }
	friend SvStream& operator>>(SvStream& rIn, SdrLayerSet& rSet);
	friend SvStream& operator<<(SvStream& rOut, const SdrLayerSet& rSet);
};

#define SDRLAYER_MAXCOUNT 255
class SdrLayerAdmin {
friend class SdrView;
friend class SdrModel;
friend class SdrPage;

protected:
	Container      aLayer;
	Container      aLSets;
	SdrLayerAdmin* pParent; 
	SdrModel*      pModel; 
	String         aControlLayerName;
protected:

	SdrLayerID           GetUniqueLayerID() const;

	void                 Broadcast(FASTBOOL bLayerSet) const;
public:
	SdrLayerAdmin(SdrLayerAdmin* pNewParent=NULL);
	SdrLayerAdmin(const SdrLayerAdmin& rSrcLayerAdmin);
	~SdrLayerAdmin();
	const SdrLayerAdmin& operator=(const SdrLayerAdmin& rSrcLayerAdmin);
	FASTBOOL             operator==(const SdrLayerAdmin& rCmpLayerAdmin) const;
	FASTBOOL             operator!=(const SdrLayerAdmin& rCmpLayerAdmin) const       { return !operator==(rCmpLayerAdmin); }
	SdrLayerAdmin*       GetParent() const                                           { return pParent; }
	void                 SetParent(SdrLayerAdmin* pNewParent)                        { pParent=pNewParent; }
	void                 SetModel(SdrModel* pNewModel);
	SdrModel*            GetModel() const                                            { return pModel; }
	void                 InsertLayer(SdrLayer* pLayer, USHORT nPos=0xFFFF)           { aLayer.Insert(pLayer,nPos); pLayer->SetModel(pModel); Broadcast(FALSE); }
	SdrLayer*            RemoveLayer(USHORT nPos);

	void               ClearLayer();

	SdrLayer*          NewLayer(const String& rName, USHORT nPos=0xFFFF);
	void               DeleteLayer(SdrLayer* pLayer)                                 { aLayer.Remove(pLayer); delete pLayer; Broadcast(FALSE); }
	void               MoveLayer(SdrLayer* pLayer, USHORT nNewPos=0xFFFF);
	SdrLayer*          MoveLayer(USHORT nPos, USHORT nNewPos);

	SdrLayer*          NewStandardLayer(USHORT nPos=0xFFFF);

	USHORT             GetLayerCount() const                                         { return USHORT(aLayer.Count()); }
	SdrLayer*          GetLayer(USHORT i)                                            { return (SdrLayer*)(aLayer.GetObject(i)); }
	const SdrLayer*    GetLayer(USHORT i) const                                      { return (SdrLayer*)(aLayer.GetObject(i)); }

	USHORT             GetLayerPos(SdrLayer* pLayer) const;

	SdrLayer*          GetLayer(const String& rName, FASTBOOL bInherited)            { return (SdrLayer*)(((const SdrLayerAdmin*)this)->GetLayer(rName,bInherited)); }
	const SdrLayer*    GetLayer(const String& rName, FASTBOOL bInherited) const;
	SdrLayerID   GetLayerID(const String& rName, FASTBOOL bInherited) const;
	SdrLayer*    GetLayerPerID(USHORT nID)                                     { return (SdrLayer*)(((const SdrLayerAdmin*)this)->GetLayerPerID(nID)); }
	const SdrLayer*    GetLayerPerID(USHORT nID) const;

	void               InsertLayerSet(SdrLayerSet* pSet, USHORT nPos=0xFFFF)         { aLSets.Insert(pSet,nPos); pSet->SetModel(pModel); Broadcast(TRUE); }

	void               ClearLayerSets();

	SdrLayerSet*       NewLayerSet(const String& rName, USHORT nPos=0xFFFF);
	SdrLayerSet*       RemoveLayerSet(SdrLayerSet* pSet);
	void               DeleteLayerSet(SdrLayerSet* pSet)                             { aLSets.Remove(pSet); delete pSet; Broadcast(TRUE); }
	void               MoveLayerSet(SdrLayerSet* pSet, USHORT nNewPos=0xFFFF);

	USHORT             GetLayerSetCount() const                                      { return USHORT(aLSets.Count()); }
	SdrLayerSet*       GetLayerSet(USHORT i)                                         { return (SdrLayerSet*)(aLSets.GetObject(i)); }
	const SdrLayerSet* GetLayerSet(USHORT i) const                                   { return (SdrLayerSet*)(aLSets.GetObject(i)); }

	SdrLayerSet*       GetLayerSet(const String& rName, FASTBOOL bInherited)         { return (SdrLayerSet*)(((const SdrLayerAdmin*)this)->GetLayerSet(rName,bInherited)); }
	const SdrLayerSet* GetLayerSet(const String& rName, FASTBOOL bInherited) const;

	void     	       SetControlLayerName(const String& rNewName) { aControlLayerName=rNewName; }
	const String& 	   GetControlLayerName() const                 { return aControlLayerName; }
};

#endif 

