

#ifndef _SVDMODEL_HXX
#define _SVDMODEL_HXX

#ifndef _CPPUHELPER_WEAKREF_HXX_
#include <cppuhelper/weakref.hxx>
#endif
#ifndef _SOT_STORAGE_HXX
#include <sot/storage.hxx>
#endif
#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif
#ifndef _CONTNR_HXX 
#include <tools/contnr.hxx>
#endif
#ifndef _MAPMOD_HXX 
#include <vcl/mapmod.hxx>
#endif
#ifndef _SFXBRDCST_HXX 
#include <svtools/brdcst.hxx>
#endif
#ifndef _SYSTEM_HXX 
#include <vcl/system.hxx>
#endif
#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _DATETIME_HXX 
#include <tools/datetime.hxx>
#endif
#ifndef _SFXHINT_HXX 
#include <svtools/hint.hxx>
#endif

#ifdef VCL
#ifndef _SV_FIELD_HXX
#include <vcl/field.hxx>
#endif
#endif

class OutputDevice;

#ifndef _SVDTYPES_HXX
#include <svx/svdtypes.hxx> 
#endif

#ifndef _SV_FIELD_HXX 
#include <vcl/field.hxx>
#endif

#include <vos/ref.hxx>

#if defined(UNX) || defined(WIN) || defined(WNT)
#define DEGREE_CHAR ((sal_Unicode)176)   
#endif

#if defined(DOS) || defined(OS2)
#define DEGREE_CHAR ((sal_Unicode)248)   
#endif

#if defined(MAC)
#define DEGREE_CHAR ((sal_Unicode)161)   
#endif

#ifndef DEGREE_CHAR
#error unbekannte Plattrorm
#endif

class SdrOutliner;
class SdrIOHeader;
class SdrLayerAdmin;
class SdrObjList;
class SdrObject;
class SdrPage;
class SdrPageView;
class SdrTextObj;
class SdrUndoAction;
class SdrUndoGroup;
class AutoTimer;
class SfxItemPool;
class SfxItemSet;
class SfxRepeatTarget;
class SfxStyleSheet;
class SfxStyleSheetBasePool;
class SfxUndoAction;
class SvPersist;
class SvxLinkManager;
class XBitmapList;
class XBitmapTable;
class XColorTable;
class XDashList;
class XDashTable;
class XGradientList;
class XGradientTable;
class XHatchList;
class XHatchTable;
class XLineEndList;
class XLineEndTable;
class SvxForbiddenCharactersTable;
class SvNumberFormatter;
class SotStorage;

#define SDR_SWAPGRAPHICSMODE_NONE		0x00000000
#define SDR_SWAPGRAPHICSMODE_TEMP		0x00000001
#define SDR_SWAPGRAPHICSMODE_DOC		0x00000002
#define SDR_SWAPGRAPHICSMODE_PURGE		0x00000100
#define SDR_SWAPGRAPHICSMODE_DEFAULT	(SDR_SWAPGRAPHICSMODE_TEMP|SDR_SWAPGRAPHICSMODE_DOC|SDR_SWAPGRAPHICSMODE_PURGE)

enum SdrHintKind {HINT_UNKNOWN,         
	  HINT_LAYERCHG,        
	  HINT_LAYERORDERCHG,   
	  HINT_LAYERSETCHG,     
	  HINT_LAYERSETORDERCHG,
	  HINT_PAGECHG,         
	  HINT_PAGEORDERCHG,    
	  HINT_OBJCHG,          
	  HINT_OBJINSERTED,     
	  HINT_OBJREMOVED,      
	  HINT_MODELCLEARED,    
	  HINT_OBJLISTCLEARED,  
	  HINT_REFDEVICECHG,    
	  HINT_DEFAULTTABCHG,   
	  HINT_DEFFONTHGTCHG,   
	  HINT_MODELSAVED,      
	  HINT_CONTROLINSERTED, 
	  HINT_CONTROLREMOVED}; 

class SdrHint: public SfxHint {
public:
	Rectangle         aRect;
	const SdrPage*    pPage;
	const SdrObject*  pObj;
	const SdrObjList* pObjList;
	FASTBOOL          bNeedRepaint;
	SdrHintKind       eHint;
	FASTBOOL          bAniRepaint;
public:
	TYPEINFO();
	SdrHint(): pPage(NULL),pObj(NULL),pObjList(NULL),bNeedRepaint(FALSE),eHint(HINT_UNKNOWN),bAniRepaint(FALSE) {}
	SdrHint(SdrHintKind eNewHint): pPage(NULL),pObj(NULL),pObjList(NULL),bNeedRepaint(FALSE),eHint(eNewHint),bAniRepaint(FALSE) {}
	SdrHint(const SdrPage& rNewPage);
	SdrHint(const SdrObject& rNewObj);
	SdrHint(const SdrObject& rNewObj, const Rectangle& rRect);
	void              SetPage(const SdrPage* pNewPage)     { pPage=pNewPage; }
	void              SetObjList(const SdrObjList* pNewOL) { pObjList=pNewOL; }
	void              SetObject(const SdrObject* pNewObj)  { pObj=pNewObj; }
	void              SetKind(SdrHintKind eNewKind)        { eHint=eNewKind; }
	void              SetRect(const Rectangle& rNewRect)   { aRect=rNewRect; }
	void              SetNeedRepaint(FASTBOOL bOn)         { bNeedRepaint=bOn; }
	void              SetAniRepaint(FASTBOOL bOn)          { bAniRepaint=bOn; }
	const SdrPage*    GetPage() const                      { return pPage; }
	const SdrObjList* GetObjList() const                   { return pObjList; }
	const SdrObject*  GetObject() const                    { return pObj;  }
	SdrHintKind       GetKind() const                      { return eHint; }
	const Rectangle&  GetRect() const                      { return aRect; }
	FASTBOOL          IsNeedRepaint() const                { return bNeedRepaint; }
	FASTBOOL          IsAniRepaint() const                 { return bAniRepaint; }
};

#define LOADREFCOUNTS (FALSE)

class SdrModelInfo {
public:
	DateTime   aCreationDate;
	DateTime   aLastWriteDate;
	DateTime   aLastReadDate;
	DateTime   aLastPrintDate;
	rtl_TextEncoding eCreationCharSet;
	rtl_TextEncoding eLastWriteCharSet;
	rtl_TextEncoding eLastReadCharSet;
	UINT16     nCompressMode;
	UINT16     nNumberFormat;

public:

	SdrModelInfo(FASTBOOL bInit=FALSE);
	friend SvStream& operator<<(SvStream& rOut, const SdrModelInfo& rModInfo);
	friend SvStream& operator>>(SvStream& rIn, SdrModelInfo& rModInfo);
};

struct SdrDocumentStreamInfo
{
	FASTBOOL	mbDeleteAfterUse;
	String		maUserData;
	void*		mpDummy1;
	BOOL		mbDummy1 : 1;
};

class SdrModel: public SfxBroadcaster {
protected:
	SdrModelInfo   aInfo;
	DateTime       aReadDate;  
	Container      aMaPag;     
	Container      aPages;
	Link           aUndoLink;  
	Link           aIOProgressLink;
	String         aTablePath;
	String         aLoadedModelFN; 
	Size           aMaxObjSize; 
	Fraction       aObjUnit;   
	MapUnit        eObjUnit;   
	FieldUnit      eUIUnit;      
	Fraction       aUIScale;     
	String         aUIUnitStr;   
	Fraction       aUIUnitFact;  
	int            nUIUnitKomma; 
	FASTBOOL       bUIOnlyKomma; 

	SdrLayerAdmin*  pLayerAdmin;
	SfxItemPool*    pItemPool;
	FASTBOOL        bMyPool;        
	SvPersist*      pPersist;
	SdrOutliner*    pDrawOutliner;  
	SdrOutliner*    pHitTestOutliner;
	ULONG           nDefTextHgt;    
	OutputDevice*   pRefOutDev;     
	ULONG           nProgressAkt;   
	ULONG           nProgressMax;   
	ULONG           nProgressOfs;   
	SfxStyleSheetBasePool* pStyleSheetPool;
	SfxStyleSheet*  pDefaultStyleSheet;
	SvxLinkManager* pLinkManager;   
	SdrModel*       pLoadedModel;   
	Container*      pUndoStack;
	Container*      pRedoStack;
	SdrUndoGroup*   pAktUndoGroup;  
	USHORT          nUndoLevel;     
	USHORT          nProgressPercent; 
	USHORT          nLoadVersion;   
	FASTBOOL        bExtColorTable; 
	FASTBOOL        bChanged;
	FASTBOOL        bInfoChanged;
	FASTBOOL        bPagNumsDirty;
	FASTBOOL        bMPgNumsDirty;
	FASTBOOL        bPageNotValid;  
	FASTBOOL        bSavePortable;  
	FASTBOOL        bNoBitmapCaching;   
	FASTBOOL        bLoading;           
	FASTBOOL        bStreamingSdrModel; 
#if SUPD > 356
	FASTBOOL        bReadOnly;
#endif
	FASTBOOL        bTransparentTextFrames;
	FASTBOOL        bSaveCompressed;
	FASTBOOL        bSwapGraphics;
	FASTBOOL        bPasteResize; 
	FASTBOOL		bSaveOLEPreview;      
	UINT16          nStreamCompressMode;  
	UINT16          nStreamNumberFormat;
	UINT16          nDefaultTabulator;
	SdrPageView*    pAktPaintPV;
	UINT32          nMaxUndoCount;
	FASTBOOL        bSaveNative;
	BOOL            bStarDrawPreviewMode;
	UINT16          nStarDrawPreviewMasterPageNum;

	SotStorage*		pModelStorage;
	SvxForbiddenCharactersTable* mpForbiddenCharactersTable;
	ULONG			nSwapGraphicsMode;

	UINT32          nReserveUInt2;
	UINT32          nReserveUInt3;
	UINT16          mnCharCompressType;
	UINT16          nReserveUInt5;
	UINT16          nReserveUInt6;
	UINT16          nReserveUInt7;
	FASTBOOL        mbModelLocked;
	FASTBOOL        bReserveBool2;
	FASTBOOL        bReserveBool3;

	XColorTable*    pColorTable;
	XDashList*      pDashList;
	XLineEndList*   pLineEndList;
	XHatchList*     pHatchList;
	XGradientList*  pGradientList;
	XBitmapList*    pBitmapList;

	SvNumberFormatter* mpNumberFormatter;
public:
	const SvNumberFormatter& GetNumberFormatter() const;
protected:

	virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > createUnoModel();

private:

	SdrModel(const SdrModel& rSrcModel);
	void operator=(const SdrModel& rSrcModel);
	FASTBOOL operator==(const SdrModel& rCmpModel) const;
#if 0 // _SOLAR__PRIVATE

	ULONG ImpCountAllSteamComponents() const;

	void ImpPostUndoAction(SdrUndoAction* pUndo);
	void ImpSetUIUnit();
	void ImpSetOutlinerDefaults( SdrOutliner* pOutliner, BOOL bInit = FALSE );
	void ImpReformatAllTextObjects();
	void ImpCreateTables();
	void ImpCtor(SfxItemPool* pPool, SvPersist* pPers, FASTBOOL bUseExtColorTable,
		FASTBOOL bLoadRefCounts = TRUE);

#endif 

	::com::sun::star::uno::WeakReference< ::com::sun::star::uno::XInterface > mxUnoModel;

public:
#if 0 // _SOLAR__PRIVATE
	FASTBOOL IsPasteResize() const        { return bPasteResize; }
	void     SetPasteResize(FASTBOOL bOn) { bPasteResize=bOn; }
#endif 
	TYPEINFO();

	SdrModel(SfxItemPool* pPool=NULL, SvPersist* pPers=NULL, INT32 bLoadRefCounts = LOADREFCOUNTS);
	SdrModel(const String& rPath, SfxItemPool* pPool=NULL, SvPersist* pPers=NULL, INT32 bLoadRefCounts = LOADREFCOUNTS);
	SdrModel(SfxItemPool* pPool, SvPersist* pPers, FASTBOOL bUseExtColorTable, INT32 bLoadRefCounts = LOADREFCOUNTS);
	SdrModel(const String& rPath, SfxItemPool* pPool, SvPersist* pPers, FASTBOOL bUseExtColorTable, INT32 bLoadRefCounts = LOADREFCOUNTS);
	virtual ~SdrModel();
	void    Clear();

	void    DoProgress(ULONG nVal);
	void    IncProgress()                       { DoProgress(nProgressAkt+1); }

	const SdrModelInfo& GetInfo() const         { return aInfo; }
	SdrModelInfo&       GetInfo()               { return aInfo; }

	FASTBOOL IsLoading() const                  { return bLoading; }

	FASTBOOL IsStreamingSdrModel() const        { return bStreamingSdrModel; }
	void     SetStreamingSdrModel(FASTBOOL bOn) { bStreamingSdrModel=bOn; }

	virtual SvStream* GetDocumentStream( SdrDocumentStreamInfo& rStreamInfo ) const;

	void     BurnInStyleSheetAttributes( BOOL bPseudoSheetsOnly=FALSE );

	void     RemoveNotPersistentObjects(FASTBOOL bNoBroadcast);

	virtual SdrPage*  AllocPage(FASTBOOL bMasterPage);
	virtual SdrModel* AllocModel() const;

	virtual const SdrModel* LoadModel(const String& rFileName);
	virtual void            DisposeLoadedModels();

	const SdrLayerAdmin& GetLayerAdmin() const                  { return *pLayerAdmin; }
	SdrLayerAdmin&       GetLayerAdmin()                        { return *pLayerAdmin; }

	const SfxItemPool&   GetItemPool() const                    { return *pItemPool; }
	SfxItemPool&         GetItemPool()                          { return *pItemPool; }

	SdrOutliner&         GetDrawOutliner(const SdrTextObj* pObj=NULL) const;
	SdrOutliner&         GetHitTestOutliner() const { return *pHitTestOutliner; }
	const SdrTextObj*    GetFormattingTextObj() const;

	void         		 SetTextDefaults() const;
	static void    		 SetTextDefaults( SfxItemPool* pItemPool, ULONG nDefTextHgt );

	void                 SetRefDevice(OutputDevice* pDev);
	OutputDevice*        GetRefDevice() const                   { return pRefOutDev; }

	void                 RefDeviceChanged(); 

	void                 SetDefaultFontHeight(ULONG nVal);
	ULONG                GetDefaultFontHeight() const           { return nDefTextHgt; }

	void                 SetDefaultTabulator(USHORT nVal);
	USHORT               GetDefaultTabulator() const            { return nDefaultTabulator; }

	SfxStyleSheet*       GetDefaultStyleSheet() const             { return pDefaultStyleSheet; }
	void                 SetDefaultStyleSheet(SfxStyleSheet* pDefSS) { pDefaultStyleSheet=pDefSS; }

	SvxLinkManager*      GetLinkManager()                         { return pLinkManager; }
	void                 SetLinkManager(SvxLinkManager* pLinkMgr) { pLinkManager = pLinkMgr; }

	SvPersist*           GetPersist() const                       { return pPersist; }

	MapUnit          GetScaleUnit() const                       { return eObjUnit; }
	void             SetScaleUnit(MapUnit eMap);
	const Fraction&  GetScaleFraction() const                   { return aObjUnit; }
	void             SetScaleFraction(const Fraction& rFrac);

	void             SetScaleUnit(MapUnit eMap, const Fraction& rFrac);

	const Size&      GetMaxObjSize() const                      { return aMaxObjSize; }
	void             SetMaxObjSize(const Size& rSiz)            { aMaxObjSize=rSiz; }

	void             SetUIUnit(FieldUnit eUnit);
	FieldUnit        GetUIUnit() const                          { return eUIUnit; }

	void             SetUIScale(const Fraction& rScale);
	const Fraction&  GetUIScale() const                         { return aUIScale; }

	void             SetUIUnit(FieldUnit eUnit, const Fraction& rScale);

	const Fraction&  GetUIUnitFact() const                      { return aUIUnitFact; }
	const String&    GetUIUnitStr() const                       { return aUIUnitStr; }
	int              GetUIUnitKomma() const                     { return nUIUnitKomma; }
	FASTBOOL         IsUIOnlyKomma() const                      { return bUIOnlyKomma; }

	void             TakeUnitStr(FieldUnit eUnit, String& rStr) const;
	void             TakeMetricStr(long nVal, String& rStr, FASTBOOL bNoUnitChars=FALSE) const;
	void             TakeWinkStr(long nWink, String& rStr, FASTBOOL bNoDegChar=FALSE) const;
	void             TakePercentStr(const Fraction& rVal, String& rStr, FASTBOOL bNoPercentChar=FALSE) const;

	FASTBOOL         IsPagNumsDirty() const                     { return bPagNumsDirty; };
	FASTBOOL         IsMPgNumsDirty() const                     { return bMPgNumsDirty; };
	void             RecalcPageNums(FASTBOOL bMaster);

	virtual void     InsertPage(SdrPage* pPage, USHORT nPos=0xFFFF);
	virtual void     DeletePage(USHORT nPgNum);

	virtual SdrPage* RemovePage(USHORT nPgNum);
	virtual void     MovePage(USHORT nPgNum, USHORT nNewPos);
	const SdrPage*   GetPage(USHORT nPgNum) const               { return (SdrPage*)(aPages.GetObject(nPgNum)); }
	SdrPage*         GetPage(USHORT nPgNum)                     { return (SdrPage*)(aPages.GetObject(nPgNum)); }

	USHORT           GetPageCount() const                       { return USHORT(aPages.Count()); }

	virtual void     InsertMasterPage(SdrPage* pPage, USHORT nPos=0xFFFF);
	virtual void     DeleteMasterPage(USHORT nPgNum);

	virtual SdrPage* RemoveMasterPage(USHORT nPgNum);
	virtual void     MoveMasterPage(USHORT nPgNum, USHORT nNewPos);
	const SdrPage*   GetMasterPage(USHORT nPgNum) const         { return (SdrPage*)(aMaPag.GetObject(nPgNum)); }
	SdrPage*         GetMasterPage(USHORT nPgNum)               { return (SdrPage*)(aMaPag.GetObject(nPgNum)); }

	USHORT           GetMasterPageCount() const                 { return USHORT(aMaPag.Count()); }

	FASTBOOL        IsChanged() const                           { return bChanged; }
	virtual void    SetChanged(FASTBOOL bFlg=TRUE);

	FASTBOOL        IsPageNotValid() const                     { return bPageNotValid; }
	void            SetPageNotValid(FASTBOOL bJa=TRUE)         { bPageNotValid=bJa; }

	FASTBOOL        IsSavePortable() const                     { return bSavePortable; }
	void            SetSavePortable(FASTBOOL bJa=TRUE)         { bSavePortable=bJa; }

	FASTBOOL        IsSaveCompressed() const                   { return bSaveCompressed; }
	void            SetSaveCompressed(FASTBOOL bJa=TRUE)       { bSaveCompressed=bJa; }

	FASTBOOL        IsSaveNative() const                       { return bSaveNative; }
	void            SetSaveNative(FASTBOOL bJa=TRUE)           { bSaveNative=bJa; }

	FASTBOOL        IsSwapGraphics() const { return bSwapGraphics; }
	void            SetSwapGraphics(FASTBOOL bJa=TRUE);
	void			SetSwapGraphicsMode(ULONG nMode) { nSwapGraphicsMode = nMode; }
	ULONG			GetSwapGraphicsMode() const { return nSwapGraphicsMode; }

	FASTBOOL        IsSaveOLEPreview() const          { return bSaveOLEPreview; }
	void            SetSaveOLEPreview( FASTBOOL bSet) { bSaveOLEPreview = bSet; }

	FASTBOOL        IsBitmapCaching() const                     { return !bNoBitmapCaching; }
	void            SetBitmapCaching(FASTBOOL bJa=TRUE)         { bNoBitmapCaching=!bJa; }

	FASTBOOL        IsPickThroughTransparentTextFrames() const  { return bTransparentTextFrames; }
	void            SetPickThroughTransparentTextFrames(FASTBOOL bOn) { bTransparentTextFrames=bOn; }

	SdrPageView*    GetPaintingPageView() const                 { return pAktPaintPV; }
	void            SetPaintingPageView(SdrPageView* pPV)       { pAktPaintPV=pPV; }

	virtual FASTBOOL IsReadOnly() const;
#if SUPD > 356
	virtual void     SetReadOnly(FASTBOOL bYes);
#endif

	void Merge(SdrModel& rSourceModel,
	  USHORT nFirstPageNum=0, USHORT nLastPageNum=0xFFFF,
	  USHORT nDestPos=0xFFFF,
	  FASTBOOL bMergeMasterPages=FALSE, FASTBOOL bAllMasterPages=FALSE,
	  FASTBOOL bUndo=TRUE, FASTBOOL bTreadSourceAsConst=FALSE);

	void CopyPages(USHORT nFirstPageNum, USHORT nLastPageNum,
	   USHORT nDestPos,
	   FASTBOOL bUndo=TRUE, FASTBOOL bMoveNoCopy=FALSE);

	void BegUndo();                       
	void BegUndo(const String& rComment); 
	void BegUndo(const String& rComment, const String& rObjDescr, SdrRepeatFunc eFunc=SDRREPFUNC_OBJ_NONE); 
	void BegUndo(SdrUndoGroup* pUndoGrp); 
	void EndUndo();                       
	void AddUndo(SdrUndoAction* pUndo);
	USHORT GetUndoBracketLevel() const                       { return nUndoLevel; }
	const SdrUndoGroup* GetAktUndoGroup() const              { return pAktUndoGroup; }

	void SetUndoComment(const String& rComment);
	void SetUndoComment(const String& rComment, const String& rObjDescr);

	void  SetMaxUndoActionCount(ULONG nAnz);
	ULONG GetMaxUndoActionCount() const { return nMaxUndoCount; }
	void  ClearUndoBuffer();

	ULONG GetUndoActionCount() const                      { return pUndoStack!=NULL ? pUndoStack->Count() : 0; }
	const SfxUndoAction* GetUndoAction(ULONG nNum) const  { return (SfxUndoAction*)(pUndoStack!=NULL ? pUndoStack->GetObject(nNum) : NULL); }

	ULONG GetRedoActionCount() const                      { return pRedoStack!=NULL ? pRedoStack->Count() : 0; }
	const SfxUndoAction* GetRedoAction(ULONG nNum) const  { return (SfxUndoAction*)(pRedoStack!=NULL ? pRedoStack->GetObject(nNum) : NULL); }

	FASTBOOL Undo();
	FASTBOOL Redo();
	FASTBOOL Repeat(SfxRepeatTarget&);

	void        SetNotifyUndoActionHdl(const Link& rLink)    { aUndoLink=rLink; }
	const Link& GetNotifyUndoActionHdl() const               { return aUndoLink; }

	void        SetIOProgressHdl(const Link& rLink)          { aIOProgressLink=rLink; }
	const Link& GetIOProgressHdl() const                     { return aIOProgressLink; }

	void            SetColorTable(XColorTable* pTable)       { pColorTable=pTable; }
	XColorTable*    GetColorTable() const                    { return pColorTable; }
	void            SetDashList(XDashList* pList)            { pDashList=pList; }
	XDashList*      GetDashList() const                      { return pDashList; }
	void            SetLineEndList(XLineEndList* pList)      { pLineEndList=pList; }
	XLineEndList*   GetLineEndList() const                   { return pLineEndList; }
	void            SetHatchList(XHatchList* pList)          { pHatchList=pList; }
	XHatchList*     GetHatchList() const                     { return pHatchList; }
	void            SetGradientList(XGradientList* pList)    { pGradientList=pList; }
	XGradientList*  GetGradientList() const                  { return pGradientList; }
	void            SetBitmapList(XBitmapList* pList)        { pBitmapList=pList; }
	XBitmapList*    GetBitmapList() const                    { return pBitmapList; }

	SfxStyleSheetBasePool* GetStyleSheetPool() const         { return pStyleSheetPool; }
	void SetStyleSheetPool(SfxStyleSheetBasePool* pPool)     { pStyleSheetPool=pPool; }

	friend SvStream& operator<<(SvStream& rOut, const SdrModel& rMod);

	friend SvStream& operator>>(SvStream& rIn, SdrModel& rMod);

	FASTBOOL WriteModelInfo(SvStream& rOut) const;

	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrIOHeader& rHead, SvStream& rIn);

	virtual void AfterRead();

	FASTBOOL CheckConsistence() const;

	void 	SetStarDrawPreviewMode(BOOL bPreview);
	BOOL 	IsStarDrawPreviewMode() { return bStarDrawPreviewMode; }

	SotStorage*	GetModelStorage() const { return pModelStorage; }
	void		SetModelStorage( SotStorage* pStor ) { pModelStorage = pStor; }

	void 	PrepareStore();

	void PreSave();
	void PostSave();

	::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > getUnoModel();

	BOOL isLocked() const { return mbModelLocked; }
	void setLock( BOOL bLock );

	void			SetForbiddenCharsTable( vos::ORef<SvxForbiddenCharactersTable> xForbiddenChars );
	vos::ORef<SvxForbiddenCharactersTable>	GetForbiddenCharsTable() const;

	void SetCharCompressType( UINT16 nType );
	UINT16 GetCharCompressType() const { return mnCharCompressType; }

	void ReformatAllTextObjects();

	FASTBOOL HasTransparentObjects( BOOL bCheckForAlphaChannel = FALSE ) const;
};

#endif 

