

#ifndef _SVDOSMEV_HXX
#define _SVDOSMEV_HXX

#ifndef _BIGINT_HXX
#include <tools/bigint.hxx>
#endif

#ifndef _SV_FIELD_HXX 
#include <vcl/field.hxx>
#endif

class SdrObject;
class SdrObjSmartData;
class ImpSdrExprTokenList;
class ImpSdrExprToken;
class SdrObjList;
class SdrSmartGeometricList;
class SdrSmartGeometric;
class SdrSmartGeometricLine;
class SdrSmartDraftLineList;
class SdrSmartDraftLine;
class SdrSmartGluePointList;
class SdrSmartGluePoint;
class SdrSmartDragPointList;
class SdrSmartDragPoint;
class Date;
class Time;
class DateTime;

enum SdrExprErrKind {
	SDREXPRERR_NONE,            
	SDREXPRERR_DIVZERO,         
	SDREXPRERR_CIRCREF,         
	SDREXPRERR_VALUEEXPECTED,   
	SDREXPRERR_VAREXPECTED,     
	SDREXPRERR_VALUETOOBIG,     
	SDREXPRERR_UNEXPECTEDCHAR,  
	SDREXPRERR_UNEXPECTEDEOLN,  
	SDREXPRERR_UNKNOWNIDENT,    
	SDREXPRERR_BRKOPEXPECTED,   
	SDREXPRERR_BRKCLEXPECTED,   
	SDREXPRERR_SQBRKOPEXPECTED, 
	SDREXPRERR_SQBRKCLEXPECTED, 
	SDREXPRERR_SPBRKOPEXPECTED, 
	SDREXPRERR_SPBRKCLEXPECTED, 
	SDREXPRERR_PARAMEXPECTED,   
	SDREXPRERR_ELSEEXPECTED,    
	SDREXPRERR_MEMBEREXPECTED,  
	SDREXPRERR_FALSEMEMBER,     
	SDREXPRERR_BASEMUSTBEINT,   
	SDREXPRERR_BASEMUSTBEPOS,   
	SDREXPRERR_DUPLICATEBASE,   
	SDREXPRERR_BASECLEXPECTED,  
	SDREXPRERR_INVALIDFUNCARG,  
	SDREXPRERR_INVALIDDIGIT,    
	SDREXPRERR_COMPLEXVALUE,    
	SDREXPRERR_DOUBLEDECPOINT   
};

enum SdrExprValueKind {
	SDREXPRVALUE_SIMPLE,     
	SDREXPRVALUE_COMPLEX,    
	SDREXPRVALUE_UNITS,      
	SDREXPRVALUE_ANGLE,      

	SDREXPRVALUE_POINT,      
	SDREXPRVALUE_DATE,       
	SDREXPRVALUE_TIME,       
	SDREXPRVALUE_DATETIME    
};

class SdrExprValue {
friend class ImpSdrSmartObjEvaluator;

protected:
	BigInt    nX,nY; 
	double    fX,fY; 
	ULONG     nMSecsUsed;
	FASTBOOL  bFloat; 
	FASTBOOL  bUIMap;
	FASTBOOL  bDirty;
	FASTBOOL  bHasValue;
	FASTBOOL  bLocked;
	FASTBOOL  bIsField;     
	FASTBOOL  bIsFieldPtr;  
	SdrExprValueKind eKind;
	FieldUnit eUIMap;
	MapUnit   eCoreMap;
	USHORT    nUnitPower;
private:
	virtual void Undirty(const SdrObject* pObj);
	void CTor();
	FASTBOOL HasUnit() const                    { return nUnitPower!=0; }
	FASTBOOL IsSameUnit(const SdrExprValue& rVal) const;
public:
	SdrExprValue()                              { CTor(); }
	SdrExprValue(double fVal)                   { CTor(); SetValue(fVal);         }
	SdrExprValue(long nVal)                     { CTor(); SetValue(nVal);         }
	SdrExprValue(const BigInt& nVal)            { CTor(); SetValue(nVal);         }
	SdrExprValue(double fVal, FieldUnit eU)     { CTor(); SetUnits(fVal,eU);      }
	SdrExprValue(long nVal, FieldUnit eU)       { CTor(); SetUnits(nVal,eU);      }
	SdrExprValue(const BigInt& n, FieldUnit eU) { CTor(); SetUnits(n,eU);         }
	SdrExprValue(double fVal, MapUnit eU)       { CTor(); SetUnits(fVal,eU);      }
	SdrExprValue(long nVal, MapUnit eU)         { CTor(); SetUnits(nVal,eU);      }
	SdrExprValue(const BigInt& n, MapUnit eU)   { CTor(); SetUnits(n,eU);         }
	SdrExprValue(const Point& P, MapUnit eU)    { CTor(); SetUnits(P,eU);         }
	SdrExprValue(FASTBOOL bVal)                 { CTor(); SetBool(bVal);          }
	SdrExprValue(const Time& rTime)             { CTor(); SetTime(rTime);         }
	SdrExprValue(const Date& rDate)             { CTor(); SetDate(rDate);         }
	SdrExprValue(const DateTime& rDateTime)     { CTor(); SetDateTime(rDateTime); }
	virtual ~SdrExprValue();

	const SdrExprValue& ForceUndirty(const SdrObject* pObj) const { if (bDirty) ((SdrExprValue*)this)->Undirty(pObj); return *this; }
	void             SetDirty()                             { bDirty=TRUE; }
	FASTBOOL         IsDirty() const                        { return bDirty; }
	FASTBOOL         IsField() const                        { return bIsField; }
	FASTBOOL         IsFieldPtr() const                     { return bIsFieldPtr; }

	FASTBOOL         IsLocked(const SdrObject* pObj) const  { ForceUndirty(pObj); return bLocked; }
	ULONG            GetMSecsUsed(const SdrObject* pO) const{ ForceUndirty(pO  ); return nMSecsUsed; }
	FASTBOOL         HasValue(const SdrObject* pObj) const  { ForceUndirty(pObj); return bHasValue; }
	SdrExprValueKind GetExprKind(const SdrObject* pObj) const{ForceUndirty(pObj); return eKind; }
	FASTBOOL         IsLong(const SdrObject* pObj) const    { ForceUndirty(pObj); return !bFloat ; }
	FASTBOOL         IsBigInt(const SdrObject* pObj) const  { ForceUndirty(pObj); return !bFloat ; }
	FASTBOOL         IsFloat(const SdrObject* pObj) const   { ForceUndirty(pObj); return bFloat; }
	double           GetFloat(const SdrObject* pObj) const  { ForceUndirty(pObj); return fX; }
	const BigInt&    GetBigInt(const SdrObject* pObj) const { ForceUndirty(pObj); return nX; }
	long             GetLong(const SdrObject* pObj) const   { ForceUndirty(pObj); return long(nX); }
	FASTBOOL         GetBool(const SdrObject* pObj) const   { ForceUndirty(pObj); return !nX.IsZero(); }
	MapUnit          GetCoreMap(const SdrObject* pObj) const{ ForceUndirty(pObj); return eCoreMap; }
	FieldUnit        GetUIMap(const SdrObject* pObj) const  { ForceUndirty(pObj); return eUIMap; }
	FASTBOOL         QuickIsLocked() const                  { return bLocked; }
	ULONG            QuickGetMSecsUsed() const              { return nMSecsUsed; }
	FASTBOOL         QuickHasValue() const                  { return bHasValue; }
	SdrExprValueKind QuickGetExprKind() const               { return eKind; }
	FASTBOOL         QuickIsLong() const                    { return !bFloat ; }
	FASTBOOL         QuickIsBigInt() const                  { return !bFloat ; }
	FASTBOOL         QuickIsFloat() const                   { return bFloat; }
	double           QuickGetFloat() const                  { return fX; }
	const BigInt&    QuickGetBigInt() const                 { return nX; }
	long             QuickGetLong() const                   { return long(nX); }
	FASTBOOL         QuickGetBool() const                   { return !nX.IsZero(); }
	MapUnit          QuickGetCoreMap() const                { return eCoreMap; }
	FieldUnit        QuickGetUIMap() const                  { return eUIMap; }

	long             QuickGetUnits(const BigInt& rVal, MapUnit eU) const;
	long             QuickGetUnits(double nVal, MapUnit eU) const;
	double           QuickGetFloatNoUnits(const BigInt& rVal) const;
	double           QuickGetFloatNoUnits(double nVal) const;

	long             GetUnits(MapUnit eU, const SdrObject* pObj) const  { ForceUndirty(pObj); return QuickGetUnits(eU); }
	long             QuickGetUnits(MapUnit eU) const;
	long             QuickGetLongNoUnits() const { return (long)QuickGetFloatNoUnits(); }
	double           QuickGetFloatNoUnits() const;

	Time             QuickGetTime() const;
	Date             QuickGetDate() const;
	DateTime         QuickGetDateTime() const;

	FASTBOOL         QuickIsDate() const       { return eKind==SDREXPRVALUE_DATE; }
	FASTBOOL         QuickIsTime() const       { return eKind==SDREXPRVALUE_TIME; }
	FASTBOOL         QuickIsDateTime() const   { return eKind==SDREXPRVALUE_DATETIME; }
	FASTBOOL         QuickIsDateOrTime() const { return eKind==SDREXPRVALUE_DATE || eKind==SDREXPRVALUE_TIME || eKind==SDREXPRVALUE_DATETIME; }
	BigInt           QuickGetDateTimeMSecs() const;

	void SetExprKind(SdrExprValueKind eNew)  { eKind=eNew; }

	void Negate();

	void SetValue(double fVal);
	void SetValue(long nVal);
	void SetValue(const BigInt& nVal);

	void SetUnits(double fVal, FieldUnit eU);
	void SetUnits(long nVal, FieldUnit eU);
	void SetUnits(const BigInt& n, FieldUnit eU);

	void SetUnits(double fVal, MapUnit eU);
	void SetUnits(long nVal, MapUnit eU);
	void SetUnits(const BigInt& n, MapUnit eU);
	void SetUnits(const Point& P, MapUnit eU);

	void SetBool(FASTBOOL bVal);
	void SetTime(const Time& rTime);
	void SetDate(const Date& rDate);
	void SetDateTime(const DateTime& rDateTime);

	void operator+=(const SdrExprValue& rVal);
	void operator-=(const SdrExprValue& rVal);
	void operator*=(const SdrExprValue& rVal);
	void operator/=(const SdrExprValue& rVal);
	void operator%=(const SdrExprValue& rVal);
	FASTBOOL operator==(const SdrExprValue& rVal) const;
	FASTBOOL operator!=(const SdrExprValue& rVal) const    { return !operator==(rVal); }
	FASTBOOL operator==(double fVal) const;
	FASTBOOL operator!=(double fVal) const                 { return !operator==(fVal); }
	FASTBOOL operator==(long nVal) const;
	FASTBOOL operator!=(long nVal) const                   { return !operator==(nVal); }
	FASTBOOL operator==(const BigInt& nVal) const;
	FASTBOOL operator!=(const BigInt& nVal) const          { return !operator==(nVal); }
	FASTBOOL operator> (const SdrExprValue& rVal) const;
	FASTBOOL operator>=(const SdrExprValue& rVal) const;
	FASTBOOL operator< (const SdrExprValue& rVal) const    { return !operator>=(rVal); }
	FASTBOOL operator<=(const SdrExprValue& rVal) const    { return !operator>(rVal); }
	FASTBOOL operator> (double fVal) const;
	FASTBOOL operator>=(double fVal) const;
	FASTBOOL operator< (double fVal) const                 { return !operator>=(fVal); }
	FASTBOOL operator<=(double fVal) const                 { return !operator>(fVal); }
	FASTBOOL operator> (long nVal) const;
	FASTBOOL operator>=(long nVal) const;
	FASTBOOL operator< (long nVal) const                   { return !operator>=(nVal); }
	FASTBOOL operator<=(long nVal) const                   { return !operator>(nVal); }
	FASTBOOL operator> (const BigInt& nVal) const;
	FASTBOOL operator>=(const BigInt& nVal) const;
	FASTBOOL operator< (const BigInt& nVal) const          { return !operator>=(nVal); }
	FASTBOOL operator<=(const BigInt& nVal) const          { return !operator>(nVal); }
};

class ImpSdrSmartObjEvaluator {
friend class ImpZwischenergebnis;
	const ImpSdrExprTokenList* pTokenList;
	const SdrObject* pObj;
	const SdrObjSmartData* pSmart;
	const ImpSdrExprToken* pAktTok;
	const SdrExprValue* pTargetField; 
	SdrExprErrKind eError;
	ULONG          nErrPos;
	ULONG          nAnz;
	ULONG          nPos;
	SdrExprValue   aResult;
	FASTBOOL       bMetric;
	FASTBOOL       bMetricX;
	FASTBOOL       bMetricY;
	FieldUnit      eUIMap;
	MapUnit        eCoreMap;

	FASTBOOL       bFieldLocationSearched;
	FASTBOOL       bFieldLocationFound;
	USHORT         nFieldArea; 
	ULONG          nGeoFieldAreaNum;
	ULONG          nFieldLine;
	ULONG          nFieldColumn;
	ULONG          nMSecsUsed; 
	FASTBOOL       bLocked;
private:
	void         SetMSecsUsed(ULONG n);
	void         SearchFieldLocation();
	void         ForceFieldLocation() const { if (!bFieldLocationSearched) ((ImpSdrSmartObjEvaluator*)this)->SearchFieldLocation(); }
	void         Error(ULONG nEPos, SdrExprErrKind eECode);
	FASTBOOL     TokensLeft() const { return nPos<nAnz; }
	const ImpSdrExprToken* NextToken();
	FASTBOOL     BracketOpen();
	FASTBOOL     BracketClose(USHORT eOpen);
	FASTBOOL     NextArg(SdrExprValue*& pArg, FASTBOOL bSkip);
	void         FuncRound(FASTBOOL bSkip, double nRet, SdrExprValue* pRet, SdrExprValue*& pArg);
	void         FuncRandom(FASTBOOL bSkip, FASTBOOL bBracket, double nRet, SdrExprValue* pRet, SdrExprValue*& pArg);
	SdrExprValue* Memb(FASTBOOL bSkip);
	SdrExprValue* Func(FASTBOOL bSkip);
	SdrExprValue* Wert(FASTBOOL bSkip);
	SdrExprValue* Fakt(FASTBOOL bSkip);
	SdrExprValue* Term(FASTBOOL bSkip);
	SdrExprValue* Expr2(FASTBOOL bSkip);
	SdrExprValue* CompTerm(FASTBOOL bSkip);
	SdrExprValue* BoolFakt(FASTBOOL bSkip);
	SdrExprValue* BoolXTerm(FASTBOOL bSkip);
	SdrExprValue* BoolTerm(FASTBOOL bSkip);
	SdrExprValue* Condition(FASTBOOL bSkip);
	SdrExprValue* Expr(FASTBOOL bSkip);
public:
	ImpSdrSmartObjEvaluator(const SdrObject* pObj_);
	void SetMetric(FASTBOOL bOn)  { bMetric=bOn; bMetricX=FALSE; bMetricY=FALSE; }
	void SetMetricX(FASTBOOL bOn) { bMetric=bOn; bMetricX=TRUE;  bMetricY=FALSE; }
	void SetMetricY(FASTBOOL bOn) { bMetric=bOn; bMetricX=FALSE; bMetricY=TRUE;  }
	FASTBOOL Evaluate(const ImpSdrExprTokenList* pTokenList_, const SdrExprValue* pTargetField_);
	FASTBOOL HasValue() const       { return aResult.HasValue(pObj); }
	FASTBOOL IsLocked() const       { return aResult.IsLocked(pObj); }
	FASTBOOL IsError() const        { return eError!=SDREXPRERR_NONE; }
	SdrExprErrKind GetError() const { return eError; }
	static XubString GetErrorStr(SdrExprErrKind eErr);
	XubString GetErrorStr() const    { return GetErrorStr(eError); }
	ULONG    GetErrorPos() const    { return nErrPos; }
	const SdrExprValue& GetResult() const { return aResult; }
	MapUnit GetCoreMap() const      { return eCoreMap; }
	const SdrObject* GetObject() const { return pObj; }
	const SdrObjSmartData* GetSmartData() const { return pSmart; }
	ULONG GetMSecsUsed() const  { return nMSecsUsed; }
};

#endif 

