

#ifndef _SVDOTEXT_HXX
#define _SVDOTEXT_HXX

#ifndef VCL
#ifndef _SV_FIELD_HXX
#include <vcl/field.hxx>
#endif
#else
#ifndef _SV_FIELD_HXX
#include <vcl/field.hxx>
#endif
#endif

#ifndef _SVDOATTR_HXX
#include <svx/svdoattr.hxx>
#endif

#ifndef _SVDTRANS_HXX
#include <svx/svdtrans.hxx> 
#endif

#ifndef _DATETIME_HXX 
#include <tools/datetime.hxx>
#endif

#ifndef _SVX_XTXASIT_HXX 
#include <svx/xtxasit.hxx>
#endif

#ifndef _SVX_TEXTIT0_HXX 
#include <svx/xtextit0.hxx>
#endif

class OutlinerParaObject;
class SdrOutliner;
class SvxFieldItem;
class ImpSdrObjTextLink;

#define SDRUSERDATA_OBJTEXTLINK			(1)

class SdrTextObjGeoData : public SdrObjGeoData
{
public:
	Rectangle					aRect;
	GeoStat						aGeo;
};

class ImpSdrObjTextLinkUserData : public SdrObjUserData
{
	friend class				SdrTextObj;
	friend class				ImpSdrObjTextLink;

	SdrTextObj*					pObj;
	String						aFileName;   
	String						aFilterName; 
	DateTime					aFileDate0;  
	ImpSdrObjTextLink*			pLink;
	rtl_TextEncoding			eCharSet;

public:
	TYPEINFO();
	ImpSdrObjTextLinkUserData(SdrTextObj* pObj1);
	virtual ~ImpSdrObjTextLinkUserData();

	virtual SdrObjUserData* Clone(SdrObject* pObj1) const;
	virtual void WriteData(SvStream& rOut);
	virtual void ReadData(SvStream& rIn);
	virtual void AfterRead();
};

class SdrTextObj : public SdrAttrObj
{
	friend class				ImpTextPortionHandler;
	friend class				ImpSdrObjTextLink;
	friend class				ImpSdrObjTextLinkUserData;
	friend class				SdrPowerPointImport; 
	friend class				SdrExchangeView; 
	friend class				SdrView;         
	friend class				SdrObjEditView;  
	friend class				SdrMeasureObj;   
	friend class				SvxMSDffManager; 

protected:

	Rectangle					aRect;

	GeoStat						aGeo;

	OutlinerParaObject*			pOutlinerParaObject;

	Size						aTextSize;

	SdrOutliner*   				pEdtOutl;

	Rectangle*					pFormTextBoundRect;

	SdrObjKind					eTextKind;

	BOOL						bTextFrame : 1;
	BOOL						bPortionInfoChecked : 1; 
	BOOL						bNoShear : 1;            
	BOOL						bNoRotate : 1;           
	BOOL						bNoMirror : 1;           
	BOOL						bTextSizeDirty : 1;

	BOOL bDisableAutoWidthOnDragging : 1;

private:
	void ImpCheckMasterCachable();
	SdrOutliner& ImpGetDrawOutliner() const;
	SdrObject* ImpConvertObj(FASTBOOL bToPoly) const;
	void ImpLinkAnmeldung();
	void ImpLinkAbmeldung();
	ImpSdrObjTextLinkUserData* GetLinkUserData() const;

protected:
	FASTBOOL ImpCanConvTextToCurve() const { return pOutlinerParaObject!=NULL && pModel!=NULL && !IsOutlText() && !IsFontwork(); }
	void ImpConvertSetAttrAndLayer(SdrObject* pObj, FASTBOOL bNoSetAttr=FALSE) const;
	SdrObject* ImpConvertMakeObj(const XPolyPolygon& rXPP, FASTBOOL bClosed, FASTBOOL bBezier, FASTBOOL bNoSetAttr=FALSE) const;
	SdrObject* ImpConvertAddText(SdrObject* pObj, FASTBOOL bBezier) const;
	void ImpSetTextStyleSheetListeners();
	void ImpSetCharStretching(SdrOutliner& rOutliner, const Rectangle& rTextRect, const Rectangle& rAnchorRect, Fraction& rFitXKorreg) const;
	void ImpAddTextToBoundRect();
	void ImpJustifyRect(Rectangle& rRect) const;
	void ImpCheckShear();
	Rectangle ImpDragCalcRect(const SdrDragStat& rDrag) const;
	void ImpSetTextEditParams() const;
	void SetTextSizeDirty() { bTextSizeDirty=TRUE; }
	FASTBOOL ImpPaintAnimatedText(OutputDevice& rOut, const Point& rOfs, SdrOutliner& rOutliner,
		const Rectangle& rAnchorRect, const Rectangle& rPaintRect, const SdrPaintInfoRec& rInfoRec) const;

	void ImpSetContourPolygon( SdrOutliner& rOutliner, Rectangle& rAnchorRect, BOOL bLineWidth ) const;

	DECL_LINK(ImpAnimationHdl,ImpSdrMtfAnimator*);
	virtual void SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType, const SfxHint& rHint, const TypeId& rHintType);
	virtual SdrObjGeoData* NewGeoData() const;
	virtual void SaveGeoData(SdrObjGeoData& rGeo) const;
	virtual void RestGeoData(const SdrObjGeoData& rGeo);
	FASTBOOL NbcSetEckenradius(long nRad);
	FASTBOOL NbcSetAutoGrowHeight(FASTBOOL bAuto);
	FASTBOOL NbcSetMinTextFrameHeight(long nHgt);
	FASTBOOL NbcSetMaxTextFrameHeight(long nHgt);
	FASTBOOL NbcSetAutoGrowWidth(FASTBOOL bAuto);
	FASTBOOL NbcSetMinTextFrameWidth(long nWdt);
	FASTBOOL NbcSetMaxTextFrameWidth(long nWdt);
	FASTBOOL NbcSetFitToSize(SdrFitToSizeType eFit);

	SdrTextObj();
	SdrTextObj(const Rectangle& rNewRect);

	SdrTextObj(SdrObjKind eNewTextKind);
	SdrTextObj(SdrObjKind eNewTextKind, const Rectangle& rNewRect);

	SdrTextObj(SdrObjKind eNewTextKind, const Rectangle& rNewRect, SvStream& rInput, USHORT eFormat);
	virtual ~SdrTextObj();

public:
	TYPEINFO();

	void SetTextLink(const String& rFileName, const String& rFilterName, rtl_TextEncoding eCharSet);
	void ReleaseTextLink();
	FASTBOOL IsLinkedText() const { return pPlusData!=NULL && GetLinkUserData()!=NULL; }
	FASTBOOL ReloadLinkedText(FASTBOOL bForceLoad=FALSE);
	FASTBOOL LoadText(const String& rFileName, const String& rFilterName, rtl_TextEncoding eCharSet);

	FASTBOOL AdjustTextFrameWidthAndHeight(Rectangle& rR, FASTBOOL bHgt=TRUE, FASTBOOL bWdt=TRUE) const;
	FASTBOOL NbcAdjustTextFrameWidthAndHeight(FASTBOOL bHgt=TRUE, FASTBOOL bWdt=TRUE);
	FASTBOOL AdjustTextFrameWidthAndHeight(FASTBOOL bHgt=TRUE, FASTBOOL bWdt=TRUE);
	void NbcResizeTextAttributes(const Fraction& xFact, const Fraction& yFact);
	FASTBOOL IsTextFrame() const { return bTextFrame; }
	FASTBOOL IsOutlText() const { return bTextFrame && (eTextKind==OBJ_OUTLINETEXT || eTextKind==OBJ_TITLETEXT); }
	BOOL GetTextKind() const { return eTextKind; }
	FASTBOOL HasText() const { return pEdtOutl==NULL ? pOutlinerParaObject!=NULL : HasEditText(); }
	FASTBOOL HasEditText() const;
	FASTBOOL IsTextEditActive() const { return pEdtOutl!=NULL; }
	void SetDisableAutoWidthOnDragging(FASTBOOL bOn) { bDisableAutoWidthOnDragging=bOn; }
	FASTBOOL IsDisableAutoWidthOnDragging() { return bDisableAutoWidthOnDragging; }
	void NbcSetText(const String& rStr);
	void SetText(const String& rStr);
	void NbcSetText(SvStream& rInput, USHORT eFormat);
	void SetText(SvStream& rInput, USHORT eFormat);

	virtual const Size& GetTextSize() const;
	void FitFrameToTextSize();

	virtual void TakeTextRect( SdrOutliner& rOutliner, Rectangle& rTextRect, FASTBOOL bNoEditText=FALSE,
		Rectangle* pAnchorRect=NULL, BOOL bLineWidth=TRUE ) const;
	virtual void TakeTextAnchorRect(Rectangle& rAnchorRect) const;
	const GeoStat& GetGeoStat() const { return aGeo; }
	inline long GetEckenradius() const;
	FASTBOOL IsAutoGrowHeight() const;
	inline long GetMinTextFrameHeight() const;
	inline long GetMaxTextFrameHeight() const;
	FASTBOOL IsAutoGrowWidth() const;
	inline long GetMinTextFrameWidth() const;
	inline long GetMaxTextFrameWidth() const;
	SdrFitToSizeType GetFitToSize() const;

	inline FASTBOOL IsFontwork() const;

	inline FASTBOOL IsHideContour() const;

	inline FASTBOOL IsContourTextFrame() const;

	SdrTextHorzAdjust GetTextHorizontalAdjust() const;

	SdrTextVertAdjust GetTextVerticalAdjust() const;

	inline long GetTextLeftDistance() const;
	inline long GetTextRightDistance() const;
	inline long GetTextUpperDistance() const;
	inline long GetTextLowerDistance() const;
	inline SdrTextAniKind GetTextAniKind() const;
	inline SdrTextAniDirection GetTextAniDirection() const;

	virtual void SetPage(SdrPage* pNewPage);
	virtual void SetModel(SdrModel* pNewModel);
	virtual void TakeObjInfo(SdrObjTransformInfoRec& rInfo) const;
	virtual UINT16 GetObjIdentifier() const;
	virtual FASTBOOL Paint(ExtOutputDevice& rOut, const SdrPaintInfoRec& rInfoRec) const;
	virtual void RecalcBoundRect();

	virtual void TakeUnrotatedSnapRect(Rectangle& rRect) const;
	virtual SdrObject* CheckHit(const Point& rPnt, USHORT nTol, const SetOfByte* pVisiLayer) const;
	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;
	virtual void operator=(const SdrObject& rObj);
	virtual void TakeXorPoly(XPolyPolygon& rPoly, FASTBOOL bDetail) const;
	virtual void TakeContour(XPolyPolygon& rPoly) const;
	virtual void TakeContour(XPolyPolygon& rXPoly, SdrContourType eType) const;
	virtual void RecalcSnapRect();
	virtual void NbcSetSnapRect(const Rectangle& rRect);
	virtual void NbcSetLogicRect(const Rectangle& rRect);
	virtual const Rectangle& GetLogicRect() const;
	virtual long GetRotateAngle() const;
	virtual long GetShearAngle(FASTBOOL bVertical=FALSE) const;

	virtual USHORT GetSnapPointCount() const;
	virtual Point GetSnapPoint(USHORT i) const;

	virtual USHORT GetHdlCount() const;
	virtual SdrHdl* GetHdl(USHORT nHdlNum) const;
	virtual FASTBOOL HasSpecialDrag() const;
	virtual FASTBOOL BegDrag(SdrDragStat& rDrag) const;
	virtual FASTBOOL MovDrag(SdrDragStat& rDrag) const;
	virtual FASTBOOL EndDrag(SdrDragStat& rDrag);
	virtual void BrkDrag(SdrDragStat& rDrag) const;
	virtual String GetDragComment(const SdrDragStat& rDrag, FASTBOOL bUndoDragComment, FASTBOOL bCreateComment) const;
	virtual void TakeDragPoly(const SdrDragStat& rDrag, XPolyPolygon& rXPP) const;

	virtual FASTBOOL BegCreate(SdrDragStat& rStat);
	virtual FASTBOOL MovCreate(SdrDragStat& rStat);
	virtual FASTBOOL EndCreate(SdrDragStat& rStat, SdrCreateCmd eCmd);
	virtual FASTBOOL BckCreate(SdrDragStat& rStat);
	virtual void BrkCreate(SdrDragStat& rStat);
	virtual void TakeCreatePoly(const SdrDragStat& rDrag, XPolyPolygon& rXPP) const;
	virtual Pointer GetCreatePointer() const;

	virtual void NbcMove(const Size& rSiz);
	virtual void NbcResize(const Point& rRef, const Fraction& xFact, const Fraction& yFact);
	virtual void NbcRotate(const Point& rRef, long nWink, double sn, double cs);
	virtual void NbcMirror(const Point& rRef1, const Point& rRef2);
	virtual void NbcShear(const Point& rRef, long nWink, double tn, FASTBOOL bVShear);

	virtual FASTBOOL HasTextEdit() const;
	virtual FASTBOOL BegTextEdit(SdrOutliner& rOutl);
	virtual void TakeTextEditArea(Size* pPaperMin, Size* pPaperMax, Rectangle* pViewInit, Rectangle* pViewMin) const;
	virtual void EndTextEdit(SdrOutliner& rOutl);
	virtual SdrObject* CheckTextEditHit(const Point& rPnt, USHORT nTol, const SetOfByte* pVisiLayer) const;
	virtual USHORT GetOutlinerViewAnchorMode() const;

	FASTBOOL IsTextAnimated() const { return GetTextAniKind()!=SDRTEXTANI_NONE; }
	void StartTextAnimation(OutputDevice* pOutDev, const Point& rOffset, long nExtraData=0L);
	void StopTextAnimation(OutputDevice* pOutDev=NULL, long nExtraData=0L);
	void SetTextAnimationSupervisor(OutputDevice* pDisplayDev, BOOL bObjSupervises);

	virtual void NbcSetOutlinerParaObject(OutlinerParaObject* pTextObject);
	virtual OutlinerParaObject* GetOutlinerParaObject() const;
	OutlinerParaObject* GetEditOutlinerParaObject() const;

	virtual void NbcReformatText();
	virtual void ReformatText();
	virtual void RestartAnimation(SdrPageView* pPageView) const;

	virtual FASTBOOL CalcFieldValue(const SvxFieldItem& rField, USHORT nPara, USHORT nPos,
		FASTBOOL bEdit, Color*& rpTxtColor, Color*& rpFldColor, String& rRet) const;

	virtual SdrObject* DoConvertToPolyObj(BOOL bBezier) const;

	virtual void NbcSetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);

	virtual void ItemSetChanged(const SfxItemSet& rSet);
	virtual void ItemChange(const sal_uInt16 nWhich, const SfxPoolItem* pNewItem = 0);

	virtual void BurnInStyleSheetAttributes( BOOL bPseudoSheetsOnly=FALSE );

	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);

	void SetTextEditOutliner(SdrOutliner* pOutl) { pEdtOutl=pOutl; }

	virtual void PreSave();

	void ForceOutlinerParaObject();
	BOOL IsVerticalWriting() const;
	void SetVerticalWriting( BOOL bVertical );

	virtual BOOL TRGetBaseGeometry(Matrix3D& rMat, XPolyPolygon& rPolyPolygon) const;

	virtual void TRSetBaseGeometry(const Matrix3D& rMat, const XPolyPolygon& rPolyPolygon);
};

inline long SdrTextObj::GetEckenradius() const
{
	return ((SdrEckenradiusItem&)(GetItemSet().Get(SDRATTR_ECKENRADIUS))).GetValue();
}

inline long SdrTextObj::GetMinTextFrameHeight() const
{
	return ((SdrTextMinFrameHeightItem&)(GetItemSet().Get(SDRATTR_TEXT_MINFRAMEHEIGHT))).GetValue();
}

inline long SdrTextObj::GetMaxTextFrameHeight() const
{
	return ((SdrTextMaxFrameHeightItem&)(GetItemSet().Get(SDRATTR_TEXT_MAXFRAMEHEIGHT))).GetValue();
}

inline long SdrTextObj::GetMinTextFrameWidth() const
{
	return ((SdrTextMinFrameWidthItem&)(GetItemSet().Get(SDRATTR_TEXT_MINFRAMEWIDTH))).GetValue();
}

inline long SdrTextObj::GetMaxTextFrameWidth() const
{
	return ((SdrTextMaxFrameWidthItem&)(GetItemSet().Get(SDRATTR_TEXT_MAXFRAMEWIDTH))).GetValue();
}

inline FASTBOOL SdrTextObj::IsFontwork() const
{
	return (bTextFrame) ? FALSE 
		: ((XFormTextStyleItem&)(GetItemSet().Get(XATTR_FORMTXTSTYLE))).GetValue()!=XFT_NONE;
}

inline FASTBOOL SdrTextObj::IsHideContour() const
{
	return (bTextFrame) ? FALSE 
		: ((XFormTextHideFormItem&)(GetItemSet().Get(XATTR_FORMTXTHIDEFORM))).GetValue();
}

inline FASTBOOL SdrTextObj::IsContourTextFrame() const
{
	return (bTextFrame) ? FALSE 
		: ((SdrTextContourFrameItem&)(GetItemSet().Get(SDRATTR_TEXT_CONTOURFRAME))).GetValue();
}

inline long SdrTextObj::GetTextLeftDistance() const
{
	return ((SdrTextLeftDistItem&)(GetItemSet().Get(SDRATTR_TEXT_LEFTDIST))).GetValue();
}

inline long SdrTextObj::GetTextRightDistance() const
{
	return ((SdrTextRightDistItem&)(GetItemSet().Get(SDRATTR_TEXT_RIGHTDIST))).GetValue();
}

inline long SdrTextObj::GetTextUpperDistance() const
{
	return ((SdrTextUpperDistItem&)(GetItemSet().Get(SDRATTR_TEXT_UPPERDIST))).GetValue();
}

inline long SdrTextObj::GetTextLowerDistance() const
{
	return ((SdrTextLowerDistItem&)(GetItemSet().Get(SDRATTR_TEXT_LOWERDIST))).GetValue();
}

inline SdrTextAniKind SdrTextObj::GetTextAniKind() const
{
	return ((SdrTextAniKindItem&)(GetItemSet().Get(SDRATTR_TEXT_ANIKIND))).GetValue();
}

inline SdrTextAniDirection SdrTextObj::GetTextAniDirection() const
{
	return ((SdrTextAniDirectionItem&)(GetItemSet().Get(SDRATTR_TEXT_ANIDIRECTION))).GetValue();
}

#endif 

