

#ifndef _SVDPAGE_HXX
#define _SVDPAGE_HXX

#ifndef _SV_BITMAP_HXX
#include <vcl/bitmap.hxx>
#endif

#ifndef _PRINT_HXX 
#include <vcl/print.hxx>
#endif
#ifndef _GDIMTF_HXX 
#include <vcl/gdimtf.hxx>
#endif

#ifndef _CPPUHELPER_WEAKREF_HXX_
#include <cppuhelper/weakref.hxx>
#endif

#ifndef _SVDTYPES_HXX
#include <svx/svdtypes.hxx>
#endif
#ifndef _SVDLAYER_HXX
#include <svx/svdlayer.hxx>
#endif

#if 0 // _SOLAR__PRIVATE

#define IMP_PAGEPAINT_NORMAL			0
#define IMP_PAGEPAINT_PREPARE_CACHE		1
#define IMP_PAGEPAINT_PAINT_CACHE		2
#define IMP_PAGEPAINT_PREPARE_BG_CACHE	3
#define IMP_PAGEPAINT_PAINT_BG_CACHE	4

#endif 

class SdrObject;
class SdrPage;
class ExtOutputDevice;
class SdrPaintInfoRec;
class SdrIOHeader;
class SdrModel;
class SfxItemPool;
class SdrPageView;
class SdrLayerAdmin;
class SetOfByte;
class Color;
class SfxStyleSheet;

struct SdrPaintProcRec {
	SdrObject* pObj;
	ExtOutputDevice& rOut;
	const SdrPaintInfoRec& rInfoRec;
	SdrPaintProcRec(SdrObject* pObj_, ExtOutputDevice& rOut_, const SdrPaintInfoRec& rInfoRec_):
		pObj(pObj_), rOut(rOut_), rInfoRec(rInfoRec_) {}
};

enum SdrInsertReasonKind {SDRREASON_UNKNOWN,    
			  SDRREASON_STREAMING,  
			  SDRREASON_UNDO,       
			  SDRREASON_COPY,       
			  SDRREASON_VIEWCREATE, 
			  SDRREASON_VIEWCALL};  

class SdrInsertReason {
	const SdrObject* pRefObj;
	SdrInsertReasonKind eReason;
public:
	SdrInsertReason(): pRefObj(NULL),eReason(SDRREASON_UNKNOWN) {}
	SdrInsertReason(SdrInsertReasonKind eR,const SdrObject* pO=NULL): pRefObj(pO),eReason(eR) {}
	void SetReferenceObject(const SdrObject* pO)  { pRefObj=pO; }
	const SdrObject* GetReferenceObject() const   { return pRefObj; }
	void SetReason(SdrInsertReasonKind eR)        { eReason=eR; }
	SdrInsertReasonKind GetReason() const         { return eReason; }
};

class SdrObjList {
protected:
friend class SdrObjListIter;
friend class SdrEditView;
	Container   aList;
	SdrObjList* pUpList;   
	SdrModel*   pModel;    
	SdrPage*    pPage;     
	SdrObject*  pOwnerObj; 
	Rectangle   aOutRect;
	Rectangle   aSnapRect;
	SdrObjListKind eListKind;
	FASTBOOL    bObjOrdNumsDirty;
	FASTBOOL    bRectsDirty;
protected:
	virtual void RecalcRects();
#if 0 // _SOLAR__PRIVATE
	FASTBOOL ImpGetFillColor(SdrObject* pObj, Color& rCol) const;
#endif 
public:
	TYPEINFO();
	SdrObjList(SdrModel* pNewModel, SdrPage* pNewPage, SdrObjList* pNewUpList=NULL);
	SdrObjList(const SdrObjList& rSrcList);
	virtual ~SdrObjList();

	Container& GetContainer()                           { return aList; }
	void SetObjOrdNumsDirty()                           { bObjOrdNumsDirty=TRUE; }

	void operator=(const SdrObjList& rSrcList);
	void CopyObjects(const SdrObjList& rSrcList);

	void    Clear();
	SdrObjListKind GetListKind() const                  { return eListKind; }
	void           SetListKind(SdrObjListKind eNewKind) { eListKind=eNewKind; }
	SdrObjList*    GetUpList() const                    { return pUpList; }
	void           SetUpList(SdrObjList* pNewUpList)    { pUpList=pNewUpList; }
	SdrObject*     GetOwnerObj() const                  { return pOwnerObj; }
	void           SetOwnerObj(SdrObject* pNewOwner)    { pOwnerObj=pNewOwner; }
	virtual SdrPage* GetPage() const;
	virtual void     SetPage(SdrPage* pNewPage);
	virtual SdrModel* GetModel() const;
	virtual void      SetModel(SdrModel* pNewModel);

	void     RecalcObjOrdNums();
	FASTBOOL IsObjOrdNumsDirty() const        { return bObjOrdNumsDirty; }
	virtual void NbcInsertObject(SdrObject* pObj, ULONG nPos=CONTAINER_APPEND
#if SUPD>356
					 , const SdrInsertReason* pReason=NULL
#endif
														  );
	virtual void InsertObject(SdrObject* pObj, ULONG nPos=CONTAINER_APPEND
#if SUPD>356
				  , const SdrInsertReason* pReason=NULL
#endif
														 );
#if SUPD<=356
	void NbcInsertObject(SdrObject* pObj, ULONG nPos, const SdrInsertReason* pReason)
	{ NbcInsertObject(pObj,nPos); }
	void InsertObject(SdrObject* pObj, ULONG nPos, const SdrInsertReason* pReason)
	{ InsertObject(pObj,nPos); }
#endif

	virtual SdrObject* NbcRemoveObject(ULONG nObjNum);
	virtual SdrObject* RemoveObject(ULONG nObjNum);

	virtual SdrObject* NbcReplaceObject(SdrObject* pNewObj, ULONG nObjNum);
	virtual SdrObject* ReplaceObject(SdrObject* pNewObj, ULONG nObjNum);

	virtual SdrObject* NbcSetObjectOrdNum(ULONG nOldObjNum, ULONG nNewObjNum);
	virtual SdrObject* SetObjectOrdNum(ULONG nOldObjNum, ULONG nNewObjNum);

	virtual void SetRectsDirty();

	const Rectangle& GetAllObjSnapRect() const;
	const Rectangle& GetAllObjBoundRect() const;

	FASTBOOL Paint(ExtOutputDevice& rXOut, const SdrPaintInfoRec& rInfoRec, FASTBOOL bRestoreColors=FALSE) const;
	FASTBOOL Paint(ExtOutputDevice& rXOut, const SdrPaintInfoRec& rInfoRec, FASTBOOL bRestoreColors, USHORT nImpMode) const;

	SdrObject* CheckHit(const Point& rPnt, USHORT nTol, const SetOfByte* pVisiLayer, FASTBOOL bBackward=FALSE) const;
	SdrObject* CheckHit(const Point& rPnt, USHORT nTol, FASTBOOL bBackward=FALSE) const { return CheckHit(rPnt,nTol,NULL,bBackward); }
	FASTBOOL IsHit(const Point& rPnt, USHORT nTol, const SetOfByte* pVisiLayer) const { return CheckHit(rPnt,nTol,pVisiLayer)!=NULL; }
	FASTBOOL IsHit(const Point& rPnt, USHORT nTol) const                              { return CheckHit(rPnt,nTol,NULL)!=NULL; }

	void NbcReformatAllTextObjects();
	void ReformatAllTextObjects();

	void BurnInStyleSheetAttributes( BOOL bPseudoSheetsOnly=FALSE );

	void RemoveNotPersistentObjects(FASTBOOL bNoBroadcast);

	void RestartAllAnimations(SdrPageView* pPageView) const;

	FASTBOOL GetFillColor(const Point& rPnt, const SetOfByte& rVisLayers,
			  FASTBOOL bLayerSorted, Color& rCol) const;

	ULONG      GetObjCount() const           { return aList.Count(); }
	SdrObject* GetObj(ULONG nNum) const      { return (SdrObject*)aList.GetObject(nNum); }

	virtual FASTBOOL IsReadOnly() const;

	ULONG   CountAllObjects() const;

	void	ForceSwapInObjects() const;
	void	SwapInAll() const { ForceSwapInObjects(); }

	void    Save(SvStream& rOut) const;
	void    Load(SvStream& rIn, SdrPage& rPage);

	virtual void AfterRead();
};

#define SDRPAGE_NOTFOUND 0xFFFF

class SdrMasterPageDescriptor {
	USHORT    nPgNum;
	SetOfByte aVisLayers;
public:
	SdrMasterPageDescriptor(USHORT nPageNum=0): nPgNum(nPageNum) { aVisLayers.SetAll(); }
	FASTBOOL         operator==(const SdrMasterPageDescriptor& rCmp) const { return nPgNum==rCmp.nPgNum && aVisLayers==rCmp.aVisLayers; }
	FASTBOOL         operator!=(const SdrMasterPageDescriptor& rCmp) const { return !operator==(rCmp); }
	USHORT           GetPageNum() const                          { return nPgNum; }
	void             SetPageNum(USHORT nNum)                     { nPgNum=nNum; }
	const SetOfByte& GetVisibleLayers() const                    { return aVisLayers; }
	void             SetVisibleLayers(const SetOfByte& rVisLay)  { aVisLayers=rVisLay; }

	friend SvStream& operator<<(SvStream& rOut, const SdrMasterPageDescriptor& rMDP);
	friend SvStream& operator>>(SvStream& rIn, SdrMasterPageDescriptor& rMDP);
};

class SdrMasterPageDescriptorList {
	Container aList;
protected:
	SdrMasterPageDescriptor* GetObject(USHORT i) const { return (SdrMasterPageDescriptor*)(aList.GetObject(i)); }
public:
	SdrMasterPageDescriptorList(): aList(1024,4,4) {}
	SdrMasterPageDescriptorList(const SdrMasterPageDescriptorList& rSrcList): aList(1024,4,4) { *this=rSrcList; }
	~SdrMasterPageDescriptorList()                                         { Clear(); }
	void     Clear();
	void     operator=(const SdrMasterPageDescriptorList& rSrcList);
	FASTBOOL operator==(const SdrMasterPageDescriptorList& rCmpList) const;
	FASTBOOL operator!=(const SdrMasterPageDescriptorList& rCmpList) const   { return !operator==(rCmpList); }
	USHORT   GetCount() const                                                { return USHORT(aList.Count()); }
	void     Insert(USHORT nPgNum, USHORT nPos=0xFFFF)                       { aList.Insert(new SdrMasterPageDescriptor(nPgNum),nPos); }
	void     Insert(const SdrMasterPageDescriptor& rMPD, USHORT nPos=0xFFFF) { aList.Insert(new SdrMasterPageDescriptor(rMPD),nPos); }
	void     Remove(USHORT nPos)                                             { delete aList.Remove(nPos); }
	void     Move(USHORT nPos, USHORT nNewPos)                               { aList.Insert(aList.Remove(nPos),nNewPos); }
	SdrMasterPageDescriptor&       operator[](USHORT nPos)                   { return *GetObject(nPos); }
	const SdrMasterPageDescriptor& operator[](USHORT nPos) const             { return *GetObject(nPos); }
	friend SvStream& operator<<(SvStream& rOut, const SdrMasterPageDescriptorList& rMPDL);
	friend SvStream& operator>>(SvStream& rIn, SdrMasterPageDescriptorList& rMPDL);
};

class SdrPageGridFrame {
	Rectangle aPaper;
	Rectangle aUserArea;
public:
	SdrPageGridFrame(const Rectangle& rPaper): aPaper(rPaper), aUserArea(rPaper) {}
	SdrPageGridFrame(const Rectangle& rPaper, const Rectangle& rUser): aPaper(rPaper), aUserArea(rUser) {}
	void             SetPaperRect(const Rectangle& rPaper) { aPaper=rPaper; }
	void             SetUserArea(const Rectangle& rUser)   { aUserArea=rUser; }
	const Rectangle& GetPaperRect() const                  { return aPaper; }
	const Rectangle& GetUserArea() const                   { return aUserArea; }
};

class SdrPageGridFrameList {
	Container aList;
private:
	SdrPageGridFrameList(const SdrPageGridFrameList& rSrcList): aList(1024,4,4) {}
	void           operator=(const SdrPageGridFrameList& rSrcList)              {}
protected:
	SdrPageGridFrame* GetObject(USHORT i) const { return (SdrPageGridFrame*)(aList.GetObject(i)); }
public:
	SdrPageGridFrameList(): aList(1024,4,4)                            {}
	~SdrPageGridFrameList()                                            { Clear(); }
	void           Clear();
	USHORT         GetCount() const                                    { return USHORT(aList.Count()); }
	void           Insert(const SdrPageGridFrame& rGF, USHORT nPos=0xFFFF) { aList.Insert(new SdrPageGridFrame(rGF),nPos); }
	void           Delete(USHORT nPos)                                 { delete aList.Remove(nPos); }
	void           Move(USHORT nPos, USHORT nNewPos)                   { aList.Insert(aList.Remove(nPos),nNewPos); }
	SdrPageGridFrame&       operator[](USHORT nPos)                    { return *GetObject(nPos); }
	const SdrPageGridFrame& operator[](USHORT nPos) const              { return *GetObject(nPos); }
};

class SdrPage: public SdrObjList {
friend SdrModel;
	INT32 nWdt;     
	INT32 nHgt;     
	INT32 nBordLft; 
	INT32 nBordUpp; 
	INT32 nBordRgt; 
	INT32 nBordLwr; 

	::com::sun::star::uno::WeakReference< ::com::sun::star::uno::XInterface > mxUnoPage;

protected:

	SdrLayerAdmin* 	pLayerAdmin;
	SdrObject*		pBackgroundObj;

	SdrMasterPageDescriptorList aMasters; 
	SetOfByte  aPrefVisiLayers;
	USHORT     nPageNum;
	FASTBOOL   bMaster;  
	FASTBOOL   bInserted;
#if SUPD>=361
	FASTBOOL   bObjectsNotPersistent;
#endif
	FASTBOOL   bSwappingLocked;

	virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > createUnoPage();

#if 0 // _SOLAR__PRIVATE
protected:
	void ImpMasterPageRemoved(USHORT nMasterPageNum);
	void ImpMasterPageInserted(USHORT nMasterPageNum);
	void ImpMasterPageMoved(USHORT nMasterPageNum, USHORT nNewMasterPageNum);
#endif 
public:
	TYPEINFO();
	SdrPage(SdrModel& rNewModel, FASTBOOL bMasterPage=FALSE);

	SdrPage(const SdrPage& rSrcPage);
	virtual ~SdrPage();

	virtual void operator=(const SdrPage& rSrcPage);
	virtual SdrPage* Clone() const;
	virtual SdrPage* Clone(SdrModel* pNewModel) const;
	SfxItemPool& GetItemPool() const;
			FASTBOOL IsMasterPage() const       { return bMaster; }
			void SetInserted(FASTBOOL bJa=TRUE);
			FASTBOOL IsInserted() const         { return bInserted; }
	virtual void SetChanged();
			void   SetPageNum(USHORT nNum) { nPageNum=nNum; }; 
			USHORT GetPageNum() const;

	virtual void SetSize(const Size& aSiz);
	virtual Size GetSize() const;
	virtual void SetOrientation(Orientation eOri);
	virtual Orientation GetOrientation() const;
	virtual INT32 GetWdt() const;
	virtual INT32 GetHgt() const;
	virtual void  SetBorder(INT32 nLft, INT32 nUpp, INT32 nRgt, INT32 Lwr);
	virtual void  SetLftBorder(INT32 nBorder);
	virtual void  SetUppBorder(INT32 nBorder);
	virtual void  SetRgtBorder(INT32 nBorder);
	virtual void  SetLwrBorder(INT32 nBorder);
	virtual INT32 GetLftBorder() const;
	virtual INT32 GetUppBorder() const;
	virtual INT32 GetRgtBorder() const;
	virtual INT32 GetLwrBorder() const;
	void SendRepaintBroadcast() const;

	virtual Point GetOffset() const;

	virtual void SetModel(SdrModel* pNewModel);
	USHORT           GetMasterPagePos(const SdrPage& rPage) const     { return GetMasterPagePos(rPage.GetPageNum()); }
			USHORT           GetMasterPagePos(USHORT nPgNum) const;
	virtual void             InsertMasterPage(USHORT nPgNum, USHORT nPos=0xFFFF);
	virtual void             RemoveMasterPage(USHORT nPos);
	virtual void             MoveMasterPage(USHORT nPos, USHORT nNewPos);

	virtual void             InsertMasterPage(const SdrMasterPageDescriptor& rMPD, USHORT nPos=0xFFFF);
	SdrPage*         GetMasterPage(USHORT nPos) const;
			USHORT           GetMasterPageNum(USHORT nPos) const              { return aMasters[nPos].GetPageNum(); }
			const SetOfByte& GetMasterPageVisibleLayers(USHORT nPos) const    { return aMasters[nPos].GetVisibleLayers(); }
			USHORT           GetMasterPageCount() const                       { return aMasters.GetCount(); }
	SdrMasterPageDescriptor& GetMasterPageDescriptor(USHORT nPos)             { return aMasters[nPos]; }
	const SdrMasterPageDescriptor& GetMasterPageDescriptor(USHORT nPos) const { return aMasters[nPos]; }
	const SdrMasterPageDescriptorList& GetMasterPageDescriptorList() const    { return aMasters; }
	virtual void             SetMasterPageNum(USHORT nPgNum, USHORT nPos);
	virtual void             SetMasterPageVisibleLayers(const SetOfByte& rVL, USHORT nPos);
	virtual void             SetMasterPageDescriptor(const SdrMasterPageDescriptor& rMPD, USHORT nPos);
	virtual void             SetMasterPageDescriptorList(const SdrMasterPageDescriptorList& rMPDL);

	const         SdrLayerAdmin& GetLayerAdmin() const                  { return *pLayerAdmin; }
	  SdrLayerAdmin& GetLayerAdmin()                        { return *pLayerAdmin; }

	FASTBOOL GetFillColor(const Point& rPnt, const SetOfByte& rVisLayers,
			  FASTBOOL bLayerSorted, Color& rCol) const;

	Bitmap        GetBitmap(FASTBOOL bTrimBorders=TRUE) const               { return GetBitmap(aPrefVisiLayers,bTrimBorders); }
	Bitmap        GetBitmap(const SetOfByte& rVisibleLayers, FASTBOOL bTrimBorders=TRUE) const;

	GDIMetaFile   GetMetaFile(FASTBOOL bTrimBorders=TRUE)                   { return GetMetaFile(aPrefVisiLayers,bTrimBorders); }
	GDIMetaFile   GetMetaFile(const SetOfByte& rVisibleLayers, FASTBOOL bTrimBorders=TRUE);

	virtual String GetLayoutName() const;

	virtual const SdrPageGridFrameList* GetGridFrameList(const SdrPageView* pPV, const Rectangle* pRect) const;
#if SUPD>=361
	FASTBOOL IsObjectsNotPersistent() const          { return bObjectsNotPersistent; }
	void     SetObjectsNotPersistent(FASTBOOL b)     { bObjectsNotPersistent=b; }
#endif

	FASTBOOL IsSwappingLocked() const                { return bSwappingLocked; }
	void     SetSwappingLocked(FASTBOOL bLock)       { bSwappingLocked=bLock; }

	SdrObject* GetBackgroundObj() const { return pBackgroundObj; }
	void 	   SetBackgroundObj( SdrObject* pObj );

	virtual void  WriteData(SvStream& rOut) const;
	virtual void  ReadData(const SdrIOHeader& rHead, SvStream& rIn);
	friend SvStream& operator<<(SvStream& rOut, const SdrPage& rPg);
	friend SvStream& operator>>(SvStream& rIn, SdrPage& rPg);

	::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > getUnoPage();

	virtual SfxStyleSheet* GetTextStyleSheetForObject( SdrObject* pObj ) const;

	FASTBOOL HasTransparentObjects( BOOL bCheckForAlphaChannel = FALSE ) const;
};

#endif 

