

#ifndef _SVDPNTV_HXX
#define _SVDPNTV_HXX

#ifndef _TIMER_HXX 
#include <vcl/timer.hxx>
#endif

#ifndef _VIRDEV_HXX 
#include <vcl/virdev.hxx>
#endif

#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif

#ifndef _SFXITEMSET_HXX 
#include <svtools/itemset.hxx>
#endif

#ifndef _UNDO_HXX 
#include <svtools/undo.hxx>
#endif

#ifndef _SVDDRAG_HXX
#include <svx/svddrag.hxx>
#endif

#ifndef _SVDLAYER_HXX
#include <svx/svdlayer.hxx>  
#endif

#ifndef _APP_HXX 
#include <vcl/svapp.hxx>
#endif

#ifndef _WINDOW_HXX 
#include <vcl/window.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

namespace com { namespace sun { namespace star { namespace awt {
class XControlContainer;
}}}}

class SdrPageViewWinRec;
class OutputDevice;
class VirtualDevice;
class SdrView;
class SfxItemSet;
class SfxStyleSheet;
class ExtOutputDevice;
class SdrNamedSubRecord;
class SdrIOHeader;
class SdrOle2Obj;
class ImpMasterBmp;
class SdrItemBrowser;
class SdrObject;
class B2dIAOManager;

#define SDR_MASTERPAGECACHE_NONE		0x00000000UL
#define SDR_MASTERPAGECACHE_FULL		0x00000001UL
#define SDR_MASTERPAGECACHE_BACKGROUND	0x00000002UL
#define SDR_MASTERPAGECACHE_DEFAULT		0xFFFFFFFFUL

enum SdrAnimationMode
{
	SDR_ANIMATION_ANIMATE,
	SDR_ANIMATION_DONT_ANIMATE,
	SDR_ANIMATION_DISABLE
};

typedef unsigned char TRISTATE;
#define FUZZY					(2)
#define SDR_ANYFORMAT			(0xFFFFFFFF)
#define SDR_ANYITEM				(0xFFFF)
#define SDRVIEWWIN_NOTFOUND		(0xFFFF)

class RollingRect
{
	Point						aP1;
	Point						aP2;

	USHORT						nIdx;
	USHORT						nLen;

	unsigned					bStripes : 1;
	unsigned					bCrossHair : 1;

public:
	RollingRect(USHORT nNewLen=8) { Reset(nNewLen); }
	void Reset(USHORT nNewLen=8) { nIdx=0; nLen=nNewLen; bStripes=FALSE; bCrossHair=FALSE; }
	void Invert(OutputDevice& rOut) const { DrawRect(rOut,FALSE); }
	void RollIt(OutputDevice& rOut, BOOL bIncIdx=TRUE) { DrawRect(rOut,TRUE); if (bIncIdx) IncRollIdx(); }
	void IncRollIdx() { nIdx++; if (nIdx>=2*nLen) nIdx=0; }
	const Point& GetP1() const { return aP1; }
	const Point& GetP2() const { return aP2; }
	void SetP1(const Point& rP) const { ((RollingRect*)this)->aP1=rP; }
	void SetP2(const Point& rP) const { ((RollingRect*)this)->aP2=rP; }
	BOOL IsCrossHair() const { return bCrossHair; }
	BOOL IsStripes() const { return bStripes; }
	void SetCrossHair(BOOL bOn) { bCrossHair=bOn; }
	void SetStripes(BOOL bOn) { bStripes=bOn; }

protected:
	void DrawRect(OutputDevice& rOut, BOOL bMov) const;
	void DrawHor(OutputDevice& rOut, long x1, long x2, long y, BOOL bMov) const;
	void DrawVer(OutputDevice& rOut, long x, long y1, long y2, BOOL bMov) const;
};

class FrameAnimator: public RollingRect
{
	OutputDevice*				pOut;
	SdrView&					rView;
	AutoTimer					aTim;

public:
	FrameAnimator(SdrView& rNewView);
	DECL_LINK(Hdl,AutoTimer*);
	void Invert(OutputDevice* pOut) const;
	void Start() const;
	void Stop() const;
};

class ImpMasterBmp
{
	friend class				SdrPageView;

	Fraction					aMapX;
	Fraction					aMapY;
	Rectangle					aLogBound;
	Rectangle					aPixBound;
	VirtualDevice				aVD;
	SetOfByte					aVisLayers;
	USHORT						nMasterPageNum;

public:
	ImpMasterBmp(): nMasterPageNum(0) {}
	ImpMasterBmp(const OutputDevice& rCompDev): aVD(rCompDev),nMasterPageNum(0) {}
	VirtualDevice& GetVirtDev() { return aVD; }
	const VirtualDevice& GetVirtDev() const { return aVD; }
	USHORT GetMasterPageNum() const { return nMasterPageNum; }
};

class SdrViewWinRec
{
public:
	OutputDevice*				pWin;

	B2dIAOManager*				pIAOManager;
	unsigned					bXorVisible : 1;

	SdrViewWinRec(OutputDevice* pW);
	~SdrViewWinRec();
};

class SdrViewWinList
{
	Container					aList;

	SdrViewWinList(const SdrViewWinList& rSource)
	:	aList(1024,4,4) {}
	void operator=(const SdrViewWinList& rSource) {}

protected:
	SdrViewWinRec* GetObject(USHORT i) const { return (SdrViewWinRec*)(aList.GetObject(i)); }

public:
	SdrViewWinList(): aList(1024,4,4) {}
	~SdrViewWinList() { Clear(); }
	void Clear();
	USHORT GetCount() const { return (USHORT)aList.Count(); }
	void Insert(SdrViewWinRec* pVWR, USHORT nPos=0xFFFF) { aList.Insert(pVWR,nPos); }
	void Delete(USHORT nPos) { delete (SdrViewWinRec*)aList.Remove(nPos); }
	SdrViewWinRec& operator[](USHORT nPos) { return *GetObject(nPos); }
	const SdrViewWinRec& operator[](USHORT nPos) const { return *GetObject(nPos); }
	USHORT Find(OutputDevice* pW) const;
};

class SdrPaintView: public SfxListener, public SfxRepeatTarget
{
	friend class				SdrPageView;
	friend class				FrameAnimator;
	friend class				SdrViewUserMarker;
	friend class				SdrGrafObj;

protected:
	SdrModel*					pMod;
	ExtOutputDevice*			pXOut;
	SdrItemBrowser*				pItemBrowser;
	const OutputDevice*			pActualOutDev; 
	OutputDevice*				pDragWin;
	SfxStyleSheet*				pDefaultStyleSheet;
	SfxItemSet*					pDisabledAttr;
	ImpMasterBmp*				pMasterBmp;			

	String						aAktLayer;     
	String						aMeasureLayer; 

	Container					aPagV;         
	Container					aPagHide;      
	SdrViewWinList				aWinList;    
	MapMode						aActualMapMode;
	Size						aGridBig; 
	Size						aGridFin; 
	Size						aGridSubdiv;
	Fraction					aGridWdtX;
	Fraction					aGridWdtY;
	FrameAnimator				aAni;
	SdrDragStat					aDragStat;
	Rectangle					aMaxWorkArea;
	SfxItemSet					aDefaultAttr;
	Timer						aComeBackTimer;
	Timer						aAfterPaintTimer;
	AutoTimer					aUserMarkerAnimator;
	Container					aUserMarkers;
	List						aAsyncPaintList;
	Timer						aAsyncPaintTimer;

	SdrAnimationMode			eAnimationMode;

	USHORT						nHitTolPix;
	USHORT						nMinMovPix;
	USHORT						nHitTolLog;
	USHORT						nMinMovLog;
	ULONG						nMasterCacheMode;	

	unsigned					bForeignXOut : 1;
	unsigned					bLayerSortedRedraw : 1;
	unsigned					bPageVisible : 1;
	unsigned					bBordVisible : 1;
	unsigned					bGridVisible : 1;
	unsigned					bGridFront : 1;
	unsigned					bHlplVisible : 1;
	unsigned					bHlplFront : 1;
	unsigned					bGlueVisible : 1;    
	unsigned					bGlueVisible2 : 1;   
	unsigned					bGlueVisible3 : 1;   
	unsigned					bGlueVisible4 : 1;   
	unsigned					bLineDraft : 1;
	unsigned					bFillDraft : 1;
	unsigned					bTextDraft : 1;
	unsigned					bGrafDraft : 1;
	unsigned					bLineDraftPrn : 1;
	unsigned					bFillDraftPrn : 1;
	unsigned					bTextDraftPrn : 1;
	unsigned					bGrafDraftPrn : 1;
	unsigned					bSaveHiddenPages : 1; 
	unsigned					bPageTwice : 1;       
	unsigned					bRestoreColors : 1;   
	unsigned					bObjectPaintIgnoresClipping : 1;
	unsigned					bEncircle : 1;
	unsigned					bSomeObjChgdFlag : 1;
	unsigned					bMasterBmp : 1;			
	unsigned					bSwapAsynchron : 1;
	unsigned					bPrintPreview : 1;

	unsigned					bVisualizeEnteredGroup : 1;
	unsigned					bHideGrafDraft : 1;
	unsigned					bAnimationPause : 1;

private:
	void ImpClearVars();
	DECL_LINK(ImpComeBackHdl,Timer*);
	DECL_LINK(ImpAfterPaintHdl,Timer*);
	DECL_LINK(ImpUserMarkerAnimatorHdl,AutoTimer*);
	void ImpInsertUserMarker(SdrViewUserMarker* pMarker);
	void ImpRemoveUserMarker(SdrViewUserMarker* pMarker);
	void ImpCheckMarkerAnimator();
	USHORT ImpGetUserMarkerCount() const { return (USHORT)aUserMarkers.Count(); }
	SdrViewUserMarker* ImpGetUserMarker(USHORT nNum) const { return (SdrViewUserMarker*)aUserMarkers.GetObject(nNum); }
	void ImpAddAsyncObj( const SdrObject* pObj, const OutputDevice* pOut );

protected:
	USHORT ImpGetMinMovLogic(short nMinMov, const OutputDevice* pOut) const;
	USHORT ImpGetHitTolLogic(short nHitTol, const OutputDevice* pOut) const;

	void FlushComeBackTimer() const;
	void TheresNewMapMode();
	void ImpSetGlueVisible2(BOOL bOn) { if (bGlueVisible2!=bOn) { bGlueVisible2=bOn; if (!bGlueVisible && !bGlueVisible3 && !bGlueVisible4) GlueInvalidate(); } }
	void ImpSetGlueVisible3(BOOL bOn) { if (bGlueVisible3!=bOn) { bGlueVisible3=bOn; if (!bGlueVisible && !bGlueVisible2 && !bGlueVisible4) GlueInvalidate(); } }
	void ImpSetGlueVisible4(BOOL bOn) { if (bGlueVisible4!=bOn) { bGlueVisible4=bOn; if (!bGlueVisible && !bGlueVisible2 && !bGlueVisible3) GlueInvalidate(); } }
	BOOL ImpIsGlueVisible2() const { return bGlueVisible2; }
	BOOL ImpIsGlueVisible3() const { return bGlueVisible3; }
	BOOL ImpIsGlueVisible4() const { return bGlueVisible4; }
	BOOL ImpIsGlueVisible() { return bGlueVisible || bGlueVisible2 || bGlueVisible3 || bGlueVisible4; }

	virtual void SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType, const SfxHint& rHint, const TypeId& rHintType);
	BOOL IsShownXorVisibleWinNum(USHORT nNum) const { return aWinList[nNum].bXorVisible; }
	void SetShownXorVisible(USHORT nNum, BOOL bAn) { aWinList[nNum].bXorVisible=bAn; }
	USHORT GetHiddenPV(const SdrPage* pPage) const;
	void GlueInvalidate() const;

	void ShowEncirclement(OutputDevice* pOut);
	void HideEncirclement(OutputDevice* pOut);
	void DrawEncirclement(OutputDevice* pOut) const;

	virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead, SvStream& rIn);

	virtual void ModelHasChanged();

public:
	TYPEINFO();
	SdrPaintView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrPaintView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrPaintView();

	virtual void ClearPageViews();
	virtual void ClearHideViews();
	virtual void ClearAll();
	virtual void Clear(); 
	SdrModel* GetModel() const { return pMod; }

	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BckAction();
	virtual void BrkAction(); 
	virtual void TakeActionRect(Rectangle& rRect) const;

	void SetActualWin(const OutputDevice* pWin);
	void SetMinMoveDistancePixel(USHORT nVal) { nMinMovPix=nVal; TheresNewMapMode(); }
	USHORT GetMinMoveDistancePixel() const { return (USHORT)nMinMovPix; }
	void SetHitTolerancePixel(USHORT nVal) { nHitTolPix=nVal; TheresNewMapMode(); }
	USHORT GetHitTolerancePixel() const { return (USHORT)nHitTolPix; }

	BOOL DoVisualizeEnteredGroup() const { return bVisualizeEnteredGroup; }
	void SetVisualizeEnteredGroup(BOOL bNew) { bVisualizeEnteredGroup = bNew; }

	virtual void ToggleShownXor(OutputDevice* pOut, const Region* pRegion) const; 
	void ShowShownXor(OutputDevice* pOut, BOOL bShow=TRUE);
	void HideShownXor(OutputDevice* pOut) { ShowShownXor(pOut,FALSE); }
	BOOL IsShownXorVisible(OutputDevice* pOut) const;

	void RestartAfterPaintTimer();

	void PostPaint();

	const SdrDragStat& GetDragStat() const { return aDragStat; }

	virtual SdrPageView* ShowPage(SdrPage* pPage, const Point& rOffs);
	SdrPageView* ShowPagePgNum(USHORT nPgNum, const Point& rOffs);
	SdrPageView* ShowMasterPagePgNum(USHORT nPgNum, const Point& rOffs);
	virtual void HidePage(SdrPageView* pPV);
	void HidePage(const SdrPage* pPage) { HidePage(GetPageView(pPage)); }
	void HidePagePgNum(USHORT nPgNum);
	void HidePagePvNum(USHORT nPvNum) { HidePage(GetPageViewPvNum(nPvNum)); }
	virtual void HideAllPages();
	virtual void SetPagePos(SdrPageView* pPV, const Point& rOffs);
	void SetPagePos(SdrPage* pPage, const Point& rOffs) { SetPagePos(GetPageView(pPage),rOffs); }
	void SetPagePosPgNum(USHORT nPgNum, const Point& rOffs);
	void SetPagePosPvNum(USHORT nPvNum, const Point& rOffs) { SetPagePos(GetPageViewPvNum(nPvNum),rOffs); }

	USHORT GetPageViewCount() const { return USHORT(aPagV.Count()); }
	SdrPageView* GetPageViewPvNum(USHORT nPvNum) const { return ((SdrPageView*)aPagV.GetObject(nPvNum)); }

	SdrPageView* GetPageView(const SdrPage* pPage) const;
	SdrPageView* GetPageViewPgNum(USHORT nPgNum) const;
	USHORT GetPageViewNum(const SdrPageView* pPV) const;

	SdrPageView* HitPage(const Point& rPnt) const;

	SdrPageView* GetPageView(const Point& rPnt) const;

	USHORT GetPageHideCount() const { return USHORT(aPagHide.Count()); }
	SdrPageView* GetPageHidePvNum(USHORT nPvNum) const { return ((SdrPageView*)aPagHide.GetObject(nPvNum)); }
	SdrPageView* GetPageHide(const SdrPage* pPage) const { return ((SdrPageView*)aPagHide.GetObject(GetHiddenPV(pPage))); }
	USHORT GetPageHideNum(const SdrPageView* pPV) const;

	virtual void AddWin(OutputDevice* pWin1);
	virtual void DelWin(OutputDevice* pWin1);

	USHORT GetWinCount() const { return aWinList.GetCount(); }
	OutputDevice* GetWin(USHORT nNum) const { return nNum<aWinList.GetCount() ? aWinList[nNum].pWin : NULL; }
	USHORT FindWin( OutputDevice* pWin ) { return aWinList.Find( pWin ); }

	const SdrViewWinRec& GetWinRec(USHORT nNum) const { return aWinList[nNum]; }
	SdrViewWinRec& GetWinRec(USHORT nNum) { return aWinList[nNum]; }

	void ShowLayerSet(const String& rName, BOOL bShow=TRUE);

	TRISTATE IsLayerSetVisible(const String& rName) const;

	void SetLayerVisible(const String& rName, BOOL bShow=TRUE);
	TRISTATE IsLayerVisible(const String& rName) const;
	void SetAllLayersVisible(BOOL bShow=TRUE);

	void SetLayerLocked(const String& rName, BOOL bLock=TRUE);
	TRISTATE IsLayerLocked(const String& rName) const;
	void SetAllLayersLocked(BOOL bLock=TRUE);

	void SetLayerPrintable(const String& rName, BOOL bPrn=TRUE);
	TRISTATE IsLayerPrintable(const String& rName) const;
	void SetAllLayersPrintable(BOOL bPrn=TRUE);

	virtual void InitRedraw(OutputDevice* pOut, const Region& rReg, USHORT nPaintMode=0);
	virtual void InitRedraw(USHORT nWinNum, const Region& rReg, USHORT nPaintMode=0);
	BOOL IsRedrawReady() const;
	BOOL RedrawOne(USHORT nBrkEvent=INPUT_MOUSEANDKEYBOARD);
	BOOL RedrawUntilInput(USHORT nBrkEvent=INPUT_MOUSEANDKEYBOARD);

	BOOL IsPageVisible() const { return bPageVisible; } 
	BOOL IsBordVisible() const { return bBordVisible; } 
	BOOL IsGridVisible() const { return bGridVisible; } 
	BOOL IsGridFront() const { return bGridFront  ; } 
	BOOL IsHlplVisible() const { return bHlplVisible; } 
	BOOL IsHlplFront() const { return bHlplFront  ; } 
	BOOL IsGlueVisible() const { return bGlueVisible; } 
	void SetPageVisible(BOOL bOn=TRUE) { bPageVisible=bOn; InvalidateAllWin(); }
	void SetBordVisible(BOOL bOn=TRUE) { bBordVisible=bOn; InvalidateAllWin(); }
	void SetGridVisible(BOOL bOn=TRUE) { bGridVisible=bOn; InvalidateAllWin(); }
	void SetGridFront(BOOL bOn=TRUE) { bGridFront  =bOn; InvalidateAllWin(); }
	void SetHlplVisible(BOOL bOn=TRUE) { bHlplVisible=bOn; InvalidateAllWin(); }
	void SetHlplFront(BOOL bOn=TRUE) { bHlplFront  =bOn; InvalidateAllWin(); }
	void SetGlueVisible(BOOL bOn=TRUE) { if (bGlueVisible!=bOn) { bGlueVisible=bOn; if (!bGlueVisible2 && !bGlueVisible3 && !bGlueVisible4) GlueInvalidate(); } }

	BOOL IsLineDraft() const { return bLineDraft; } 
	BOOL IsFillDraft() const { return bFillDraft; } 
	BOOL IsTextDraft() const { return bTextDraft; } 
	BOOL IsGrafDraft() const { return bGrafDraft; } 
	BOOL IsHideGrafDraft() const { return bHideGrafDraft; } 
	BOOL IsLineDraftPrn() const { return bLineDraftPrn; } 
	BOOL IsFillDraftPrn() const { return bFillDraftPrn; } 
	BOOL IsTextDraftPrn() const { return bTextDraftPrn; } 
	BOOL IsGrafDraftPrn() const { return bGrafDraftPrn; } 
	void SetLineDraft(BOOL bOn) { bLineDraft=bOn; InvalidateAllWin(); }
	void SetFillDraft(BOOL bOn) { bFillDraft=bOn; InvalidateAllWin(); }
	void SetTextDraft(BOOL bOn) { bTextDraft=bOn; InvalidateAllWin(); }
	void SetGrafDraft(BOOL bOn) { bGrafDraft=bOn; InvalidateAllWin(); }
	void SetHideGrafDraft(BOOL bOn) { bHideGrafDraft=bOn; InvalidateAllWin(); }
	void SetLineDraftPrn(BOOL bOn) { bLineDraftPrn=bOn; }
	void SetFillDraftPrn(BOOL bOn) { bFillDraftPrn=bOn; }
	void SetTextDraftPrn(BOOL bOn) { bTextDraftPrn=bOn; }
	void SetGrafDraftPrn(BOOL bOn) { bGrafDraftPrn=bOn; }

	void SetGridCoarse(const Size& rSiz) { aGridBig=rSiz; SetGridWidth(Fraction(rSiz.Width(),1),Fraction(rSiz.Height(),1)); }
	void SetGridFine(const Size& rSiz) { aGridFin=rSiz; if (aGridFin.Height()==0) aGridFin.Height()=aGridFin.Width(); if (bGridVisible) InvalidateAllWin(); } 
	const Size& GetGridCoarse() const { return aGridBig; }
	const Size& GetGridFine() const { return aGridFin; }

	void SetGridWidth(const Fraction& rX, const Fraction& rY) { aGridWdtX=rX; aGridWdtY=rY; if (bGridVisible) InvalidateAllWin(); }
	void SetGridSubdivision(const Size& rSubdiv) { aGridSubdiv=rSubdiv; if (bGridVisible) InvalidateAllWin(); }
	const Fraction& GetGridWidthX() const { return aGridWdtX; }
	const Fraction& GetGridWidthY() const { return aGridWdtY; }
	const Size& GetGridSubdivision() const { return aGridSubdiv; }

	BOOL IsHiddenPageSaveMode() const { return bSaveHiddenPages; }
	void SetHiddenPageSaveMode(BOOL bOn=TRUE) { bSaveHiddenPages=bOn; }

	void InvalidateAllWin();
	void InvalidateAllWin(const Rectangle& rRect, BOOL bPlus1Pix=FALSE);

	virtual void InvalidateOneWin(Window& rWin);
	virtual void InvalidateOneWin(Window& rWin, const Rectangle& rRect);

	void SetActiveLayer(const String& rName) { aAktLayer=rName; }
	const String&  GetActiveLayer() const { return aAktLayer; }

	void SetLayerSortedRedraw(BOOL bOn) { bLayerSortedRedraw=TRUE; InvalidateAllWin(); }
	BOOL IsLayerSortedRedraw() const { return bLayerSortedRedraw; }

	void SetObjectPaintIgnoresClipping(BOOL bOn=TRUE) { bObjectPaintIgnoresClipping=bOn; }
	BOOL IsObjectPaintIgnoresClipping() const { return bObjectPaintIgnoresClipping; }

	void LeaveOneGroup();

	void LeaveAllGroup();

	BOOL IsGroupEntered() const;

	void SetDisabledAttr(const SfxItemSet* pNewDisabledAttr);

	void SetDefaultAttr(const SfxItemSet& rAttr, BOOL bReplaceAll);
	const SfxItemSet& GetDefaultAttr() const { return aDefaultAttr; }
	void SetDefaultStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);
	SfxStyleSheet* GetDefaultStyleSheet() const { return pDefaultStyleSheet; }

	void SetNotPersistDefaultAttr(const SfxItemSet& rAttr, BOOL bReplaceAll);
	void MergeNotPersistDefaultAttr(SfxItemSet& rAttr, BOOL bOnlyHardAttr) const;

	void BegEncirclement(const Point& rPnt, OutputDevice* pOut=NULL, short nMinMov=-2);
	void MovEncirclement(const Point& rPnt);
	Rectangle EndEncirclement(BOOL bNoJustify=FALSE);
	void BrkEncirclement();
	BOOL IsEncirclement() const { return bEncircle; }

	BOOL IsMasterPagePaintCaching() { return bMasterBmp; }
	ULONG GetMasterPagePaintCacheMode() const { return nMasterCacheMode; }
	void SetMasterPagePaintCaching( BOOL bOn, ULONG nCacheMode = SDR_MASTERPAGECACHE_DEFAULT );
	BOOL HasMasterPagePaintCache() { return pMasterBmp!=NULL; }
	void ReleaseMasterPagePaintCache();

	BOOL IsSwapAsynchron() const { return bSwapAsynchron; }
	void SetSwapAsynchron(BOOL bJa=TRUE) { bSwapAsynchron=bJa; }

	B2dIAOManager* GetIAOManager(OutputDevice* pOut);
	B2dIAOManager* GetFirstIAOManager();
	void RefreshAllIAOManagers();
	BOOL KeyInput(const KeyEvent& rKEvt, Window* pWin);

	BOOL MouseButtonDown(const MouseEvent& rMEvt, Window* pWin) { return FALSE; }
	BOOL MouseButtonUp(const MouseEvent& rMEvt, Window* pWin) { return FALSE; }
	BOOL MouseMove(const MouseEvent& rMEvt, Window* pWin) { return FALSE; }
	BOOL Command(const CommandEvent& rCEvt, Window* pWin) { return FALSE; }
	BOOL Cut(ULONG nFormat=SDR_ANYFORMAT) { return FALSE; }
	BOOL Yank(ULONG nFormat=SDR_ANYFORMAT) { return FALSE; }
	BOOL Paste(Window* pWin=NULL, ULONG nFormat=SDR_ANYFORMAT) { return FALSE; }

	BOOL GetAttributes(SfxItemSet& rTargetSet, BOOL bOnlyHardAttr=FALSE) const;

	BOOL SetAttributes(const SfxItemSet& rSet, BOOL bReplaceAll);
	SfxStyleSheet* GetStyleSheet(BOOL& rOk) const;
	BOOL SetStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);

	virtual void MakeVisible(const Rectangle& rRect, Window& rWin);

	virtual void DoConnect(SdrOle2Obj* pOleObj);

	BOOL IsAnimationEnabled() const { return ( SDR_ANIMATION_ANIMATE == eAnimationMode ); }
	void SetAnimationEnabled( BOOL bEnable=TRUE ) { SetAnimationMode( bEnable ? SDR_ANIMATION_ANIMATE : SDR_ANIMATION_DISABLE ); }
	 BOOL IsAnimationPause() const { return bAnimationPause; }
	void SetAnimationPause( BOOL bSet ) { bAnimationPause = bSet; }

	void SetAnimationMode( const SdrAnimationMode eMode ) { eAnimationMode = eMode; }
	SdrAnimationMode GetAnimationMode() const { return eAnimationMode; }

	void ShowItemBrowser(BOOL bShow=TRUE);
	BOOL IsItemBrowserVisible() const { return pItemBrowser!=NULL && ((Window*)pItemBrowser)->IsVisible(); }
	Window* GetItemBrowser() const { return (Window*)pItemBrowser; }

	void VisAreaChanged(const OutputDevice* pOut=NULL);
	void VisAreaChanged(const SdrPageViewWinRec& rPVWR);

	virtual void InsertControlContainer( ::com::sun::star::uno::Reference< ::com::sun::star::awt::XControlContainer > xCC) {}
	virtual void RemoveControlContainer( ::com::sun::star::uno::Reference< ::com::sun::star::awt::XControlContainer > xCC) {}

	BOOL IsPrintPreview() const { return bPrintPreview; }
	void SetPrintPreview(BOOL bOn=TRUE) { bPrintPreview=bOn; }
};

#endif 

