

#ifndef _SVDPOEV_HXX
#define _SVDPOEV_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdedtv.hxx>

enum SdrPathSmoothKind  {SDRPATHSMOOTH_DONTCARE,   
			 SDRPATHSMOOTH_ANGULAR,    
			 SDRPATHSMOOTH_ASYMMETRIC, 
			 SDRPATHSMOOTH_SYMMETRIC}; 

enum SdrPathSegmentKind {SDRPATHSEGMENT_DONTCARE,  
			 SDRPATHSEGMENT_LINE,      
			 SDRPATHSEGMENT_CURVE,     
			 SDRPATHSEGMENT_TOGGLE};   

enum SdrObjClosedKind   {SDROBJCLOSED_DONTCARE,    
			 SDROBJCLOSED_OPEN,        
			 SDROBJCLOSED_CLOSED,      
			 SDROBJCLOSED_TOGGLE};     

class SdrPolyEditView: public SdrEditView
{
	friend class				SdrEditView;

protected:
	BOOL						bSetMarkedPointsSmoothPossible : 1;
	BOOL						bSetMarkedSegmentsKindPossible : 1;

	SdrPathSmoothKind			eMarkedPointsSmooth;
	SdrPathSegmentKind			eMarkedSegmentsKind;

private:
	void ImpClearVars();
	void ImpResetPolyPossibilityFlags();
	void ImpCheckPolyPossibilities();

	void ImpCopyMarkedPoints();
	typedef void (*PPolyTrFunc)(Point&, Point*, Point*, const void*, const void*, const void*, const void*, const void*);
	void ImpTransformMarkedPoints(PPolyTrFunc pTrFunc, const void* p1=NULL, const void* p2=NULL, const void* p3=NULL, const void* p4=NULL, const void* p5=NULL);

public:
	SdrPolyEditView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrPolyEditView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrPolyEditView();

	BOOL IsSetMarkedPointsSmoothPossible() const { ForcePossibilities(); return bSetMarkedPointsSmoothPossible; }
	SdrPathSmoothKind GetMarkedPointsSmooth() const { ForcePossibilities(); return eMarkedPointsSmooth; }
	void SetMarkedPointsSmooth(SdrPathSmoothKind eKind);

	BOOL IsSetMarkedSegmentsKindPossible() const { ForcePossibilities(); return bSetMarkedSegmentsKindPossible; }
	SdrPathSegmentKind GetMarkedSegmentsKind() const { ForcePossibilities(); return eMarkedSegmentsKind; }
	void SetMarkedSegmentsKind(SdrPathSegmentKind eKind);

	void DeleteMarkedPoints();
	BOOL IsDeleteMarkedPointsPossible() const { return HasMarkedPoints(); }

	void MoveMarkedPoints(const Size& rSiz, BOOL bCopy=FALSE);
	void ResizeMarkedPoints(const Point& rRef, const Fraction& xFact, const Fraction& yFact, BOOL bCopy=FALSE);
	void RotateMarkedPoints(const Point& rRef, long nWink, BOOL bCopy=FALSE);

	void RipUpAtMarkedPoints();
	BOOL IsRipUpAtMarkedPointsPossible() const;

	void ShutMarkedObjects();
	void CloseMarkedObjects(BOOL bToggle=FALSE, BOOL bOpen=FALSE, long nOpenDistance=0);
	void OpenMarkedObjects(long nOpenDistance=0) { CloseMarkedObjects(FALSE,TRUE,nOpenDistance); }
	BOOL IsOpenCloseMarkedObjectsPossible() const;
	SdrObjClosedKind GetMarkedObjectsClosedState() const;
};

#endif 

