

#ifndef _SVDSNPV_HXX
#define _SVDSNPV_HXX

#include <svx/svdpntv.hxx>

#ifndef _SVDHLPLN_HXX
#include <svx/svdhlpln.hxx>
#endif

#define SDRSNAP_NOTSNAPPED  0x0000
#define SDRSNAP_XSNAPPED    0x0001
#define SDRSNAP_YSNAPPED    0x0002
#define SDRSNAP_XYSNAPPED   0x0003

enum SdrCrookMode {
	SDRCROOK_ROTATE,
	SDRCROOK_SLANT,
	SDRCROOK_STRETCH
};

class SdrSnapView: public SdrPaintView
{
protected:
	SdrPageView*				pDragHelpLinePV;  

	Size						aMagnSiz;
	Size						aSnapSiz; 
#ifndef TF_FRACTIONGRID
	Fraction					aSnapWdtX;
	Fraction					aSnapWdtY;
#endif
	SdrHelpLine					aDragHelpLine;    
	Point						aLastMausPosNoSnap;

	USHORT						nMagnSizPix;
	USHORT						nDragHelpLineNum; 
	long						nSnapAngle;
	long						nEliminatePolyPointLimitAngle;

	SdrCrookMode				eCrookMode;

	unsigned					bSnapEnab : 1;
	unsigned					bGridSnap : 1;
	unsigned					bSnapTo1Pix : 1;             
	unsigned					bBordSnap : 1;
	unsigned					bHlplSnap : 1;
	unsigned					bOFrmSnap : 1;
	unsigned					bOPntSnap : 1;
	unsigned					bOConSnap : 1;
	unsigned					bMoveMFrmSnap : 1;
	unsigned					bMoveOFrmSnap : 1;
	unsigned					bMoveOPntSnap : 1;
	unsigned					bMoveOConSnap : 1;
	unsigned					bMoveSnapOnlyTopLeft : 1;    
	unsigned					bOrtho : 1;
	unsigned					bBigOrtho : 1;
	unsigned					bAngleSnapEnab : 1;
	unsigned					bMoveOnlyDragging : 1;       
	unsigned					bSlantButShear : 1;          
	unsigned					bCrookNoContortion : 1;      
	unsigned					bSetPageOrg : 1;             
	unsigned					bDragHelpLine : 1;    
	unsigned					bHlplFixed : 1;       
	unsigned					bEliminatePolyPoints : 1;

private:
	void ClearVars();

protected:

	void ShowSetPageOrg(OutputDevice* pOut);
	void HideSetPageOrg(OutputDevice* pOut);
	void DrawSetPageOrg(OutputDevice* pOut) const;
	void ShowDragHelpLine(OutputDevice* pOut);
	void HideDragHelpLine(OutputDevice* pOut);
	void DrawDragHelpLine(OutputDevice* pOut) const;
	virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead,SvStream& rIn);
public:
	SdrSnapView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrSnapView(SdrModel* pModel1, ExtOutputDevice* pXOut);

	virtual void ToggleShownXor(OutputDevice* pOut, const Region* pRegion) const;
	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BckAction();
	virtual void BrkAction(); 
	virtual void TakeActionRect(Rectangle& rRect) const;

	void SetSnapGrid(const Size& rSiz) { aSnapSiz=rSiz; SetSnapGridWidth(Fraction(rSiz.Width(),1),Fraction(rSiz.Height(),1)); }
	const Size& GetSnapGrid() const { return aSnapSiz; }

	void SetSnapGridWidth(const Fraction& rX, const Fraction& rY) { aSnapWdtX=rX; aSnapWdtY=rY; SnapMove(); }
	const Fraction& GetSnapGridWidthX() const { return aSnapWdtX; }
	const Fraction& GetSnapGridWidthY() const { return aSnapWdtY; }

	void SetSnapMagnetic(const Size& rSiz) { if (rSiz!=aMagnSiz) { aMagnSiz=rSiz; SnapMove(); } }
	const Size& GetSnapMagnetic() const { return aMagnSiz; }
	void SetSnapMagneticPixel(USHORT nPix) { nMagnSizPix=nPix; }
	USHORT GetSnapMagneticPixel() const { return nMagnSizPix; }

	void RecalcLogicSnapMagnetic(const OutputDevice& rOut) { SetSnapMagnetic(rOut.PixelToLogic(Size(nMagnSizPix,nMagnSizPix))); }
	void SetActualWin(const OutputDevice* pWin) { SdrPaintView::SetActualWin(pWin); if (pWin!=NULL) RecalcLogicSnapMagnetic(*pWin); }

	USHORT SnapPos(Point& rPnt, const SdrPageView* pPV) const;
	Point GetSnapPos(const Point& rPnt, const SdrPageView* pPV) const;
	USHORT SnapRect(const Rectangle& rRect, const SdrPageView* pPV, long& rDX, long& rDY) const;
	void CheckSnap(const Point& rPt, const SdrPageView* pPV,
		long& nBestXSnap, long& nBestYSnap, BOOL& bXSnapped, BOOL& bYSnapped) const;
	void SnapMove();

	BOOL IsSnapEnabled() const { return bSnapEnab; }
	BOOL IsGridSnap() const { return bGridSnap; } 
	BOOL IsBordSnap() const { return bBordSnap; } 
	BOOL IsHlplSnap() const { return bHlplSnap; } 
	BOOL IsOFrmSnap() const { return bOFrmSnap; } 
	BOOL IsOPntSnap() const { return bOPntSnap; } 
	BOOL IsOConSnap() const { return bOConSnap; } 
	void SetSnapEnabled(BOOL bOn) { bSnapEnab=bOn; SnapMove(); }
	void SetGridSnap(BOOL bOn) { bGridSnap=bOn; SnapMove(); }
	void SetBordSnap(BOOL bOn) { bBordSnap=bOn; SnapMove(); }
	void SetHlplSnap(BOOL bOn) { bHlplSnap=bOn; SnapMove(); }
	void SetOFrmSnap(BOOL bOn) { bOFrmSnap=bOn; SnapMove(); }
	void SetOPntSnap(BOOL bOn) { bOPntSnap=bOn; SnapMove(); }
	void SetOConSnap(BOOL bOn) { bOConSnap=bOn; SnapMove(); }

	void SetMoveSnapOnlyTopLeft(BOOL bOn) { bMoveSnapOnlyTopLeft=bOn; SnapMove(); }
	BOOL IsMoveSnapOnlyTopLeft() const { return bMoveSnapOnlyTopLeft; }

	BOOL IsHlplFixed() const { return bHlplFixed; }
	void SetHlplFixed(BOOL bOn) { bHlplFixed=bOn; }

	BOOL IsMoveMFrmSnap() const { return bMoveMFrmSnap; } 
	BOOL IsMoveOFrmSnap() const { return bMoveOFrmSnap; } 
	BOOL IsMoveOPntSnap() const { return bMoveOPntSnap; } 
	BOOL IsMoveOConSnap() const { return bMoveOConSnap; } 

	void SetMoveMFrmSnap(BOOL bOn) { bMoveMFrmSnap=bOn; SnapMove(); }
	void SetMoveOFrmSnap(BOOL bOn) { bMoveOFrmSnap=bOn; SnapMove(); }
	void SetMoveOPntSnap(BOOL bOn) { bMoveOPntSnap=bOn; SnapMove(); }
	void SetMoveOConSnap(BOOL bOn) { bMoveOConSnap=bOn; SnapMove(); }

	BOOL BegSetPageOrg(const Point& rPnt, OutputDevice* pOut=NULL, short nMinMov=0);
	void MovSetPageOrg(const Point& rPnt);
	BOOL EndSetPageOrg();
	void BrkSetPageOrg();
	BOOL IsSetPageOrg() const { return bSetPageOrg; }

	BOOL PickHelpLine(const Point& rPnt, short nTol, const OutputDevice& rOut, USHORT& rnHelpLineNum, SdrPageView*& rpPV) const;
	BOOL PickHelpLine(const Point& rPnt, const OutputDevice& rOut, USHORT& rnHelpLineNum, SdrPageView*& rpPV) const { return PickHelpLine(rPnt,-2,rOut,rnHelpLineNum,rpPV); }

	BOOL BegDragHelpLine(USHORT nHelpLineNum, SdrPageView* pPV, OutputDevice* pOut=NULL, short nMinMov=-3);

	BOOL BegDragHelpLine(const Point& rPnt, SdrHelpLineKind eNewKind, OutputDevice* pOut=NULL, short nMinMov=0);
	const SdrHelpLine& GetDraggedHelpLine() const { return aDragHelpLine; }
	SdrHelpLineKind GetDraggedHelpLineKind() const { return aDragHelpLine.GetKind(); }

	USHORT GetDraggedHelpLineIndex() const { return nDragHelpLineNum; }
	SdrPageView* GetDraggedHelpLinePageView() const { return pDragHelpLinePV; }

	void SetDraggedHelpLineKind(SdrHelpLineKind eNewKind);
	void MovDragHelpLine(const Point& rPnt);
	BOOL EndDragHelpLine();
	void BrkDragHelpLine();
	BOOL IsDragHelpLine() const { return bDragHelpLine; }

	void SetAngleSnapEnabled(BOOL bOn) { bAngleSnapEnab=bOn; SnapMove(); }
	BOOL IsAngleSnapEnabled() const { return bAngleSnapEnab; }
	void SetSnapAngle(long nWink) { nSnapAngle=nWink;   SnapMove(); }
	long GetSnapAngle() const { return nSnapAngle; }

	void SetOrtho(BOOL bOn) { bOrtho=bOn; } 
	BOOL IsOrtho() const { return bOrtho; }

	void SetBigOrtho(BOOL bOn) { bBigOrtho=bOn; SnapMove(); }
	BOOL IsBigOrtho() const { return bBigOrtho; }

	void SetMoveOnlyDragging(BOOL bOn) { bMoveOnlyDragging=bOn; }
	BOOL IsMoveOnlyDragging() const { return bMoveOnlyDragging; }

	void SetSlantButShear(BOOL bOn) { bSlantButShear=bOn; SnapMove(); }
	BOOL IsSlantButShear() const { return bSlantButShear; }

	void SetCrookNoContortion(BOOL bOn) { bCrookNoContortion=bOn; SnapMove(); }
	BOOL IsCrookNoContortion() const { return bCrookNoContortion; }

	void SetCrookMode(SdrCrookMode eMode) { eCrookMode=eMode; SnapMove(); }
	SdrCrookMode GetCrookMode() const { return eCrookMode; }

	void SetEliminatePolyPoints(BOOL bOn) { bEliminatePolyPoints=bOn; SnapMove(); }
	BOOL IsEliminatePolyPoints() const { return bEliminatePolyPoints; }
	void SetEliminatePolyPointLimitAngle(long nAngle) { nEliminatePolyPointLimitAngle=nAngle; SnapMove(); }
	long GetEliminatePolyPointLimitAngle() const { return nEliminatePolyPointLimitAngle; }
};

#endif 

