

#ifndef _SVDVIEW_HXX
#define _SVDVIEW_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdcrtv.hxx>

class SvxURLField;

enum SdrViewContext {SDRCONTEXT_STANDARD,
		 SDRCONTEXT_POINTEDIT,
		 SDRCONTEXT_GLUEPOINTEDIT,
		 SDRCONTEXT_TEXTEDIT,
		 SDRCONTEXT_GRAPHIC};

enum SdrEventKind  {SDREVENT_NONE,
		SDREVENT_TEXTEDIT,
		SDREVENT_MOVACTION,
		SDREVENT_ENDACTION,
		SDREVENT_BCKACTION,
		SDREVENT_BRKACTION,
		SDREVENT_ENDCREATE,
		SDREVENT_ENDDRAG,
		SDREVENT_MARKOBJ,
		SDREVENT_MARKPOINT,
		SDREVENT_MARKGLUEPOINT,
		SDREVENT_BEGMARK,
		SDREVENT_BEGINSOBJPOINT,
		SDREVENT_ENDINSOBJPOINT,
		SDREVENT_BEGINSGLUEPOINT,
		SDREVENT_BEGDRAGHELPLINE,
		SDREVENT_BEGDRAGOBJ,
		SDREVENT_BEGCREATEOBJ,
		SDREVENT_BEGMACROOBJ,
		SDREVENT_BEGTEXTEDIT,
		SDREVENT_ENDMARK,
		SDREVENT_BRKMARK,
		SDREVENT_EXECUTEURL};

#define SDRMOUSEBUTTONDOWN 1
#define SDRMOUSEMOVE       2
#define SDRMOUSEBUTTONUP   3

struct SdrViewEvent
{
	SdrHdl*						pHdl;
	SdrObject*					pObj;
	SdrObject*					pRootObj;        
	SdrPageView*				pPV;
	const SvxURLField*			pURLField;

	Point						aLogicPos;
	SdrHitKind					eHit;
	SdrEventKind				eEvent;
	SdrHdlKind					eHdlKind;
	SdrCreateCmd				eEndCreateCmd;   

	UINT16						nMouseClicks;
	UINT16						nMouseMode;
	UINT16						nMouseCode;
	UINT16						nHlplIdx;
	UINT16						nGlueId;

	unsigned					bMouseDown : 1;
	unsigned					bMouseUp : 1;
	unsigned					bDoubleHdlSize : 1;  
	unsigned					bIsAction : 1;       
	unsigned					bIsTextEdit : 1;     
	unsigned					bTextEditHit : 1;    
	unsigned					bAddMark : 1;
	unsigned					bUnmark : 1;
	unsigned					bPrevNextMark : 1;
	unsigned					bMarkPrev : 1;
	unsigned					bInsPointNewObj : 1;
	unsigned					bDragWithCopy : 1;
	unsigned					bCaptureMouse : 1;
	unsigned					bReleaseMouse : 1;

public:
	SdrViewEvent();
	~SdrViewEvent();

	void SetMouseEvent(const MouseEvent& rMEvt, USHORT nEventKind);
};

class SdrView: public SdrCreateView
{
	friend class				SdrPageView;

	unsigned					bNoExtendedMouseDispatcher : 1;
	unsigned					bNoExtendedKeyDispatcher : 1;
	unsigned					bNoExtendedCommandDispatcher : 1;
	unsigned					bTextEditOnObjectsWithoutTextIfTextTool : 1;

protected:

	virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead, SvStream& rIn);

public:
	TYPEINFO();
	SdrView(SdrModel* pModel1, OutputDevice* pOut);
	SdrView(SdrModel* pModel1, ExtOutputDevice* pOut);
	SdrView(SdrModel* pModel1);

	void EnableExtendedMouseEventDispatcher(BOOL bOn) { bNoExtendedMouseDispatcher = !bOn; }
	BOOL IsExtendedMouseEventDispatcherEnabled() const { return bNoExtendedMouseDispatcher; }

	void EnableExtendedKeyInputDispatcher(BOOL bOn) { bNoExtendedKeyDispatcher=!bOn; }
	BOOL IsExtendedKeyInputDispatcherEnabled() const { return bNoExtendedKeyDispatcher; }

	void EnableExtendedCommandEventDispatcher(BOOL bOn) { bNoExtendedCommandDispatcher=!bOn; }
	BOOL IsExtendedCommandEventDispatcherEnabled() const { return bNoExtendedCommandDispatcher; }

	void EnableTextEditOnObjectsWithoutTextIfTextTool(BOOL bOn) { bTextEditOnObjectsWithoutTextIfTextTool=bOn; }
	BOOL IsEnableTextEditOnObjectsWithoutTextIfTextToolEnabled() const { return bTextEditOnObjectsWithoutTextIfTextTool; }

	BOOL KeyInput(const KeyEvent& rKEvt, Window* pWin);
	BOOL MouseButtonDown(const MouseEvent& rMEvt, Window* pWin);
	BOOL MouseButtonUp(const MouseEvent& rMEvt, Window* pWin);
	BOOL MouseMove(const MouseEvent& rMEvt, Window* pWin);
	BOOL Command(const CommandEvent& rCEvt, Window* pWin);

	BOOL SetAttributes(const SfxItemSet& rSet, BOOL bReplaceAll=FALSE) { return SdrCreateView::SetAttributes(rSet,bReplaceAll); }
	BOOL SetStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr=FALSE) { return SdrCreateView::SetStyleSheet(pStyleSheet,bDontRemoveHardAttr); }

	BOOL GetAttributes(SfxItemSet& rTargetSet, BOOL bOnlyHardAttr=FALSE) const;

	SfxStyleSheet* GetStyleSheet() const;

	Pointer GetPreferedPointer(const Point& rMousePos, const OutputDevice* pOut, USHORT nModifier=0, BOOL bLeftDown=FALSE) const;
	SdrHitKind PickAnything(const MouseEvent& rMEvt, USHORT nMouseDownOrMoveOrUp, SdrViewEvent& rVEvt) const;
	SdrHitKind PickAnything(const Point& rLogicPos, SdrViewEvent& rVEvt) const;
	BOOL DoMouseEvent(const SdrViewEvent& rVEvt);
	SdrViewContext GetContext() const;

	BOOL IsMarkPossible() const;
	BOOL HasMarked() const { return HasMarkedObj(); }
	void MarkAll();
	void UnmarkAll();
	BOOL IsAllMarked() const;
	BOOL IsAllMarkPrevNextPossible() const; 
	BOOL MarkNext(BOOL bPrev=FALSE);
	BOOL MarkNext(const Point& rPnt, BOOL bPrev=FALSE);

	const Rectangle& GetMarkedRect() const;
	void SetMarkedRect(const Rectangle& rRect);

	virtual void DeleteMarked();
	BOOL IsDeleteMarkedPossible() const;
	BOOL IsDeletePossible() const { return IsDeleteMarkedPossible(); }

	BOOL BegMark(const Point& rPnt, BOOL bAddMark=FALSE, BOOL bUnmark=FALSE);

	String GetStatusText();

	friend SvStream& operator<<(SvStream& rOut, const SdrView& rView);

	friend SvStream& operator>>(SvStream& rIn, SdrView& rView);
};

#endif 

