

#ifndef _SVDVMARK_HXX
#define _SVDVMARK_HXX

#ifndef _SV_OUTDEV_HXX 
#include <vcl/outdev.hxx>
#endif

#ifndef _VCL_POINTR_HXX
#include <vcl/pointr.hxx>
#endif

class XPolygon;
class XPolyPolygon;
class SdrObject;
class SdrPageView;

enum SdrViewUserMarkerAlign
{
	SDRMARKER_ALIGNCENTER,
	SDRMARKER_ALIGNINNER,
	SDRMARKER_ALIGNOUTER
};

class SdrViewUserMarker
{
	friend class				SdrPaintView;

	SdrPaintView*				pView;
	OutputDevice*				pForcedOutDev;
	Point*						pPoint;
	Rectangle*					pRect;
	Polygon*					pPoly;
	PolyPolygon*				pPolyPoly;
	XPolygon*					pXPoly;
	XPolyPolygon*				pXPolyPoly;

	SdrViewUserMarkerAlign		eAlign;
	USHORT						nPixelDistance;      
	USHORT						nLineWdt;            
	USHORT						nCrossSize;          
	FASTBOOL					bLineWdtLog;
	FASTBOOL					bCrossSizeLog;
	FASTBOOL					bSolidArea;
	FASTBOOL					bDashed;
	FASTBOOL					bCrossHair;          
	FASTBOOL					bStripes;            
	FASTBOOL					bEllipse;            
	FASTBOOL					bPolyLine;           
	FASTBOOL					bAnimate;
	FASTBOOL					bVisible;

	USHORT						nAnimateDelay;
	USHORT						nAnimateSpeed;
	USHORT						nAnimateAnz;
	FASTBOOL					bAnimateBwd;
	FASTBOOL					bAnimateToggle;
	USHORT						nAnimateDelayCountDown;
	USHORT						nAnimateSpeedCountDown;
	USHORT						nAnimateNum;

	FASTBOOL					bHasPointer;
	FASTBOOL					bMouseMovable;

protected:
	void ImpDelGeometrics();
	void ImpDraw(FASTBOOL bHiding, FASTBOOL bNoSaveDC);
	virtual void Draw(OutputDevice* pOut, FASTBOOL bHiding, FASTBOOL bNoSaveDC);

public:
	SdrViewUserMarker(SdrPaintView* pView);

	virtual ~SdrViewUserMarker();

	void SetOutputDevice(OutputDevice* pOut) { pForcedOutDev=pOut; }
	OutputDevice* GetOutputDevice() const { return pForcedOutDev; }

	void SetView(SdrPaintView* pView_);
	SdrPaintView* GetView() const { return pView; }

	void SetPoint(const Point& rPoint); 
	void SetRectangle(const Rectangle& rRect); 
	void SetRectangle(const SdrObject* pObj, const SdrPageView* pPV=NULL);
	void SetPolygon(const Polygon& rPoly);
	void SetPolyPolygon(const PolyPolygon& rPolyPoly);
	void SetXPolygon(const XPolygon& rXPoly);
	void SetXPolyPolygon(const XPolyPolygon& rXPolyPoly);
	void SetXPolyPolygon(const SdrObject* pObj, const SdrPageView* pPV=NULL);

	void SetCrossHair(FASTBOOL bOn);
	void SetStripes(FASTBOOL bOn);
	void SetEllipse(FASTBOOL bOn);
	void SetPolyLine(FASTBOOL bOn);
	void SetSolidArea(FASTBOOL bOn);
	void SetDashed(FASTBOOL bOn);
	void SetPixelDistance(USHORT nDistance);
	void SetLineWidth(USHORT nWdt);
	void SetLineWidthIsLogic(FASTBOOL bOn);
	void SetCrossHairSize(USHORT nSize);
	void SetCrossSizeIsLogic(FASTBOOL bOn);

	void SetAnimate(FASTBOOL bOn); 
	void SetAnimateDelay(USHORT nTime) { nAnimateDelay=(nTime+25)/50; }
	void SetAnimateSpeed(USHORT nTime) { nAnimateSpeed=(nTime+25)/50; if (nAnimateSpeed>0) nAnimateSpeed--; }
	void SetAnimateCount(USHORT nAnz) { nAnimateAnz=nAnz; }
	void SetAnimateBackward(FASTBOOL bOn) { bAnimateBwd=bOn; }
	void SetAnimateToggle(FASTBOOL bOn) { bAnimateToggle=bOn; }

	const Point* GetPoint() const { return pPoint; }
	const Rectangle* GetRectangle() const { return pRect; }
	const Polygon* GetPolygon() const { return pPoly; }
	const PolyPolygon* GetPolyPolygon() const { return pPolyPoly; }
	const XPolygon* GetXPolygon() const { return pXPoly; }
	const XPolyPolygon* GetXPolyPolygon() const { return pXPolyPoly; }

	FASTBOOL IsDot() const { return pPoint!=NULL && !bCrossHair; }
	FASTBOOL IsCrossHair() const { return pPoint!=NULL && bCrossHair; }
	FASTBOOL IsRectangle() const { return pRect!=NULL && !bEllipse && !bStripes; }
	FASTBOOL IsStripes() const { return pRect!=NULL && bStripes; }
	FASTBOOL IsEllipse() const { return pRect!=NULL && bEllipse; }
	FASTBOOL IsPolyLine() const { return (pPoly!=NULL || pPolyPoly!=NULL || pXPoly!=NULL || pXPolyPoly!=NULL) && bPolyLine; }

	FASTBOOL IsSolidArea() const { return bSolidArea && pPoint==NULL && !bPolyLine; }
	FASTBOOL IsDashed() const { return bDashed; }
	USHORT GetPixelDistance() const { return nPixelDistance; }
	USHORT GetLineWidth() const { return nLineWdt; }
	FASTBOOL IsLineWidthIsLogic() const { return bLineWdtLog; }
	USHORT GetCrossHairSize() const { return nCrossSize; }
	FASTBOOL IsCrossHairSizeIsLogic() const { return bCrossSizeLog; }

	FASTBOOL IsAnimate() const { return bAnimate; }
	USHORT GetAnimateDelay() const { return nAnimateDelay*50; }
	USHORT GetAnimateSpeed() const { return (nAnimateSpeed+1)*50; }
	USHORT GetAnimateCount() const { return nAnimateAnz; }
	FASTBOOL IsAnimateBackward() const { return bAnimateBwd; }
	FASTBOOL IsAnimateToggle() const { return bAnimateToggle; }

	virtual void Move(long nXMove, long nYMove);
	void Show();
	void Hide();
	FASTBOOL IsVisible() const { return bVisible; }
	virtual void DoAnimateOneStep();
	FASTBOOL IncAnimateCounters();

	virtual FASTBOOL IsHit(const Point& rPnt, short nTol=-2) const;
	FASTBOOL HasPointer() const { return bHasPointer; }
	virtual Pointer  GetPointer() const;
	FASTBOOL IsMouseMovable() const { return bMouseMovable; }
};

#endif 

