

#ifndef _MySVXACORR_HXX
#define _MySVXACORR_HXX

#ifndef _RTTI_HXX 
#include <tools/rtti.hxx>
#endif
#ifndef _SVARRAY_HXX 
#include <svtools/svarray.hxx>
#endif
#ifndef _LANG_HXX 
#include <tools/lang.hxx>
#endif
#ifndef _TIME_HXX 
#include <tools/time.hxx>
#endif
#ifndef _DATE_HXX 
#include <tools/date.hxx>
#endif
#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _SVXSWAFOPT_HXX
#include <svx/swafopt.hxx>
#endif

class CharClass;
class SfxPoolItem;
class SvxAutoCorrect;
class SvStringsISortDtor;
class SvStorage;
class SvStorageRef;
class SfxObjectShell;
class SvxAutoCorrLanguageTable_Impl;
class SvxAutoCorrLastFileAskTable_Impl;

const long CptlSttSntnc		= 0x00000001;	
const long CptlSttWrd		= 0x00000002;	
const long ChgFractionSymbol= 0x00000004;	
const long ChgOrdinalNumber = 0x00000008;	
const long ChgToEnEmDash    = 0x00000010;	
const long ChgWeightUnderl  = 0x00000020;	
const long SetINetAttr      = 0x00000040;	
const long Autocorrect 		= 0x00000080;	
const long ChgQuotes		= 0x00000100;	
const long SaveWordCplSttLst= 0x00000200;	
const long SaveWordWrdSttLst= 0x00000400;	
const long IngnoreDoubleSpace= 0x00000800;	
const long ChgSglQuotes		= 0x00001000;	

const long ChgWordLstLoad	= 0x20000000;	
const long CplSttLstLoad	= 0x40000000;	
const long WrdSttLstLoad	= 0x80000000;	

class SvxAutoCorrDoc
{
public:
	SvxAutoCorrDoc() {}
	virtual ~SvxAutoCorrDoc();

	virtual BOOL Delete( xub_StrLen nStt, xub_StrLen nEnd ) = 0;
	virtual BOOL Insert( xub_StrLen nPos, const String& rTxt ) = 0;
	virtual BOOL Replace( xub_StrLen nPos, const String& rTxt ) = 0;

	virtual BOOL SetAttr( xub_StrLen nStt, xub_StrLen nEnd, USHORT nSlotId,
				SfxPoolItem& ) = 0;

	virtual BOOL SetINetAttr( xub_StrLen nStt, xub_StrLen nEnd, const String& rURL ) = 0;

	virtual const String* GetPrevPara( BOOL bAtNormalPos ) = 0;

	virtual BOOL ChgAutoCorrWord( xub_StrLen& rSttPos, xub_StrLen nEndPos,
					  SvxAutoCorrect& rACorrect,
					  const String** ppPara ) = 0;

	virtual void SaveCpltSttWord( ULONG nFlag, xub_StrLen nPos,
						const String& rExceptWord,
						sal_Unicode cChar );

	virtual LanguageType GetLanguage( xub_StrLen nPos, BOOL bPrevPara = FALSE ) const;
};

class SvxAutocorrWord
{
	String sShort, sLong;
	BOOL bIsTxtOnly;				
public:
	SvxAutocorrWord( const String& rS, const String& rL, BOOL bFlag = TRUE )
		: sShort( rS ), sLong( rL ), bIsTxtOnly( bFlag )
	{}

	const String& GetShort() const					{ return sShort; }
	const String& GetLong() const					{ return sLong; }
	BOOL IsTextOnly() const							{ return bIsTxtOnly; }
};

typedef SvxAutocorrWord* SvxAutocorrWordPtr;
SV_DECL_PTRARR_SORT_DEL( SvxAutocorrWordList, SvxAutocorrWordPtr, 10, 10 )

class SvxAutoCorrectLanguageLists
{
	String sShareAutoCorrFile, sUserAutoCorrFile;

	Date aModifiedDate;
	Time aModifiedTime, aLastCheckTime;

	LanguageType 			eLanguage; 
	SvStringsISortDtor* 	pCplStt_ExcptLst;
	SvStringsISortDtor* 	pWrdStt_ExcptLst;
	SvxAutocorrWordList* 	pAutocorr_List;
	SvxAutoCorrect&			rAutoCorrect;

	long nFlags;

	BOOL IsFileChanged_Imp();
	void LoadExceptList_Imp( SvStringsISortDtor*& rpLst,
					const sal_Char* pStrmName,
					SvStorageRef &rStg);
	void LoadXMLExceptList_Imp( SvStringsISortDtor*& rpLst,
					const sal_Char* pStrmName,
					SvStorageRef &rStg);
	void SaveExceptList_Imp( const SvStringsISortDtor& rLst,
					const sal_Char* pStrmName,
					SvStorageRef &rStg,
					BOOL bConvert = FALSE);

	BOOL MakeBlocklist_Imp( SvStorage& rStg );
	void RemoveStream_Imp( const String& rName );
	void MakeUserStorage_Impl();

public:
	SvxAutoCorrectLanguageLists( SvxAutoCorrect& rParent,
			const String& rShareAutoCorrectFile,
			const String& rUserAutoCorrectFile,
			LanguageType eLang);
	~SvxAutoCorrectLanguageLists();

	SvxAutocorrWordList* LoadAutocorrWordList();
	void SetAutocorrWordList( SvxAutocorrWordList* pList );
	const SvxAutocorrWordList* GetAutocorrWordList();

	SvStringsISortDtor* LoadCplSttExceptList();
	void SaveCplSttExceptList();
	void SetCplSttExceptList( SvStringsISortDtor* pList );
	const SvStringsISortDtor* GetCplSttExceptList();
	BOOL AddToCplSttExceptList(const String& rNew);

	SvStringsISortDtor* LoadWrdSttExceptList();
	void SaveWrdSttExceptList();
	void SetWrdSttExceptList( SvStringsISortDtor* pList );
	const SvStringsISortDtor* GetWrdSttExceptList();
	BOOL AddToWrdSttExceptList(const String& rNew);

	BOOL PutText( const String& rShort, const String& rLong );

	BOOL PutText( const String& rShort, SfxObjectShell& );

	BOOL DeleteText( const String& rShort );
};

class SvxAutoCorrect
{
	friend class SvxAutoCorrectLanguageLists;

	String sShareAutoCorrFile, sUserAutoCorrFile;

	SvxSwAutoFmtFlags aSwFlags;		

	SvxAutoCorrLanguageTable_Impl* pLangTable;
	SvxAutoCorrLastFileAskTable_Impl* pLastFileTable;
	CharClass* pCharClass;

	LanguageType eCharClassLang;

	long nFlags;
	sal_Unicode cStartDQuote, cEndDQuote, cStartSQuote, cEndSQuote,
	c1Div2, c1Div4, c3Div4,
	cEmDash, cEnDash;

	sal_Unicode _GetQuote( sal_Unicode cInsChar, BOOL bSttQuote,
				LanguageType eLang ) const;
	SvxAutoCorrectLanguageLists& _GetLanguageList( LanguageType eLang );

	void _GetCharClass( LanguageType eLang );

protected:

	virtual BOOL PutText( SvStorage&, const String& rShort, SfxObjectShell& ,
				String& );

	BOOL 	CreateLanguageFile(LanguageType eLang, BOOL bNewFile = TRUE);

public:
	virtual BOOL GetLongText( SvStorageRef&, const String& rShort, String& rLong );

	TYPEINFO();

	SvxAutoCorrect( const String& rShareAutocorrFile,
		const String& rUserAutocorrFile );
	SvxAutoCorrect( const SvxAutoCorrect& );
	virtual ~SvxAutoCorrect();

	ULONG AutoCorrect( SvxAutoCorrDoc& rDoc, const String& rTxt,
			xub_StrLen nPos, sal_Unicode cInsChar, BOOL bInsert );

	BOOL GetPrevAutoCorrWord( SvxAutoCorrDoc& rDoc, const String& rTxt,
					xub_StrLen nPos, String& rWord ) const;

	const SvxAutocorrWord* SearchWordsInList( const String& rTxt,
						xub_StrLen& rStt, xub_StrLen nEndPos,
						SvxAutoCorrDoc& rDoc,
						LanguageType& rLang );

	sal_Unicode GetStartSingleQuote() const 		{ return cStartSQuote; }
	sal_Unicode GetEndSingleQuote() const			{ return cEndSQuote; }
	sal_Unicode GetStartDoubleQuote() const 		{ return cStartDQuote; }
	sal_Unicode GetEndDoubleQuote() const			{ return cEndDQuote; }

	void SetStartSingleQuote( const sal_Unicode cStart ) 	{ cStartSQuote = cStart; }
	void SetEndSingleQuote( const sal_Unicode cEnd ) 		{ cEndSQuote = cEnd; }
	void SetStartDoubleQuote( const sal_Unicode cStart ) 	{ cStartDQuote = cStart; }
	void SetEndDoubleQuote( const sal_Unicode cEnd ) 		{ cEndDQuote = cEnd; }

	String GetQuote( SvxAutoCorrDoc& rDoc, xub_StrLen nInsPos,
		sal_Unicode cInsChar, BOOL bSttQuote );
	void InsertQuote( SvxAutoCorrDoc& rDoc, xub_StrLen nInsPos,
		sal_Unicode cInsChar, BOOL bSttQuote, BOOL bIns );

	String GetAutoCorrFileName( LanguageType eLang = LANGUAGE_SYSTEM,
					BOOL bNewFile = FALSE,
					BOOL bTstUserExist = FALSE ) const;
	void SetUserAutoCorrFileName( const String& rNew );
	void SetShareAutoCorrFileName( const String& rNew );

	long GetFlags() const						{ return nFlags; }
	inline SvxSwAutoFmtFlags&	GetSwFlags()	{ return aSwFlags;}
	BOOL IsAutoCorrFlag( long nFlag ) const
					{ return nFlags & nFlag ? TRUE : FALSE; }
	void SetAutoCorrFlag( long nFlag, BOOL bOn = TRUE );

	SvxAutocorrWordList* LoadAutocorrWordList(
						LanguageType eLang = LANGUAGE_SYSTEM )
		{ return _GetLanguageList( eLang ).LoadAutocorrWordList(); }
	const SvxAutocorrWordList* GetAutocorrWordList(
						LanguageType eLang = LANGUAGE_SYSTEM )
		{ return _GetLanguageList( eLang ).GetAutocorrWordList(); }

	BOOL PutText( const String& rShort, const String& rLong, LanguageType eLang = LANGUAGE_SYSTEM );

	BOOL PutText( const String& rShort, SfxObjectShell& rShell,
	 LanguageType eLang = LANGUAGE_SYSTEM )
		{ return _GetLanguageList( eLang ).PutText(rShort, rShell ); }

	BOOL DeleteText( const String& rShort, LanguageType eLang = LANGUAGE_SYSTEM);

	void SaveCplSttExceptList( LanguageType eLang = LANGUAGE_SYSTEM );
	SvStringsISortDtor* LoadCplSttExceptList(
						LanguageType eLang = LANGUAGE_SYSTEM)
		{	return _GetLanguageList( eLang ).LoadCplSttExceptList(); }
	const SvStringsISortDtor* GetCplSttExceptList(
						LanguageType eLang = LANGUAGE_SYSTEM )
		{	return _GetLanguageList( eLang ).GetCplSttExceptList(); }

	BOOL AddCplSttException( const String& rNew,
					LanguageType eLang = LANGUAGE_SYSTEM );

	void SaveWrdSttExceptList( LanguageType eLang = LANGUAGE_SYSTEM );
	SvStringsISortDtor* LoadWrdSttExceptList(
						LanguageType eLang = LANGUAGE_SYSTEM )
		{	return _GetLanguageList( eLang ).LoadWrdSttExceptList(); }
	const SvStringsISortDtor* GetWrdSttExceptList(
						LanguageType eLang = LANGUAGE_SYSTEM )
		{	return _GetLanguageList( eLang ).LoadWrdSttExceptList(); }

	BOOL AddWrtSttException( const String& rNew, LanguageType eLang = LANGUAGE_SYSTEM);

	BOOL FindInWrdSttExceptList( LanguageType eLang, const String& sWord );
	BOOL FindInCplSttExceptList( LanguageType eLang, const String& sWord,
						BOOL bAbbreviation = FALSE);

	BOOL FnCptlSttWrd( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang = LANGUAGE_SYSTEM );
	BOOL FnChgFractionSymbol( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos );
	BOOL FnChgOrdinalNumber( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang = LANGUAGE_SYSTEM );
	BOOL FnChgToEnEmDash( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang = LANGUAGE_SYSTEM );
	BOOL FnSetINetAttr( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang = LANGUAGE_SYSTEM );
	BOOL FnChgWeightUnderl( SvxAutoCorrDoc&, const String&,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang = LANGUAGE_SYSTEM );
	BOOL FnCptlSttSntnc( SvxAutoCorrDoc&, const String&, BOOL bNormalPos,
					xub_StrLen nSttPos, xub_StrLen nEndPos,
					LanguageType eLang  = LANGUAGE_SYSTEM);
	static long 	GetDefaultFlags();

	CharClass& GetCharClass( LanguageType eLang )
	{
		if( !pCharClass || eLang != eCharClassLang )
			_GetCharClass( eLang );
		return *pCharClass;
	}
};

#endif
