

#ifndef _SVXRTF_HXX
#define _SVXRTF_HXX

#ifndef _TABLE_HXX 
#include <tools/table.hxx>
#endif
#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _SFXITEMSET_HXX 
#include <svtools/itemset.hxx>
#endif
#ifndef _PARRTF_HXX 
#include <svtools/parrtf.hxx>
#endif

#define _SVSTDARR_USHORTS
#include <svtools/svstdarr.hxx>

class Font;
class Color;
class Graphic;
class DateTime;
struct SvxRTFStyleType;
class SvxRTFItemStackType;
class SvxRTFItemStackList;
class SfxDocumentInfo;

class SvxNodeIdx
{
public:
	virtual ~SvxNodeIdx() {}
	virtual ULONG	GetIdx() const = 0;
	virtual SvxNodeIdx* Clone() const = 0;	
};

class SvxPosition
{
public:
	virtual ~SvxPosition() {}

	virtual ULONG		GetNodeIdx() const = 0;
	virtual xub_StrLen	GetCntIdx() const = 0;

	virtual SvxPosition* Clone() const = 0; 
	virtual SvxNodeIdx* MakeNodeIdx() const = 0; 
};

typedef Color* ColorPtr;
SV_DECL_PTRARR( SvxRTFColorTbl, ColorPtr, 16, 4 )
DECLARE_TABLE( SvxRTFFontTbl, Font* )
DECLARE_TABLE( SvxRTFStyleTbl, SvxRTFStyleType* )
typedef SvxRTFItemStackType* SvxRTFItemStackTypePtr;
SV_DECL_PTRARR_DEL( SvxRTFItemStackList, SvxRTFItemStackTypePtr, 1, 1 )
SV_DECL_PTRARR_STACK( SvxRTFItemStack, SvxRTFItemStackTypePtr, 0, 1 )

struct SvxRTFStyleType
{
	SfxItemSet aAttrSet;		
	String sName;
	USHORT nBasedOn, nNext;
	BYTE nOutlineNo;
	BOOL bIsCharFmt;

	SvxRTFStyleType( SfxItemPool& rPool, const USHORT* pWhichRange );
};

struct SvxRTFPictureType
{

	enum RTF_BMPSTYLE
	{
		RTF_BITMAP, 		
		WIN_METAFILE,		
		MAC_QUICKDRAW,		
		OS2_METAFILE,		
		RTF_DI_BMP			
	} eStyle;

	enum RTF_BMPMODE
	{
		BINARY_MODE,
		HEX_MODE
	} nMode;

	USHORT	nType;
	USHORT	uPicLen;
	USHORT	nWidth, nHeight;
	USHORT	nGoalWidth, nGoalHeight;
	USHORT	nBitsPerPixel;
	USHORT	nPlanes;
	USHORT	nWidthBytes;
	USHORT	nScalX, nScalY;
	short 	nCropT, nCropB, nCropL, nCropR;

	SvxRTFPictureType() { ResetValues(); }

	void ResetValues();
};

struct RTFPlainAttrMapIds
{
	USHORT 	nCaseMap,
			nBgColor,
			nColor,
			nContour,
			nCrossedOut,
			nEscapement,
			nFont,
			nFontHeight,
			nKering,
			nLanguage,
			nPosture,
			nShadowed,
			nUnderline,
			nWeight,
			nWordlineMode,
			nAutoKerning,
			nCJKFont,
			nCJKFontHeight,
			nCJKLanguage,
			nCJKPosture,
			nCJKWeight,
			nCTLFont,
			nCTLFontHeight,
			nCTLLanguage,
			nCTLPosture,
			nCTLWeight,
			nEmphasis,
			nTwoLines,
			nCharScaleX,
			nHorzVert,
			nRuby,
			nRelief
			;
	RTFPlainAttrMapIds( const SfxItemPool& rPool );
};

struct RTFPardAttrMapIds
{
	USHORT	nLinespacing,
			nAdjust,
			nTabStop,
			nHyphenzone,
			nLRSpace,
			nULSpace,
			nBrush,
			nBox,
			nShadow,
			nOutlineLvl,
			nSplit,
			nKeep,
			nFontAlign,
			nScriptSpace,
			nHangPunct,
			nForbRule
			;
	RTFPardAttrMapIds( const SfxItemPool& rPool );
};

class SvxRTFParser : public SvRTFParser
{
	SvxRTFColorTbl	aColorTbl;
	SvxRTFFontTbl	aFontTbl;
	SvxRTFStyleTbl	aStyleTbl;
	SvxRTFItemStack aAttrStack;
	SvxRTFItemStackList aAttrSetList;

	SvUShorts aPlainMap;
	SvUShorts aPardMap;
	SvUShorts aWhichMap;
	String	sBaseURL;

	SvxPosition* pInsPos;
	SfxItemPool* pAttrPool;
	Color*  pDfltColor;
	Font*	pDfltFont;
	SfxDocumentInfo* pSfxInfo;
	SfxItemSet *pRTFDefaults;

	long	nVersionNo;
	int 	nDfltFont;

	BOOL	bNewDoc : 1;			
	BOOL	bNewGroup : 1;			
	BOOL	bIsSetDfltTab : 1;		
	BOOL	bChkStyleAttr : 1;		
	BOOL	bCalcValue : 1; 		
	BOOL	bPardTokenRead : 1; 	
	BOOL	bReadDocInfo : 1;		
	BOOL	bIsLeftToRightDef : 1; 	

	BOOL 	bIsInReadStyleTab : 1;	

	void ClearColorTbl();
	void ClearFontTbl();
	void ClearStyleTbl();
	void ClearAttrStack();

	SvxRTFItemStackTypePtr _GetAttrSet( int bCopyAttr=FALSE );	
	void _ClearStyleAttr( SvxRTFItemStackType& rStkType );

	void SetAttrSet( SfxItemSet& rAttrSet, SvxPosition& rSttPos );
	void SetAttrSet( SvxRTFItemStackType &rSet );
	void SetDefault( int nToken, short nValue );

	void RTFPardPlain( int bPard, SfxItemSet** ppSet );

	void BuildWhichTbl();

protected:

	String& DelCharAtEnd( String& rStr, const sal_Unicode cDel );

	virtual void NextToken( int nToken );

	virtual void ReadBitmapData();
	virtual void ReadOLEData();

	void ReadStyleTable();
	void ReadColorTable();
	void ReadFontTable();
	void ReadAttr( int nToken, SfxItemSet* pSet );
	void ReadTabAttr( int nToken, SfxItemSet& rSet );

	DateTime& GetDateTimeStamp( DateTime& rDT );
	String& GetTextToEndGroup( String& rStr );
	virtual void ReadInfo( const sal_Char* pChkForVerNo = 0 );

	inline SfxItemSet& GetAttrSet();

	int IsAttrSttPos();
	void AttrGroupEnd();		
	void SetAllAttrOfStk();		

	virtual void InsertPara() = 0;
	virtual void InsertText() = 0;
	virtual void MovePos( int bForward = TRUE ) = 0;
	virtual void SetEndPrevPara( SvxNodeIdx*& rpNodePos,
					 xub_StrLen& rCntPos )=0;
	virtual void SetAttrInDoc( SvxRTFItemStackType &rSet );

	virtual void UnknownAttrToken( int nToken, SfxItemSet* pSet );

	virtual void CalcValue();

	SvxRTFParser( SfxItemPool& rAttrPool,
		SvStream& rIn,
		int bReadNewDoc = TRUE );
	virtual ~SvxRTFParser();

	int IsNewDoc() const				{ return bNewDoc; }
	void SetNewDoc( int bFlag ) 		{ bNewDoc = bFlag; }
	int IsNewGroup() const				{ return bNewGroup; }
	void SetNewGroup( int bFlag )		{ bNewGroup = bFlag; }
	int IsChkStyleAttr() const			{ return bChkStyleAttr; }
	void SetChkStyleAttr( int bFlag )	{ bChkStyleAttr = bFlag; }
	int IsCalcValue() const 			{ return bCalcValue; }
	void SetCalcValue( int bFlag )		{ bCalcValue = bFlag; }
	int IsPardTokenRead() const 		{ return bPardTokenRead; }
	void SetPardTokenRead( int bFlag )	{ bPardTokenRead = bFlag; }
	int IsReadDocInfo() const			{ return bReadDocInfo; }
	void SetReadDocInfo( int bFlag )	{ bReadDocInfo = bFlag; }

	SvxPosition& GetInsPos() const		{ return *pInsPos; }
	void SetInsPos( const SvxPosition& rNew );

	long GetVersionNo() const 			{ return nVersionNo; }

	void AddPardAttr( USHORT nWhich ) { aPardMap.Insert( nWhich, aPardMap.Count() ); }
	void AddPlainAttr( USHORT nWhich ) { aPlainMap.Insert( nWhich, aPlainMap.Count() ); }

	SvxRTFStyleTbl& GetStyleTbl()				{ return aStyleTbl; }
	SvxRTFItemStack& GetAttrStack() 			{ return aAttrStack; }
	SvxRTFColorTbl&	GetColorTbl()				{ return aColorTbl; }
	SvxRTFFontTbl& GetFontTbl()					{ return aFontTbl; }

	SfxDocumentInfo* GetDocInfo() const 		{ return pSfxInfo; }

	const String& GetBaseURL() const			{ return sBaseURL; }

	BOOL ReadBmpData( Graphic& rGrf, SvxRTFPictureType& rPicType );

	xub_StrLen HexToBin( String& rToken );

	void SetScriptAttr( USHORT eType, SfxItemSet& rSet, SfxPoolItem& rItem );

public:

	virtual SvParserState CallParser(); 

	inline const Color& GetColor( USHORT nId ) const;
	const Font& GetFont( USHORT nId );		

	virtual int IsEndPara( SvxNodeIdx* pNd, xub_StrLen nCnt ) const = 0;

	void SetAttrPool( SfxItemPool* pNewPool )	{ pAttrPool = pNewPool; }

	RTFPardAttrMapIds& GetPardMap()
			{ return (RTFPardAttrMapIds&)*aPardMap.GetData(); }
	RTFPlainAttrMapIds& GetPlainMap()
			{ return (RTFPlainAttrMapIds&)*aPlainMap.GetData(); }

	void ReadBorderAttr( int nToken, SfxItemSet& rSet, int bTableDef=FALSE );
	void ReadBackgroundAttr( int nToken, SfxItemSet& rSet, int bTableDef=FALSE	);

	virtual void Continue( int nToken );

	const SfxItemSet& GetRTFDefaults();
};

class SvxRTFItemStackType
{
	friend class SvxRTFParser;
	friend class SvxRTFItemStackList;

	SfxItemSet	aAttrSet;
	SvxNodeIdx	*pSttNd, *pEndNd;
	xub_StrLen nSttCnt, nEndCnt;
	SvxRTFItemStackList* pChildList;
	USHORT nStyleNo;

	SvxRTFItemStackType( SfxItemPool&, const USHORT* pWhichRange,
				const SvxPosition& );
	~SvxRTFItemStackType();

	void Add( SvxRTFItemStackTypePtr );
	void Compress( const SvxRTFParser& );

public:
	SvxRTFItemStackType( const SvxRTFItemStackType&, const SvxPosition&,
			int bCopyAttr = FALSE );
	void SetStartPos( const SvxPosition& rPos );

	ULONG GetSttNodeIdx() const { return pSttNd->GetIdx(); }
	ULONG GetEndNodeIdx() const { return pEndNd->GetIdx(); }

	const SvxNodeIdx& GetSttNode() const { return *pSttNd; }
	const SvxNodeIdx& GetEndNode() const { return *pEndNd; }

	xub_StrLen GetSttCnt() const { return nSttCnt; }
	xub_StrLen GetEndCnt() const { return nEndCnt; }
	 SfxItemSet& GetAttrSet()			{ return aAttrSet; }
	const SfxItemSet& GetAttrSet() const	{ return aAttrSet; }

	USHORT StyleNo() const	{ return nStyleNo; }

	void SetRTFDefaults( const SfxItemSet& rDefaults );
};

inline const Color& SvxRTFParser::GetColor( USHORT nId ) const
{
	ColorPtr pColor = (ColorPtr)pDfltColor;
	if( nId < aColorTbl.Count() )
		pColor = aColorTbl[ nId ];
	return *pColor;
}

inline SfxItemSet& SvxRTFParser::GetAttrSet()
{
	SvxRTFItemStackTypePtr pTmp;
	if( bNewGroup || 0 == ( pTmp = aAttrStack.Top()) )
		pTmp = _GetAttrSet();
	return pTmp->aAttrSet;
}

#endif

