
#ifndef _SVX_TRANSFRM_HXX
#define _SVX_TRANSFRM_HXX

#include <svx/dlgctrl.hxx>

#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif
class SdrView;

class SvxTransformTabDialog : public SfxTabDialog
{
private:
	const SdrView*		pView;

	USHORT				nAnchorCtrls;

	virtual void		PageCreated( USHORT nId, SfxTabPage &rPage );

public:
	SvxTransformTabDialog( Window* pParent, const SfxItemSet* pAttr,
				const SdrView* pView,
				USHORT nAnchorTypes = 0);
			~SvxTransformTabDialog();
};

class SvxPositionTabPage : public SvxTabPage
{
private:
	FixedLine			aFlPosition;
	FixedText			aFtPosX;
	MetricField			aMtrPosX;
	FixedText			aFtPosY;
	MetricField			aMtrPosY;
	TriStateBox			aTsbProtect;
	FixedText			aFtReference;
	SvxRectCtl			aCtl;

	FixedLine			aAnchorBox;
	FixedText			aFtAnchor;
	ListBox				aDdLbAnchor;
	FixedText 			aFtOrient;
	ListBox				aDdLbOrient;
	const SfxItemSet&	rOutAttrs;

	const SdrView*		pView;
	Rectangle			aRect;
	Rectangle			aWorkArea;

	Point				aAnchorPos;
	SfxMapUnit			ePoolUnit;
	FieldUnit			eDlgUnit;
	MapUnit				eMapUnit;
	BOOL				bPageDisabled;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( ChangePosXHdl, void * );
	DECL_LINK( ChangePosYHdl, void * );
	DECL_LINK( ClickProtectHdl, void * );
	DECL_LINK( SetAnchorHdl, ListBox * );
	DECL_LINK( SetOrientHdl, ListBox * );

	void SetMinMaxPosition();
	void GetTopLeftPosition( INT32& rX, INT32& rY, const Rectangle& rRect );
#endif
public:
		 SvxPositionTabPage( Window* pParent, const SfxItemSet& rInAttrs  );

	static SfxTabPage* Create( Window*, const SfxItemSet& );
	static USHORT*	   GetRanges();

	virtual BOOL FillItemSet( SfxItemSet& );
	virtual void Reset( const SfxItemSet & );

	virtual void ActivatePage( const SfxItemSet& rSet );
	virtual int  DeactivatePage( SfxItemSet* pSet );

	virtual void PointChanged( Window* pWindow, RECT_POINT eRP );

	void 		 Construct();
	void		 SetView( const SdrView* pSdrView ) { pView = pSdrView; }

	void		 ShowAnchorCtrls(USHORT nAnchorCtrls); 
};

class SvxSizeTabPage : public SvxTabPage
{
private:
	FixedLine			aFlSize;
	FixedText			aFtWidth;
	MetricField			aMtrWidth;
	FixedText			aFtHeight;
	MetricField			aMtrHeight;

	CheckBox   			aCbxScale;
	TriStateBox			aTsbProtect;
	FixedText			aFtReference;
	SvxRectCtl			aCtl;

	FixedLine			aFlAdjust;
	TriStateBox         aTsbAutoGrowWidth;
	TriStateBox         aTsbAutoGrowHeight;

	const SfxItemSet&	rOutAttrs;

	const SdrView*		pView;
	Rectangle			aRect;
	UINT32				lOldWidth;
	UINT32				lOldHeight;
	RECT_POINT			eRP;

	Rectangle			aWorkArea;
	SfxMapUnit			ePoolUnit;
	FieldUnit			eDlgUnit;
	MapUnit				eMapUnit;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( ChangeWidthHdl, void * );
	DECL_LINK( ChangeHeightHdl, void * );
	DECL_LINK( ClickProtectHdl, void * );
	DECL_LINK( ClickAutoHdl, void * );

	void		SetMaxSize( Rectangle aRect );
	Rectangle	GetRect();
#endif
public:
		 SvxSizeTabPage( Window* pParent, const SfxItemSet& rInAttrs  );

	static SfxTabPage* Create( Window*, const SfxItemSet& );
	static USHORT*	   GetRanges();

	virtual BOOL FillItemSet( SfxItemSet& );
	virtual void Reset( const SfxItemSet & );

	virtual void ActivatePage( const SfxItemSet& rSet );
	virtual int  DeactivatePage( SfxItemSet* pSet );

	virtual void PointChanged( Window* pWindow, RECT_POINT eRP );

	void 		 Construct();
	void		 SetView( const SdrView* pSdrView ) { pView = pSdrView; }

	virtual void FillUserData();
};

class SvxAngleTabPage : public SvxTabPage
{
private:
	FixedLine			aFlPosition;
	FixedText			aFtPosX;
	MetricField			aMtrPosX;
	FixedText			aFtPosY;
	MetricField			aMtrPosY;
	FixedText			aFtPosPresets;
	SvxRectCtl			aCtlRect;

	FixedLine			aFlAngle;
	FixedText			aFtAngle;
	MetricField			aMtrAngle;
	FixedText			aFtAnglePresets;
	SvxRectCtl			aCtlAngle;

	const SfxItemSet&	rOutAttrs;

	const SdrView*		pView;
	Rectangle			aRect;

	Point				aAnchorPos;
	SfxMapUnit			ePoolUnit;
	FieldUnit			eDlgUnit;
	MapUnit				eMapUnit;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( ModifiedHdl, void * );
#endif
public:
		 SvxAngleTabPage( Window* pParent, const SfxItemSet& rInAttrs  );

	static SfxTabPage* Create( Window*, const SfxItemSet& );
	static USHORT*	   GetRanges();

	virtual BOOL FillItemSet( SfxItemSet& );
	virtual void Reset( const SfxItemSet & );

	virtual void ActivatePage( const SfxItemSet& rSet );
	virtual int  DeactivatePage( SfxItemSet* pSet );

	virtual void PointChanged( Window* pWindow, RECT_POINT eRP );

	void 		 Construct();
	void		 SetView( const SdrView* pSdrView ) { pView = pSdrView; }
};

class SvxSlantTabPage : public SvxTabPage
{
private:
	FixedLine			aFlRadius;
	FixedText			aFtRadius;
	MetricField			aMtrRadius;

	FixedLine			aFlAngle;
	FixedText			aFtAngle;
	MetricField			aMtrAngle;

	const SfxItemSet&	rOutAttrs;

	const SdrView*		pView;
	Rectangle			aRect;

	SfxMapUnit			ePoolUnit;
	FieldUnit			eDlgUnit;
	MapUnit				eMapUnit;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( ModifiedHdl, void * );
#endif
public:
		 SvxSlantTabPage( Window* pParent, const SfxItemSet& rInAttrs  );

	static SfxTabPage* Create( Window*, const SfxItemSet& );
	static USHORT*	   GetRanges();

	virtual BOOL FillItemSet( SfxItemSet& );
	virtual void Reset( const SfxItemSet & );

	virtual void ActivatePage( const SfxItemSet& rSet );
	virtual int  DeactivatePage( SfxItemSet* pSet );

	virtual void PointChanged( Window* pWindow, RECT_POINT eRP );

	void 		 Construct();
	void		 SetView( const SdrView* pSdrView ) { pView = pSdrView; }
};

#endif 

