
#ifndef _SVX_TSPTITEM_HXX
#define _SVX_TSPTITEM_HXX

#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif
#ifndef _SVX_SVXENUM_HXX 
#include <svx/svxenum.hxx>
#endif
#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#define SVX_TAB_DEFCOUNT    10
#define SVX_TAB_DEFDIST     1134            
#define SVX_TAB_NOTFOUND    USHRT_MAX
#define cDfltDecimalChar    (sal_Unicode(0x00)) 
#define cDfltFillChar       (sal_Unicode(' '))

class SvxTabStop
{
private:
	long            nTabPos;

	SvxTabAdjust    eAdjustment;
	sal_Unicode		cDecimal;
	sal_Unicode		cFill;

	friend SvStream& operator<<( SvStream&, SvxTabStop& );

public:
	SvxTabStop();
	SvxTabStop( const long nPos,
	const SvxTabAdjust eAdjst = SVX_TAB_ADJUST_LEFT,
	const sal_Unicode cDec = cDfltDecimalChar,
	const sal_Unicode cFil = cDfltFillChar );

	long&           GetTabPos() { return nTabPos; }
	long            GetTabPos() const { return nTabPos; }

	SvxTabAdjust&   GetAdjustment() { return eAdjustment; }
	SvxTabAdjust    GetAdjustment() const { return eAdjustment; }

	sal_Unicode&  GetDecimal() { return cDecimal; }
	sal_Unicode   GetDecimal() const { return cDecimal; }

	sal_Unicode&  GetFill() { return cFill; }
	sal_Unicode   GetFill() const { return cFill; }

	String			GetValueString() const;

	BOOL			IsEqual( const SvxTabStop& rTS ) const
			{
				return ( nTabPos     == rTS.nTabPos     &&
						 eAdjustment == rTS.eAdjustment &&
						 cDecimal    == rTS.cDecimal    &&
						 cFill       == rTS.cFill );
			}

	BOOL            operator==( const SvxTabStop& rTS ) const
			{ return nTabPos == rTS.nTabPos; }
	BOOL            operator <( const SvxTabStop& rTS ) const
			{ return nTabPos < rTS.nTabPos; }

	SvxTabStop&     operator=( const SvxTabStop& rTS )
			{
				nTabPos = rTS.nTabPos;
				eAdjustment = rTS.eAdjustment;
				cDecimal = rTS.cDecimal;
				cFill = rTS.cFill;
				return *this;
			}
};

SV_DECL_VARARR_SORT( SvxTabStopArr, SvxTabStop, SVX_TAB_DEFCOUNT, 1 )

#ifndef MAC
class SvxTabStopItem : public SfxPoolItem, private SvxTabStopArr
#else
class SvxTabStopItem : public SfxPoolItem, public SvxTabStopArr
#endif
{

public:
	TYPEINFO();

	SvxTabStopItem( USHORT nWhich = ITEMID_TABSTOP );
	SvxTabStopItem( const USHORT nTabs,
		const USHORT nDist,
		const SvxTabAdjust eAdjst = SVX_TAB_ADJUST_DEFAULT,
		USHORT nWhich = ITEMID_TABSTOP );
	SvxTabStopItem( const SvxTabStopItem& rTSI );

	USHORT          GetPos( const SvxTabStop& rTab ) const;

	USHORT          GetPos( const long nPos ) const;

	USHORT          Count() const { return SvxTabStopArr::Count(); }
	BOOL            Insert( const SvxTabStop& rTab );
	void            Insert( const SvxTabStopItem* pTabs, USHORT nStart = 0,
				USHORT nEnd = USHRT_MAX );
	void            Remove( SvxTabStop& rTab )
			{ SvxTabStopArr::Remove( rTab ); }
	void            Remove( const USHORT nPos, const USHORT nLen = 1 )
			{ SvxTabStopArr::Remove( nPos, nLen ); }

	SvxTabStopItem& operator=( const SvxTabStopItem& rTSI );

	int             operator!=( const SvxTabStopItem& rTSI ) const
			{ return !( operator==( rTSI ) ); }

	const SvxTabStop& operator[]( const USHORT nPos ) const
			{
				DBG_ASSERT( GetStart() &&
							nPos < Count(), "op[]" );
				return *( GetStart() + nPos );
			}
	const SvxTabStop*  GetStart() const
			{   return SvxTabStopArr::GetData(); }

	virtual int 			 operator==( const SfxPoolItem& ) const;
	virtual	sal_Bool        	 QueryValue( com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 ) const;
	virtual	sal_Bool			 PutValue( const com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 );

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;
	virtual SfxPoolItem*	 Create( SvStream&, USHORT ) const;
	virtual SvStream&		 Store( SvStream& , USHORT nItemVersion ) const;
};

#endif

