

#ifndef _E3D_VIEW3D_HXX
#define _E3D_VIEW3D_HXX

#include <svx/svdview.hxx>

#ifndef _SVX_DEF3D_HXX
#include <svx/def3d.hxx>
#endif

#ifndef _E3D_DEFLT3D_HXX
#include <svx/deflt3d.hxx>
#endif

#ifndef _SVX_VECTOR3D_HXX
#include <goodies/vector3d.hxx>
#endif

class E3dObject;
class E3dScene;
class SceneList;

class E3dView : public SdrView
{
protected:
	SdrObject**					pMarkedObjs;
	XPolyPolygon*				pMirrorPolygon;
	XPolyPolygon*				pMirroredPolygon;
	SdrPageView*				pMyPV;

	E3dDefaultAttributes		a3DDefaultAttr;
	MouseEvent					aMouseEvent;					
	Vector3D					aDefaultTranslation;            
	Vector3D					aDefaultLightPos;               

	Color						aDefaultLightColor;             
	Color						aDefaultAmbientColor;

	double						fDefaultScaleX;                 
	double						fDefaultScaleY;
	double						fDefaultScaleZ;
	double						fDefaultRotateX;                
	double						fDefaultRotateY;
	double						fDefaultRotateZ;
	double						fDefaultExtrusionDeepth;        

	double						fDefaultLightIntensity;         
	double						fDefaultAmbientIntensity;       
	long						nHDefaultSegments;              
	long						nVDefaultSegments;              
	long						nPolyCnt;

	E3dDragConstraint			eDragConstraint;
	E3dDragDetail				eDragDetail;

	BOOL						b3dCreationActive;
	BOOL						bDoubleSided;

	void InitView();

	void ImpCreate3DObject(E3dScene* pScene, SdrObject* pObj, BOOL bExtrude, double fDepth, Matrix4D& rLatheMat);
	void ImpCreateSingle3DObjectFlat(E3dScene* pScene, SdrObject* pObj, BOOL bExtrude, double fDepth, Matrix4D& rLatheMat);
	void ImpChangeSomeAttributesFor3DConversion(SdrObject* pObj);
	void ImpChangeSomeAttributesFor3DConversion2(SdrObject* pObj);

	void InitScene(E3dScene* pScene, double fW, double fH, double fCamZ);
	void ImpIsConvertTo3DPossible(SdrObject* pObj, BOOL& rAny3D, BOOL& rGroupSelected) const;
	void BreakSingle3DObj(E3dObject* pObj);

public:
	TYPEINFO();

	E3dView(SdrModel* pModel, OutputDevice* pOut);
	E3dView(SdrModel* pModel, ExtOutputDevice* pExtOut);
	E3dView(SdrModel* pModel);
	virtual ~E3dView();

	virtual void DrawMarkedObj(OutputDevice& rOut, const Point& rOfs) const;

	E3dDefaultAttributes& Get3DDefaultAttributes() { return a3DDefaultAttr; }
	virtual BOOL BegDragObj(const Point& rPnt, OutputDevice* pOut = NULL, SdrHdl* pHdl = NULL, short nMinMov = -3, SdrDragMethod* pForcedMeth = NULL);
	virtual	void CheckPossibilities();

	void SetMouseEvent(const MouseEvent& rNew) { aMouseEvent = rNew; }
	const MouseEvent& GetMouseEvent() { return aMouseEvent; }

	virtual SdrModel* GetMarkedObjModel() const;

	virtual BOOL Paste(const SdrModel& rMod, const Point& rPos, SdrObjList* pLst=NULL, UINT32 nOptions=0);

	BOOL ImpCloneAll3DObjectsToDestScene(E3dScene* pSrcScene, E3dScene* pDstScene, Point aOffset);

	BOOL HasMarkedScene();
	E3dScene* GetMarkedScene();

	BOOL IsConvertTo3DObjPossible() const;
	void ConvertMarkedObjTo3D(BOOL bExtrude=TRUE, Vector3D aPnt1 = Vector3D(), Vector3D aPnt2 = Vector3D(0.0, 1.0, 0.0));

	void DoDepthArrange(E3dScene* pScene, double fDepth);
	void ConvertMarkedToPolyObj(BOOL bLineToArea);
	void Set3DDragConstraint(E3dDragConstraint eConstr) { eDragConstraint = eConstr; }
	E3dDragConstraint Get3DDragConstraint() { return eDragConstraint; }
	void Set3DDragDetail(E3dDragDetail eDetail)	{ eDragDetail = eDetail; }
	E3dDragDetail Get3DDragDetail() { return eDragDetail; }
	void SetCurrent3DObj(E3dObject* p3DObj);
	void Start3DCreation();
	BOOL IsCreationActive() const { return b3dCreationActive; }
	virtual void MovAction(const Point& rPnt);
	void End3DCreation(BOOL bUseDefaultValuesForMirrorAxes=FALSE);
	void ResetCreationActive();
	void ShowMirrored();
	void CreateMirrorPolygons();
	void ShowMirrorPolygons(Point aMirrorPoint1, Point aMirrorPoint2);

	const Vector3D &DefaultTranslation () const
	{
		return aDefaultTranslation;
	}

	Vector3D &DefaultTranslation ()
	{
		return aDefaultTranslation;
	}

	const Vector3D &DefaultLightPos () const
	{
		return aDefaultLightPos;
	}

	Vector3D &DefaultLightPos ()
	{
		return aDefaultLightPos;
	}

	double GetDefaultCamPosZ();

	double &DefaultScaleX ()
	{
		return fDefaultScaleX;
	}

	double DefaultScaleX () const
	{
		return fDefaultScaleX;
	}

	double &DefaultScaleY ()
	{
		return fDefaultScaleY;
	}

	double DefaultScaleY () const
	{
		return fDefaultScaleY;
	}

	double &DefaultScaleZ ()
	{
		return fDefaultScaleZ;
	}

	double DefaultScaleZ () const
	{
		return fDefaultScaleZ;
	}

	double &DefaultRotateX ()
	{
		return fDefaultRotateX;
	}

	double DefaultRotateX () const
	{
		return fDefaultRotateX;
	}

	double &DefaultRotateY ()
	{
		return fDefaultRotateY;
	}

	double DefaultRotateY () const
	{
		return fDefaultRotateY;
	}

	double &DefaultRotateZ ()
	{
		return fDefaultRotateZ;
	}

	double DefaultRotateZ () const
	{
		return fDefaultRotateZ;
	}

	double &DefaultExtrusionDeepth ()
	{
		return fDefaultExtrusionDeepth;
	}

	double DefaultExtrusionDeepth () const
	{
		return fDefaultExtrusionDeepth;
	}

	double GetDefaultCamFocal();

	double &DefaultLightIntensity ()
	{
		return fDefaultLightIntensity;
	}

	double DefaultLightIntensity () const
	{
		return fDefaultLightIntensity;
	}

	double &DefaultAmbientIntensity ()
	{
		return fDefaultAmbientIntensity;
	}

	double DefaultAmbientIntensity () const
	{
		return fDefaultAmbientIntensity;
	}

	const Color &DefaultLightColor () const
	{
		return aDefaultLightColor;
	}

	Color DefaultLightColor ()
	{
		return aDefaultLightColor;
	}

	const Color &DefaultAmbientColor () const
	{
		return aDefaultAmbientColor;
	}

	Color DefaultAmbientColor ()
	{
		return aDefaultAmbientColor;
	}

	long GetHDefaultSegments() const { return nHDefaultSegments; }
	void SetHDefaultSegments(long nSegs) { nHDefaultSegments = nSegs; }

	long GetVDefaultSegments() const { return nVDefaultSegments; }
	void SetVDefaultSegments(long nSegs) { nVDefaultSegments = nSegs; }

	virtual void ShowDragObj(OutputDevice* pOut);
	virtual void HideDragObj(OutputDevice* pOut);
	virtual void DrawDragObj(OutputDevice* pOut, BOOL bFull) const;

	BOOL IsBreak3DObjPossible() const;
	void Break3DObj();

	BOOL DoubleSided () const
	{
		return bDoubleSided;
	}

	BOOL &DoubleSided ()
	{
		return bDoubleSided;
	}

	void MergeScenes();
	SfxItemSet Get3DAttributes(E3dScene* pInScene = NULL, BOOL bOnly3DAttr=FALSE) const;
	void Set3DAttributes(const SfxItemSet& rAttr, E3dScene* pInScene = NULL, BOOL bOnly3DAttr=FALSE);
};

#endif			
