

#ifndef _BTCPCON_HXX
#define _BTCPCON_HXX

#define USE_SOCKETS

#include <tools/tcpdef.hxx>

#ifndef _COMMUNI_HXX 
#include <tools/simplecm.hxx>
#endif

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif

class ClientContext

{
private:
	CommunicationLink *pCommunicationLink;

public:
	ClientContext( CommunicationLink *pLink ) { pCommunicationLink = pLink; }

	BOOL IsEqual( CommunicationLink *pLink ) { return pLink == pCommunicationLink; }
	CommunicationLink *GetCommunicationLink() { return pCommunicationLink; }

	ByteString GetPeerName() { return pCommunicationLink->GetCommunicationPartner( CM_DOTTED ); }

	ByteString GetPeerAddress() { return pCommunicationLink->GetCommunicationPartner( CM_FQDN ); }

};

class DataPack

{
public:
	void*	pDataBuffer;		
	ULONG	nDataSize;			
	ULONG	nDataType;			
	ClientContext*	pConId;		

	DataPack() {pDataBuffer = NULL; nDataSize = 0; nDataType = TCP_DATATYPE_NULL; pConId = NULL; }
	~DataPack() { if (pDataBuffer) delete pDataBuffer;}
};

DECLARE_LIST( ClientCtxList, ClientContext * );

class TcpConBase

{
protected:
	ByteString sTcpNr;
	ByteString sAddress;

	CommunicationManager *pComManager;

	BOOL bIsServer;
	BOOL bSynchronesEmpfangen;

	ClientCtxList aClientContextList;
	ClientContext *pClientContext;

	DECL_LINK( ConnectionOpenedHdl, CommunicationLink* pCL );
	DECL_LINK( ConnectionClosedHdl, CommunicationLink* pCL );
	DECL_LINK( DataReceivedHdl, CommunicationLink* pCL );

#define MaybeCallLink( Link, pData ) { if ( Link.IsSet() ) return (USHORT)Link.Call( pData ); else return 0; }

	Link aPacketRecievedLnk;
	Link aConnectionOpenedLnk;
	Link aConnectionClosedLnk;

	virtual USHORT PacketRecieved( DataPack* pData ) { MaybeCallLink( aPacketRecievedLnk, pData ) }

	virtual USHORT ConnectionOpened( ClientContext* pCtx ) { MaybeCallLink( aConnectionOpenedLnk, pCtx ) }
	virtual USHORT ConnectionClosed( ClientContext* pCtx ) { MaybeCallLink( aConnectionClosedLnk, pCtx ) }

	void LateInit();

public:
	TcpConBase( BOOL bDoSynchronesEmpfangen );

	virtual ~TcpConBase();

	void StartCommunication() { pComManager->StartCommunication(); }
	ClientContext *GetClientContext() { return pClientContext; }

	long int SendData( DataPack* pDPack );
	void ReceiveData( CommunicationLink* pCL );

	ByteString&	GetAddress() { sAddress = pComManager->GetMyName(CM_FQDN); return sAddress; }
	ByteString& GetTcpNr() { sTcpNr = pComManager->GetMyName(CM_DOTTED); return sTcpNr; }

	BOOL	IsServer(){ return bIsServer; };
	BOOL	ClientHasCon(){ return GetClientContext() != 0; }

	void 	SetStatusHdl( const Link &rLink ) { pComManager->SetInfoMsgHdl( rLink ); }

	void SetPacketRecievedHdl( Link aLink ) { aPacketRecievedLnk = aLink; }
	void SetConnectionOpenedHdl( Link aLink ) { aConnectionOpenedLnk = aLink; }
	void SetConnectionClosedHdl( Link aLink ) { aConnectionClosedLnk = aLink; }

	USHORT GetCommunicationLinkCount() { return pComManager->GetCommunicationLinkCount(); }
	CommunicationLinkRef GetCommunicationLink( USHORT nNr ) { return pComManager->GetCommunicationLink( nNr ); }

	void SetApplication( const ByteString& aApp ) { pComManager->SetApplication( aApp, TRUE ); }

};

#endif

