

#ifndef _DATETIME_HXX
#define _DATETIME_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _DATE_HXX
#include <tools/date.hxx>
#endif

#ifndef _TIME_HXX
#include <tools/time.hxx>
#endif

class DateTime : public Date, public Time
{
public:
	                DateTime() : Date(), Time() {}
	                DateTime( const DateTime& rDateTime ) :
	                    Date( rDateTime ), Time( rDateTime ) {}
	                DateTime( const Date& rDate ) : Date( rDate ), Time(0) {}
	                DateTime( const Time& rTime ) : Date(0), Time( rTime ) {}
	                DateTime( const Date& rDate, const Time& rTime ) :
	                    Date( rDate ), Time( rTime ) {}
	 BOOL            IsBetween( const DateTime& rFrom,
	                           const DateTime& rTo ) const;
	 BOOL            operator ==( const DateTime& rDateTime ) const
	                    { return (Date::operator==( rDateTime ) &&
	                              Time::operator==( rDateTime )); }
	BOOL            operator !=( const DateTime& rDateTime ) const
	                    { return (Date::operator!=( rDateTime ) ||
	                              Time::operator!=( rDateTime )); }
	BOOL            operator  >( const DateTime& rDateTime ) const;
	BOOL            operator  <( const DateTime& rDateTime ) const;
	BOOL            operator >=( const DateTime& rDateTime ) const;
	BOOL            operator <=( const DateTime& rDateTime ) const;
	 ULONG           GetSecFromDateTime( const Date& rDate ) const;
	void            MakeDateTimeFromSec( const Date& rDate, ULONG nSec );
	 void            ConvertToUTC()       { *this -= Time::GetUTCOffset(); }
	void            ConvertToLocalTime() { *this += Time::GetUTCOffset(); }
	 DateTime&       operator +=( long nDays )
	                    { Date::operator+=( nDays ); return *this; }
	DateTime&       operator -=( long nDays )
	                    { Date::operator-=( nDays ); return *this; }
	DateTime&		operator +=( double fTimeInDays );
	DateTime&		operator -=( double fTimeInDays )
			{ return operator+=( -fTimeInDays ); }
	DateTime&       operator +=( const Time& rTime );
	DateTime&       operator -=( const Time& rTime );
	 friend DateTime operator +( const DateTime& rDateTime, long nDays );
	friend DateTime operator -( const DateTime& rDateTime, long nDays );
	friend DateTime operator +( const DateTime& rDateTime, double fTimeInDays );
	friend DateTime operator -( const DateTime& rDateTime, double fTimeInDays )
			{ return operator+( rDateTime, -fTimeInDays ); }
	friend DateTime operator +( const DateTime& rDateTime, const Time& rTime );
	friend DateTime operator -( const DateTime& rDateTime, const Time& rTime );
	friend double	operator -( const DateTime& rDateTime1, const DateTime& rDateTime2 );
	friend long		operator -( const DateTime& rDateTime, const Date& rDate )
			{ return (const Date&) rDateTime - rDate; }
	 DateTime&       operator =( const DateTime& rDateTime );
};

inline DateTime& DateTime::operator =( const DateTime& rDateTime )
{
	Date::operator=( rDateTime );
	Time::operator=( rDateTime );
	return *this;
}

#endif 
