

#ifndef _FRACT_HXX
#define _FRACT_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class SvStream;

class Fraction
{
private:
	long            nNumerator;
	long            nDenominator;

public:
	                Fraction() { nNumerator = 0; nDenominator = 1; }
	                Fraction( const Fraction & rFrac );
	                Fraction( long nNum, long nDen=1 );
	                Fraction( long nN1, long nN2, long nD1, long nD2 );
	                Fraction( double dVal );
	 BOOL            IsValid() const;
	 long            GetNumerator() const { return nNumerator; }
	long            GetDenominator() const { return nDenominator; }
	 operator        long() const;
	operator        double() const;
	 Fraction&       operator=( const Fraction& rfrFrac );
	 Fraction&       operator+=( const Fraction& rfrFrac );
	Fraction&       operator-=( const Fraction& rfrFrac );
	Fraction&       operator*=( const Fraction& rfrFrac );
	Fraction&       operator/=( const Fraction& rfrFrac );
	 void            ReduceInaccurate( unsigned nSignificantBits );
#ifdef __BORLANDC__
	friend          Fraction operator+( const Fraction& rVal1, const Fraction& rVal2 );
	friend          Fraction operator-( const Fraction& rVal1, const Fraction& rVal2 );
	friend          Fraction operator*( const Fraction& rVal1, const Fraction& rVal2 );
	friend          Fraction operator/( const Fraction& rVal1, const Fraction& rVal2 );
	 friend          BOOL operator==( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator!=( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator< ( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator> ( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator<=( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator>=( const Fraction& rVal1, const Fraction& rVal2 );
#else
	friend inline   Fraction operator+( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   Fraction operator-( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   Fraction operator*( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   Fraction operator/( const Fraction& rVal1, const Fraction& rVal2 );
	 friend          BOOL operator==( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   BOOL operator!=( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator< ( const Fraction& rVal1, const Fraction& rVal2 );
	friend          BOOL operator> ( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   BOOL operator<=( const Fraction& rVal1, const Fraction& rVal2 );
	friend inline   BOOL operator>=( const Fraction& rVal1, const Fraction& rVal2 );
#endif
	friend SvStream& operator>>( SvStream& rIStream, Fraction& rFract );
	friend SvStream& operator<<( SvStream& rOStream, const Fraction& rFract );
};

inline Fraction::Fraction( const Fraction& rFrac )
{
	nNumerator   = rFrac.nNumerator;
	nDenominator = rFrac.nDenominator;
}

inline Fraction& Fraction::operator=( const Fraction& rFrac )
{
	nNumerator   = rFrac.nNumerator;
	nDenominator = rFrac.nDenominator;
	return *this;
}

inline BOOL Fraction::IsValid() const
{
	return (nDenominator > 0);
}

inline Fraction::operator long() const
{
	if ( nDenominator > 0 )
	    return (nNumerator / nDenominator);
	else
	    return 0;
}

inline Fraction operator+( const Fraction& rVal1, const Fraction& rVal2 )
{
	Fraction aErg( rVal1 );
	aErg += rVal2;
	return aErg;
}

inline Fraction operator-( const Fraction& rVal1, const Fraction& rVal2 )
{
	Fraction aErg( rVal1 );
	aErg -= rVal2;
	return aErg;
}

inline Fraction operator*( const Fraction& rVal1, const Fraction& rVal2 )
{
	Fraction aErg( rVal1 );
	aErg *= rVal2;
	return aErg;
}

inline Fraction operator/( const Fraction& rVal1, const Fraction& rVal2 )
{
	Fraction aErg( rVal1 );
	aErg /= rVal2;
	return aErg;
}

inline BOOL operator !=( const Fraction& rVal1, const Fraction& rVal2 )
{
	return !(rVal1 == rVal2);
}

inline BOOL operator <=( const Fraction& rVal1, const Fraction& rVal2 )
{
	return !(rVal1 > rVal2);
}

inline BOOL operator >=( const Fraction& rVal1, const Fraction& rVal2 )
{
	return !(rVal1 < rVal2);
}

#endif 
