

#ifndef _INFO_COMMUNICATION_HXX
#define _INFO_COMMUNICATION_HXX

#include <string.h>

#if SUPD == 358
#ifndef _TOOLERR_HXX
#include <tools/toolerr.hxx>
#endif
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#else
#include <tools/solar.h>
#include <tools/string.hxx>
#endif

#define INFODATA_MAX_INFO 0xFFFF
#define INFODATA_MAX_STR_LEN (INFODATA_MAX_INFO - 1 - (3*sizeof(SVBT16)) - sizeof(SVBT8) -(2*sizeof(BOOL)))

#define INFODATA_CHARARRAY_LEN (INFODATA_MAX_STR_LEN + 1)

class InfoDataPack
{
public:

  struct InfoPack {
	 SVBT16 nKeylen;
	 SVBT8 nCommand; 
	 BOOL bSubList, bLastPacket; 
	 SVBT16 nAktuellPacket, nGesamtPacket;
	 char sInfoString[INFODATA_CHARARRAY_LEN];
  };

  enum CommandType{ none,
	  info_request, info_request_list, info_request_whole_list,
	  info_result, info_result_whole_list,
	  info_reject, info_ak,
	  info_write, info_write_new,
	  command_req, command_ack, command_deny,
	  notify,
	  handshake,
			dependency_create, dependency_close,
			dependency_getdependinfo, dependency_touch,
			dependency_getdependlist, dependency_ack,
			dependency_getdata, dependency_reopen, dependency_deny,
			dependency_changetitle, dependency_delete };
public:

  InfoDataPack( );
  InfoDataPack( const char *pInfo, CommandType eCommand,
		USHORT nAktuell = 1, USHORT nGesamt = 1 );

  InfoDataPack( const char *pKey, const char *pValue, CommandType eCommand,
		BOOL bHasLink = FALSE, BOOL bLastPacket = TRUE,
		USHORT nAktuell = 1, USHORT nGesamt = 1 );
  InfoDataPack( const InfoPack &rInfoPack );

  InfoDataPack& operator =( const InfoPack &rInfoPack );
  InfoDataPack& operator =( const ByteString &rStr );
  InfoDataPack& operator =( const char *pChar );

  ULONG Size() const { return sizeof( InfoPack )            
	- INFODATA_CHARARRAY_LEN           
	+ strlen( aInfoPack.sInfoString )  
	+ 1; }                             
  ULONG InfoSize() const { return strlen( aInfoPack.sInfoString ); }

  const InfoPack &DataPack() const { return aInfoPack; }
  InfoPack *DataPackCopy() const { return (InfoPack *)(memcpy( new InfoPack, &aInfoPack, Size() )); }

  void PutPacketNum( USHORT nAktuell, USHORT nGesamt );
  void PutCommand( CommandType eCommand ) { ByteToSVBT8( BYTE(eCommand), aInfoPack.nCommand ); }
  void SetInfoStr( const char *pInfo );

  void SetKeyValueStr( const char *pKey, const char *pValue );
  void SetKeyValueStr( const ByteString &aKey, const ByteString &aValue )
		{ SetKeyValueStr( aKey.GetBuffer(), aValue.GetBuffer() ); }
  void SetHasLink( BOOL bHasLink ) { aInfoPack.bSubList = bHasLink; }
  void SetLastPacket( BOOL bLastPacket ) { aInfoPack.bLastPacket = bLastPacket; }

  CommandType GetPacketType() const { return CommandType( SVBT8ToByte( aInfoPack.nCommand)); }
  USHORT      GetCurrPaket() const { return SVBT16ToShort(aInfoPack.nAktuellPacket); }
  USHORT      GetMaxPaket() const { return SVBT16ToShort(aInfoPack.nGesamtPacket); }

  const char *GetKeyString() const { return sKey.GetBuffer(); } 
  const char *GetValueString() const { return pValue; } 
  const char *GetInfoString() const { return aInfoPack.sInfoString; }
  BOOL        HasSubList() const { return aInfoPack.bSubList; }
  BOOL        IsLastPaket() const { return aInfoPack.bLastPacket; }

private:
  void SetKeyLen( USHORT nLen ) { ShortToSVBT16( nLen, aInfoPack.nKeylen ); }
  USHORT GetKeyLen() { return SVBT16ToShort( aInfoPack.nKeylen ); }

  ByteString sKey;
  char *pValue;
  struct InfoPack aInfoPack;
};

#endif
