

#ifndef _LINK_HXX
#define _LINK_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

typedef long (*PSTUB)( void*, void* );

#define DECL_LINK( Method, ArgType ) \
	long Method( ArgType ); \
	static long LinkStub##Method( void* pThis, void* )

#define DECL_STATIC_LINK( Class, Method, ArgType ) \
	static long Method( Class*, ArgType )

#define IMPL_METHOD( Class, Method, ArgType, ArgName ) \
	long Class::Method( ArgType ArgName )

#define IMPL_STUB(Class, Method, ArgType) \
	long __EXPORT Class##::LinkStub##Method( void* pThis, void* pCaller) \
	{ \
	    return ((Class*)pThis )->Method( (ArgType)pCaller ); \
	}

#define IMPL_STATIC_LINK( Class, Method, ArgType, ArgName ) \
	long __EXPORT Class::Method( Class* pThis, ArgType ArgName )

#define LINK( Inst, Class, Member ) \
	Link( (Class*)Inst, (PSTUB)&Class##::LinkStub##Member )

#define STATIC_LINK( Inst, Class, Member ) \
	Link( (Class*)Inst, (PSTUB)&Class::Member )

#define IMPL_LINK( Class, Method, ArgType, ArgName ) \
	IMPL_STUB( Class, Method, ArgType ) \
	long Class::Method( ArgType ArgName )

#if defined GCC && defined NO_OPTIMIZE
#define IMPL_LINK_INLINE_START( Class, Method, ArgType, ArgName ) \
	IMPL_LINK( Class, Method, ArgType, ArgName )

#define IMPL_LINK_INLINE_END( Class, Method, ArgType, ArgName )
#else
#define IMPL_LINK_INLINE_START( Class, Method, ArgType, ArgName ) \
	inline long Class::Method( ArgType ArgName )

#define IMPL_LINK_INLINE_END( Class, Method, ArgType, ArgName ) \
	IMPL_STUB( Class, Method, ArgType )
#endif

#define IMPL_LINK_INLINE( Class, Method, ArgType, ArgName, Body ) \
	long Class::Method( ArgType ArgName ) \
	Body \
	IMPL_STUB( Class, Method, ArgType )

#define EMPTYARG

class Link
{
	void*       pInst;
	PSTUB       pFunc;

public:
	            Link();
	            Link( void* pLinkHdl, PSTUB pMemFunc );
	 long        Call( void* pCaller ) const;
	 BOOL        IsSet() const;
	BOOL        operator !() const;
	 BOOL        operator==( const Link& rLink ) const;
	BOOL        operator!=( const Link& rLink ) const
	                { return !(Link::operator==( rLink )); }
};

inline Link::Link()
{
	pFunc = 0;
}

inline Link::Link( void* pLinkHdl, PSTUB pMemFunc )
{
	pInst = pLinkHdl;
	pFunc = pMemFunc;
}

inline long Link::Call(void *pCaller) const
{
	return pFunc ? (*pFunc)(pInst, pCaller) : 0;
}

inline BOOL Link::IsSet() const
{
	if ( pFunc )
	    return TRUE;
	else
	    return FALSE;
}

inline BOOL Link::operator !() const
{
	if ( !pFunc )
	    return TRUE;
	else
	    return FALSE;
}

#endif  
