

#ifndef _PSTM_HXX
#define _PSTM_HXX

#ifndef _TABLE_HXX
#include <tools/table.hxx>
#endif

#ifndef _INQIDX_HXX
#include <tools/unqidx.hxx>
#endif

#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#define ERRCODE_IO_NOFACTORY ERRCODE_IO_WRONGFORMAT

class SvPersistBase;
typedef void * (*SvCreateInstancePersist)( SvPersistBase ** );
#define SV_CLASS_REGISTER( Class )                          \
	Register( Class::StaticClassId(), Class::CreateInstance )

class SvClassManager
{
	Table       aAssocTable;
public:
	void        Register( USHORT nClassId, SvCreateInstancePersist pFunc );
	SvCreateInstancePersist Get( USHORT nClassId );
};

class SvRttiBase : public SvRefBase
{
public:
	        TYPEINFO();
};
SV_DECL_IMPL_REF(SvRttiBase)

#define SV_DECL_PERSIST( Class, CLASS_ID )                          \
	TYPEINFO();                                                     \
	static  USHORT  StaticClassId() { return CLASS_ID; }            \
	static  void *  CreateInstance( SvPersistBase ** ppBase );      \
	friend SvPersistStream& operator >> ( SvPersistStream & rStm,   \
	                                      Class *& rpObj);          \
	virtual USHORT  GetClassId() const;                             \
	virtual void    Load( SvPersistStream & );                      \
	virtual void    Save( SvPersistStream & );

#define SV_DECL_PERSIST1( Class, Super1, CLASS_ID )                 \
	SV_DECL_PERSIST( Class, CLASS_ID )

#define PRV_SV_IMPL_PERSIST( Class )                                \
	void *          Class::CreateInstance( SvPersistBase ** ppBase )\
	                {                                               \
	                    Class * p = new Class();                    \
	                    *ppBase = p;                                \
	                    return p;                                   \
	                }                                               \
	USHORT          Class::GetClassId() const                       \
	                { return StaticClassId(); }                     \
	SvPersistStream& operator >> (SvPersistStream & rStm, Class *& rpObj)\
	                {                                               \
	                    SvPersistBase * pObj;                       \
	                    rStm >> pObj;                               \
	                    rpObj = PTR_CAST( Class, pObj );            \
	                    return rStm;                                \
	                }

#define SV_IMPL_PERSIST( Class )                                    \
	TYPEINIT0( Class )                                              \
	PRV_SV_IMPL_PERSIST( Class )

#define SV_IMPL_PERSIST1( Class, Super1 )                           \
	TYPEINIT1( Class, Super1 )                                      \
	PRV_SV_IMPL_PERSIST( Class )

class SvPersistStream;
class SvPersistBase : public SvRttiBase
{
public:
	virtual USHORT  GetClassId() const = 0;
	virtual void    Load( SvPersistStream & ) = 0;
	virtual void    Save( SvPersistStream & ) = 0;
	friend SvPersistStream& operator >> ( SvPersistStream & rStm,
	                                      SvPersistBase *& rpObj );
};
SV_DECL_IMPL_REF(SvPersistBase)

typedef SvPersistBase SuperSvPersistBase;
#define inline
SV_DECL_REF_LIST(SuperSvPersistBase,SuperSvPersistBase*)
#undef inline

class SvPersistBaseMemberList : public SuperSvPersistBaseMemberList
{
public:
			SvPersistBaseMemberList(){}
			SvPersistBaseMemberList(USHORT nInitSize, USHORT nResize )
	: SuperSvPersistBaseMemberList( nInitSize, nResize ){}
	 void   WriteObjects( SvPersistStream &, BOOL bOnlyStreamedObj = FALSE ) const;
	friend SvPersistStream& operator << (SvPersistStream &, const SvPersistBaseMemberList &);
	friend SvPersistStream& operator >> (SvPersistStream &, SvPersistBaseMemberList &);
};

#define SV_DECL_PERSIST_LIST(ClassName,EntryName)\
class ClassName##MemberList : public SvPersistBaseMemberList\
{\
public:\
	PRV_SV_DECL_MEMBER_LIST(ClassName,EntryName)\
};

#define SV_IMPL_PERSIST_LIST(ClassName,EntryName)\
	PRV_SV_IMPL_MEMBER_LIST(ClassName,EntryName,SvPersistBaseMemberList)

#define SV_DECL_IMPL_PERSIST_LIST(ClassName,EntryName)\
SV_DECL_PERSIST_LIST(ClassName,EntryName)\
SV_IMPL_PERSIST_LIST(ClassName,EntryName)

#if 0 // _SOLAR__PRIVATE
DECLARE_UNIQUEINDEX( SvPersistUIdx,SvPersistBase *)
#else
typedef UniqueIndex SvPersistUIdx;
#endif

class SvStream;
class SvPersistStream : public SvStream

{
	SvClassManager &        rClassMgr;
	SvStream *              pStm;
	Table                   aPTable; 
	SvPersistUIdx           aPUIdx;
	ULONG                   nStartIdx;
	const SvPersistStream * pRefStm;
	UINT32                  nFlags;
	 virtual ULONG       GetData( void* pData, ULONG nSize );
	virtual ULONG       PutData( const void* pData, ULONG nSize );
	virtual ULONG       SeekPos( ULONG nPos );
	virtual void        FlushData();
protected:
	ULONG               GetCurMaxIndex( const SvPersistUIdx & ) const;
	ULONG               GetCurMaxIndex() const
	                    { return GetCurMaxIndex( aPUIdx ); }
	 void				WriteObj( BYTE nHdr, SvPersistBase * pObj );
	UINT32  			ReadObj( SvPersistBase * & rpObj,
					BOOL bRegister );
public:
	BOOL                IsStreamed( SvPersistBase * pObj ) const
	                    { return 0 != GetIndex( pObj ); }
	virtual void        ResetError();
	                     SvPersistStream( SvClassManager &, SvStream * pStream,
	                                     UINT32 nStartIdx = 1 );
	                    SvPersistStream( SvClassManager &, SvStream * pStream,
	                                     const SvPersistStream & rPersStm );
	                    ~SvPersistStream();

	void                SetStream( SvStream * pStream );
	SvStream *			GetStream() const { return pStm; }
	virtual USHORT		IsA() const;
	 SvPersistBase *     GetObject( ULONG nIdx ) const;
	ULONG               GetIndex( SvPersistBase * ) const;
	 void                SetContextFlags( UINT32 n ) { nFlags = n; }
	UINT32              GetContextFlags() const { return nFlags; }
	 static void         WriteCompressed( SvStream & rStm, UINT32 nVal );
	static UINT32       ReadCompressed( SvStream & rStm );
	 UINT32              WriteDummyLen();
	void                WriteLen( UINT32 nLenPos );
	UINT32              ReadLen( UINT32 * pTestPos );
	 SvPersistStream&    WritePointer( SvPersistBase * pObj );
	SvPersistStream&    ReadPointer( SvPersistBase * & rpObj );
	friend SvPersistStream& operator << (SvPersistStream &, SvPersistBase *);
	friend SvPersistStream& operator >> (SvPersistStream &, SvPersistBase * &);
	 friend SvStream& operator >> ( SvStream &, SvPersistStream & );
	friend SvStream& operator << ( SvStream &, SvPersistStream & );
	ULONG   			InsertObj( SvPersistBase * );
	ULONG   			RemoveObj( SvPersistBase * );
};

#endif 
