

#ifndef _TOOLS_SOLMATH_HXX
#define _TOOLS_SOLMATH_HXX

#ifdef MAC

#include <mac_start.h>
#include <fp.h>
#include <mac_end.h>
#else
#include <float.h>
#endif

#ifdef SOLARIS
#include <ieeefp.h>
#endif

#include <math.h>

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#define SOMA_MIN_PRECISION	(1.0 / (16777216.0 * 16777216.0))

enum SolarMathRoundingMode
{
	SolarMathRoundCorrected = 0,	
	SolarMathRoundDown,		   		
	SolarMathRoundUp,				
	SolarMathRoundFloor,			
	SolarMathRoundCeiling,			
	SolarMathRoundHalfDown,			
	SolarMathRoundHalfUp,  			
	SolarMathRoundHalfEven			
};

class International;

class SolarMath
{
public:

	static	inline	BOOL	ApproxEqual( double a, double b );

	static	inline	double	ApproxAdd( double a, double b );

	static	inline	double	ApproxSub( double a, double b );

	static	inline	double	ApproxFloor( double a );

	static	inline	double	ApproxCeil( double a );

	static	inline	int		Finite( double d );

	static	inline	BOOL	IsINF( double d );

	static	inline	BOOL	IsNAN( double d );
	 static  inline  BOOL    IsIND( double d );

	static	inline	BOOL	IsSignBitSet( double d );

	static	inline	void	SetINF( double& rd, BOOL bNegative );

	static	inline	void	SetNAN( double& rd, BOOL bNegative );

	static	void			DoubleToString(
					XubString& sStr,
					double fVal,
					xub_Unicode cType,			
					int nDec,					
					xub_Unicode cDec = '.',		
					BOOL bEraseTrailingDecZero = FALSE
					);

	static	double			StringToDouble(
					const xub_Unicode* pStr,
					const International& rIntl,
					int& nErrno,
					const xub_Unicode** ppEnd = NULL
					);

	static	double			StringToDouble(
					const xub_Unicode* pStr,
					const xub_Unicode cGrpSep,	
					const xub_Unicode cDecSep,	
					int& nErrno,
					const xub_Unicode** ppEnd = NULL
					);

	static	double			Round(
					double fVal,
					short nDec = 0,
					SolarMathRoundingMode eMode = SolarMathRoundCorrected
					);
	 static  double          Pow10Exp( double fVal, int nExp );

};

#if 0
#define SOMA_FPSIGNAL_JUMP 1
#else
#define SOMA_FPSIGNAL_JUMP 0
#endif

#ifdef RS6000
#define SOMA_SIGFPE SIGTRAP
#else
#define SOMA_SIGFPE SIGFPE
#endif

#if defined(WNT) || defined(WIN)
#define SOMA_FPEXCEPTIONS_ON()	_control87( _MCW_EM, 0 )
#define SOMA_FPEXCEPTIONS_OFF()	_control87( _MCW_EM, _MCW_EM )
#define SOMA_FPRESET()			_fpreset()

#elif defined(OS2)
#define SOMA_FPEXCEPTIONS_ON()	_control87( MCW_EM, 0 )
#define SOMA_FPEXCEPTIONS_OFF()	_control87( MCW_EM, MCW_EM )
#define SOMA_FPRESET()			_fpreset()

#elif defined(RS6000)
#define SOMA_FPEXCEPTIONS_ON()	fp_enable_all()
#define SOMA_FPEXCEPTIONS_OFF()	fp_disable_all()
#define SOMA_FPRESET()

#elif defined(LINUX)
#include <fpu_control.h>

extern inline void glibc_setfpucw( fpu_control_t set )
{
	fpu_control_t cw;

	_FPU_GETCW(cw);

	cw &= _FPU_RESERVED;
	cw |= set & ~_FPU_RESERVED;

	_FPU_SETCW(cw);
}

#define __setfpucw( control_word ) glibc_setfpucw( control_word )

#if defined(POWERPC)

#define SOMA_FPEXCEPTIONS_ON()	__setfpucw( _FPU_DEFAULT | 0x000000F0 )
#define SOMA_FPEXCEPTIONS_OFF()	__setfpucw( _FPU_DEFAULT )
#else
#define SOMA_FPEXCEPTIONS_ON()	__setfpucw( _FPU_DEFAULT & ~0x001F )
#define SOMA_FPEXCEPTIONS_OFF()	__setfpucw( _FPU_IEEE )
#endif
#define SOMA_FPRESET()

#else
#define SOMA_FPEXCEPTIONS_ON()
#define SOMA_FPEXCEPTIONS_OFF()
#define SOMA_FPRESET()
#endif

#if SOMA_FPSIGNAL_JUMP
#define SOMA_FPCONTROL() SOMA_FPEXCEPTIONS_ON()
#else
#define SOMA_FPCONTROL() SOMA_FPEXCEPTIONS_OFF()
#endif

#if defined(WNT)
#define SOMA_FINITE(d) _finite(d)
#elif defined(MAC)
#define SOMA_FINITE(d) isfinite(d)
#elif defined(LINUX)
#define SOMA_FINITE(d) finite(d)
#elif defined(UNX)
#define SOMA_FINITE(d) finite(d)
#else

#define SOMA_FINITE(d) SolarMath::Finite(d)
#endif

#if defined(__IEEEDOUBLE) && (__SIZEOFDOUBLE == 8)

#if defined(__BIGENDIAN)

typedef union
{
	struct
	{
		UINT32 sign			: 1;
		UINT32 exponent		:11;
		UINT32 fraction_hi	:20;
		UINT32 fraction_lo	:32;
	} inf_parts;
	struct
	{
		UINT32 sign			: 1;
		UINT32 exponent		:11;
		UINT32 qnan_bit		: 1;
		UINT32 bits			:19;
		UINT32 fraction_lo	:32;
	} nan_parts;
	struct
	{
		UINT32 msw			:32;
		UINT32 lsw			:32;
	} w32_parts;
	double value;
} SolarMathDouble;

#elif defined(__LITTLEENDIAN)

typedef union
{
	struct {
		UINT32 fraction_lo	:32;
		UINT32 fraction_hi	:20;
		UINT32 exponent		:11;
		UINT32 sign			: 1;
	} inf_parts;
	struct {
		UINT32 fraction_lo	:32;
		UINT32 bits			:19;
		UINT32 qnan_bit		: 1;
		UINT32 exponent		:11;
		UINT32 sign			: 1;
	} nan_parts;
	struct
	{
		UINT32 lsw			:32;
		UINT32 msw			:32;
	} w32_parts;
	double value;
} SolarMathDouble;

#else
#error __FILE__ ": unsupported ENDIAN for SolarMath IEEE 754"
#endif

#else	
#error __FILE__ ": don't know how to handle SolarMath IEEE 754"
#endif

inline BOOL SolarMath::ApproxEqual( double a, double b )
{
	if ( a == b )
		return TRUE;
	double x = a - b;
	return (x < 0.0 ? -x : x)  <  ((a < 0.0 ? -a : a) * SOMA_MIN_PRECISION);
}

inline double SolarMath::ApproxAdd( double a, double b )
{
	if ( ((a < 0.0 && b > 0.0) || (b < 0.0 && a > 0.0)) && ApproxEqual( a, -b ) )
		return 0.0;
	return a + b;
}

inline double SolarMath::ApproxSub( double a, double b )
{
	if ( ((a < 0.0 && b < 0.0) || (a > 0.0 && b > 0.0)) && ApproxEqual( a, b ) )
		return 0.0;
	return a - b;
}

inline double SolarMath::ApproxFloor( double a )
{
	double b = floor( a );
	if ( ApproxEqual( a - 1.0, b ) )
		return b + 1.0;
	return b;
}

inline double SolarMath::ApproxCeil( double a )
{
	double b = ceil( a );
	if ( ApproxEqual( a + 1.0, b ) )
		return b - 1.0;
	return b;
}

inline int SolarMath::Finite( double d )
{	
	 return ((SolarMathDouble*)&d)->inf_parts.exponent != 0x7ff;
}

inline BOOL SolarMath::IsINF( double d )
{	
	return (SOMA_FINITE(d) == 0) &&
	    (((SolarMathDouble*)&d)->inf_parts.fraction_hi == 0)
	    && (((SolarMathDouble*)&d)->inf_parts.fraction_lo == 0);
}

inline BOOL SolarMath::IsNAN( double d )
{	
	return (SOMA_FINITE(d) == 0) && (
	    (((SolarMathDouble*)&d)->inf_parts.fraction_hi != 0)
	    || (((SolarMathDouble*)&d)->inf_parts.fraction_lo != 0) );
}

inline BOOL SolarMath::IsIND( double d )
{   
	return (((SolarMathDouble*)&d)->w32_parts.msw == 0xfff80000)
	    && (((SolarMathDouble*)&d)->w32_parts.lsw == 0);
}

inline BOOL SolarMath::IsSignBitSet( double d )
{
	return ((SolarMathDouble*)&d)->inf_parts.sign != 0;
}

inline void SolarMath::SetINF( double& rd, BOOL bNegative )
{
	((SolarMathDouble&)rd).w32_parts.msw = ( bNegative ? 0xfff00000 : 0x7ff00000 );
	((SolarMathDouble&)rd).w32_parts.lsw = 0;
}

inline void SolarMath::SetNAN( double& rd, BOOL bNegative )
{
	((SolarMathDouble&)rd).w32_parts.msw = ( bNegative ? 0xffffffff : 0x7fffffff );
	((SolarMathDouble&)rd).w32_parts.lsw = 0xffffffff ;
}

#endif	

