

#ifndef _TOOLS_TIME_HXX
#define _TOOLS_TIME_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class ResId;

class Time
{
private:
	long			nTime;

public:
		Time();
		Time( const ResId & rResId );
		Time( long _nTime ) { Time::nTime = _nTime; }
		Time( const Time& rTime )
			{ nTime = rTime.nTime; }
		Time( ULONG nHour, ULONG nMin,
			  ULONG nSec = 0, ULONG n100Sec = 0 );

	void			SetTime( long nNewTime ) { nTime = nNewTime; }
	long			GetTime() const { return nTime; }

	void			SetHour( USHORT nNewHour );
	void			SetMin( USHORT nNewMin );
	void			SetSec( USHORT nNewSec );
	void			Set100Sec( USHORT nNew100Sec );
	USHORT			GetHour() const
			{ ULONG nTempTime = (nTime >= 0) ? nTime : nTime*-1;
			  return (USHORT)(nTempTime / 1000000); }
	USHORT			GetMin() const
			{ ULONG nTempTime = (nTime >= 0) ? nTime : nTime*-1;
			  return (USHORT)((nTempTime / 10000) % 100); }
	USHORT			GetSec() const
			{ ULONG nTempTime = (nTime >= 0) ? nTime : nTime*-1;
			  return (USHORT)((nTempTime / 100) % 100); }
	USHORT			Get100Sec() const
			{ ULONG nTempTime = (nTime >= 0) ? nTime : nTime*-1;
			  return (USHORT)(nTempTime % 100); }

	long			GetMSFromTime() const;
	void			MakeTimeFromMS( long nMS );
	 double          GetTimeInDays() const;

	BOOL			IsBetween( const Time& rFrom, const Time& rTo ) const
			{ return ((nTime >= rFrom.nTime) && (nTime <= rTo.nTime)); }

	BOOL			operator ==( const Time& rTime ) const
			{ return (nTime == rTime.nTime); }
	BOOL			operator !=( const Time& rTime ) const
			{ return (nTime != rTime.nTime); }
	BOOL			operator  >( const Time& rTime ) const
			{ return (nTime > rTime.nTime); }
	BOOL			operator  <( const Time& rTime ) const
			{ return (nTime < rTime.nTime); }
	BOOL			operator >=( const Time& rTime ) const
			{ return (nTime >= rTime.nTime); }
	BOOL			operator <=( const Time& rTime ) const
			{ return (nTime <= rTime.nTime); }

	static Time 	GetUTCOffset();
	static ULONG	GetSystemTicks();		
	static ULONG	GetProcessTicks();		

	void			ConvertToUTC()		 { *this -= Time::GetUTCOffset(); }
	void			ConvertToLocalTime() { *this += Time::GetUTCOffset(); }

	Time&			operator =( const Time& rTime )
			{ nTime = rTime.nTime; return *this; }
	Time			operator -() const
			{ return Time( nTime * -1 ); }
	Time&			operator +=( const Time& rTime );
	Time&			operator -=( const Time& rTime );
	friend Time 	operator +( const Time& rTime1, const Time& rTime2 );
	friend Time 	operator -( const Time& rTime1, const Time& rTime2 );

	void setTime( UINT32 nTime ) { SetTime( nTime ); }
	UINT32 getTime() const { return GetTime(); }

#ifdef SMART_UNO_GENERATION
	UINT16			getHundredthSeconds() const { return Get100Sec(); }
	UINT16			getSeconds() const { return GetSec(); }
	UINT16			getMinutes() const { return GetMin(); }
	UINT16			getHours() const { return GetHour(); }

	void			setHundredthSeconds( UINT16 n ) { Set100Sec( n ); }
	void			setSeconds( UINT16 n ) { SetSec( n ); }
	void			setMinutes( UINT16 n ) { SetMin( n ); }
	void			setHours( UINT16 n ) { SetHour( n ); }
#endif
};

#endif 
