

#ifndef _SV_ACCEL_HXX
#define _SV_ACCEL_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_RESID_HXX
#include <vcl/resid.hxx>
#endif
#ifndef _SV_RC_HXX
#include <vcl/rc.hxx>
#endif
#ifndef _SV_KEYCOD_HXX
#include <vcl/keycod.hxx>
#endif

class ImplAccelData;
class ImplAccelEntry;

class Accelerator : public Resource
{
private:
	ImplAccelData*	mpData;
	XubString		maHelpStr;
	Link			maActivateHdl;
	Link			maDeactivateHdl;
	Link			maSelectHdl;

	KeyCode 		maCurKeyCode;
	USHORT			mnCurId;
	USHORT			mnCurRepeat;
	BOOL			mbIsCancel;
	BOOL*			mpDel;

#if 0 // _SOLAR__PRIVATE
#ifdef _SV_ACCEL_CXX
	void			ImplInit();
	void			ImplCopyData( ImplAccelData& rAccelData );
	void			ImplDeleteData();
	void			ImplInsertAccel( USHORT nItemId, const KeyCode& rKeyCode,
						 BOOL bEnable, Accelerator* pAutoAccel );
#endif

	friend class ImplAccelManager;
	ImplAccelEntry* ImplGetAccelData( const KeyCode& rKeyCode ) const;
#endif

protected:
	void			ImplLoadRes( const ResId& rResId );

public:
		Accelerator();
		Accelerator( const Accelerator& rAccel );
		Accelerator( const ResId& rResId );
		 ~Accelerator();

	virtual void	Activate();
	virtual void	Deactivate();
	virtual void	Select();

	void			InsertItem( USHORT nItemId, const KeyCode& rKeyCode );
	void			InsertItem( const ResId& rResId );
	void			RemoveItem( USHORT nItemId );
	void			RemoveItem( const KeyCode rKeyCode );
	void			Clear();

	USHORT			GetCurItemId() const { return mnCurId; }
	const KeyCode&	GetCurKeyCode() const { return maCurKeyCode; }
	USHORT			GetCurRepeat() const { return mnCurRepeat; }
	BOOL			IsCancel() const { return mbIsCancel; }

	USHORT			GetItemCount() const;
	USHORT			GetItemId( USHORT nPos ) const;
	KeyCode 		GetItemKeyCode( USHORT nPos ) const;
	USHORT			GetItemId( const KeyCode& rKeyCode ) const;
	KeyCode 		GetKeyCode( USHORT nItemId ) const;
	BOOL			IsIdValid( USHORT nItemId ) const;
	BOOL			IsKeyCodeValid( const KeyCode rKeyCode ) const;
	BOOL			Call( const KeyCode& rKeyCode, USHORT nRepeat = 0 );

	void			SetAccel( USHORT nItemId, Accelerator* pAccel );
	Accelerator*	GetAccel( USHORT nItemId ) const;
	void			SetAccel( const KeyCode rKeyCode, Accelerator* pAccel );
	Accelerator*	GetAccel( const KeyCode rKeyCode ) const;

	void			EnableItem( USHORT nItemId, BOOL bEnable = TRUE );
	BOOL			IsItemEnabled( USHORT nItemId ) const;
	void			EnableItem( const KeyCode rKeyCode, BOOL bEnable = TRUE );
	BOOL			IsItemEnabled( const KeyCode rKeyCode ) const;

	void			SetHelpText( const XubString& rHelpText ) { maHelpStr = rHelpText; }
	const XubString& GetHelpText() const { return maHelpStr; }

	void			SetActivateHdl( const Link& rLink ) { maActivateHdl = rLink; }
	const Link& 	GetActivateHdl() const { return maActivateHdl; }
	void			SetDeactivateHdl( const Link& rLink ) { maDeactivateHdl = rLink; }
	const Link& 	GetDeactivateHdl() const { return maDeactivateHdl; }
	void			SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link& 	GetSelectHdl() const { return maSelectHdl; }

	Accelerator&	operator=( const Accelerator& rAccel );
};

#endif	
