
#ifndef _VCL_FILEDLG_HXX
#define _VCL_FILEDLG_HXX

#include <vcl/dialog.hxx>

struct ControlChain_Impl;
class Control;

class VclFileDialog : public ModalDialog
{
private:
	ControlChain_Impl*			_pUserControls;

protected:

	Link						_aOKHdl;
	Link						_aFileSelectHdl;
	Link						_aFilterSelectHdl;

	String						_aPath;
	String						_aDefExt;

	void						ReleaseOwnerShip( Window* pUserControl );

public:
	VclFileDialog( Window* pParent, const ResId& rResId );
	~VclFileDialog();

	virtual void				FileSelect() = 0;
	virtual void				FilterSelect() = 0;
	virtual void				AddFilter( const String& rFilter,
							   const String& rType,ULONG nFlags = 0 ) = 0;
	virtual void				AddFilter( const String& rFilter, const String& rType,
							   const String& rSysType, ULONG nFlags = 0 ) = 0;
	virtual void				RemoveFilter( const String& rFilter ) = 0;
	virtual void				RemoveAllFilter() = 0;
	virtual void				SetCurFilter( const String& rFilter ) = 0;
	virtual String				GetCurFilter() const = 0;
	virtual USHORT				GetFilterCount() const = 0;
	virtual const String&		GetFilterName( USHORT nPos ) const = 0;
	virtual const String&		GetFilterType( USHORT nPos ) const = 0;
	virtual ULONG				GetFilterFlags( USHORT nPos ) const = 0;

#if (SUPD < 540)
	virtual void
#else
	virtual BOOL
#endif
					AddControl( Window* pControl, BOOL bNewLine = FALSE );

	inline void					SetPath( const String& rNewURL );
	inline const String&		GetPath() const;
	inline void					SetDefaultExt( const String& rExt );
	inline const String&		GetDefaultExt() const;
	inline void					SetOKHdl( const Link& rLink );
	inline const Link&			GetOKHdl() const;
	inline void					SetFileSelectHdl( const Link& rLink );
	inline const Link&			GetFileSelectHdl() const;
	inline void					SetFilterSelectHdl( const Link& rLink );
	inline const Link&			GetFilterSelectHdl() const;
};

inline void VclFileDialog::SetPath( const String& rNewURL )
{
	_aPath = rNewURL;
}

inline const String& VclFileDialog::GetPath() const
{
	return _aPath;
}

inline void VclFileDialog::SetDefaultExt( const String& rExt )
{
	_aDefExt = rExt;
}

inline const String& VclFileDialog::GetDefaultExt() const
{
	return _aDefExt;
}

inline void VclFileDialog::SetOKHdl
(
	const Link& rLink
)
{
	_aOKHdl = rLink;
}

inline const Link& VclFileDialog::GetOKHdl() const
{
	return _aOKHdl;
}

inline void VclFileDialog::SetFileSelectHdl
(
	const Link& rLink
)
{
	_aFileSelectHdl = rLink;
}

inline const Link& VclFileDialog::GetFileSelectHdl() const
{
	return _aFileSelectHdl;
}

inline void VclFileDialog::SetFilterSelectHdl
(
	const Link& rLink
)
{
	_aFilterSelectHdl = rLink;
}

inline const Link& VclFileDialog::GetFilterSelectHdl() const
{
	return _aFilterSelectHdl;
}

#endif

