

#ifndef NOOLDSV
#ifndef _SV_GDIOBJ_HXX
#define _SV_GDIOBJ_HXX

#ifndef _SV_COLOR_HXX
#include <vcl/color.hxx>
#endif

enum PenStyle { PEN_NULL, PEN_SOLID, PEN_DOT, PEN_DASH, PEN_DASHDOT };

class Pen
{
	Color			aColor;
	PenStyle		eStyle;
	ULONG			nWidth;

public:
		Pen() { eStyle = PEN_SOLID; nWidth = 0; }
		Pen( PenStyle _eStyle ) { eStyle = _eStyle; nWidth = 0; }
		Pen( const Color& rColor, ULONG _nWidth = 0,
			PenStyle _eStyle = PEN_SOLID ) :
			aColor( rColor )
			{ eStyle = _eStyle; nWidth = _nWidth; }

	void			SetColor( const Color& rColor ) { aColor = rColor; }
	const Color&	GetColor() const { return aColor; }

	void			SetWidth( ULONG _nWidth ) { nWidth = _nWidth; }
	ULONG			GetWidth() const { return nWidth; }

	void			SetStyle( PenStyle _eStyle ) { eStyle = _eStyle; }
	PenStyle		GetStyle() const { return eStyle; }

	BOOL			operator==( const Pen& rPen ) const
			{ return ((eStyle == rPen.eStyle) && (nWidth == rPen.nWidth) && (aColor == rPen.aColor)); }
	BOOL			operator!=( const Pen& rPen ) const
			{ return !(Pen::operator==( rPen )); }
	BOOL			IsSameInstance( const Pen& rPen ) const
			{ return (this == &rPen); }
};

#define BRUSH_SIZE		8

enum BrushStyle { BRUSH_NULL, BRUSH_SOLID, BRUSH_HORZ, BRUSH_VERT,
	  BRUSH_CROSS, BRUSH_DIAGCROSS, BRUSH_UPDIAG, BRUSH_DOWNDIAG,
	  BRUSH_25, BRUSH_50, BRUSH_75,
	  BRUSH_BITMAP };

class Brush
{
private:
	Color			aColor;
	Color			aFillColor;
	BrushStyle		eStyle;
	BOOL			bTransparent;

public:
		Brush() : aColor( COL_WHITE )
			{  eStyle = BRUSH_SOLID; bTransparent = FALSE; }
		Brush( BrushStyle _eStyle ) : aColor( COL_WHITE )
			{  eStyle = _eStyle; bTransparent = FALSE; }
		Brush( const Color& rColor,
			BrushStyle _eStyle = BRUSH_SOLID ) :
			aColor( rColor )
			{  eStyle = _eStyle; bTransparent = TRUE; }
		Brush( const Color& rColor, const Color& rFillColor,
			BrushStyle _eStyle ) :
			aColor( rColor ),
			aFillColor( rFillColor )
			{  eStyle = _eStyle; bTransparent = FALSE; }

	void			SetColor( const Color& rColor ) { aColor = rColor; }
	const Color&	GetColor() const { return aColor; }
	void			SetFillColor( const Color& rColor ) { aFillColor = rColor; }
	const Color&	GetFillColor() const { return aFillColor; }

	void			SetStyle( BrushStyle _eStyle ) { eStyle = _eStyle; }
	BrushStyle		GetStyle() const { return eStyle; }

	void			SetTransparent( BOOL _bTransparent ) { bTransparent = _bTransparent; }
	BOOL			GetTransparent() const { return bTransparent; }
	BOOL			IsTransparent() const;
	BOOL			IsHatched() const;

	Color			GetVCLFillColor() const;

	BOOL			operator==( const Brush& rBrush ) const
			{ return ((eStyle == rBrush.eStyle) && (aColor == rBrush.aColor)); }
	BOOL			operator!=( const Brush& rBrush ) const
			{ return !(Brush::operator==( rBrush )); }
	BOOL			IsSameInstance( const Brush& rBrush ) const
			{ return (this == &rBrush); }
};

inline BOOL Brush::IsTransparent() const
{
	if ( eStyle == BRUSH_SOLID )
		return FALSE;
	else
	{
		if ( eStyle == BRUSH_NULL )
			return TRUE;
		else
			return bTransparent;
	}
}

inline BOOL Brush::IsHatched() const
{
	if ( (eStyle != BRUSH_SOLID)  &&
		 (eStyle != BRUSH_BITMAP) &&
		 (eStyle != BRUSH_NULL) )
		return TRUE;
	else
		return FALSE;
}

#endif 
#endif
