

#ifndef _SV_GRADIENT_HXX
#define _SV_GRADIENT_HXX

#include <vcl/color.hxx>
#include <vcl/vclenum.hxx>

class SvStream;

class Impl_Gradient
{
public:
	USHORT          mnRefCount;
	GradientStyle   meStyle;
	Color           maStartColor;
	Color           maEndColor;
	USHORT          mnAngle;
	USHORT          mnBorder;
	USHORT          mnOfsX;
	USHORT          mnOfsY;
	USHORT          mnIntensityStart;
	USHORT          mnIntensityEnd;
	USHORT          mnStepCount;
	 friend SvStream& operator>>( SvStream& rIStm, Impl_Gradient& rImplGradient );
	friend SvStream& operator<<( SvStream& rOStm, const Impl_Gradient& rImplGradient );
	                 Impl_Gradient();
	                Impl_Gradient( const Impl_Gradient& rImplGradient );
};

class Gradient
{
private:	
	Impl_Gradient*	mpImplGradient;
	void			MakeUnique();

public:
	                Gradient();
	                Gradient( const Gradient& rGradient );
	                Gradient( GradientStyle eStyle );
	                Gradient( GradientStyle eStyle,
	                          const Color& rStartColor,
	                          const Color& rEndColor );
	                ~Gradient();
	 void            SetStyle( GradientStyle eStyle );
	GradientStyle   GetStyle() const { return mpImplGradient->meStyle; }
	 void            SetStartColor( const Color& rColor );
	const Color&    GetStartColor() const { return mpImplGradient->maStartColor; }
	void            SetEndColor( const Color& rColor );
	const Color&    GetEndColor() const { return mpImplGradient->maEndColor; }
	 void            SetAngle( USHORT nAngle );
	USHORT          GetAngle() const { return mpImplGradient->mnAngle; }
	 void            SetBorder( USHORT nBorder );
	USHORT          GetBorder() const { return mpImplGradient->mnBorder; }
	void            SetOfsX( USHORT nOfsX );
	USHORT          GetOfsX() const { return mpImplGradient->mnOfsX; }
	void            SetOfsY( USHORT nOfsY );
	USHORT          GetOfsY() const { return mpImplGradient->mnOfsY; }
	 void            SetStartIntensity( USHORT nIntens );
	USHORT          GetStartIntensity() const { return mpImplGradient->mnIntensityStart; }
	void            SetEndIntensity( USHORT nIntens );
	USHORT          GetEndIntensity() const { return mpImplGradient->mnIntensityEnd; }
	 void            SetSteps( USHORT nSteps );
	USHORT          GetSteps() const { return mpImplGradient->mnStepCount; }
	 Gradient&       operator=( const Gradient& rGradient );
	BOOL            operator==( const Gradient& rGradient ) const;
	BOOL            operator!=( const Gradient& rGradient ) const
	                    { return !(Gradient::operator==( rGradient )); }
	BOOL            IsSameInstance( const Gradient& rGradient ) const
	                    { return (mpImplGradient == rGradient.mpImplGradient); }
	 friend SvStream& operator>>( SvStream& rIStm, Gradient& rGradient );
	friend SvStream& operator<<( SvStream& rOStm, const Gradient& rGradient );

#ifdef SMART_UNO_GENERATION
	GradientStyle	getStyle() const				{ return GetStyle(); }
	void 			setStyle( GradientStyle e ) 	{ SetStyle( e ); }
	long 			getStartColor() const			{ return GetStartColor().GetColor(); }
	void 			setStartColor( long n ) 		{ SetStartColor( Color( n ) ); }
	long 			getEndColor() const				{ return GetEndColor().GetColor(); }
	void 			setEndColor( long n ) 			{ SetEndColor( Color( n ) ); }
	USHORT 			getAngle() const				{ return GetAngle(); }
	void 			setAngle( USHORT n )			{ SetAngle( n ); }
	USHORT			getBorder() const				{ return GetBorder(); }
	void 			setBorder( USHORT n )			{ SetBorder( n ); }
	USHORT			getXOffset() const				{ return GetOfsX(); }
	void 			setXOffset( USHORT n )			{ SetOfsX( n ); }
	USHORT			getYOffset() const				{ return GetOfsY(); }
	void 			setYOffset( USHORT n )			{ SetOfsY( n ); }
	USHORT			getStartIntensity() const		{ return GetStartIntensity(); }
	void 			setStartIntensity( USHORT n )	{ SetStartIntensity( n ); }
	USHORT			getEndIntensity() const			{ return GetEndIntensity(); }
	void 			setEndIntensity( USHORT n )		{ SetEndIntensity( n ); }
	USHORT			getStepCount() const			{ return GetSteps(); }
	void 			setStepCount( USHORT n )		{ SetSteps( n ); }
#endif
};

#endif  
