

#ifndef _SV_GRAPH_HXX
#define _SV_GRAPH_HXX

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_BITMAP_HXX
#include <vcl/bitmap.hxx>
#endif
#ifndef _SV_BITMAPEX_HXX
#include <vcl/bitmapex.hxx>
#endif
#ifndef _SV_ANIMATE_HXX
#include <vcl/animate.hxx>
#endif
#ifndef _SV_GDIMTF_HXX
#include <vcl/gdimtf.hxx>
#endif
#ifndef _SV_GRAPH_H
#include <vcl/graph.h>
#endif
#ifndef _GFXLINK_HXX
#include <vcl/gfxlink.hxx>
#endif

class   ImpGraphic;
class   OutputDevice;
class   Font;
class   GfxLink;

class Graphic : public SvDataCopyStream
{
private:
	 ImpGraphic*         mpImpGraphic;

#if 0 // _SOLAR__PRIVATE

public:
	 void                ImplTestRefCount();
	ImpGraphic*         ImplGetImpGraphic() const { return mpImpGraphic; }

#endif

public:
	                     TYPEINFO();
	                     Graphic();
	                    Graphic( const Graphic& rGraphic );
	                    Graphic( const Bitmap& rBmp );
	                    Graphic( const BitmapEx& rBmpEx );
	                    Graphic( const Animation& rAnimation );
	                    Graphic( const GDIMetaFile& rMtf );
	virtual             ~Graphic();
	 Graphic&            operator=( const Graphic& rGraphic );
	BOOL                operator==( const Graphic& rGraphic ) const;
	BOOL                operator!=( const Graphic& rGraphic ) const;
	BOOL                operator!() const;
	 void                Clear();
	 GraphicType         GetType() const;
	void                SetDefaultType();
	BOOL                IsSupportedGraphic() const;
	 BOOL                IsTransparent() const;
	BOOL                IsAlpha() const;
	BOOL                IsAnimated() const;
	 Bitmap              GetBitmap() const;
	BitmapEx            GetBitmapEx() const;
	Animation           GetAnimation() const;
	const GDIMetaFile&  GetGDIMetaFile() const;
	 Size                GetPrefSize() const;
	void                SetPrefSize( const Size& rPrefSize );
	 MapMode             GetPrefMapMode() const;
	void                SetPrefMapMode( const MapMode& rPrefMapMode );
	 ULONG               GetSizeBytes() const;
	 void                Draw( OutputDevice* pOutDev,
	                          const Point& rDestPt ) const;
	void                Draw( OutputDevice* pOutDev,
	                          const Point& rDestPt,
	                          const Size& rDestSize ) const;
	static void         Draw( OutputDevice* pOutDev, const String& rText,
	                          Font& rFont, const Bitmap& rBitmap,
	                          const Point& rDestPt, const Size& rDestSize );
	 void                StartAnimation( OutputDevice* pOutDev,
	                                    const Point& rDestPt,
	                                    long nExtraData = 0L,
	                                    OutputDevice* pFirstFrameOutDev = NULL );
	void                StartAnimation( OutputDevice* pOutDev,
	                                    const Point& rDestPt,
	                                    const Size& rDestSize,
	                                    long nExtraData = 0L,
	                                    OutputDevice* pFirstFrameOutDev = NULL );
	void                StopAnimation( OutputDevice* pOutputDevice = NULL,
	                                   long nExtraData = 0L );
	 void                SetAnimationNotifyHdl( const Link& rLink );
	Link                GetAnimationNotifyHdl() const;
	 ULONG               GetAnimationLoopCount() const;
	void                ResetAnimationLoopCount();
	 List*               GetAnimationInfoList() const;
	 ULONG               GetChecksum() const;

public:
	 GraphicReader*      GetContext();
	void                SetContext( GraphicReader* pReader );

public:
	 static USHORT       GetGraphicsCompressMode( SvStream& rIStm );
	 void                SetDocFileName( const String& rName, ULONG nFilePos );
	const String&       GetDocFileName() const;
	ULONG               GetDocFilePos() const;
	 BOOL                ReadEmbedded( SvStream& rIStream, BOOL bSwap = FALSE );
	BOOL                WriteEmbedded( SvStream& rOStream );
	 BOOL                SwapOut();
	BOOL                SwapOut( SvStream* pOStm );
	BOOL                SwapIn();
	BOOL                SwapIn( SvStream* pIStm );
	BOOL                IsSwapOut() const;
	 void                SetLink( const GfxLink& );
	GfxLink             GetLink();
	BOOL                IsLink() const;
	 friend SvStream&    operator<<( SvStream& rOStream, const Graphic& rGraphic );
	friend SvStream&    operator>>( SvStream& rIStream, Graphic& rGraphic );

public:
	 virtual void        Load( SvStream& );
	virtual void        Save( SvStream& );
	virtual void        Assign( const SvDataCopyStream& );
};

#endif  

