

#ifndef _SV_SLIDER_HXX
#define _SV_SLIDER_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif

#ifndef _SV_SCRBAR_HXX
#include <vcl/scrbar.hxx>
#endif

class Slider : public Control
{
private:
	Rectangle		maChannel1Rect;
	Rectangle		maChannel2Rect;
	Rectangle		maThumbRect;
	long			mnStartPos;
	long			mnMouseOff;
	long			mnThumbPixOffset;
	long			mnThumbPixRange;
	long			mnThumbPixPos;
	long			mnChannelPixOffset;
	long			mnChannelPixRange;
	long			mnChannelPixTop;
	long			mnChannelPixBottom;
	long			mnMinRange;
	long			mnMaxRange;
	long			mnThumbPos;
	long			mnLineSize;
	long			mnPageSize;
	long			mnDelta;
	USHORT			mnDragDraw;
	USHORT			mnStateFlags;
	ScrollType		meScrollType;
	BOOL			mbCalcSize;
	BOOL			mbFullDrag;
	Link			maSlideHdl;
	Link			maEndSlideHdl;

#ifdef _SV_SLIDER_CXX
	void			ImplInit( Window* pParent, WinBits nStyle );
	void			ImplInitStyle( WinBits nStyle );
	void			ImplLoadRes( const ResId& rResId );
	void			ImplInitSettings();
	void			ImplUpdateRects( BOOL bUpdate = TRUE );
	long			ImplCalcThumbPos( long nPixPos );
	long			ImplCalcThumbPosPix( long nPos );
	void			ImplCalc( BOOL bUpdate = TRUE );
	void			ImplDraw( USHORT nDrawFlags );
	BOOL			ImplIsPageUp( const Point& rPos );
	BOOL			ImplIsPageDown( const Point& rPos );
	long			ImplSlide( long nNewPos, BOOL bCallEndSlide );
	long			ImplDoAction( BOOL bCallEndSlide );
	void			ImplDoMouseAction( const Point& rPos, BOOL bCallAction = TRUE );
	long			ImplDoSlide( long nNewPos );
	long			ImplDoSlideAction( ScrollType eScrollType );
#endif

public:
		Slider( Window* pParent, WinBits nStyle = WB_HORZ );
		Slider( Window* pParent, const ResId& rResId );

	virtual void	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void	Tracking( const TrackingEvent& rTEvt );
	virtual void	KeyInput( const KeyEvent& rKEvt );
	virtual void	Paint( const Rectangle& rRect );
	virtual void	Resize();
	virtual void	RequestHelp( const HelpEvent& rHEvt );
	virtual void	StateChanged( StateChangedType nType );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

	virtual void	Slide();
	virtual void	EndSlide();

	void			EnableDrag( BOOL bEnable = TRUE )
			{ mbFullDrag = bEnable; }
	BOOL			IsDragEnabled() const { return mbFullDrag; }

	void			SetRangeMin( long nNewRange );
	long			GetRangeMin() const { return mnMinRange; }
	void			SetRangeMax( long nNewRange );
	long			GetRangeMax() const { return mnMaxRange; }
	void			SetRange( const Range& rRange );
	Range			GetRange() const { return Range( GetRangeMin(), GetRangeMax() ); }
	void			SetThumbPos( long nThumbPos );
	long			GetThumbPos() const { return mnThumbPos; }
	void			SetLineSize( long nNewSize ) { mnLineSize = nNewSize; }
	long			GetLineSize() const { return mnLineSize; }
	void			SetPageSize( long nNewSize ) { mnPageSize = nNewSize; }
	long			GetPageSize() const { return mnPageSize; }

	long			GetDelta() const { return mnDelta; }

	Size			CalcWindowSizePixel();

	void			SetSlideHdl( const Link& rLink ) { maSlideHdl = rLink; }
	const Link& 	GetSlideHdl() const { return maSlideHdl;	}
	void			SetEndSlideHdl( const Link& rLink ) { maEndSlideHdl = rLink; }
	const Link& 	GetEndSlideHdl() const { return maEndSlideHdl; }
};

#endif 
