

#ifndef _SV_TABCTRL_HXX
#define _SV_TABCTRL_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif

struct ImplTabItem;
struct ImplTabCtrlData;
class ImplTabItemList;
class TabPage;
class PushButton;

#define TAB_APPEND          ((USHORT)0xFFFF)
#define TAB_PAGE_NOTFOUND   ((USHORT)0xFFFF)

class TabControl : public Control
{
private:
	ImplTabItemList*    mpItemList;
	ImplTabCtrlData*    mpTabCtrlData;
	long                mnLastWidth;
	long                mnLastHeight;
	long                mnBtnSize;
	long                mnMaxPageWidth;
	USHORT              mnActPageId;
	USHORT              mnCurPageId;
	USHORT              mnFirstPagePos;
	USHORT              mnLastFirstPagePos;
	BOOL                mbFormat;
	BOOL                mbRestoreHelpId;
	BOOL                mbRestoreUnqId;
	BOOL                mbSingleLine;
	BOOL                mbScroll;
	BOOL                mbColored;
	BOOL                mbSmallInvalidate;
	BOOL                mbDummy2;
	Link                maActivateHdl;
	Link                maDeactivateHdl;

#ifdef _SV_TABCTRL_CXX
private:
	void                ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	ImplTabItem*        ImplGetItem( USHORT nId ) const;
	void                ImplScrollBtnsColor();
	void                ImplSetScrollBtnsState();
	void                ImplPosScrollBtns();
	Size                ImplGetItemSize( ImplTabItem* pItem, long nMaxWidth ) const;
	Rectangle           ImplGetTabRect( USHORT nPos, long nWidth = -1, long nHeight = -1 );
	void                ImplChangeTabPage( USHORT nId, USHORT nOldId );
	BOOL                ImplPosCurTabPage();
	void                ImplActivateTabPage( BOOL bNext );
	void                ImplSetFirstPagePos( USHORT nPagePos );
	void                ImplShowFocus();
	void                ImplDrawItem( ImplTabItem* pItem, const Rectangle& rCurRect );
	                    DECL_LINK( ImplScrollBtnHdl, PushButton* pBtn );
#endif
protected:
	void                ImplInit( Window* pParent, WinBits nStyle );
	void                ImplLoadRes( const ResId& rResId );

public:
	                    TabControl( Window* pParent,
	                                WinBits nStyle = WB_STDTABCONTROL );
	                    TabControl( Window* pParent, const ResId& rResId );
	                    ~TabControl();
	 virtual void        MouseButtonDown( const MouseEvent& rMEvt );
	virtual void        KeyInput( const KeyEvent& rKEvt );
	virtual void        Paint( const Rectangle& rRect );
	virtual void        Resize();
	virtual void        GetFocus();
	virtual void        LoseFocus();
	virtual void        RequestHelp( const HelpEvent& rHEvt );
	virtual void        Command( const CommandEvent& rCEvt );
	virtual long        Notify( NotifyEvent& rNEvt );
	virtual void        StateChanged( StateChangedType nType );
	virtual void        DataChanged( const DataChangedEvent& rDCEvt );
	 virtual void        ActivatePage();
	virtual long        DeactivatePage();
	 void                SetTabPageSizePixel( const Size& rSize );
	Size                GetTabPageSizePixel() const;

#ifndef NOOLDSV
	void                SetOutputSizePixel( const Size& rSize )
	                        { SetTabPageSizePixel( rSize ); }
	Size                GetOutputSizePixel() const
	                        { return GetTabPageSizePixel(); }
#endif
	 void                InsertPage( const ResId& rResId,
	                                USHORT nPos = TAB_APPEND );
	void                InsertPage( USHORT nPageId, const XubString& rText,
	                                USHORT nPos = TAB_APPEND );
	void                RemovePage( USHORT nPageId );
	void                Clear();
	 USHORT              GetPageCount() const;
	USHORT              GetPageId( USHORT nPos ) const;
	USHORT              GetPagePos( USHORT nPageId ) const;
	USHORT              GetPageId( const Point& rPos ) const;
	 void                SetCurPageId( USHORT nPageId );
	USHORT              GetCurPageId() const;
	 void                SetFirstPageId( USHORT nPageId );
	USHORT              GetFirstPageId() const { return GetPageId( mnFirstPagePos ); }
	 void                SelectTabPage( USHORT nPageId );
	 void                SetMaxPageWidth( long nMaxWidth ) { mnMaxPageWidth = nMaxWidth; }
	long                GetMaxPageWidth() const { return mnMaxPageWidth; }
	void                ResetMaxPageWidth() { SetMaxPageWidth( 0 ); }
	BOOL                IsMaxPageWidth() const { return mnMaxPageWidth != 0; }
	 void                SetTabPage( USHORT nPageId, TabPage* pPage );
	TabPage*            GetTabPage( USHORT nPageId ) const;
	USHORT              GetTabPageResId( USHORT nPageId ) const;
	 void                SetPageText( USHORT nPageId, const XubString& rText );
	XubString           GetPageText( USHORT nPageId ) const;
	 void                SetHelpText( USHORT nPageId, const XubString& rText );
	const XubString&    GetHelpText( USHORT nPageId ) const;
	 void                SetHelpId( USHORT nPageId, ULONG nHelpId );
	ULONG               GetHelpId( USHORT nPageId ) const;
	 void                SetHelpText( const XubString& rText )
	                        { Control::SetHelpText( rText ); }
	const XubString&    GetHelpText() const
	                        { return Control::GetHelpText(); }
	 void                SetHelpId( ULONG nId )
	                        { Control::SetHelpId( nId ); }
	ULONG               GetHelpId() const
	                        { return Control::GetHelpId(); }
	 void                SetActivatePageHdl( const Link& rLink ) { maActivateHdl = rLink; }
	const Link&         GetActivatePageHdl() const { return maActivateHdl; }
	void                SetDeactivatePageHdl( const Link& rLink ) { maDeactivateHdl = rLink; }
	const Link&         GetDeactivatePageHdl() const { return maDeactivateHdl; }
};

#endif  
