

#ifndef _SV_WALL_HXX
#define _SV_WALL_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_COLOR_HXX
#include <vcl/color.hxx>
#endif

class Rectangle;
class Gradient;
class BitmapEx;

#define	WALLPAPER_NULL 					WallpaperStyle_NULL
#define	WALLPAPER_TILE					WallpaperStyle_TILE
#define	WALLPAPER_CENTER				WallpaperStyle_CENTER
#define	WALLPAPER_SCALE					WallpaperStyle_SCALE
#define	WALLPAPER_TOPLEFT				WallpaperStyle_TOPLEFT
#define	WALLPAPER_TOP					WallpaperStyle_TOP
#define	WALLPAPER_TOPRIGHT				WallpaperStyle_TOPRIGHT
#define	WALLPAPER_LEFT					WallpaperStyle_LEFT
#define	WALLPAPER_RIGHT					WallpaperStyle_RIGHT
#define	WALLPAPER_BOTTOMLEFT			WallpaperStyle_BOTTOMLEFT
#define	WALLPAPER_BOTTOM				WallpaperStyle_BOTTOM
#define	WALLPAPER_BOTTOMRIGHT			WallpaperStyle_BOTTOMRIGHT
#define	WALLPAPER_FORCE_EQUAL_SIZE		WallpaperStyle_FORCE_EQUAL_SIZE

#ifndef ENUM_WALLPAPERSTYLE_DECLARED
#define ENUM_WALLPAPERSTYLE_DECLARED

enum WallpaperStyle 
{
	WALLPAPER_NULL, 
	WALLPAPER_TILE,
	WALLPAPER_CENTER,
	WALLPAPER_SCALE,
	WALLPAPER_TOPLEFT,		
	WALLPAPER_TOP,		
	WALLPAPER_TOPRIGHT,
	WALLPAPER_LEFT,								
	WALLPAPER_RIGHT,
	WALLPAPER_BOTTOMLEFT,	
	WALLPAPER_BOTTOM,	
	WALLPAPER_BOTTOMRIGHT,
	WALLPAPER_FORCE_EQUAL_SIZE = 0x7fffffff
};

#endif

class SvStream;

class ImplWallpaper
{
	friend class Wallpaper;

private:
	Color			maColor;
	BitmapEx*		mpBitmap;
	Gradient*		mpGradient;
	Rectangle*		mpRect;
	WallpaperStyle	meStyle;
	USHORT			mnRefCount;
	BitmapEx*		mpCache;

	friend SvStream& operator>>( SvStream& rIStm, ImplWallpaper& rImplWallpaper );
	friend SvStream& operator<<( SvStream& rOStm, const ImplWallpaper& rImplWallpaper );

#ifdef _SV_WALL_CXX
public:
		ImplWallpaper();
		ImplWallpaper( const ImplWallpaper& rImplWallpaper );
		~ImplWallpaper();

	void			ImplSetCachedBitmap( BitmapEx& rBmp );
	const BitmapEx*	ImplGetCachedBitmap() { return mpCache; }
	void			ImplReleaseCachedBitmap();
#endif
};

class Wallpaper
{
private:
	ImplWallpaper*	mpImplWallpaper;

#ifdef _SV_WALL_CXX
	void			ImplMakeUnique( BOOL bReleaseCache = TRUE );
public:
	ImplWallpaper*	ImplGetImpWallpaper() const { return mpImplWallpaper; }
#endif

public:
		Wallpaper();
		Wallpaper( const Wallpaper& rWallpaper );
		Wallpaper( const Color& rColor );
		Wallpaper( const BitmapEx& rBmpEx );
		Wallpaper( const Gradient& rGradient );
		~Wallpaper();

	void			SetColor( const Color& rColor );
	const Color&	GetColor() const { return mpImplWallpaper->maColor; }

	void			SetStyle( WallpaperStyle eStyle );
	WallpaperStyle	GetStyle() const { return mpImplWallpaper->meStyle; }

	void			SetBitmap( const BitmapEx& rBitmap );
	void			SetBitmap();
	BitmapEx		GetBitmap() const;
	BOOL			IsBitmap() const { return (mpImplWallpaper->mpBitmap != 0); }

	void			SetGradient( const Gradient& rGradient );
	void			SetGradient();
	Gradient		GetGradient() const;
	BOOL			IsGradient() const { return (mpImplWallpaper->mpGradient != 0); }

	void			SetRect( const Rectangle& rRect );
	void			SetRect();
	Rectangle		GetRect() const;
	BOOL			IsRect() const { return (mpImplWallpaper->mpRect != 0); }

	BOOL			IsFixed() const;
	BOOL			IsScrollable() const;

	Wallpaper&		operator=( const Wallpaper& rWallpaper );
	BOOL			operator==( const Wallpaper& rWallpaper ) const;
	BOOL			operator!=( const Wallpaper& rWallpaper ) const
			{ return !(Wallpaper::operator==( rWallpaper )); }
	BOOL			IsSameInstance( const Wallpaper& rWallpaper ) const
			{ return (mpImplWallpaper == rWallpaper.mpImplWallpaper); }

	friend SvStream& operator>>( SvStream& rIStm, Wallpaper& rWallpaper );
	friend SvStream& operator<<( SvStream& rOStm, const Wallpaper& rWallpaper );
};

#endif	
