

#ifndef _XMLOFF_TXTVFLDI_HXX
#define _XMLOFF_TXTVFLDI_HXX

#ifndef _XMLOFF_TXTVFLDI_HXX
#include <xmloff/txtvfldi.hxx>
#endif

#ifndef _XMLOFF_TXTFLDI_HXX
#include <xmloff/txtfldi.hxx>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSETINFO_HPP_
#include <com/sun/star/beans/XPropertySetInfo.hpp>
#endif

enum VarType 
{ 
	VarTypeSimple, 
	VarTypeUserField, 
	VarTypeSequence
};

class XMLValueImportHelper
{

	const ::rtl::OUString sPropertyContent;
	const ::rtl::OUString sPropertyValue;
	const ::rtl::OUString sPropertyFormula;
	const ::rtl::OUString sPropertyNumberFormat;

	SvXMLImport& rImport;
	XMLTextImportHelper& rHelper;

	::rtl::OUString sValue;		
	double fValue;				
	sal_Int32 nFormatKey;		
	::rtl::OUString sFormula;	
	::rtl::OUString sDefault;	

	sal_Bool bStringType; 		
	sal_Bool bFormatOK;			
	sal_Bool bTypeOK;			
	sal_Bool bStringValueOK;	
	sal_Bool bFloatValueOK;		
	sal_Bool bFormulaOK;		

	const sal_Bool bSetType;	
	const sal_Bool bSetValue;	
	const sal_Bool bSetStyle;	
	const sal_Bool bSetFormula;	

	const sal_Bool bStringDefault;	
	const sal_Bool bFormulaDefault;	

public:
	XMLValueImportHelper(
		SvXMLImport& rImprt,					
		XMLTextImportHelper& rHlp,				
		sal_Bool bType, 						
		sal_Bool bStyle,						
		sal_Bool bValue, 						
		sal_Bool bFormula);						

	virtual void ProcessAttribute( sal_uInt16 nAttrToken,
					   const ::rtl::OUString& sAttrValue );

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);

	inline sal_Bool IsStringValue() { return bStringType; }

	inline sal_Bool IsFormatOK() { return bFormatOK; }

	inline void SetDefault(const ::rtl::OUString& sStr) { sDefault = sStr; }
};

class XMLVarFieldImportContext : public XMLTextFieldImportContext
{
protected:
	const ::rtl::OUString sPropertyContent;
	const ::rtl::OUString sPropertyHint;
	const ::rtl::OUString sPropertyIsVisible;
	const ::rtl::OUString sPropertyIsDisplayFormula;
	const ::rtl::OUString sPropertyCurrentPresentation;

private:
	::rtl::OUString sName;				
	::rtl::OUString sFormula;			
	::rtl::OUString sDescription;		
	XMLValueImportHelper aValueHelper;	
	sal_Bool bDisplayFormula;			
	sal_Bool bDisplayNone;				

	sal_Bool bNameOK;					
	sal_Bool bFormulaOK;				
	sal_Bool bDescriptionOK;			
	sal_Bool bDisplayOK;				

	sal_Bool bSetName;					
	sal_Bool bSetFormula;				
	sal_Bool bSetFormulaDefault;		
	sal_Bool bSetDescription;			
	sal_Bool bSetVisible;				
	sal_Bool bSetDisplayFormula;		
	sal_Bool bSetPresentation;			

public:

	TYPEINFO();

	XMLVarFieldImportContext(

		SvXMLImport& rImport,			
		XMLTextImportHelper& rHlp,		
		const sal_Char* pServiceName,	
		sal_uInt16 nPrfx,				
		const ::rtl::OUString& rLocalName,	

		sal_Bool bName,					
		sal_Bool bFormula,				
		sal_Bool bFormulaDefault,		
		sal_Bool bDescription,			
		sal_Bool bVisible,				
		sal_Bool bDisplayFormula,		
		sal_Bool bType,					
		sal_Bool bStyle,				
		sal_Bool bValue,				
		sal_Bool bPresentation);		

protected:

	virtual void ProcessAttribute( sal_uInt16 nAttrToken,
					   const ::rtl::OUString& sAttrValue );

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);

	inline ::rtl::OUString GetName()	{ return sName; }
	inline sal_Bool IsStringValue()		{ return aValueHelper.IsStringValue();}
	inline sal_Bool IsNameOK()			{ return bNameOK; }
	inline sal_Bool IsFormulaOK()		{ return bFormulaOK; }
	inline sal_Bool IsDescriptionOK()	{ return bDescriptionOK; }
	inline sal_Bool IsDisplayOK()		{ return bDisplayOK; }
};

class XMLVariableGetFieldImportContext : public XMLVarFieldImportContext
{
	const ::rtl::OUString sPropertySubType;

public:

	TYPEINFO();

	XMLVariableGetFieldImportContext(
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		

protected:

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLExpressionFieldImportContext : public XMLVarFieldImportContext
{
	const ::rtl::OUString sPropertySubType;

public:

	TYPEINFO();

	XMLExpressionFieldImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& sLocalName);		

protected:
	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLTextInputFieldImportContext : public XMLVarFieldImportContext
{
	const ::rtl::OUString sPropertyContent;

public:

	TYPEINFO();

	XMLTextInputFieldImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& sLocalName);		

protected:
	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLSetVarFieldImportContext : public XMLVarFieldImportContext
{
	const VarType eFieldType;

public:

	TYPEINFO();

	XMLSetVarFieldImportContext( 

		SvXMLImport& rImport,			
		XMLTextImportHelper& rHlp,		
		const sal_Char* pServiceName,	
		sal_uInt16 nPrfx,				
		const ::rtl::OUString& rLocalName, 

		VarType eVarType,				

		sal_Bool bName,					
		sal_Bool bFormula,				
		sal_Bool bFormulaDefault,		
		sal_Bool bDescription,			
		sal_Bool bVisible,				
		sal_Bool bDisplayFormula,		
		sal_Bool bType,					
		sal_Bool bStyle,				
		sal_Bool bValue,				
		sal_Bool bPresentation);		

protected:

	virtual void EndElement();

	sal_Bool FindFieldMaster(
		::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xMaster);
};

class XMLVariableSetFieldImportContext : public XMLSetVarFieldImportContext
{
	const ::rtl::OUString sPropertySubType;

public:

	TYPEINFO();

	XMLVariableSetFieldImportContext(
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		

protected:

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLVariableInputFieldImportContext : public XMLSetVarFieldImportContext
{
	const ::rtl::OUString sPropertySubType;
	const ::rtl::OUString sPropertyIsInput;

public:

	TYPEINFO();

	XMLVariableInputFieldImportContext(
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx, 						
		const ::rtl::OUString& rLocalName);		

protected:

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLUserFieldImportContext : public XMLSetVarFieldImportContext
{

public:

	TYPEINFO();

	XMLUserFieldImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		
};

class XMLUserFieldInputImportContext : public XMLVarFieldImportContext
{

public:

	TYPEINFO();

	XMLUserFieldInputImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLSequenceFieldImportContext : public XMLSetVarFieldImportContext
{
	const ::rtl::OUString sPropertyFormula;
	const ::rtl::OUString sPropertyNumberFormat;
	const ::rtl::OUString sPropertySequenceValue;
	::rtl::OUString sFormula;
	::rtl::OUString sNumFormat;
	::rtl::OUString sNumFormatSync;
	::rtl::OUString sRefName;

	sal_Bool bRefNameOK;

public:

	TYPEINFO();

	XMLSequenceFieldImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		

protected:

	virtual void ProcessAttribute( sal_uInt16 nAttrToken,
					   const ::rtl::OUString& sAttrValue );

	virtual void PrepareField( 
		const ::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet> & xPropertySet);
};

class XMLVariableDeclsImportContext : public SvXMLImportContext
{
	enum VarType eVarDeclsContextType;
	XMLTextImportHelper& rImportHelper;

public:

	TYPEINFO();

	XMLVariableDeclsImportContext(
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName,		
		enum VarType eVarType);					

	virtual SvXMLImportContext *CreateChildContext( 
		sal_uInt16 nPrefix,
		const ::rtl::OUString& rLocalName,
		const ::com::sun::star::uno::Reference< 
		::com::sun::star::xml::sax::XAttributeList> & xAttrList );
};

class XMLVariableDeclImportContext : public SvXMLImportContext
{
	const ::rtl::OUString sPropertyName;
	const ::rtl::OUString sPropertySubType;
	const ::rtl::OUString sPropertyNumberingLevel;
	const ::rtl::OUString sPropertyNumberingSeparator;
	const ::rtl::OUString sPropertyIsExpression;

	::rtl::OUString sName;
	XMLValueImportHelper aValueHelper;
	sal_Int8 nNumLevel;
	sal_Unicode cSeparationChar;

public:

	TYPEINFO();

	XMLVariableDeclImportContext( 
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName,		
		const ::com::sun::star::uno::Reference< 
		::com::sun::star::xml::sax::XAttributeList> & xAttrList,
		enum VarType eVarType);					

	static sal_Bool FindFieldMaster(::com::sun::star::uno::Reference<
						::com::sun::star::beans::XPropertySet> & xMaster,
						SvXMLImport& rImport, 
						XMLTextImportHelper& rHelper,
						const ::rtl::OUString& sVarName,
						enum VarType eVarType);
};

class XMLDatabaseDisplayImportContext : public XMLDatabaseFieldImportContext
{
	const ::rtl::OUString sPropertyColumnName;
	const ::rtl::OUString sPropertyDatabaseFormat;
	const ::rtl::OUString sPropertyCurrentPresentation;

	XMLValueImportHelper aValueHelper;

	::rtl::OUString sColumnName;
	sal_Bool bColumnOK;

public:

	TYPEINFO();

	XMLDatabaseDisplayImportContext(
		SvXMLImport& rImport,					
		XMLTextImportHelper& rHlp,				
		sal_uInt16 nPrfx,						
		const ::rtl::OUString& rLocalName);		

protected:

	virtual void ProcessAttribute( sal_uInt16 nAttrToken,
					   const ::rtl::OUString& sAttrValue );

	virtual void EndElement();
};

#endif
